@echo off

setlocal

set LOG_FILE=%cd%\samurai-graph.log

rem set directory

set CURRENT_PATH=%cd%\
set SCRIPT_PATH=%~d0%~p0
set CONTRIB_PATH=%SCRIPT_PATH%..
set TOOLS_PATH=%SCRIPT_PATH%..\..\tools
set FREEHEP_PATH=%SCRIPT_PATH%

set FREEHEP_EXPORT_PATH=%FREEHEP_PATH%freehep-export
set FREEHEP_VECTORGRAPHICS_PATH=%FREEHEP_PATH%vectorgraphics\src

rem set absolute directory

cd /d %SCRIPT_PATH%
set SCRIPT_PATH=%cd%\
cd /d %CONTRIB_PATH%
set CONTRIB_PATH=%cd%\
cd /d %TOOLS_PATH%
set TOOLS_PATH=%cd%\
cd /d %FREEHEP_PATH%
set FREEHEP_PATH=%cd%\
cd /d %FREEHEP_EXPORT_PATH%
set FREEHEP_EXPORT_PATH=%cd%\
cd /d %FREEHEP_VECTORGRAPHICS_PATH%
set FREEHEP_VECTORGRAPHICS_PATH=%cd%\

cd /d %CURRENT_PATH%

rem set done file

set DONE_FILE=%CONTRIB_PATH%_build_freehep_done

rem set maven

set MAVEN_HOME=%TOOLS_PATH%maven\apache-maven-2.2.1
cd /d %MAVEN_HOME%
set MAVEN_HOME=%cd%\

set MVN=%MAVEN_HOME%bin\mvn.bat


goto check_arguments



rem build functions

:build_freehep
set errcode=0
cd /d "%SCRIPT_PATH%"
if exist "%DONE_FILE%" goto build_freehep_done

  echo [freehep] creating freehep-export jar file ...
  cd %FREEHEP_EXPORT_PATH%
  call %MVN%  > "%LOG_FILE%" 2>&1
  if errorlevel 1 goto build_freehep_failure
  
  echo [freehep] creating vectorgraphics jar files ...
  cd %FREEHEP_VECTORGRAPHICS_PATH%
  call %MVN%  install assembly:attached  > "%LOG_FILE%" 2>&1
  if errorlevel 1 goto build_freehep_failure
  
  goto build_freehep_success
  :build_freehep_failure
  set errcode=1
  echo error
  type "%LOG_FILE%"
  goto build_freehep_clean
  :build_freehep_success
  copy nul "%DONE_FILE%" > nul 2>&1
  :build_freehep_clean
  del /f "%LOG_FILE%"
:build_freehep_done
cd /d "%CURRENT_PATH%"
goto %RETURN_LABEL%



rem clean

:clean_freehep
set errcode=0
cd /d "%SCRIPT_PATH%"
if not exist "%DONE_FILE%" goto clean_freehep_done

  echo [freehep] deleting freehep-export jar file ...
  cd %FREEHEP_EXPORT_PATH%
  call %MVN% clean  > "%LOG_FILE%" 2>&1
  echo [freehep] deleting vectorgraphics jar files ...
  cd %FREEHEP_VECTORGRAPHICS_PATH%
  call %MVN% clean  > "%LOG_FILE%" 2>&1

  if errorlevel 1 goto clean_freehep_failure
  goto clean_freehep_success
  :clean_freehep_failure
  set errcode=1
  echo error
  type "%LOG_FILE%"
  goto clean_freehep_clean
  :clean_freehep_success
  del /f "%DONE_FILE%"
  :clean_freehep_clean
  del /f "%LOG_FILE%"
:clean_freehep_done
cd /d "%CURRENT_PATH%"
goto %RETURN_LABEL%



:check_arguments
if "%1" == ""        goto print_usage
if "%1" == "build"   goto do_build
if "%1" == "rebuild" goto do_rebuild
if "%1" == "clean"   goto do_clean

:print_usage
echo Usage: %0 { build, rebuild, clean }
goto finised

:do_build
set RETURN_LABEL=finised
goto build_freehep

:do_rebuild
set RETURN_LABEL=do_rebuild2
goto clean_freehep
:do_rebuild2
if %errcode% == 1 goto finised
set RETURN_LABEL=finised
goto build_freehep

:do_clean
set RETURN_LABEL=finised
goto clean_freehep

:finised

endlocal
