package jp.riken.brain.ni.samuraigraph.application;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.swing.AbstractButton;
import javax.swing.ImageIcon;

import javax.swing.JRadioButton;


/**
 * A wizard dialog to select the data type.
 *
 */
public abstract class SGDataTypeSelectionWizardDialog extends SGWizardDialog
        implements SGIApplicationTextConstants {

	private static final long serialVersionUID = 871471670381557184L;
	
    /**
     * The title of this dialog.
     */
    public static final String TITLE = "Select The Data Type";

    protected static final String SINGLE_XY_ICON_FILENAME = "SingleSXY.png";

    protected static final String MULTIPLE_XY_ICON_FILENAME = "MultipleSXY.png";

    protected static final String ORTHOGONAL_ICON_FILENAME = "OrthogonalVXY.png";

    protected static final String POLAR_ICON_FILENAME = "PolarVXY.png";

    protected static final String PSEUDOCOLOR_MAP_ICON_FILENAME = "PseudocolorMap.png";

    // array of filename of icons
    protected static final String[] ICON_FILENAME_ARRAY = {
            SINGLE_XY_ICON_FILENAME, MULTIPLE_XY_ICON_FILENAME,
            ORTHOGONAL_ICON_FILENAME, POLAR_ICON_FILENAME,
            PSEUDOCOLOR_MAP_ICON_FILENAME
    };
    
    protected static final String SINGLE_SXY = "Single Scalar-XY";

    protected static final String MULTIPLE_SXY = "Multiple Scalar-XY";
    
    protected static final String ORTHOGONAL_VXY = "Orthogonal Vector-XY";
    
    protected static final String POLAR_VXY = "Polar Vector-XY";
    
    protected static final String PSEUDOCOLOR_MAP = "Pseudocolor Map";
    
    /**
     * Builds a dialog.
     * 
     */
    public SGDataTypeSelectionWizardDialog() {
        super();
    }
    
    /**
     * Builds a dialog.
     * 
     * @param owner
     *           the owner of the dialog
     * @param modal
     *           true for modal dialog
     */
    public SGDataTypeSelectionWizardDialog(Frame owner, boolean modal) {
        super(owner, modal);
    }

    /**
     * A class with data type and a radio button.
     */
    protected static class DataTypeButton {
        String dataType;
        AbstractButton btn;
        DataTypeButton(String dataType, AbstractButton btn) {
            super();
            this.dataType = dataType;
            this.btn = btn;
        }
    }

    /**
     * An array of data type buttons.
     */
    protected DataTypeButton[] mDataTypeButtonArray = null;

    /**
     * Returns the selected data type.
     */
    public String getSelectedDataType() {
        String str = null;
        for (int ii = 0; ii < this.mDataTypeButtonArray.length; ii++) {
            if (this.mDataTypeButtonArray[ii].btn.isSelected()) {
                str = this.mDataTypeButtonArray[ii].dataType;
                break;
            }
        }
        return str;
    }
    
    /**
     * A list of available data types.
     */
    private List<String> mAvailableDataTypeList = new ArrayList<String>();

    /**
     * Set available data types.
     * 
     * @param avList
     *             a list of available data types.
     * @return
     *             true if succeeded
     */
    public boolean setAvailableDataType(final List<String> avList) {
        
        // check input values
        if (avList == null) {
            return false;
        }
        
        // clear attributes
        this.mAvailableDataTypeList.clear();

        // set available buttons
        for (int ii = 0; ii < this.mDataTypeButtonArray.length; ii++) {
            DataTypeButton db = this.mDataTypeButtonArray[ii];
            final boolean available = avList.contains(db.dataType);
            db.btn.setEnabled(available);
        }

        // avoid the selection of unavailable buttons
        for (int ii = 0; ii < this.mDataTypeButtonArray.length; ii++) {
            DataTypeButton db = this.mDataTypeButtonArray[ii];
            if (avList.contains(db.dataType)) {
                db.btn.setSelected(true);
                break;
            }
        }
        
        // set to attribute
        this.mAvailableDataTypeList.addAll(avList);

        return true;
    }
    
    /**
     * Returns the data type for a given radio button. 
     * @param btn
     *            a radio button
     * @return
     *            the data type.
     */
    public String getDataType(JRadioButton btn) {
        String str = null;
        for (int ii = 0; ii < this.mDataTypeButtonArray.length; ii++) {
            if (this.mDataTypeButtonArray[ii].btn.equals(btn)) {
                str = this.mDataTypeButtonArray[ii].dataType;
                break;
            }
        }
        return str;
    }

    /**
     * Returns the button for a given data type.
     *
     * @param dataTypes
     *           the data type
     * @return the button for a given data type
     */
    public AbstractButton getButton(String dataType) {
        AbstractButton btn = null;
        for (int ii = 0; ii < this.mDataTypeButtonArray.length; ii++) {
            if (this.mDataTypeButtonArray[ii].dataType.equals(dataType)) {
                btn = this.mDataTypeButtonArray[ii].btn;
                break;
            }
        }
        return btn;
    }

    /**
     * Returns the button to select the single scalar type graph.
     * 
     * @return the button to select the single scalar type graph
     */
    protected abstract AbstractButton getSingleSXYGraphButton();

    /**
     * Returns the button to select the multiple scalar type graph.
     * 
     * @return the button to select the multiple scalar type graph
     */
    protected abstract AbstractButton getMultipleSXYGraphButton();

    /**
     * Returns the button to select the orthogonal vector graph.
     * 
     * @return the button to select the orthogonal vector graph
     */
    protected abstract AbstractButton getOrthogonalVectorGraphButton();

    /**
     * Returns the button to select the polar vector graph.
     *
     * @return the button to select the polar vector graph
     */
    protected abstract AbstractButton getPolarVectorGraphButton();

    /**
     * Returns the button to select the pseudocolor map.
     *
     * @return the button to select the pseudocolor map
     */
    protected abstract AbstractButton getPseudocolorMapButton();

    /**
     * Returns whether polar mode is selected.
     * 
     * @return true if polar mode is selected
     */
    public boolean isPolarSelected() {
        return this.getPolarVectorGraphButton().isSelected();
    }

    /**
     * Overrode to set availability of the text field.
     */
    public void setVisible(final boolean b) {
        this.updateComponents();
        super.setVisible(b);
    }

    /**
     * Update the components.
     *
     */
    protected void updateComponents() {
    }
    
    /**
     * Overrode to set availability of the text field.
     */
    public void actionPerformed(final ActionEvent e) {
        this.updateComponents();
        super.actionPerformed(e);
    }
    
    /**
	 * Returns whether the radio button for multiple data is selected.
	 * 
	 * @return true if the radio button for multiple data is selected, false not selected and null if the radio button is disabled
	 */
    public Boolean isMultipleSelected() {
        AbstractButton btn = this.getMultipleSXYGraphButton();
        if (btn.isEnabled()) {
            return Boolean.valueOf(btn.isSelected());
        } else {
            return null;
        }
    }

    protected ImageIcon createIcon(final String name) {
        URL url = this.getClass().getResource(RESOURCES_DIRNAME + name);
        ImageIcon icon = new ImageIcon(url);
        return icon;
    }

    protected void setupButton(AbstractButton btn, Map<String, ImageIcon> iconMap, String name, String tooltipText) {
        btn.setIcon(iconMap.get(name));
        btn.setToolTipText(tooltipText);
    }

}
