package jp.riken.brain.ni.samuraigraph.application;

import java.io.File;
import java.util.ArrayList;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import jp.riken.brain.ni.samuraigraph.base.SGFileChooser;

/**
 * A wizard dialog to choose a file.
 */
public abstract class SGFileChooserWizardDialog extends SGWizardDialog {

    /**
     * Serial Version UID
     */
    private static final long serialVersionUID = -7199760553826426756L;
    
    /**
     * The file chooser.
     */
    protected JFileChooser mFileChooser = null;

    /**
     * Builds this dialog.
     */
    public SGFileChooserWizardDialog() {
        super();
        this.initProperty();
    }

    /**
     * Creates new form SGFileChooserWizardDialog
     * 
     */
    public SGFileChooserWizardDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        this.initProperty();
    }

    /**
     * Creates new form SGFileChooserWizardDialog
     * 
     */
    public SGFileChooserWizardDialog(java.awt.Dialog parent, boolean modal) {
        super(parent, modal);
        this.initProperty();
    }

    /**
     * Initialize this dialog.
     */
    private void initProperty() {
        this.mFileChooser = new SGFileChooser();
    }

    /**
     * Sets the file path and returns a file.
     * 
     * @param dir
     *             directory name
     * @param name
     *             file name
     * @return
     *             a file object
     */
    public File setSelectedFile(String dir, String name) {
        if (name == null) {
            name = "";
        }

        // set current directory
        this.mFileChooser.setCurrentDirectory(new File(dir));

        // create the full path name
        StringBuffer sb = new StringBuffer();
        sb.append(dir);
        sb.append(FILE_SEPARATOR);
        sb.append(name);
        String path = sb.toString();
        
        // set the path name
        return this.setSelectedFile(path);
    }
    
    /**
     * Sets the file name and returns a file.
     * 
     * @param path
     *             file path
     * @return
     *             a file object
     */
    public File setSelectedFile(String path) {
        File f = new File(path);
        return this.setSelectedFile(f);
    }

    /**
     * Sets the file name and returns a file.
     * 
     * @param path
     *             file path
     * @return
     *             a file object
     */
    public File setSelectedFile(File f) {
        if (f.exists()) {
            this.mFileChooser.setSelectedFile(f);
        }
        return f;
    }

    /**
     * Returns the selected file.
     * 
     * @return the selected file
     */
    public File getSelectedFile() {
        return this.mFileChooser.getSelectedFile();
    }

    /**
     * Sets the current file filter to the attribute.
     * 
     * @param ff
     */
    public void setFileFilter(FileFilter ff) {
        this.mFileChooser.setFileFilter(ff);
    }

    /**
     * 
     */
    long mShowFileChooserTime = 0L;

    long lastUsed() {
        return this.mShowFileChooserTime;
    }

    /**
     * Open the file chooser dialog.
     * 
     * @return List of selected files.
     */
    protected ArrayList openFileChooser() {
        // show open dialog
        this.mShowFileChooserTime = System.currentTimeMillis();
        int ret = this.mFileChooser.showOpenDialog(this);

        // get selected files
        ArrayList fileList = new ArrayList();
        switch (ret) {
        // selected
        case JFileChooser.APPROVE_OPTION: {
            if (this.mFileChooser.isMultiSelectionEnabled()) {
                File[] files = this.mFileChooser.getSelectedFiles();
                for (int ii = 0; ii < files.length; ii++) {
                    fileList.add(files[ii]);
                }
            } else {
                fileList.add(this.mFileChooser.getSelectedFile());
            }
            break;
        }

        // canceled
        case JFileChooser.CANCEL_OPTION: {
            break;
        }

        // error
        case JFileChooser.ERROR_OPTION: {
            return null;
        }

        default: {

        }
        }

        return fileList;

    }
    
    /**
     * Returns whether a given file is acceptable as a property file.
     * @param f
     *         a file
     * @return
     *         true if a given file is acceptable
     */
    protected boolean isAcceptable(File f) {
        // the file exists and is a file
        return (f.exists()) && (f.isFile());
    }

    /**
     * Returns whether a given file is acceptable as a property file.
     * @param fileName
     *         the name of a file
     * @return
     *         true if a given file is acceptable
     */
    protected boolean isAcceptable(String fileName) {
        File f = new File(fileName);
        return this.isAcceptable(f);
    }

}
