package jp.riken.brain.ni.samuraigraph.application;

import java.awt.Color;
import java.awt.Component;
import java.awt.Insets;
import java.awt.Point;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.prefs.Preferences;

import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JScrollBar;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import jp.riken.brain.ni.samuraigraph.application.SGDataCreator.FileColumn;
import jp.riken.brain.ni.samuraigraph.base.SGButton;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfoSet;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.data.SGDataTypeConstants;
import jp.riken.brain.ni.samuraigraph.data.SGDataUtility;
import jp.riken.brain.ni.samuraigraph.data.SGIDataColumnTypeConstants;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFFile;
import jp.riken.brain.ni.samuraigraph.data.SGVariable;
import ucar.nc2.NetcdfFile;

/**
 * A dialog to choose multiple data files for the property file.
 * 
 */
public class SGPropertyDataFileChooserWizardDialog extends SGWizardDialog
        implements TableModelListener, DropTargetListener, ComponentListener,
        CellEditorListener, MouseListener, MouseMotionListener,
        MouseWheelListener, SGIDataColumnTypeConstants {

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = 1614748084851553730L;

    /**
     * The title of this dialog.
     */
    public static final String TITLE = "Select the Data Files";

    /**
     * The default constructor.
     */
    public SGPropertyDataFileChooserWizardDialog() {
        super();
        initComponents();
        this.initProperty();
    }

    /** Creates new form SGDataFileChooseWizardDialog */
    public SGPropertyDataFileChooserWizardDialog(java.awt.Frame parent,
            boolean modal) {
        super(parent, modal);
        initComponents();
        this.initProperty();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        mPanel = new javax.swing.JPanel();
        mScrollPane = new javax.swing.JScrollPane();
        mTable = this.createTableInstance();
        mButtonPanel = new javax.swing.JPanel();
        mPreviousButton = new jp.riken.brain.ni.samuraigraph.base.SGButton();
        mNextButton = new jp.riken.brain.ni.samuraigraph.base.SGButton();
        mOKButton = new jp.riken.brain.ni.samuraigraph.base.SGButton();
        mCancelButton = new jp.riken.brain.ni.samuraigraph.base.SGButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        mPanel.setFont(new java.awt.Font("SansSerif", 0, 11));
        mPanel.setLayout(new java.awt.GridBagLayout());

        mScrollPane.setFont(new java.awt.Font("Dialog", 0, 11));
        mScrollPane.setPreferredSize(new java.awt.Dimension(468, 200));

        mTable.setFont(new java.awt.Font("Dialog", 0, 11)); // NOI18N
        mScrollPane.setViewportView(mTable);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 0, 0, 0);
        mPanel.add(mScrollPane, gridBagConstraints);

        getContentPane().add(mPanel, new java.awt.GridBagConstraints());

        mButtonPanel.setFont(new java.awt.Font("Dialog", 0, 11));

        mPreviousButton.setText("<Prev");
        mPreviousButton.setFont(new java.awt.Font("Dialog", 1, 12));
        mButtonPanel.add(mPreviousButton);

        mNextButton.setText("Next>");
        mNextButton.setFont(new java.awt.Font("Dialog", 1, 12));
        mButtonPanel.add(mNextButton);

        mOKButton.setText("OK");
        mOKButton.setFont(new java.awt.Font("Dialog", 1, 12));
        mButtonPanel.add(mOKButton);

        mCancelButton.setText("Cancel");
        mCancelButton.setFont(new java.awt.Font("Dialog", 1, 12));
        mButtonPanel.add(mCancelButton);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        getContentPane().add(mButtonPanel, gridBagConstraints);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel mButtonPanel;
    private jp.riken.brain.ni.samuraigraph.base.SGButton mCancelButton;
    private jp.riken.brain.ni.samuraigraph.base.SGButton mNextButton;
    private jp.riken.brain.ni.samuraigraph.base.SGButton mOKButton;
    private javax.swing.JPanel mPanel;
    private jp.riken.brain.ni.samuraigraph.base.SGButton mPreviousButton;
    private javax.swing.JScrollPane mScrollPane;
    protected javax.swing.JTable mTable;
    // End of variables declaration//GEN-END:variables

    /**
     * Name of the column for the figure ID.
     */
    public static final String COLUMN_NAME_FIGURE_ID = "Figure ID";

    /**
     * Name of the column for the data type.
     */
    public static final String COLUMN_NAME_DATA_TYPE = "Data Type";

    /**
     * Name of the column for the data name.
     */
    public static final String COLUMN_NAME_DATA_NAME = "Data Name";

    /**
     * Name of the column for the file name.
     */
    public static final String COLUMN_NAME_FILE_NAME = "File Name";

    /**
     * Name of the column for the file name.
     */
    public static final String COLUMN_NAME_BUTTON = "";

    /**
     * A string to represent no data.
     */
    public static final String NO_DATA = "NO_DATA";

    /**
     * An array of column name.
     */
    public static final String[] COLUMN_NAME_ARRAY = { COLUMN_NAME_FIGURE_ID,
            COLUMN_NAME_DATA_TYPE, COLUMN_NAME_DATA_NAME, COLUMN_NAME_FILE_NAME,
            COLUMN_NAME_BUTTON };

    /**
     * The height of each row.
     */
    public static final int ROW_HEIGHT = 20;

    /**
     * A constant array of column width.
     */
    private static final int[] PREFERRED_COLUMN_WIDTH_ARRAY = { 50, 80, 80, 120, 20 };

    /**
     * The unit increment of the scroll bar.
     */
    private static final int SCROLL_BAR_UNIT_INCREMENT = ROW_HEIGHT;
    
    /**
     * The block increment of the scroll bar.
     */
    private static final int SCROLL_BAR_BLOCK_INCREMENT = 4 * SCROLL_BAR_UNIT_INCREMENT;

    /**
     * The table model.
     */
    private DefaultTableModel mTableModel = null;

    /**
     * An array of data files.
     */
    private DataFile[] mDataFileArray = null;
    
    /**
     * Data creator.
     */
    private SGDataCreator mDataCreator = null;

    /**
     * A dialog to setup the data columns.
     */
    private SGPropertyFileDataDialog mPropertyFileDataDialog = null;

    /**
     * A dialog to select the data file.
     */
    private SGSingleDataFileChooserWizardDialog mSingleDataFileChooserWizardDilaog = null;
    
    /**
     * The row index setting up the data file or URL.
     */
    private int mCurDataSetupRowIndex = -1;

    /**
     * The row index currently focused.
     */
    private int mFocusedRowIndex = -1;

    /**
     * The version number of the property file.
     */
    private String mVersionNumber = null;

    /**
     * Initialize this dialog.
     */
    private boolean initProperty() {
    	
        // set the title
        this.setTitle(SGPropertyDataFileChooserWizardDialog.TITLE);

        // table model
        this.mTableModel = new OriginalTableModel();
        this.mTable.setModel(this.mTableModel);
        this.mTableModel.addTableModelListener(this);

        // set the height of all rows
        this.mTable.setRowHeight(ROW_HEIGHT);

        // data creator
        this.mDataCreator = new SGDataCreator();

        // drag and drop
        DropTarget target = new DropTarget(this.mTable,
                DnDConstants.ACTION_COPY_OR_MOVE, this, true);
        this.setDropTarget(target);

        // add the table as an event listener
        this.mTable.addComponentListener(this);
        this.mTable.addMouseListener(this);
        this.mTable.addMouseMotionListener(this);
        this.mTable.addMouseWheelListener(this);

        // set up the scroll bar properties
        JScrollBar vertBar = this.mScrollPane.getVerticalScrollBar();
        vertBar.setUnitIncrement(SCROLL_BAR_UNIT_INCREMENT);
        vertBar.setBlockIncrement(SCROLL_BAR_BLOCK_INCREMENT);
        
        return true;
    }

    /**
     * 
     */
    public void componentResized(ComponentEvent e) {
        super.componentResized(e);
        this.updateTable();
    }

    /**
     * 
     */
    public void componentShown(ComponentEvent e) {
        super.componentShown(e);
        this.updateTable();
    }

    private void updateTable() {
        final int height = this.mTable.getHeight();
        if (height != 0) {
            this.setPreferredColumnWidth();
        }
    }

    /**
     * Returns the number of data.
     *
     * @return the number of data
     */
    public int getDataNum() {
        if (this.mDataFileArray == null) {
            return 0;
        } else {
            return this.mDataFileArray.length;
        }
    }

    /**
     * A class for a single data file.
     */
    private class DataFile {
        int figureId = -1;

        String dataType = null;

        String dataName = null;

        String fileName = null;

        Map<String, Object> infoMap = new HashMap<String, Object>();

        SGDataColumnInfoSet dataColInfoSet = null;
        
        boolean hasData = true;

        DataFile(int figureId, String dataType, String dataName, Map<String, Object> infoMap) {
            this.figureId = figureId;
            this.dataType = dataType;
            this.dataName = dataName;
            this.infoMap = infoMap;
        }
    }

    /**
     * Returns an array of property file data.
     *
     * @return an array of property file data
     */
    public SGPropertyFileData[] getPropertyFileDataArray() {
        final int dataNum = this.getDataNum();
        SGPropertyFileData[] pFile = new SGPropertyFileData[dataNum];
        for (int ii = 0; ii < dataNum; ii++) {
            DataFile df = this.mDataFileArray[ii];
            pFile[ii] = new SGPropertyFileData(df.figureId, df.dataType,
                    df.dataName, df.infoMap);
//            pFile[ii].setDataColumnInfoArray(df.dataColumnInfoArray);
            pFile[ii].setColumnInfoSet(df.dataColInfoSet);
            pFile[ii].setFileName(df.fileName);
        }
        return pFile;
    }

    /**
     *
     */
    public void actionPerformed(final ActionEvent e) {

        Object source = e.getSource();
        String command = e.getActionCommand();

        // an event from the data column selection dialog
        if (source.equals(this.mPropertyFileDataDialog)) {
            
            if (command.equals(OK_BUTTON_TEXT)) {
                // get an array of data column information for the current
                // data file and set it to an attribute
                SGPropertyFileDataDialog dg = (SGPropertyFileDataDialog) source;
                String fileName = dg.getFileName();
//                SGDataColumnInfo[] info = dg.getDataColumnInfo();
                SGDataColumnInfoSet colInfoSet = dg.getDataColumnInfoSet();
                DataFile dataFile = this.mDataFileArray[this.mCurDataSetupRowIndex];
                dataFile.dataColInfoSet = colInfoSet;
                dataFile.fileName = fileName;
                if (this.mPropertyFileDataDialog instanceof SGPropertyFileNetCDFDataDialog) {
                    SGPropertyFileNetCDFDataDialog ndg = (SGPropertyFileNetCDFDataDialog)dg;
                    Integer pstart = ndg.getMultipleDimensionStart();
                    Integer pend = ndg.getMultipleDimensionEnd();
                    Integer pstep = ndg.getMultipleDimensionStep();
                    Map<String, Object> infoMap = dataFile.infoMap;
                    if (pstart!=null && pend!=null && pstep!=null) {
                        infoMap.put(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_START, pstart);
                        infoMap.put(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_END, pend);
                        infoMap.put(SGIDataColumnTypeConstants.KEY_MULTIPLE_DIMENSION_STEP, pstep);
                    }
                }
                
                // set to the table
                final int colIndexFileName = this.getFileNameColumnIndex();
                this.mTable.setValueAt(fileName, this.mCurDataSetupRowIndex, colIndexFileName);

                // update the buttons
                this.updateAvailableButtons();
            } else if (command.equals(CANCEL_BUTTON_TEXT)) {
                if (this.mSingleDataFileChooserWizardDilaog!=null) {
                    this.mSingleDataFileChooserWizardDilaog.setVisible(true);
                }
            }
            
            this.mCurDataSetupRowIndex = -1;
        } else if (source.equals(this.mSingleDataFileChooserWizardDilaog)) {
            
            if (command.equals(OK_BUTTON_TEXT)) {
                
                if (this.mSingleDataFileChooserWizardDilaog==null) {
                    return;
                }
                
                this.mSingleDataFileChooserWizardDilaog.setVisible(false);

                // selected data file object
                DataFile dataFile = mDataFileArray[this.mCurDataSetupRowIndex];
                String dataType = dataFile.dataType;
                Map<String, Object> infoMap = dataFile.infoMap;
                SGDataColumnInfoSet dataColumnInfoSet = dataFile.dataColInfoSet;
                final boolean isArrayData = SGDataUtility.isArrayData(dataType);
                
                String fileName = this.mSingleDataFileChooserWizardDilaog.getFileName();
                if (this.showPropertyFileDataDialog(
                        fileName, isArrayData, dataType, dataColumnInfoSet, infoMap)==false) {
                    this.mSingleDataFileChooserWizardDilaog.setVisible(true);
                }
            }
            
        }

/*        
        // an event from the data column selction dialog
        if (source instanceof SGDataColumnSelectionDialog) {
            if (OK_BUTTON_TEXT.equals(command)) {
                // get an array of data column information for the current
                // data file and set it to an attribute
                SGDataColumnSelectionDialog dg = (SGDataColumnSelectionDialog) source;
                SGDataColumnInfo[] info = dg.getDataColumnInfo();
                DataFile dataFile = this.mDataFileArray[this.mCurDataFileIndex];
                dataFile.dataColumnInfoArray = info;

                // update the buttons
                this.updateAvailableButtons();
            }
        }
*/
        
/*        
        JButton[] array = this.mSingleFileChooserButtonArray;
        for (int ii = 0; ii < array.length; ii++) {
            if (source.equals(array[ii])) {
                // disable multi selection
                this.mFileChooser.setMultiSelectionEnabled(false);

                // show the file chooser
                File[] files = this.showFileChooser();
                if (files == null) {
                    return;
                }
                File f = files[0];

                // set to the table
                final int colIndex = this.getFileNameColumnIndex();
                this.mTable.setValueAt(f.getPath(), ii, colIndex);

                // set selected file
                this.setSelectedFile(ii, f);

                return;
            }
        }

        array = this.mDataColumnSelectionButtonArray;
        for (int ii = 0; ii < array.length; ii++) {
            if (source.equals(array[ii])) {
                this.mCurDataFileIndex = ii;

                if (mDataFileArray == null) {
                    SGUtility.showFileNotFoundMessageDialog(this);
                    return;
                }

                // create a dialog
                SGDataColumnSelectionDialog dg = new SGDataColumnSelectionDialog(
                        this, true);
                dg.addActionListener(this);

                // selected data file object
                DataFile dataFile = mDataFileArray[mCurDataFileIndex];

                // parse the data file
                FileColumn[] colInfo = null;
                try {
                    colInfo = mDataCreator
                            .parseFileComlumnType(dataFile.fileName);
                } catch (FileNotFoundException e1) {
                    SGUtility.showMessageDialog(this,
                            "Failed to open the data file.",
                            SGIApplicationTextConstants.TITLE_ERROR,
                            JOptionPane.WARNING_MESSAGE);
                    return;
                }

                // initialize the dialog
                final int colNum = colInfo.length;
                dg.init(colNum);

                // set information to the dialog for data column
                // selection
//                String[] titles = new String[colNum];
//                String[] valueTypes = new String[colNum];
//                for (int jj = 0; jj < colNum; jj++) {
//                    titles[jj] = new String(colInfo[jj].title);
//                    valueTypes[jj] = new String(colInfo[jj].valueType);
//                }
                SGDataColumnInfo[] colArray = new SGDataColumnInfo[colNum];
                for (int jj = 0; jj < colNum; jj++) {
                    colArray[jj] = new SGDataColumnInfo(colInfo[jj].title, colInfo[jj].valueType);
                }
                dg.setData(dataFile.dataType, colArray, dataFile.infoMap);

                // show the dialog
                dg.setLocation(this.getX() + 20, this.getY() + 20);
                dg.setVisible(true);

                dg.removeActionListener(this);

                return;
            }
        }
*/        
        
        // calls the method of the super class
        super.actionPerformed(e);
    }

    public void mouseDragged(MouseEvent e) {
    }

    public void mouseMoved(MouseEvent e) {
    	/*
        // shows the tooltip text
        Point pos = e.getPoint();
        JTable table = this.mTable;
        final int row = table.rowAtPoint(pos);
        final int column = this.getFileNameColumnIndex();
        Object obj = table.getValueAt(row, column);
        if (obj == null || "".equals(obj)) {
            table.setToolTipText(null);
            return;
        }
        String text = (String) obj;
        table.setToolTipText(text);
        */
    }

    /**
     * Scroll the vertical scroll bar.
     */
    public void mouseWheelMoved(MouseWheelEvent e) {
        final int amount = e.getScrollAmount() * e.getWheelRotation();
        JScrollBar vertBar = this.mScrollPane.getVerticalScrollBar();
        final int value = vertBar.getValue() + amount * 6;
        vertBar.setValue(value);
    }

    /**
     * Returns the column index of table.
     *
     * @param identifier
     * @return If the column with given identifier exists, returns the index.
     */
    public int getColumnIndex(final String identifier) {
        return this.mTable.getColumnModel().getColumnIndex(identifier);
    }

    /**
     * Called when the table has changed.
     */
    public void tableChanged(TableModelEvent e) {

        final int type = e.getType();
        if (type == TableModelEvent.UPDATE) {

            // if the first row does not exist, return now
            if (e.getFirstRow() == -1) {
                return;
            }
            final int firstRow = e.getFirstRow();

            // check whether all input file exists
            final int colIndex = this.getFileNameColumnIndex();
            Object obj = this.mTableModel.getValueAt(firstRow, colIndex);
            if (obj != null && this.mDataFileArray[firstRow].hasData == false) {
                String str = (String) obj;
                File f = new File(str);

                // set selected file
                this.setSelectedFile(firstRow, f.getPath());

                // update the buttons
                this.updateAvailableButtons();
            }
        }
    }

    /**
     * Update available buttons: the OK button and file selection buttons.
     */
    private void updateAvailableButtons() {

        boolean enableOK = true;
        final int colIndex = this.getFileNameColumnIndex();

        // check all cells in file name column
        for (int ii = 0; ii < this.mDataFileArray.length; ii++) {
            if (this.mDataFileArray[ii].hasData == false) {
                continue;
            }
            String filename = (String) this.mTableModel.getValueAt(ii, colIndex);
//            if (NO_DATA.equals(filename)) {
//                continue;
//            }
            if (filename == null || filename.trim().equals("")) {
                enableOK = false;
                break;
            }
            if (SGApplicationUtility.isNetCDFFile(filename) == false) {
                File file = new File(filename);
                if (this.isAcceptable(file)) {
                    enableOK = true;
                } else {
                    enableOK = false;
                    break;
                }
            }
/*
            // if data column array is not set yet
            // and the data type do not exist in the candidates,
            // the OK button is disabled
            if (this.mDataFileArray[ii].dataColumnInfoArray == null) {
                // check the data type
                String dataType = this.mDataFileArray[ii].dataType;
                List cList = new ArrayList();
                List aList = new ArrayList();
                try {
                    this.mDataCreator.getDataTypeCandidateList(filename, cList, aList);
                } catch (FileNotFoundException e1) {
                    enableOK = false;
                    break;
                }
                if (!cList.contains(dataType)) {
                    enableOK = false;
                }
            }
*/            
        }

        // update the OK button
        this.mOKButton.setEnabled(enableOK);
    }

    /**
     *
     */
    public JTable getTable() {
        return this.mTable;
    }

    /**
     *
     */
    public TableModel getTableModel() {
        return this.mTable.getModel();
    }

    /**
     *
     */
    public boolean setTableModel(final TableModel model) {
        JTable table = this.mTable;
        table.setModel(model);
        return true;
    }

    /**
     *
     * @return
     */
    public int getFigureIDColumnIndex() {
        return this.getColumnIndex(COLUMN_NAME_FIGURE_ID);
    }

    /**
     *
     * @return
     */
    public int getDataNameColumnIndex() {
        return this.getColumnIndex(COLUMN_NAME_DATA_NAME);
    }

    /**
     *
     * @return
     */
    public int getDataTypeColumnIndex() {
        return this.getColumnIndex(COLUMN_NAME_DATA_TYPE);
    }

    /**
     *
     * @return
     */
    public int getFileNameColumnIndex() {
        return this.getColumnIndex(COLUMN_NAME_FILE_NAME);
    }

    /**
    *
    * @return
    */
   public int getButtonColumnIndex() {
       return this.getColumnIndex(COLUMN_NAME_BUTTON);
   }

    /**
     * Sets the data column information.
     *
     * @param dataList
     *            a list of data column information
     * @param versionNumber
     *            the version number of property file
     */
    @SuppressWarnings("unchecked")
	public void setData(List<SGPropertyFileData> dataList, String versionNumber) {
        
        // set to attribute
        this.mVersionNumber = versionNumber;

        // the number of data
        final int dataNum = dataList.size();

        // initialize the table
        this.initTable(dataNum);

        // create arrays
        int[] idArray = new int[dataNum];
        String[] typeArray = new String[dataNum];
        String[] nameArray = new String[dataNum];
        String[] fileArray = new String[dataNum];
        SGButton[] btnArray = new SGButton[dataNum];
        Map<String, Object>[] mapArray = new Map[dataNum];
        for (int ii = 0; ii < dataNum; ii++) {
            SGPropertyFileData data = dataList.get(ii);
            idArray[ii] = data.getFigureId();
            String type = data.getDataType();
            if (type != null) {
                typeArray[ii] = type;
                nameArray[ii] = data.getDataName();
                fileArray[ii] = "";
            } else {
                typeArray[ii] = "";
                nameArray[ii] = "";
                fileArray[ii] = NO_DATA;
            }
            btnArray[ii] = new SGButton(" ");
            mapArray[ii] = new HashMap<String, Object>(data.getInfoMap());
        }

        // create an array
        this.mDataFileArray = new DataFile[dataNum];
        for (int ii = 0; ii < dataNum; ii++) {
            this.mDataFileArray[ii] = new DataFile(idArray[ii], typeArray[ii],
                    nameArray[ii], mapArray[ii]);
            this.mDataFileArray[ii].hasData = !(NO_DATA.equals(fileArray[ii]));
        }

        // set to the table
        final int idColIndex = this.getFigureIDColumnIndex();
        final int typeColIndex = this.getDataTypeColumnIndex();
        final int nameColIndex = this.getDataNameColumnIndex();
        final int fileColIndex = this.getFileNameColumnIndex();
        final int btnColIndex = this.getButtonColumnIndex();
        for (int ii = 0; ii < dataNum; ii++) {
            this.mTable.setValueAt(Integer.toString(idArray[ii]), ii,
                    idColIndex);
//            this.mTable.setValueAt(typeArray[ii], ii, typeColIndex);
            String type = SGDataTypeConstants.getLongDataTypeConstant(typeArray[ii]);
            this.mTable.setValueAt(type, ii, typeColIndex);
            this.mTable.setValueAt(nameArray[ii], ii, nameColIndex);
            this.mTable.setValueAt(fileArray[ii], ii, fileColIndex);
            this.mTable.setValueAt(btnArray[ii], ii, btnColIndex);
            this.mTable.getCellEditor(ii, fileColIndex).addCellEditorListener(this);
        }

        // update the buttons
        this.updateAvailableButtons();
    }

    /**
     * Set size of columns.
     *
     */
    private void setPreferredColumnWidth() {
        JTable table = this.mTable;
        final int[] size = PREFERRED_COLUMN_WIDTH_ARRAY;
        for (int ii = 0; ii < COLUMN_NAME_ARRAY.length; ii++) {
            TableColumn cl = table.getColumn(COLUMN_NAME_ARRAY[ii]);
            cl.setPreferredWidth(size[ii]);
        }
    }

    /**
     * Initialize the table.
     */
    private void initTable(final int dataNumber) {
        
        // init the table model
        Object[][] data = new Object[dataNumber][];
        final int len = COLUMN_NAME_ARRAY.length;
        for (int ii = 0; ii < data.length; ii++) {
            data[ii] = new Object[len];
        }
        this.mTableModel.setDataVector(data, COLUMN_NAME_ARRAY);
        
        // set up the column with buttons
        final int colIndexButton = this.getColumnIndex(COLUMN_NAME_BUTTON);
        TableColumn colButton = this.mTable.getColumnModel().getColumn(
                colIndexButton);
        ButtonColumn bcol = new ButtonColumn();
        colButton.setMinWidth(ROW_HEIGHT);
        colButton.setMaxWidth(ROW_HEIGHT);
        colButton.setResizable(false);
        colButton.setCellEditor(bcol);
        colButton.setCellRenderer(bcol);
        
        // set up the column for file names
        final int colIndexFileName = this.getColumnIndex(COLUMN_NAME_FILE_NAME);
        TableColumn colFileName = this.mTable.getColumnModel().getColumn(
                colIndexFileName);
        JTextField textField = new JTextField();
        textField.setBorder(BorderFactory.createEmptyBorder());
        FileNameColumnCellEditor fcolEditor = new FileNameColumnCellEditor(textField);
        colFileName.setCellEditor(fcolEditor);
        FileNameColumnCellRenderer fcolRenderer = new FileNameColumnCellRenderer();
        colFileName.setCellRenderer(fcolRenderer);
    }

    /**
     * Returns a TreeMap object of figure ID to a list of property file data.
     *
     * @return a tree map object
     */
    public TreeMap<Object, Object> getSelectedFilePathListMap() {

        final int idColIndex = this.getFigureIDColumnIndex();
        final int nameColIndex = this.getFileNameColumnIndex();

        TreeMap<Object, Object> map = new TreeMap<Object, Object>();
        final int rowCount = this.mTable.getRowCount();
        for (int ii = 0; ii < rowCount; ii++) {
            // figure ID
            Object id = this.mTable.getValueAt(ii, idColIndex);

            // list of the path of data files
            Object obj = map.get(id);
            if (obj == null) {
                obj = new ArrayList();
                map.put(id, obj);
            }
            ArrayList<Object> list = (ArrayList<Object>) obj;

            // add the path to the list
            Object name = this.mTable.getValueAt(ii, nameColIndex);
            list.add(name);
        }

        return map;
    }

    /**
     *
     */
    public String[] getSelectedFileNames() {
        JTable table = this.getTable();
        final int col = this.getFileNameColumnIndex();
        final int num = table.getRowCount();
        String[] array = new String[num];
        for (int ii = 0; ii < num; ii++) {
            array[ii] = (String) table.getValueAt(ii, col);
        }
        return array;
    }

    protected JButton getPreviousButton() {
        return this.mPreviousButton;
    }

    protected JButton getNextButton() {
        return this.mNextButton;
    }

    protected JButton getOKButton() {
        return this.mOKButton;
    }

    protected JButton getCancelButton() {
        return this.mCancelButton;
    }

    /**
     *
     */
    public void dragEnter(final DropTargetDragEvent dtde) {
        dtde.acceptDrag(DnDConstants.ACTION_COPY_OR_MOVE);
    }

    /**
     *
     */
    public void dragExit(final DropTargetEvent dte) {
    }

    /**
     *
     */
    public void dragOver(final DropTargetDragEvent dtde) {
    }

    /**
     *
     */
    public void drop(final DropTargetDropEvent dtde) {

        JTable table = this.mTable;
        JTableHeader header = table.getTableHeader();
//        TableModel model = this.mTableModel;
        JViewport vp = this.mScrollPane.getViewport();
        Point vpPos = vp.getViewPosition();
        Point location = dtde.getLocation();
        Insets ins = this.getInsets();
        final int originX = ins.left - vpPos.x;
        final int originY = ins.top + this.mScrollPane.getY() - vpPos.y
                + header.getHeight();
        final Point pos = new Point(location.x - originX, location.y - originY);
        final int column = this.getFileNameColumnIndex();

        // get the row index
        final int rowIndex = table.rowAtPoint(pos);
        if (rowIndex == -1) {
            return;
        }

        // if dropped onto a row with no data, returns now
//        if (NO_DATA.equals(model.getValueAt(rowIndex, column))) {
//            return;
//        }
        if (this.mDataFileArray[rowIndex].hasData == false) {
            return;
        }

        // get dropped file list
        List fileList = SGApplicationUtility.getDroppedFileList(dtde);
        if (fileList == null || fileList.size() == 0) {
            return;
        }

        // get only the first file
        File file = (File) fileList.get(0);

        // set the value to the cell
        table.setValueAt(file.getPath(), rowIndex, column);

        // set the selected file
        this.setSelectedFile(rowIndex, file.getPath());

        // update the buttons
        this.updateAvailableButtons();
    }

    /**
     * Called when some string has input to the cell for the file name.
     * @param rowIndex
     *                the row index
     * @param f
     *                a file object created with an input string
     */
    private void setSelectedFile(final int rowIndex, final String path) {

//        if (this.isAcceptable(f)) {
//
//            // for differenct file
//            String path = f.getPath();
//            if (path.equals(this.mDataFileArray[rowIndex].fileName) == false) {
//                
//                // set to attributes
//                this.mDataFileArray[rowIndex].fileName = path;
//                
//                // clear attributes
//                this.mDataFileArray[rowIndex].dataColumnInfoArray = null;
//            }
//            
//        } else {
//            // clear attributes
//            this.mDataFileArray[rowIndex].fileName = null;
//            this.mDataFileArray[rowIndex].dataColumnInfoArray = null;
//        }
        
        // for different file
        if (path.equals(this.mDataFileArray[rowIndex].fileName) == false) {
            
            // set to attributes
            this.mDataFileArray[rowIndex].fileName = path;
            
            // clear attributes
            this.mDataFileArray[rowIndex].dataColInfoSet = null;
        }

    }

    /**
     * Returns whether a given file is acceptable as a property file.
     * @param f
     *         a file
     * @return
     *         true if a given file is acceptable
     */
    protected boolean isAcceptable(File f) {
        // the file exists and is a file
        return (f.exists()) && (f.isFile());
    }

    /**
     *
     */
    public void dropActionChanged(final DropTargetDragEvent dtde) {
    }

    /**
     * The original table model.
     */
    private class OriginalTableModel extends DefaultTableModel {

        /**
         * Serial Version UID.
         */
        private static final long serialVersionUID = -7588229760968586164L;

        public OriginalTableModel() {
            super();
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            JTable table = mTable;
            String name = table.getColumnName(columnIndex);
            boolean b = true;
            if (name.equals(COLUMN_NAME_FILE_NAME)) {
//                Object type = table.getValueAt(rowIndex,
//                        getDataTypeColumnIndex());
//                if (NO_DATA.equals(type)) {
//                    b = false;
//                }
                if (mDataFileArray[rowIndex].hasData == false) {
                    b = false;
                }
            } else {
                b = false;
            }
            return b;
        }

    }


    /**
     * Cell editor class for a column with a button.
     *
     */
    private class ButtonColumn extends AbstractCellEditor implements
            TableCellRenderer, TableCellEditor {

        private static final long serialVersionUID = 4671694450428631643L;
        private static final String BUTTON_TEXT = " ";
        private final JButton renderButton;
        private final JButton editorButton;

        public ButtonColumn() {
            super();
            renderButton = new SGButton(BUTTON_TEXT);
            editorButton = new SGButton(BUTTON_TEXT);
            editorButton.setFocusPainted(false);
            editorButton.setRolloverEnabled(false);
        }

        public Component getTableCellRendererComponent(JTable table,
                Object value, boolean isSelected, boolean hasFocus, int row,
                int column) {
            renderButton.setEnabled(mDataFileArray[row].hasData);
            return renderButton;
        }

        public Component getTableCellEditorComponent(JTable table,
                Object value, boolean isSelected, int row, int column) {
            return editorButton;
        }

        public Object getCellEditorValue() {
            return BUTTON_TEXT;
        }
    }

    /**
     * A cell editor class to display file name.
     */
    private class FileNameColumnCellEditor extends DefaultCellEditor {

        /**
         * SerialVersionUID
         */
        private static final long serialVersionUID = -7226073642075293006L;

        /**
         * Builds the cell editor for a text field.
         *
         * @param textField
         */
        public FileNameColumnCellEditor(JTextField textField) {
            super(textField);
        }

        /**
         * Returns the text field component.
         */
        public Component getTableCellEditorComponent(JTable table,
                Object value, boolean isSelected, int row, int column) {

            // set up new combo box items
            final JTextField textField = (JTextField) super.getTableCellEditorComponent(table,
                    value, isSelected, row, column);
            
            // set the caret position
            if (value != null) {
                String str = value.toString();
                final int len = str.length();
                SwingUtilities.invokeLater(new Runnable(){
                    public void run() {
                        textField.setCaretPosition(len);
                    }
                });
            }

            return textField;
        }
    }

    /**
     * A cell renderer class to display file name.
     */
    private class FileNameColumnCellRenderer extends DefaultTableCellRenderer {

        /**
         * SerialVersionUID
         */
        private static final long serialVersionUID = -1161211373077578663L;

        /**
         * Default constructor.
         *
         */
        public FileNameColumnCellRenderer() {
            super();
        }
        
        /**
         * Returns this component.
         */
        public Component getTableCellRendererComponent(JTable table,
                Object value, boolean isSelected, boolean hasFocus, int row,
                int column) {
            
            // set value
            if (value != null) {
                String path = SGUtility.getFileName(value.toString());
                this.setValue(path);
            }
            
            // set color
            Color bgColor;
            Color fgColor;
            if (isSelected) {
                bgColor = table.getSelectionBackground();
                fgColor = table.getSelectionForeground();
            } else {
                bgColor = table.getBackground();
                fgColor = table.getForeground();
            }
            this.setBackground(bgColor);
            this.setForeground(fgColor);
            
            return this;
        }
    }
    
    private class PropertySingleDataFileChooserDialog extends SGSingleDataFileChooserWizardDialog {
        private static final long serialVersionUID = -224248041535744923L;
        
        PropertySingleDataFileChooserDialog(java.awt.Frame parent, boolean modal) {
            super(parent, modal);
            this.initProperty();
        }
        void initProperty() {
            this.getOKButton().setVisible(true);
        }
        @Override
        public void checkNetCDFURLTextField() {
            this.onOK();
        }
        @Override
        protected void acceptFile(final boolean b) {
            // enable the next button
            this.getOKButton().setEnabled(b);
        }
        @Override
        public void acceptNetCDFUrl(final boolean accpet) {
            this.acceptFile(accpet);
        }
    }

    /**
     * Called when the table is clicked by the mouse button.
     * 
     * @param e
     *            the mouse event
     */
    public void mouseClicked(MouseEvent e) {
        Point p = e.getPoint();
        final int colIndexButton = this.getButtonColumnIndex();
        final int colIndexFileName = this.getFileNameColumnIndex();
        final int colIndex = this.mTable.columnAtPoint(p);
        
        // find the clicked row
        final int rowIndex = this.mTable.rowAtPoint(p);
        
        // when the button is clicked
        if (colIndex == colIndexButton) {
            
            this.mCurDataSetupRowIndex = rowIndex;

            // if the current row has no data, do nothing and return
            if (this.mDataFileArray[rowIndex].hasData == false) {
                return;
            }

            // selected data file object
            DataFile dataFile = mDataFileArray[rowIndex];
            String fileName = dataFile.fileName;
            String dataType = dataFile.dataType;
            
            // create a dialog
            final boolean isArrayData = SGDataUtility.isArrayData(dataType);
            
            this.createPropertyFileDataDialog(isArrayData);
            
            if (this.mSingleDataFileChooserWizardDilaog==null) {
                this.mSingleDataFileChooserWizardDilaog =
                    new PropertySingleDataFileChooserDialog(this.getOwnerWindow(), true);
                this.mSingleDataFileChooserWizardDilaog.setPrevious(null);
                this.mSingleDataFileChooserWizardDilaog.setNext(null);
            }
            if (isArrayData) {
                this.mSingleDataFileChooserWizardDilaog.setVisibleUrlOfNetCDF(false);
            } else {
                this.mSingleDataFileChooserWizardDilaog.setVisibleUrlOfNetCDF(true);
            }
            this.mSingleDataFileChooserWizardDilaog.addActionListener(this);
            
            if (null==fileName || fileName.equals("")) {
                String path = this.getCurrentFileDirectory();
                this.mSingleDataFileChooserWizardDilaog.setSelectedFile(path, fileName);
            } else {
                File f = new File(fileName);
                if (f.exists()) {
                    this.mSingleDataFileChooserWizardDilaog.setSelectedFile(f);
                    this.mSingleDataFileChooserWizardDilaog.setFileName(fileName);
                } else {
                    String path = this.getCurrentFileDirectory();
                    this.mSingleDataFileChooserWizardDilaog.setSelectedFile(path, fileName);
                    this.mSingleDataFileChooserWizardDilaog.setFileName(fileName);
                }
            }
            
            this.mSingleDataFileChooserWizardDilaog.pack();
            
            this.mSingleDataFileChooserWizardDilaog.setLocation(this.getX() + 20, this.getY() + 20);
            this.mSingleDataFileChooserWizardDilaog.setVisible(true);
            
        } else if (colIndex == colIndexFileName) {
        	this.mFocusedRowIndex = rowIndex;
        }
    }
    
    String getCurrentFileDirectory() {
        Preferences pref = Preferences.userNodeForPackage(this.getClass());
        String currentDir = pref.get(SGIPreferencesConstants.PREF_KEY_CURRENT_DIRECTORY, null);

        boolean b = false;
        if (currentDir == null) {
            b = true;
        } else {
            File f = new File(currentDir);
            if (f.exists() == false) {
                b = true;
            }
        }

        if (b) {
            StringBuffer sb = new StringBuffer();
            sb.append(USER_HOME);
            if (SGUtility.identifyOS(OS_NAME_WINDOWS)) {
                sb.append(FILE_SEPARATOR);
                sb.append(MY_DOCUMENTS);
            }
            String path = sb.toString();
            currentDir = path;
        }

        return currentDir;
    }
    
    private void createPropertyFileDataDialog(final boolean isArrayData) {
        
        if (isArrayData) {
            this.mPropertyFileDataDialog = new SGPropertyFileArrayDataDialog(this, true);
        } else {
            this.mPropertyFileDataDialog = new SGPropertyFileNetCDFDataDialog(this, true);
        }

        // add an action listener
        this.mPropertyFileDataDialog.addActionListener(this);
        
    }
    
    private boolean showPropertyFileDataDialog(
            final String fileName, final boolean isArrayData, final String dataType,
            final SGDataColumnInfoSet dataColumnInfoSet, final Map<String, Object> infoMap) {
        
        if (fileName != null && !"".equals(fileName)) {
            // if a text string for file name is given
            
            if (this.setPropertyFileDataDialog(
                    fileName, isArrayData, dataType, dataColumnInfoSet, infoMap)==false) {
                return false;
            }

        } else {

//            //
//            // for older data type
//            //
//            if (SGDataTypeConstants.SXY_SAMPLING_DATA.equals(dataType)) {
//                  
//                  // replace the data type
//                  dataType = SGDataTypeConstants.SXY_MULTIPLE_DATA;
//                  
//            } else if (SGDataTypeConstants.SXY_DATE_DATA.equals(dataType)) {
//                  
//                  // replace the data type
//                  dataType = SGDataTypeConstants.SXY_MULTIPLE_DATA;
//            }

            // clear the dialog
            this.mPropertyFileDataDialog.clear();
            
            // set the data type
            this.mPropertyFileDataDialog.setDataType(dataType, infoMap);
        }

        // show the dialog
        this.mPropertyFileDataDialog.setPrevious(this.mSingleDataFileChooserWizardDilaog);
        this.mPropertyFileDataDialog.pack();
        this.mPropertyFileDataDialog.setLocation(this.getX() + 20, this.getY() + 20);
        this.mPropertyFileDataDialog.setVisible(true);
        
        return true;
    }
    
    private boolean setPropertyFileDataDialog(
            final String fileName, final boolean isArrayData, final String dataType,
            final SGDataColumnInfoSet dataColumnInfoSet, final Map<String, Object> infoMap) {

        NetcdfFile ncFile = null;
        
        if (dataColumnInfoSet != null) {
            //          SGDataColumnInfoSet colInfoSet = new SGDataColumnInfoSet(dataColumnInfoArray, null);

            if (isArrayData) {

                SGPropertyFileArrayDataDialog dg = (SGPropertyFileArrayDataDialog) this.mPropertyFileDataDialog;
                if (dg.setData(fileName,
                        dataType, dataColumnInfoSet,
                        infoMap, false, this.mVersionNumber) == false) {
                    return false;
                }

            } else {

                SGPropertyFileNetCDFDataDialog dg = (SGPropertyFileNetCDFDataDialog) this.mPropertyFileDataDialog;
                try {
                    ncFile = NetcdfFile.open(fileName);
                } catch (Exception e1) {
                    SGUtility.showMessageDialog(this,
                            "Failed to open netCDF file.",
                            SGIApplicationTextConstants.TITLE_ERROR,
                            JOptionPane.WARNING_MESSAGE);
                    return false;
                }
                if (dg.setData(fileName, new SGNetCDFFile(ncFile),
                        dataType, dataColumnInfoSet,
                        infoMap, false) == false) {
                    return false;
                }
            }

        } else {

            FileColumn[] colInfo = null;
            boolean valid = true;

            if (isArrayData) {
                // parse the data file
                try {
                    colInfo = mDataCreator.parseFileComlumnType(fileName, dataType, 
                            true, this.mVersionNumber);
                    if (colInfo == null) {
                        valid = false;
                        SGUtility.showMessageDialog(this, "Data file is invalid.",
                                SGIApplicationTextConstants.TITLE_ERROR, 
                                JOptionPane.WARNING_MESSAGE);
                        return false;
                    }

                } catch (FileNotFoundException e1) {
                    valid = false;
                    SGUtility.showMessageDialog(this,
                            "Data file is not found.",
                            SGIApplicationTextConstants.TITLE_ERROR,
                            JOptionPane.WARNING_MESSAGE);
                    return false;
                }
            } else {
                // if netCDF data, open the file
                try {
                    ncFile = NetcdfFile.open(fileName);
                } catch (Exception e1) {
                    valid = false;
                    SGUtility.showMessageDialog(this,
                            "Failed to open netCDF file.",
                            SGIApplicationTextConstants.TITLE_ERROR,
                            JOptionPane.WARNING_MESSAGE);
                    return false;
                }                        
            }

            if (valid) {

                // set information to the dialog for data columns
                SGDataColumnInfo[] infoArray = null;
                //            SGDataColumnInfo[] addInfoArray = null;
                if (isArrayData) {

                    final int colNum = colInfo.length;
                    infoArray = new SGDataColumnInfo[colNum];
                    for (int jj = 0; jj < colNum; jj++) {
                        infoArray[jj] = new SGDataColumnInfo(colInfo[jj].title, colInfo[jj].valueType);
                    }

                    //
                    // for older data type
                    //
                    //                if (SGDataTypeConstants.SXY_SAMPLING_DATA.equals(dataType)) {
                    //                  
                    //                  // replace the data type
                    //                  dataType = SGDataTypeConstants.SXY_MULTIPLE_DATA;
                    //                  
                    //                  // get sampling rate
                    //                  Double samplingRate = (Double) infoMap.get(KEY_SAMPLING_RATE);
                    //                  if (samplingRate == null) {
                    //                          return;
                    //                  }
                    //                  
                    //                                  // create column info for sampling
                    //                                  SGDataColumnInfo samplingRateColumnInfo = new SGDataColumnInfo(
                    //                                                  SGDataUtility.createSamplingRateTitle(samplingRate.doubleValue()), 
                    //                                                  SGIDataColumnTypeConstants.VALUE_TYPE_SAMPLING_RATE);
                    //
                    //                                  // set X_VALUE to the column type
                    //                                  samplingRateColumnInfo.setColumnType(X_VALUE);
                    //
                    //                                  // create an array of columns
                    //                                  addInfoArray = new SGDataColumnInfo[] { samplingRateColumnInfo };
                    //                                  
                    //                } else if (SGDataTypeConstants.SXY_DATE_DATA.equals(dataType)) {
                    //                  
                    //                  // replace the data type
                    //                  dataType = SGDataTypeConstants.SXY_MULTIPLE_DATA;
                    //                }
                    //                
                    //                // create a set of column information
                    //                SGDataColumnInfoSet colInfoSet = new SGDataColumnInfoSet(
                    //                                                          infoArray, addInfoArray);

                    SGDataColumnInfoSet colInfoSet =
                        SGApplicationUtility.getArrayDataColInfoSet(colInfo, dataType, infoMap);
                    String dataTypeNew = SGApplicationUtility.getArrayDataType(dataType);
                    SGPropertyFileArrayDataDialog dg = (SGPropertyFileArrayDataDialog) this.mPropertyFileDataDialog;
                    if (dg.setData(
                            fileName, dataTypeNew, colInfoSet,
                            infoMap, true, this.mVersionNumber) == false) {
                        return false;
                    }

                } else {
                    SGNetCDFFile nc = new SGNetCDFFile(ncFile);
                    List<SGVariable> varList = nc.getVariables();

                    /*                    
                    final int colNum = varList.size();
                    
                    // get the origin map
                    NamedNodeMap nodeMap = (NamedNodeMap) infoMap.get(SGIFigureElementGraph.KEY_NODE_MAP);
                    Map<String, Integer> originMap = SGDefaultColumnTypeUtility.getOriginMap(nodeMap);
                    
                    infoArray = new SGNetCDFDataColumnInfo[colNum];
                    for (int jj = 0; jj < colNum; jj++) {
                        SGVariable var = varList.get(jj);
                        
                        // get the origin
                        int origin = 0;
						if (originMap != null) {
							if (var.isCoordinateVariable()) {
								Integer num = originMap.get(var.getName());
								if (num != null) {
									Dimension dim = var.getDimension(0);
									origin = num.intValue();
									if (origin < 0 || origin >= dim.getLength()) {
										origin = 0;
									}
								}
							}
						}
						
						infoArray[jj] = new SGNetCDFDataColumnInfo(
								var, var.getNameInPriorityOrder(),
								var.getValueType(), origin);
                    }
                     */
                    
                    infoArray = SGDataUtility.getNetCDFDataColumnInfo(varList, infoMap);

                    SGDataColumnInfoSet colInfoSet = new SGDataColumnInfoSet(infoArray);
                    SGPropertyFileNetCDFDataDialog dg = (SGPropertyFileNetCDFDataDialog) this.mPropertyFileDataDialog;
                    if (dg.setData(
                            fileName, nc, 
                            dataType, colInfoSet, infoMap, true) == false) {
                        return false;
                    }
                }

            } else {

                // clear the dialog
                this.mPropertyFileDataDialog.clear();

                // set the data type
                this.mPropertyFileDataDialog.setDataType(dataType, infoMap);

                if (null!=fileName) {
                    this.mPropertyFileDataDialog.setFileName(fileName);
                }
            }
        }

        if (ncFile != null) {
            try {
                ncFile.close();
            } catch (IOException e1) {
            }
        }
        
        return true;
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Returns the selected file path.
     * 
     * @return the selected file path
     */
    public String getSelectedFilepath() {
        if (this.mPropertyFileDataDialog != null) {
            return this.mPropertyFileDataDialog.getFileName();
        } else {
            return null;
        }
    }

    /**
     * Sets the file path and returns a file.
     * 
     * @param dir
     *             directory name
     * @param name
     *             file name
     * @return
     *             a file object
     */
    public File setSelectedFile(String dir, String name) {
        if (this.mPropertyFileDataDialog != null) {
            return this.mPropertyFileDataDialog.setSelectedFile(dir, name);
        } else {
            return null;
        }
    }
    
    long lastUsed() {
        if (this.mPropertyFileDataDialog != null) {
            return this.mPropertyFileDataDialog.lastUsed();
        } else {
            return 0L;
        }
    }

    @Override
    public void editingStopped(ChangeEvent e) {
        Object source = e.getSource();
        if (source instanceof DefaultCellEditor) {
            if (this.mFocusedRowIndex == -1) {
                return;
            }
            DefaultCellEditor editor = (DefaultCellEditor) source;
            String text = (String) editor.getCellEditorValue();
            if (SGNetCDFDataFileChooserPanel.isNetcdfUrl(text) == false) {
                File f = new File(text);
                this.setSelectedFile(this.mFocusedRowIndex, f.getPath());
            } else {
                this.setSelectedFile(this.mFocusedRowIndex, text);
            }
            this.updateAvailableButtons();
        }
    }

    @Override
    public void editingCanceled(ChangeEvent e) {
        Object source = e.getSource();
        if (source instanceof DefaultCellEditor) {
            this.mFocusedRowIndex = -1;
        }
    }

    private JTable createTableInstance() {
        return new JTable() {
			private static final long serialVersionUID = 8123021601129253077L;
			public String getToolTipText(MouseEvent e) {
                final int rowIndex = rowAtPoint(e.getPoint());
                final int colIndex = columnAtPoint(e.getPoint());
				if (colIndex == 4) {
					return null;
				}
                Object obj = getModel().getValueAt(rowIndex, colIndex);
                if (obj != null && !"".equals(obj.toString())) {
                    return obj.toString();
                } else {
                    return null;
                }
            }
        };
    }

}
