package jp.riken.brain.ni.samuraigraph.base;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.Charset;

/**
 * The wrapper class of file writer using buffered writer.
 *
 */
public class SGBufferedFileReader {
	
    private FileInputStream fis = null;
    private InputStreamReader isr = null;
    private BufferedReader br = null;

    /**
     * Builds a file reader for a given file with given character set.
     * 
     * @param file
     *           the file to read
     * @param charsetName
     *           the name of character set
     * @throws IOException
     */
	public SGBufferedFileReader(final File file, final String charsetName) throws IOException {
		super();
		fis = new FileInputStream(file);
		isr = new InputStreamReader(fis, charsetName);
		br = new BufferedReader(isr);
	}

    /**
     * Builds a file reader for a given file with given character set.
     * 
	 * @param path
     *           path of the file to read
     * @param charsetName
     *           the name of character set
     * @throws IOException
     */
	public SGBufferedFileReader(final String path, final String charsetName) throws IOException {
		this(new File(path), charsetName);
	}

	/**
     * Builds a file reader for a given file with default character set.
     * 
	 * @param file
     *           the file to read
	 * @throws IOException
	 */
	public SGBufferedFileReader(final File file) throws IOException {
		this(file, Charset.defaultCharset().name());
	}

	/**
     * Builds a file reader for a file of given path with default character set.
     * 
	 * @param path
     *           path of the file to read
	 * @throws IOException
	 */
	public SGBufferedFileReader(final String path) throws IOException {
		this(new File(path));
	}

	/**
	 * Returns the buffered reader.
	 * 
	 * @return the buffered reader
	 */
	public BufferedReader getBufferedReader() {
		return this.br;
	}

	/**
	 * Closes this reader.
	 * 
	 */
    public void close() {
    	if (br != null) {
    		try {
				br.close();
			} catch (IOException e) {
			}
    	}
    	if (isr != null) {
    		try {
				isr.close();
			} catch (IOException e) {
			}
    	}
    	if (fis != null) {
    		try {
				fis.close();
			} catch (IOException e) {
			}
    	}
    }
}
