package jp.riken.brain.ni.samuraigraph.base;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;

/**
 * The wrapper class of file reader using buffered reader.
 *
 */
public class SGBufferedFileWriter {

    private FileOutputStream fos = null;
    private OutputStreamWriter osw = null;
    private BufferedWriter bw = null;

    /**
     * Builds a file writer for a given file with given character set.
     * 
     * @param file
     *           the file to write to
     * @param charsetName
     *           the name of character set
     * @throws IOException
     */
	public SGBufferedFileWriter(final File file, final String charsetName) throws IOException {
		super();
		fos = new FileOutputStream(file);
		osw = new OutputStreamWriter(fos, charsetName);
		bw = new BufferedWriter(osw);
	}

    /**
     * Builds a file writer for a given file with given character set.
     * 
	 * @param path
     *           path of the file to write to
     * @param charsetName
     *           the name of character set
     * @throws IOException
     */
	public SGBufferedFileWriter(final String path, final String charsetName) throws IOException {
		this(new File(path), charsetName);
	}

	/**
     * Builds a file writer for a given file with default character set.
     * 
	 * @param file
     *           the file to write to
	 * @throws IOException
	 */
	public SGBufferedFileWriter(final File file) throws IOException {
		this(file, Charset.defaultCharset().name());
	}

	/**
     * Builds a file writer for a file of given path with default character set.
     * 
	 * @param path
     *           path of the file to write to
	 * @throws IOException
	 */
	public SGBufferedFileWriter(final String path) throws IOException {
		this(new File(path));
	}
	
	/**
	 * Returns the buffered writer.
	 * 
	 * @return the buffered writer
	 */
	public BufferedWriter getBufferedWriter() {
		return this.bw;
	}
    
	/**
	 * Closes this writer.
	 * 
	 */
    public void close() {
    	if (bw != null) {
    		try {
				bw.close();
			} catch (IOException e) {
			}
    	}
    	if (osw != null) {
    		try {
				osw.close();
			} catch (IOException e) {
			}
    	}
    	if (fos != null) {
    		try {
				fos.close();
			} catch (IOException e) {
			}
    	}
    }
}
