package jp.riken.brain.ni.samuraigraph.base;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.InputMap;
import javax.swing.JColorChooser;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;

/**
 * A dialog with a color chooser.
 * 
 */
public class SGColorDialog extends JDialog {

    private static final long serialVersionUID = 1972378173252289790L;

    // a panel to set color chooser
    private JPanel mColorChooserPanel = new JPanel();

    // a color chooser object
    private JColorChooser mColorChooser = new JColorChooser();

//    // a color selection model
//    private ColorSelectionModel mColorSelectionModel = new DefaultColorSelectionModel();

    /**
     * Constructs a SGColorDialog.
     */
    public SGColorDialog(final JDialog parent, final String title,
            final boolean modal) {
        super(parent, title, modal);

        // initialize
        this.init();
    }

    /**
     * Initializes this object.
     * 
     */
    private void init() {

        // set close operation
        this.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);

        // hide the preview panel
        this.mColorChooser.setPreviewPanel(new JPanel());

//        // set original color selection model
//        this.mColorChooser.setSelectionModel(this.mColorSelectionModel);

        // add the color chooser to a color chooser panel
        this.mColorChooserPanel.setLayout(new BorderLayout());
        this.mColorChooserPanel.add(this.mColorChooser, BorderLayout.CENTER);

        // set color chooser panel to this dialog
        this.setContentPane(this.mColorChooserPanel);

        // disable resize
        this.setResizable(false);

        // enabled to close this dialog when the Esc key is pressed
        AbstractAction act = new AbstractAction("") {
            private static final long serialVersionUID = 4743921657783211757L;
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        };
        InputMap imap = this.getRootPane().getInputMap(
            JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "close-it");
        this.getRootPane().getActionMap().put("close-it", act);
    }

    /**
     * Returns the color chooser.
     * 
     * @return JColorChooser object
     */
    public JColorChooser getColorChooser() {
        return this.mColorChooser;
    }

    /**
     * Returns the selected color.
     * 
     * @return the selected color
     */
    public Color getSelectedColor() {
    	return this.mColorChooser.getColor();
    }

    /**
     * Sets the selected color.
     * 
     * @param color
     *           the color to set
     */
    public void setSelectedColor(final Color color) {
    	this.mColorChooser.setColor(color);
    }

//    /**
//     * The original color chooser.
//     * 
//     */
//    private static class OriginalColorSelectionModel extends
//            DefaultColorSelectionModel {
//
//        private static final long serialVersionUID = -3834864731830390762L;
//
//        // a color selected
//        private Color mSelectedColor;
//
//        /**
//         * The default constructor.
//         */
//        public OriginalColorSelectionModel() {
//            super();
//            this.mSelectedColor = Color.white;
//        }
//
//        /**
//         * Returns selected color.
//         * 
//         * @return selected color
//         */
//        public Color getSelectedColor() {
//            return this.mSelectedColor;
//        }
//
//        /**
//         * Overrode so as to throw ChangeEvent object even when the input color
//         * is equal to the selected color.
//         * 
//         * @param color
//         *            the color to be selected
//         */
//        public void setSelectedColor(final Color color) {
//            if (color != null) {
//                this.mSelectedColor = color;
//                this.fireStateChanged();
//            }
//        }
//    }

}
