package jp.riken.brain.ni.samuraigraph.base;

import java.io.File;

/**
 * The basic constants.
 * 
 */
public interface SGIConstants {

    /**
     * The ratio of cm to inch.
     */
    public static final float CM_INCH_RATIO = 2.54f;

    /**
     * The ratio of inch to point.
     */
    public static final float INCH_POINT_RATIO = 72.0f;

    /**
     * The ratio of cm to point.
     */
    public static final float CM_POINT_RATIO = CM_INCH_RATIO / INCH_POINT_RATIO;

    /**
     * The ratio of radian to degree.
     */
    public static final float RADIAN_DEGREE_RATIO = (float) (Math.PI / 180.0);

    /**
     * The golden ratio.
     */
    public static final float GOLDEN_RATIO = (float) ((1.0 + Math.sqrt(5.0)) / 2.0);

    // /**
    // * Delimiter set of lines.
    // */
    // public static char[] DELIMITER_SET = { ' ', '\t', '\n', '\r', '\f', ','
    // };

    /**
     * The units of length.
     */
    public static final String cm = "cm";

    public static final String mm = "mm";

    public static final String pt = "pt";

    public static final String inch = "inch";

    public static final String LINE_WIDTH_UNIT = pt;

    public static final String FONT_SIZE_UNIT = pt;

    /**
     * The unit of angle.
     */
    public static final char degChar = '\u00B0';

    public static final String degree = Character.toString(degChar);

    
    /**
     * The unit of percentage.
     */
    public static final String percent = "%";

    /**
     * The algebraic signs.
     */
    public static final char multiplyChar = '\u00D7';

    public static final String multiply = Character.toString(multiplyChar);

    public static final char divideChar = '\u00F7';

    public static final char pmChar = '\u00B1'; // plus and minus

    public static final char mpChar = '\u2213'; // minus and plus

    public static final char neqChar = '\u2260'; // not equal

    /**
     * Two-byte space.
     */
    public static final char twoByteSpaceChar = '\u3000';

    /**
     * OS name.
     */
    public static final String OS_NAME = System.getProperty("os.name");

    /**
     * Java home directory.
     */
    public static final String JAVA_HOME = System.getProperty("java.home");

    /**
     * File separator.
     */
    public static final String FILE_SEPARATOR = File.separator;

    /**
     * Path separator.
     */
    public static final String PATH_SEPARATOR = File.pathSeparator;

    /**
     * Line separator.
     */
    public static final String LINE_SEPARATOR = System
            .getProperty("line.separator");

    /**
     * User name.
     */
    public static final String USER_NAME = System.getProperty("user.name");

    /**
     * User home directory.
     */
    public static final String USER_HOME = System.getProperty("user.home");

    /**
     * Temporary directory.
     */
    public static final String TMP_DIR = System.getProperty("java.io.tmpdir");

    /**
     * Directory name of resources.
     */
    public static final String RESOURCES_DIRNAME = "/resources/";

    /**
     * A constant for the Windows systems.
     */
    public static final String OS_NAME_WINDOWS = "windows";

    /**
     * A constant for the Mac OS X system.
     */
    public static final String OS_NAME_MACOSX = "mac os x";

    /**
     * A constant for "My Documents" directory in the Windows systems.
     */
    public static final String MY_DOCUMENTS = "My Documents";

    /**
     * Scale Type
     */
    public static final String SCALE_TYPE_LINEAR = "Linear";

    public static final String SCALE_TYPE_LOG = "Log";

    /**
     * Return values from class method
     */
    public static final int SUCCESSFUL_COMPLETION = 0;

    public static final int PROPERTY_FILE_INCORRECT = 1;

    public static final int DATA_FILE_INVALID = 2;

    public static final int DATA_NUMBER_SHORTAGE = 3;

    public static final int DATA_NUMBER_EXCESS = 4;

    public static final int FILE_OPEN_FAILURE = 5;
    
    public static final int FILE_SAVE_FAILURE = -1;

    /**
     * Menu commands
     */
    public static final String MENUCMD_CUT = "Cut";

    public static final String MENUCMD_COPY = "Copy";

    public static final String MENUCMD_PASTE = "Paste";

    public static final String MENUCMD_DELETE = "Delete";

    public static final String MENUCMD_DUPLICATE = "Duplicate";

    public static final String MENUCMD_BRING_TO_FRONT = "Bring to Front";

    public static final String MENUCMD_BRING_FORWARD = "Bring Forward";

    public static final String MENUCMD_SEND_BACKWARD = "Send Backward";
    
    public static final String MENUCMD_SEND_TO_BACK = "Send to Back";
    
    public static final String MENUCMD_ANCHOR = "Anchored";

    public static final String MENUCMD_PROPERTY = "Property";

    /**
     * The menu command to show netCDF properties.
     */
    public static final String MENUCMD_INSERT_NETCDF_LABEL = "Insert Label of Fixed Values";
    
    /**
     * The menu command to play the animation.
     */
    public static final String MENUCMD_NETCDF_ANIMATION = "Play Animation";

    /**
     * The menu command to fit the axes to the data.
     */
    public static final String MENUCMD_FIT_AXES_TO_DATA = "Fit axes";
    
    /**
     * The menu command to transform a data object into other type data.
     */
    public static final String MENUCMD_TRANSFORM_DATA = "Transform";

    /**
     * The menu command to split a data object into multiple data objects.
     */
    public static final String MENUCMD_SPLIT_DATA = "Split";
    
    /**
     * The menu command to merge multiple data objects which was splitted
     * into the original data object.
     */
    public static final String MENUCMD_MERGE_DATA = "Merge";

    /**
     * The menu command to align all bars in the figure.
     */
    public static final String MENUCMD_ALIGN_BARS = "Align Bars";
    
    /**
     * The menu command to output data values to file.
     */
    public static final String MENUCMD_OUTPUT_TO_FILE = "Output to File";
    
    /**
     * Look & Feel
     */
    public static final String LAF_METAL = "Metal";

    public static final String LAF_MOTIF = "Motif";

    public static final String LAF_WINDOWS = "Windows";

    public static final String LAF_WINDOWSCLASSIC = "Windows Classic";

    public static final String LAF_AQUA = "Aqua";

    /**
     * Location on a rectangle.
     */
    public static final int OTHER = 0;

    public static final int NORTH = 1;

    public static final int SOUTH = 2;

    public static final int WEST = 3;

    public static final int EAST = 4;

    public static final int NORTH_WEST = 5;

    public static final int NORTH_EAST = 6;

    public static final int SOUTH_WEST = 7;

    public static final int SOUTH_EAST = 8;

    /**
     * 
     */
    public static final int OK_OPTION = 0;

    public static final int CANCEL_OPTION = 1;

    public static final int MODE_EXPORT_AS_IMAGE = 0;

    public static final int MODE_DISPLAY = 1;

    // paper size
    public static final String PAPER_SIZE_A3 = "A3";

    public static final String PAPER_SIZE_A4 = "A4";

    public static final String PAPER_SIZE_B4 = "B4";

    public static final String PAPER_SIZE_B5 = "B5";

    public static final String PAPER_SIZE_US_LETTER = "US_Letter";

    public static final String PAPER_SIZE_US_LETTER_SIMPLE = "Letter";

    public static final String PORTRAIT = "Portrait";
    
    public static final String LANDSCAPE = "Landscape";
    
    /**
     * Line width.
     */
    public static final int LINE_WIDTH_FRAC_DIGIT_MIN = 1;
    
    public static final int LINE_WIDTH_FRAC_DIGIT_MAX = 2;
    
    public static final int LINE_WIDTH_MINIMAL_ORDER = - LINE_WIDTH_FRAC_DIGIT_MAX;

    
    /**
     * Font size.
     */
    public static final int FONT_SIZE_FRAC_DIGIT_MIN = 1;
    
    public static final int FONT_SIZE_FRAC_DIGIT_MAX = 2;
    
    public static final int FONT_SIZE_MINIMAL_ORDER = - FONT_SIZE_FRAC_DIGIT_MAX;
    
    
    /**
     * Constants for the type of saving properties.
     */
    public static final int SAVE_PROPERTIES_TO_PROPERTY_FILE = 1;
    
    public static final int SAVE_PROPERTIES_TO_DATA_SET = 2;

    public static final int SAVE_PROPERTIES_FOR_DUPLICATION = 3;

    public static final int SAVE_PROPERTIES_FOR_COPYING = 4;

    public static final int SAVE_PROPERTIES_FOR_CUTTING = 5;

    public static final int SAVE_PROPERTIES_TO_DATA_SET_OLDER = 6;
    
    public static final int SAVE_PROPERTIES_TO_DATA_SET_NETCDF = 7;

    
    /**
     * Constants for the type of loading properties.
     */
    public static final int LOAD_PROPERTIES_FROM_PROPERTY_FILE = 1;
    
    public static final int LOAD_PROPERTIES_FROM_DATA_SET = 2;

    public static final int LOAD_PROPERTIES_IN_DUPLICATION = 3;

    public static final int LOAD_PROPERTIES_IN_PASTING = 4;

    
    public static final String INVALID_INPUT_VALUE = "Input value is invalid.";

    public static final String ERROR = "Error";
}
