package jp.riken.brain.ni.samuraigraph.base;

import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;

/**
 * An object which displays the data objects as a graph.
 * 
 */
public interface SGIFigureElementGraph extends SGIFigureElement, SGIFigureElementForData {

    /**
     * 
     */
    public boolean setAxisElement(SGIFigureElementAxis aElement);

    /**
     * 
     * @param data
     * @return
     */
    public SGAxis getXAxis(SGData data);

    /**
     * 
     * @param data
     * @return
     */
    public SGAxis getYAxis(SGData data);

    /**
     * Create and add Element to the Document object.
     * @param document
     *                 the document
     * @param elList
     *                 a list of Element objects
     * @param dataList
     *                 a list of data
     * @param type
     *                 type of the method to save properties
     * @return
     *                 true if succeeded
     */
    public boolean createElementOfData(Document document,
            List elList, List dataList, int type);

    /**
     * 
     */
    public int getSelectedDataNumber();

    /**
     * 
     * @param data
     * @return
     */
    public boolean getVisibleInLegendFlag(SGData data);
    
    /**
     * Returns an array of data columns of given data.
     * 
     * @param data
     *            a data
     * @return
     *            an array of data columns
     */
    public SGDataColumnInfo[] getDataColumnInfoArray(SGData data);
    
    /**
     * Returns information map of given data.
     * 
     * @param data
     *            a data
     * @return
     *            information map
     */
    public Map<String, Object> getInfoMap(SGData data);
    
    /**
     * Sets the data selection.
     * 
     * @param data
     *           the data to set the state of selection
     * @param b
     *           true to be selected
     */
    public void setDataFocused(SGData data, boolean b);
    
    /**
     * Show the netCDF data labels.
     * 
     * @param fes
     *           the string element which is added with text label.
     * @return
     *           null if succeeds. Returns data name that not have label if failed.
     */
    public String showNetCDFLabels(final SGIFigureElementString fes);
    
    /**
     * Show labels of the given netCDF data.
     * 
     * @param fes
     * @param dataList
     * @return
     *           null if succeeds. Returns data name that not have label if failed.
     */
    public String showNetCDFLabels(final SGIFigureElementString fes, final List<SGData> dataList);
    
    /**
     * Update the netCDF data labels.
     * 
     * @param fes
     *           the string element which is updated with text label.
     */
    public void updateNetCDFLabels(SGIFigureElementString fes);

    /**
     * Hide the netCDF data labels.
     * 
     * @param fes
     *           the string element which hides text label.
     * @param data
     */
    public void hideNetCDFLabels(final SGIFigureElementString fes, final SGData data);
    
    /**
     * Hide the netCDF data labels which belongs to focused data.
     * 
     * @param fes
     *           the string element which hides text label.
     */
    public void hideNetCDFLabelsOfFocusedObjects(final SGIFigureElementString fes);

    /**
     * Notify the conversion of data.
     * 
     * @param data
     * @param name
     * @param columns
     * @param message
     */
    public void notifyDataConversion(final SGData data, final String name,
            final SGDataColumnInfo[] columns, String message);
    
    public boolean convertData(final SGFigure figure);
    
    /**
     * Undo the conversion of data.
     */
    public void undoDataConversion();
    
    public boolean isBarVisible();
    
    public boolean alignVisibleBars();
    
    public static final String TAG_NAME_GRAPH = "Graph";

    public static final String TAG_NAME_DATA = "Data";

    public static final String KEY_SAMPLING_RATE = "SamplingRate";

    public static final String KEY_POLAR = "Polar";

    public static final String KEY_MAGNITUDE_PER_CM = "MagnitudePerCM";

    public static final String KEY_DIRECTION_INVARIANT = "DirectionInvariant";
    
    public static final String KEY_NODE_MAP = "NodeMap";
    
    public static final String KEY_GROUP_NAME = "GroupName";
}
