package jp.riken.brain.ni.samuraigraph.base;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import javax.swing.border.MatteBorder;
import javax.swing.colorchooser.ColorSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * The base class of the property dialogs.
 */
public abstract class SGPropertyDialog extends SGDialog implements
        ChangeListener, FocusListener, SGIDrawingElementConstants {

    /**
     * Serial Version UID
     */
    private static final long serialVersionUID = 214371996196142459L;

    /**
     * List of the property setting listeners.
     */
    protected List<SGIPropertyDialogObserver> mPropertyDialogObserverList = new ArrayList<SGIPropertyDialogObserver>();

    /**
     * 
     */
    public SGPropertyDialog() {
        super();
    }

    /**
     * 
     */
    public SGPropertyDialog(Dialog owner) {
        super(owner);
    }

    /**
     * 
     */
    public SGPropertyDialog(Dialog owner, boolean modal) {
        super(owner, modal);
    }

    /**
     * 
     */
    public SGPropertyDialog(Dialog owner, String title) {
        super(owner, title);
    }

    /**
     * 
     */
    public SGPropertyDialog(Dialog owner, String title, boolean modal) {
        super(owner, title, modal);
    }

    /**
     * 
     */
    public SGPropertyDialog(Dialog owner, String title, boolean modal,
            GraphicsConfiguration gc) {
        super(owner, title, modal, gc);
    }

    /**
     * 
     */
    public SGPropertyDialog(Frame owner) {
        super(owner);
    }

    /**
     * 
     */
    public SGPropertyDialog(Frame owner, boolean modal) {
        super(owner, modal);
    }

    /**
     * 
     */
    public SGPropertyDialog(Frame owner, String title) {
        super(owner, title);
    }

    /**
     * 
     */
    public SGPropertyDialog(Frame owner, String title, boolean modal) {
        super(owner, title, modal);
    }

    /**
     * 
     */
    public SGPropertyDialog(Frame owner, String title, boolean modal,
            GraphicsConfiguration gc) {
        super(owner, title, modal, gc);
    }

    /**
     * 
     */
    private Icon mColorSelectionButtonEmphasisIcon = null;

    /**
     * Initialization method to be called in the constructor of sub classes.
     * This method must be called after the call of the constructor of the super
     * class.
     */
    protected boolean initialize() {

        // create a color dialog
        this.mColorDialog = new SGColorDialog(this, "Color Dialog", false);
        this.mColorDialog.pack();
        this.mColorDialog.setResizable(false);

        // set this as a ChangeListener of the color selection model
        JColorChooser cc = this.mColorDialog.getColorChooser();
        ColorSelectionModel cModel = cc.getSelectionModel();
        cModel.addChangeListener(this);

        this.mColorDialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
//                clearFocusedColorSelectionButton();
            }
        });

        // create background icon for the button
        this.mColorSelectionButtonEmphasisIcon = this
                .createIcon(SGIConstants.RESOURCES_DIRNAME + "Lines.gif");
        
        // add a focus event listener to the color selection buttons
        List cbList = this.getColorSelectionButtonsList();
        for (int ii = 0; ii < cbList.size(); ii++) {
            SGColorSelectionButton cb = (SGColorSelectionButton) cbList.get(ii);
            cb.addFocusListener(this);
        }

        //
        // add this dialog as an actin event listener
        //

        // color selection buttons
        List cList = this.getColorSelectionButtonsList();
        for (int ii = 0; ii < cList.size(); ii++) {
            Object obj = cList.get(ii);
            JButton btn = (JButton) obj;
            btn.addActionListener(this);
        }
        
        // text field components
        List tList = this.getTextFieldComponentsList();
        for (int ii = 0; ii < tList.size(); ii++) {
            Object obj = tList.get(ii);
            JTextField com = (JTextField) obj;
            com.addActionListener(this);
        }
        
        // OK, Cancel and Preview buttons
        this.getOKButton().addActionListener(this);
        this.getCancelButton().addActionListener(this);
        this.getPreviewButton().addActionListener(this);

        return true;
    }

    /**
     * Called when the escape key is typed.
     *
     */
    protected void onEscKeyTyped() {
        this.onCanceled();
    }

    public void dispose() {
        this.mColorDialog = null;
        super.dispose();
    }

    /**
     * name : name of resource file
     */
    private ImageIcon createIcon(final String name) {
        if (name == null) {
            return null;
        }

        Class inClass = getClass();
        URL url = inClass.getResource(name);

        // System.out.println("URL:"+url);

        ImageIcon icon;
        if (url != null) {
            icon = new ImageIcon(url);
        } else {
            icon = new ImageIcon(name);
        }

        // System.out.println("icon:"+icon);
        // System.out.println();

        return icon;
    }

    /**
     * 
     */
    protected String getString(final String key) {
        ResourceBundle bundle = ResourceBundle.getBundle("res", Locale.JAPAN);
        // ResourceBundle bundle =
        // ResourceBundle.getBundle("res",Locale.ENGLISH);
        String val = bundle.getString(key);

        return val;
    }

    /**
     * The color selection dialog.
     * 
     */
    protected SGColorDialog mColorDialog = null;

    /**
     * A color selection button currently used.
     * 
     */
    protected SGColorSelectionButton mCurrentColorSetButton = null;

    /**
     * Returns the color selection dialog.
     * 
     * @return the color selection dialog
     */
    public SGColorDialog getColorDialog() {
        return this.mColorDialog;
    }

    /**
     * Invoked when the state is changed.
     * 
     * @param e
     *          the changed event
     */
    public void stateChanged(final ChangeEvent e) {
        Object source = e.getSource();
        if (this.mCurrentColorSetButton != null
                && (source instanceof ColorSelectionModel)) {
        	// set the selected color to the color selection button
            Color cl = this.mColorDialog.getSelectedColor();
            this.mCurrentColorSetButton.setColor(cl);
        }
    }

    /**
     * Overrode the method to set visibility of the color dialog.
     * 
     * @param b
     *          true to set visible
     */
    public void setVisible(final boolean b) {
        if (!b) {
            if (this.mColorDialog != null) {
                this.mColorDialog.setVisible(false);
            }
        }
        super.setVisible(b);
    }
    
    /**
     * Returns a list of buttons used for the color selection.
     */
    public abstract List getColorSelectionButtonsList();

    /**
     * Returns a list of text field components.
     */
    public abstract List getTextFieldComponentsList();

    /**
     * Returns a list of text fields to set number.
     * 
     * @return
     */
    public abstract List getNumberTextFieldList();

    /**
     * Returns a list of spinners.
     */
    public abstract List getSpinnerList();

    /**
     * Returns a list of JFormattedTextField from spinners in the dialog.
     */
    protected List getFormattedTextFieldsListFromSpinners() {
        final List list = new ArrayList();

        final List sList = this.getSpinnerList();
        for (int ii = 0; ii < sList.size(); ii++) {
            SGSpinner spinner = (SGSpinner) sList.get(ii);
            list.add(spinner.getFormattedTextField());
        }

        return list;
    }

    /**
     * Called when an action event is generated.
     * 
     * @param e
     *          an action event
     */
    public void actionPerformed(final ActionEvent e) {
        Object source = e.getSource();

        // if the event source is one of the color selection buttons
        if (this.getColorSelectionButtonsList().contains(source)) {

            SGColorSelectionButton btn = (SGColorSelectionButton) source;
            
            // clear old color button
            if (this.mCurrentColorSetButton != null) {
                this.mCurrentColorSetButton.setBorder(this.getUnfocusedBorder());
            }
            
            // set currently focused color button
            this.mCurrentColorSetButton = btn;
            this.mCurrentColorSetButton.setBorder(this.getFocusedBorder());

            if (!this.mColorDialog.isVisible()) {

            	// set the current color to the dialog
            	if (this.mCurrentColorSetButton != null) {
                	this.mColorDialog.setSelectedColor(
                			this.mCurrentColorSetButton.getColor());
            	}
            	
                // set the location of color dialog
                this.mColorDialog.setLocation(this.getX() + 20, this.getY() + 20);

                // show color dialog
                this.mColorDialog.setVisible(true);
            }
            
        } else {
            
            try {
                List comList = this.getTextFieldComponentsList();
                if (source.equals(this.getOKButton()) || comList.contains(source)) {
                    this.onOK();
                } else if (source.equals(this.getCancelButton())) {
                    this.onCanceled();
                } else if (source.equals(this.getPreviewButton())) {
                    this.onPreviewed();
                }
            } catch (Exception ex) {
                SGUtility.showErrorMessageDialog(this, "Unknown error occurred.\b This dialog will be closed.", "ERROR");
                this.setVisible(false);
                this.clearAllSpinners();
            }
        }
    }
    
    /**
     * Returns the OK button.
     * @return
     *        the OK button
     */
    protected abstract JButton getOKButton();
    
    /**
     * Returns the cancel button.
     * @return
     *        the cancel button
     */
    protected abstract JButton getCancelButton();
    
    /**
     * Returns the preview button.
     * @return
     *        the preview button
     */
    protected abstract JButton getPreviewButton();

    /**
     * 
     */
    public static SpinnerNumberModel getLineWidthSpinnerNumberModel() {
        return new SpinnerNumberModel(LINE_WIDTH_MIN_VALUE,
                LINE_WIDTH_MIN_VALUE, LINE_WIDTH_MAX_VALUE,
                LINE_WIDTH_STEP_SIZE);
    }

    /**
     * 
     */
    public static SpinnerNumberModel getFontSizeSpinnerNumberModel() {
        return new SpinnerNumberModel(FONT_SIZE_MIN_VALUE, FONT_SIZE_MIN_VALUE,
                FONT_SIZE_MAX_VALUE, FONT_SIZE_STEP_VALUE);
    }

    /**
     * 
     * @param l
     * @return
     */
    public boolean addPropertyDialogObserver(final SGIPropertyDialogObserver l) {
        this.mPropertyDialogObserverList.add(l);
        return true;
    }

    /**
     * 
     * @param l
     * @return
     */
    public boolean removePropertyDialogObserver(
            final SGIPropertyDialogObserver l) {
        return this.mPropertyDialogObserverList.remove(l);
    }

    /**
     * 
     * @param l
     * @return
     */
    public boolean removeAllPropertyDialogObserver() {
        this.mPropertyDialogObserverList.clear();
        return true;
    }

    /**
     * Called when all changed properties are committed.
     */
    protected boolean onOK() {
        if (this.commit() == false) {
            return false;
        }
        this.setCloseOption(OK_OPTION);
        this.setVisible(false);
        this.clearAllSpinners();
        
        return true;
    }

    /**
     * Called when all changed properties are canceled.
     */
    protected boolean onCanceled() {
        if (this.cancel() == false) {
            return false;
        }
        this.setCloseOption(CANCEL_OPTION);
        this.setVisible(false);
        this.clearAllSpinners();

        return true;
    }

    /**
     * Called when all changed properties are reflected temporarily to preview.
     */
    protected boolean onPreviewed() {
        return this.preview();
    }

    /**
     * 
     * 
     */
    protected void clearAllSpinners() {
        List list = this.getSpinnerList();
        for (int ii = 0; ii < list.size(); ii++) {
            SGSpinner sp = (SGSpinner) list.get(ii);
            sp.clearTemporaryValues();
        }
    }
    
    public static final String NOTIFY_ACTION_COMMIT = "Notify action commit";
    
    public static final String NOTIFY_ACTION_PREVIEW = "Notify action preview";
    
    public static final String NOTIFY_ACTION_CANCEL = "Notify action cancel";
    
    protected String mCommitActionStateMessage = null;

    /**
     * 
     * @return
     */
    protected boolean commit() {
        this.mCommitActionStateMessage = NOTIFY_ACTION_COMMIT;
        if (!this.checkInputValues()) {
            return false;
        }

        List list = this.mPropertyDialogObserverList;
        for (int ii = 0; ii < list.size(); ii++) {
            SGIPropertyDialogObserver l = (SGIPropertyDialogObserver) list
                    .get(ii);
            if (this.setPropertiesToListeners(l) == false) {
                String msg;
                String err = this.getInputErrorMessage();
                if (err != null) {
                    msg = err;
                } else {
                    msg = DEFAULT_INPUT_ERROR_MESSAGE;
                }
                SGUtility.showIllegalInputErrorMessageDialog(this, msg);
                return false;
            }
            if (l.commit() == false) {
                return false;
            }
        }

//        this.setVisible(false);

        return true;
    }

    /**
     * 
     * @return
     */
    protected boolean cancel() {
        this.mCommitActionStateMessage = NOTIFY_ACTION_CANCEL;
        List list = this.mPropertyDialogObserverList;
        for (int ii = 0; ii < list.size(); ii++) {
            SGIPropertyDialogObserver l = (SGIPropertyDialogObserver) list
                    .get(ii);
            if (l.cancel() == false) {
                return false;
            }
        }

        return true;
    }

    /**
     * 
     * @return
     */
    protected boolean preview() {
        this.mCommitActionStateMessage = NOTIFY_ACTION_PREVIEW;
        if (!this.checkInputValues()) {
            return false;
        }

        List list = this.mPropertyDialogObserverList;
        for (int ii = 0; ii < list.size(); ii++) {
            SGIPropertyDialogObserver l = (SGIPropertyDialogObserver) list
                    .get(ii);
            if (this.setPropertiesToListeners(l) == false) {
                String msg;
                String err = this.getInputErrorMessage();
                if (err != null) {
                    msg = err;
                } else {
                    msg = DEFAULT_INPUT_ERROR_MESSAGE;
                }
                SGUtility.showIllegalInputErrorMessageDialog(this, msg);
                return false;
            }
            if (l.preview() == false) {
                return false;
            }
        }
        return true;
    }

    /**
     * Check input values.
     * 
     * @return
     */
    private boolean checkInputValues() {
        // values in dialog
        if (!this.hasValidInputValues()) {
            // show a message dialog
            String msg = this.getInputErrorMessage();
            SGUtility.showIllegalInputErrorMessageDialog(this, msg);

            return false;
        }

        return true;
    }

    /**
     * Overrode this method to check the input values.
     * 
     * @return true if all input values are valid
     */
    protected boolean hasValidInputValues() {
        boolean ret = true;

        this.mInputErrorComponentNameList.clear();

        // spinners
        List sList = this.getSpinnerList();
        for (int ii = 0; ii < sList.size(); ii++) {
            SGSpinner s = (SGSpinner) sList.get(ii);
            if (s.hasValidValue() == false) {
                this.addInputErrorDescription(s.getDescription());
                ret = false;
            }
        }

//        // text fields to set number
//        List tList = this.getNumberTextFieldList();
//        for (int ii = 0; ii < tList.size(); ii++) {
//            SGTextField tf = (SGTextField) tList.get(ii);
//            if (this.getNumber(tf) == null) {
//                if (tf.isIndeterminate() == false) {
//                    this.addInputErrorDescription(tf.getDescription());
//                    ret = false;
//                }
//            }
//        }

        return ret;
    }
    
    /**
     * Checks whether the text field has valid number type input value.
     * 
     * @param tf
     *          the text field
     * @param min
     *          the minimum value for the input value if it exists
     * @param max
     *          the maximum value for the input value if it exists
     * @return true if the input value is valid
     */
    protected boolean hasValidNumber(final SGTextField tf,
    		final Number min, final Number max) {
        boolean valid = true;
        boolean checkNumber = false;
        if (tf.isIndeterminate()) {
        	if (tf.getText() != null) {
        		checkNumber = true;
        	}
        } else {
        	checkNumber = true;
        }
        if (checkNumber) {
            final String desc = tf.getDescription();
            Number value = this.getNumber(tf);
        	if (value != null) {
        		if (Double.isNaN(value.doubleValue())) {
                	this.addInputErrorDescription(desc);
        			valid = false;
        		} else {
            		if (min != null) {
                		if (value.doubleValue() < min.doubleValue()) {
                        	this.addInputErrorDescription(desc);
                    		valid = false;
                		}
            		}
            		if (valid) {
                		if (max != null) {
                			if (value.doubleValue() > max.doubleValue()) {
                            	this.addInputErrorDescription(desc);
                        		valid = false;
                			}
                		}
            		}
        		}
        	} else {
            	this.addInputErrorDescription(desc);
        		valid = false;
        	}
        }
        return valid;
    }

    private String mInputErrorMessage = "";

    // The list of name of components set erroneous input value.
    private final ArrayList mInputErrorComponentNameList = new ArrayList();

    private static final String ERR_MSG_HEADER = "The input values are illegal in the following components:\n";
    
    /**
     * Returns a text string used as an error message for illegal input.
     * 
     * @return
     *         a text string used as an error message for illegal input
     */
    protected String getInputErrorMessage() {
        if (this.mInputErrorComponentNameList.size() != 0) {
            ArrayList list = this.mInputErrorComponentNameList;
            StringBuffer sb = new StringBuffer();
            sb.append(ERR_MSG_HEADER);
            for (int ii = 0; ii < list.size(); ii++) {
                String name = (String) list.get(ii);
                sb.append(" - ");
                sb.append(name);
                sb.append('\n');
            }
            return sb.toString();
        }
        return this.mInputErrorMessage;
    }

    /**
     * Set the error message used as an error message on input.
     * 
     * @param str
     * @return
     */
    protected boolean addInputErrorDescription(final String name) {
        this.mInputErrorComponentNameList.add(name);
        return true;
    }

    /**
     * 
     * @param msg
     * @return
     */
    protected boolean setInputErrorMessage(final String msg) {
        this.mInputErrorMessage = msg;
        return true;
    }

    /**
     * 
     */
    protected static final String DEFAULT_INPUT_ERROR_MESSAGE = "The input value is illegal.";

    public static final String ERRMSG_AXIS_VALUE_INVALID = "Axis value is invalid";

    /**
     * Set properties to all observers.
     * 
     * @param obs
     * @return
     */
    public abstract boolean setPropertiesToListeners(
            SGIPropertyDialogObserver obs);

    /**
     * Set parameters to the components in the dialog.
     * 
     * @return
     */
    public abstract boolean setDialogProperty();

    /**
     * 
     */
    public void windowClosing(final WindowEvent e) {
        super.windowClosing(e);
        this.onCanceled();
    }

    /**
     * 
     * @param obj
     * @return
     */
    protected boolean setValue(final SGTextField tf, final Object obj) {
        return SGUtility.setValue(tf, obj);
    }

    /**
     * 
     * @param obj
     * @return
     */
    protected boolean setValue(final SGSpinner sp, final Object obj) {
        return SGUtility.setValue(sp, obj);
    }

    /**
     * Called when a component gained focus.
     * @param e
     *          focus event
     */
    public void focusGained(FocusEvent e) {
        Object source = e.getSource();
        if (source instanceof SGColorSelectionButton) {
            SGColorSelectionButton btn = (SGColorSelectionButton) source;

            // set the border
            btn.setBorder(this.getFocusedBorder());
            
            // set the current color button
            this.mCurrentColorSetButton = btn;
        }
    }
    
    /**
     * Called when a component lost focus.
     * @param e
     *          focus event
     */
    public void focusLost(FocusEvent e) {
        Object source = e.getSource();
        if (source instanceof SGColorSelectionButton) {
            SGColorSelectionButton btn = (SGColorSelectionButton) source;
            Component opposite = e.getOppositeComponent();
            if (opposite == null) {
                return;
            }
            final boolean bcc = this.isColorChooserComponent(opposite);
            
            // if the opposite component is not the part of the color chooser dialog,
            // clear the focused button
            if (!bcc) {
                // set the border
                btn.setBorder(this.getUnfocusedBorder());
                
                // clear the current color button
                this.mCurrentColorSetButton = null;
            }
        }
    }
    
    /**
     * Checks whether the given component is a part of the color chooser dialog.
     * @param com
     *             a component
     * @return
     *             true if the component is a part of the color chooser dialog
     */
    private boolean isColorChooserComponent(Component com) {
        Container parent = com.getParent();
        if (parent == null) {
            return false;
        }
        if (this.mColorDialog.equals(parent)) {
            return true;
        } else {
            return this.isColorChooserComponent(parent);
        }
    }

    /**
     * Returns the border for a focused color selection button.
     *
     * @return the border for a focused color selection button
     */
    private Border getFocusedBorder() {
        return new MatteBorder(this.mColorSelectionButtonEmphasisIcon);
    }

    /**
     * Returns the border for an unfocused color selection button.
     *
     * @return the border for an unfocused color selection button
     */
    private Border getUnfocusedBorder() {
        return new LineBorder(Color.BLACK, 1);
    }

}
