package jp.riken.brain.ni.samuraigraph.base;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;

import org.w3c.dom.Element;

/**
 * 
 */
public class SGUtility implements SGIConstants, SGIDrawingElementConstants {

	public static void main(String[] args) {
		System.out.println("*** START ***");

		GraphicsEnvironment ge = GraphicsEnvironment
				.getLocalGraphicsEnvironment();

		Font[] fonts = ge.getAllFonts();
		for (int ii = 0; ii < fonts.length; ii++) {
			Font f = fonts[ii];
			System.out.println("-- " + f + " --");
			System.out.println("Name:     " + f.getName());
			System.out.println("FontName: " + f.getFontName());
			System.out.println("Family:   " + f.getFamily());
			System.out.println();
		}
		System.out.println();

		
/*		
		String names[] = ge.getAvailableFontFamilyNames();
		for (int ii = 0; ii < names.length; ii++) {
			System.out.println(names[ii]);
		}
		System.out.println();
*/
		
/*		
		for (int ii = 0; ii < names.length; ii++) {
			System.out.println(names[ii]);
			Font f1 = new Font(names[ii], Font.PLAIN, 1);
			Font f2 = new Font(f1.getName(), Font.PLAIN, 1);
			Font f3 = new Font(f1.getFamily(), Font.PLAIN, 1);
			Font f4 = new Font(f1.getFontName(), Font.PLAIN, 1);
			System.out.println(f1);
			System.out.println(f2);
			System.out.println(f3);
			System.out.println(f4);
			// System.out.println(f.getName());
			// System.out.println(f.getFontName());
			// System.out.println(f.getFamily());
			// System.out.println(f);
			System.out.println();
		}
*/
		System.out.println("*** END ***");
	}

    /**
	 * Returns an array of available font family names.
	 * 
	 * @return available font family names
	 */
    public static String[] getAvailableFontFamilyNames() {
        return GraphicsEnvironment.getLocalGraphicsEnvironment()
                .getAvailableFontFamilyNames();
    }
    
    /**
     * Finds a right font name from the system with a given font name.
     * 
     * @param name
     *           a font name
     * @return found font family name
     */
    public static String findFontFamilyName(final String name) {
    	if (name == null) {
    		throw new IllegalArgumentException("name == null");
    	}
    	String fName = null;
    	String[] names = getAvailableFontFamilyNames();
    	for (int ii = 0; ii < names.length; ii++) {
    		if (SGUtilityText.isEqualString(names[ii], name)) {
    			fName = names[ii];
    			break;
    		}
    	}
    	return fName;
    }

    /**
     * Returns the canonical path.
     * 
     * @param path
     *            path name
     * @return the canonical path
     */
    public static String getCanonicalPath(final String path) {
        File file = new File(path);
        String cPath = null;
        try {
            cPath = file.getCanonicalPath();
        } catch (IOException ex) {
            return null;
        }

        return cPath;
    }

    /**
     * 
     */
    public static Rectangle2D createUnion(final ArrayList rectList) {

        if (rectList == null) {
            throw new IllegalArgumentException("rectList==null");
        }

        Rectangle2D rectAll = null;
        for (int ii = 0; ii < rectList.size(); ii++) {
            Rectangle2D rect = (Rectangle2D) rectList.get(ii);
            if (rect == null) {
                throw new IllegalArgumentException("rect==null");
            }

            if (rectAll == null) {
                rectAll = rect;
                continue;
            }

            rectAll = rect.createUnion(rectAll);
        }

        return rectAll;
    }

    /**
     * 
     * @param rect
     * @param direction
     * @return
     */
    public static double getRectStart(final Rectangle2D rect,
            final boolean direction) {
        if (direction) {
            return rect.getX();
        }
        return rect.getY();
    }

    /**
     * 
     * @param rect
     * @param direction
     * @return
     */
    public static double getRectSize(final Rectangle2D rect,
            final boolean direction) {
        if (direction) {
            return rect.getWidth();
        }
        return rect.getHeight();
    }

    /**
     * 
     * @param rect
     * @param direction
     * @return
     */
    public static double getRectEnd(final Rectangle2D rect,
            final boolean direction) {
        return getRectStart(rect, direction) + getRectSize(rect, direction);
    }

    /**
     * 
     * @param rect
     * @param value
     * @param direction
     * @return
     */
    public static boolean setRectStart(final Rectangle2D rect,
            final double value, final boolean direction) {

        if (direction) {
            rect.setRect(value, rect.getY(), rect.getWidth(), rect.getHeight());
        } else {
            rect.setRect(rect.getX(), value, rect.getWidth(), rect.getHeight());
        }

        return true;
    }

    /**
     * 
     * @param rect
     * @param value
     * @param direction
     * @return
     */
    public static boolean setRectEnd(final Rectangle2D rect,
            final double value, final boolean direction) {

        if (direction) {
            rect.setRect(value - rect.getWidth(), rect.getY(), rect.getWidth(),
                    rect.getHeight());
        } else {
            rect.setRect(rect.getX(), value - rect.getHeight(),
                    rect.getWidth(), rect.getHeight());
        }

        return true;
    }

    /**
     * 
     * @param rect
     * @param value
     * @param direction
     * @return
     */
    public static boolean setRectSize(final Rectangle2D rect,
            final double value, final boolean direction) {

        if (direction) {
            rect.setRect(rect.getX(), rect.getY(), value, rect.getHeight());
        } else {
            rect.setRect(rect.getX(), rect.getY(), rect.getWidth(), value);
        }

        return true;
    }

    /**
     * 
     * @param rect
     * @param value
     * @param direction
     * @return
     */
    public static boolean isRectContains(final Rectangle2D rect,
            final double value, final boolean direction) {
        return SGUtilityNumber.contains(getRectStart(rect, direction),
                getRectEnd(rect, direction), value);
    }

    /**
     * 
     * @param rect1
     * @param rect2
     * @param direction
     * @return
     */
    public static boolean isRectContains(final Rectangle2D rect1,
            final Rectangle2D rect2, final boolean direction) {
        return SGUtilityNumber.contains(getRectStart(rect1, direction),
                getRectEnd(rect1, direction), getRectStart(rect2, direction),
                getRectEnd(rect2, direction));
    }

    /**
     * 
     * @param rect1
     * @param rect2
     * @param direction
     * @return
     */
    public static double getOverlapping(final Rectangle2D rect1,
            final Rectangle2D rect2, final boolean direction) {
        return SGUtilityNumber.getOverlap(getRectStart(rect1, direction),
                getRectEnd(rect1, direction), getRectStart(rect2, direction),
                getRectEnd(rect2, direction));
    }
    
    private static boolean mMessageDialogWasVisible = false;
    
    /**
     * Set false to mMessageDialogWasVisible flag.
     */
    public static void clearMessageDialogVisible() {
        mMessageDialogWasVisible = false;
    }
    
    /**
     * Set true to mMessageDialogWasVisible flag.
     */
    public static void setMessageDialogWasVisible() {
        mMessageDialogWasVisible = true;
    }
    
    /**
     * Return whether message dialog was showed.
     * @return
     */
    public static boolean wasMessageDialogVisible() {
        return mMessageDialogWasVisible;
    }

    /**
     * Show message dialog.
     */
    public static boolean showMessageDialog(Component parentComponent,
            Object message, String title, int messageType) {
        JOptionPane.showMessageDialog(parentComponent, message, title,
                messageType);
        setMessageDialogWasVisible();
        return true;
    }

    /**
     * 
     * @param parentComponent
     * @return
     */
    public static boolean showErrorMessageDialog(Component parentComponent,
            String msg, String title) {
        Toolkit.getDefaultToolkit().beep();
        SGUtility.showMessageDialog(parentComponent, msg, title,
                JOptionPane.ERROR_MESSAGE);
        return true;
    }

    /**
     * 
     * @param parentComponent
     * @return
     */
    public static boolean showIllegalInputErrorMessageDialog(
            Component parentComponent) {
        return showIllegalInputErrorMessageDialog(parentComponent,
                "The input value is illegal.");
    }

    /**
     * 
     * @param parentComponent
     * @return
     */
    public static boolean showIllegalInputErrorMessageDialog(
            Component parentComponent, String msg) {
        return showErrorMessageDialog(parentComponent, msg, "Illegal input");
    }

    /**
     * 
     * @param parentComponent
     * @return
     */
    public static boolean showFileNotFoundMessageDialog(
            Component parentComponent) {
        SGUtility.showMessageDialog(parentComponent,
                "Failed to open the file denoted by a specified pathname.",
                "File not found", JOptionPane.ERROR_MESSAGE);
        return true;
    }

    /**
     * 
     * @param wnd
     * @return
     */
    public static void setCenter(final Window wnd, final Window owner) {
        Dimension dim = owner.getSize();
        final int width = wnd.getWidth();
        final int height = wnd.getHeight();
        final int x = owner.getX() + (dim.width - width) / 2;
        final int y = owner.getY() + (dim.height - height) / 2;
        wnd.setLocation(x, y);
    }

    /**
     * Copy the list of copied objects.
     * 
     * @param list1
     *            a list which contains copiable objects to be copied
     * @param list2
     *            a list to set the copied objects
     * @return true if succeeded
     */
    @SuppressWarnings("unchecked")
	public static boolean copyObjects(final List list1,
            final List list2) {
        if (list1 == null || list2 == null) {
            throw new IllegalArgumentException("list1==null || list2==null");
        }

        for (int ii = 0; ii < list1.size(); ii++) {
            Object obj = list1.get(ii);
            if ((obj instanceof SGICopiable) == false) {
                throw new IllegalArgumentException(
                        "Not copiable objects are included.");
            }
            SGICopiable cp = (SGICopiable) obj;
            list2.add((SGICopiable) cp.copy());
        }

        return true;
    }

    /**
     * Add a menu command to the pop-up menu.
     * 
     * @param p -
     *            a pop-up menu
     * @param l -
     *            an action listener of the command
     * @param cmd -
     *            a command to be added
     * @return added menu item
     */
    public static JMenuItem addItem(JPopupMenu p, ActionListener l, String cmd) {
        final JMenuItem item = new JMenuItem(cmd);
        p.add(item);
        item.addActionListener(l);
        return item;
    }
    
    /**
     * Add a menu command to the pop-up menu.
     * 
     * @param p -
     *            a pop-up menu
     * @param l -
     *            an action listener of the command
     * @param cmd -
     *            a command to be added
     * @param enabled -
     *            a flag whether the command is enabled
     * @return added menu item
     */
    public static JMenuItem addItem(JPopupMenu p, ActionListener l, String cmd, boolean enabled) {
        final JMenuItem item = new JMenuItem(cmd);
        p.add(item);
        item.addActionListener(l);
        item.setEnabled(enabled);
        return item;
    }

    /**
     * Add a menu command with the check box to the pop-up menu.
     * 
     * @param p -
     *            a pop-up menu
     * @param l -
     *            an action listener of the command
     * @param cmd -
     *            a command to be added
     * @return added menu item
     */
    public static JCheckBoxMenuItem addCheckBoxItem(JPopupMenu p,
            ActionListener l, String cmd) {
        final JCheckBoxMenuItem item = new JCheckBoxMenuItem(cmd);
        p.add(item);
        item.addActionListener(l);
        return item;
    }

    /**
     * Identify the OS name.
     * 
     * @param prefix -
     *            the prefix of OS name
     * @return whether the OS name starts with the prefix
     */
    public static boolean identifyOS(final String prefix) {
        if (prefix == null) {
            throw new IllegalArgumentException("prefix==null");
        }

        return OS_NAME.toLowerCase().startsWith(prefix);
    }

    /**
     * 
     * @return
     */
    public static String getLookAndFeelID() {
        LookAndFeel laf = UIManager.getLookAndFeel();
        if (laf != null) {
            return laf.getID();
        }
        return null;
    }

    /**
     * Copy an array of a String array.
     * 
     * @param src
     *            an array to be copied
     * @return a copy of an array
     */
    public static String[] copyStringArray(String[] src) {
        final String[] dest = new String[src.length];
        System.arraycopy(src, 0, dest, 0, src.length);
        return dest;
    }

    /**
     * Copy an array of a String array.
     * 
     * @param src
     *            an array to be copied
     * @return a copy of an array
     */
    public static String[][] copyStringArray(String[][] src) {
        final String[][] dest = new String[src.length][];
        for (int ii = 0; ii < src.length; ii++) {
            dest[ii] = copyStringArray(src[ii]);
        }
        return dest;
    }

    /**
     * Copy an array of a primitive integer array.
     * 
     * @param src
     *            an array to be copied
     * @return a copy of an array
     */
    public static int[] copyIntegerArray(int[] src) {
        final int[] dest = new int[src.length];
        System.arraycopy(src, 0, dest, 0, src.length);
        return dest;
    }

    /**
     * Copy an array of a primitive integer array.
     * 
     * @param src
     *            an array to be copied
     * @return a copy of an array
     */
    public static Integer[] copyIntegerArray(Integer[] src) {
        final Integer[] dest = new Integer[src.length];
        System.arraycopy(src, 0, dest, 0, src.length);
        return dest;
    }
    
    /**
     * Copy an array of a primitive double array.
     * 
     * @param src
     *            an array to be copied
     * @return a copy of an array
     */
    public static double[] copyDoubleArray(double[] src) {
        final double[] dest = new double[src.length];
        System.arraycopy(src, 0, dest, 0, src.length);
        return dest;
    }

    /**
     * Copy an array of a primitive double array.
     * 
     * @param src
     *            an array to be copied
     * @return a copy of an array
     */
    public static double[][] copyDoubleArray(double[][] src) {
        final double[][] dest = new double[src.length][];
        for (int ii = 0; ii < src.length; ii++) {
            dest[ii] = copyDoubleArray(src[ii]);
        }
        return dest;
    }
    
    /**
     * 
     * @param obj
     * @return
     */
    public static boolean setValue(final SGTextField tf, final Object obj) {
        if (obj == null) {
            tf.setText(null);
        } else {
            String str = obj.toString().trim();
            Double d = SGUtilityText.getDouble(str);
            if (d == null) {
                return false;
            }
            tf.setText(str);
        }

        return true;
    }

    /**
     * 
     * @param obj
     * @return
     */
    public static boolean setIntValue(final SGTextField tf, final Object obj) {
        if (obj == null) {
            tf.setText(null);
        } else {
            String str = obj.toString().trim();
            Integer n = SGUtilityText.getInteger(str);
            if (n == null) {
                return false;
            }
            tf.setText(str);
        }

        return true;
    }

    /**
     * 
     * @param obj
     * @return
     */
    public static boolean setValue(final SGSpinner sp, final Object obj) {
        if (obj == null) {
            sp.setText("");
            return false;
        }

        String str = obj.toString().trim();
        Double d = SGUtilityText.getDouble(str);
        if (d == null) {
            return false;
        }
        sp.setText(str);
        sp.setValue(d);

        try {
            sp.commitEditByDefault();
        } catch (ParseException e) {
        }

        return true;
    }

    /**
     * 
     * @param obj
     * @return
     */
    public static boolean setIntValue(final SGSpinner sp, final Object obj) {
        if (obj == null) {
            sp.setText("");
            return false;
        }

        String str = obj.toString().trim();
        Integer n = SGUtilityText.getInteger(str);
        if (n == null) {
            return false;
        }
        sp.setText(str);
        sp.setValue(Double.valueOf(n.doubleValue()));

        try {
            sp.commitEditByDefault();
        } catch (ParseException e) {
        }

        return true;
    }

    /**
     * Returns a location on a rectangle.
     */
    public static int getMouseLocation(final Rectangle2D rect, final int x,
            final int y, final float radius) {
        final boolean bMinX = (Math.abs(x - rect.getMinX()) < radius);
        final boolean bMidX = (Math.abs(x - rect.getCenterX()) < radius);
        final boolean bMaxX = (Math.abs(x - rect.getMaxX()) < radius);
        final boolean bMinY = (Math.abs(y - rect.getMinY()) < radius);
        final boolean bMidY = (Math.abs(y - rect.getCenterY()) < radius);
        final boolean bMaxY = (Math.abs(y - rect.getMaxY()) < radius);

        int location = OTHER;

        if (bMinX) {
            if (bMinY) {
                // north west
                location = NORTH_WEST;
            } else if (bMidY) {
                // west
                location = WEST;
            } else if (bMaxY) {
                // south west
                location = SOUTH_WEST;
            }
        } else if (bMaxX) {
            if (bMinY) {
                // north east
                location = NORTH_EAST;
            } else if (bMidY) {
                // east
                location = EAST;
            } else if (bMaxY) {
                // south east
                location = SOUTH_EAST;
            }
        } else if (bMidX) {
            if (bMinY) {
                // north
                location = NORTH;
            } else if (bMaxY) {
                // south
                location = SOUTH;
            }
        }

        return location;
    }

    /**
     * Resize the rectangle by mouse drag.
     * 
     * @param rect
     * @param pos
     * @param e
     * @param ml
     */
    public static void resizeRectangle(final Rectangle2D rect, // rectangle
            final Point pos, // mouse position
            final MouseEvent e, // mouse event on dragging
            final int ml // mouse location
    ) {

        // is Shift-key is pressed
        final boolean onShift = ((e.getModifiers() & InputEvent.SHIFT_MASK) != 0);

        final int diffX = e.getX() - pos.x;
        final int diffY = e.getY() - pos.y;

        final float xOld = (float) rect.getX();
        final float yOld = (float) rect.getY();
        final float wOld = (float) rect.getWidth();
        final float hOld = (float) rect.getHeight();

        final float sizeOldX = wOld;
        final float sizeOldY = hOld;

        float sizeNewX = 0.0f;
        float sizeNewY = 0.0f;

        // variables to be updated
        float x = xOld;
        float y = yOld;
        float w = wOld;
        float h = hOld;

        if (ml == NORTH) {
            // System.out.println("NORTH");
            pos.setLocation(pos.getX(), pos.getY() + diffY);
            sizeNewY = sizeOldY - diffY;
            y = yOld + sizeOldY - sizeNewY;
            h = sizeNewY;
        } else if (ml == SOUTH) {
            // System.out.println("SOUTH");
            pos.setLocation(pos.getX(), pos.getY() + diffY);
            sizeNewY = sizeOldY + diffY;
            h = sizeNewY;
        } else if (ml == WEST) {
            // System.out.println("WEST");
            pos.setLocation(pos.getX() + diffX, pos.getY() + diffY);
            sizeNewX = sizeOldX - diffX;
            x = xOld + sizeOldX - sizeNewX;
            w = sizeNewX;
        } else if (ml == EAST) {
            // System.out.println("EAST");
            pos.setLocation(pos.getX() + diffX, pos.getY());
            sizeNewX = sizeOldX + diffX;
            w = sizeNewX;
        } else if (ml == NORTH_WEST) {
            pos.setLocation(pos.getX() + diffX, pos.getY() + diffY);
            sizeNewX = sizeOldX - diffX;

            if (onShift) {
                sizeNewY = sizeNewX * (sizeOldY / sizeOldX);
            } else {
                sizeNewY = sizeOldY - diffY;
            }
            x = xOld + sizeOldX - sizeNewX;
            y = yOld + sizeOldY - sizeNewY;
            w = sizeNewX;
            h = sizeNewY;
            // System.out.println(sizeOldY+" "+sizeNewY);
        } else if (ml == NORTH_EAST) {
            if (!onShift) {
                pos.setLocation(pos.getX() + diffX, pos.getY() + diffY);

                sizeNewX = sizeOldX + diffX;
                sizeNewY = sizeOldY - diffY;

                y = yOld + sizeOldY - sizeNewY;
                w = sizeNewX;
                h = sizeNewY;

            } else {
                pos.setLocation(pos.getX(), pos.getY() + diffY);

                sizeNewY = sizeOldY - diffY;
                sizeNewX = sizeNewY * (sizeOldX / sizeOldY);

                y = yOld + sizeOldY - sizeNewY;
                w = sizeNewX;
                h = sizeNewY;
            }
        } else if (ml == SOUTH_WEST) {
            if (!onShift) {
                pos.setLocation(pos.getX() + diffX, pos.getY() + diffY);

                sizeNewX = sizeOldX - diffX;
                sizeNewY = sizeOldY + diffY;

                x = xOld + sizeOldX - sizeNewX;
                w = sizeNewX;
                h = sizeNewY;
            } else {
                pos.setLocation(pos.getX() + diffX, pos.getY() + diffY);

                sizeNewX = sizeOldX - diffX;
                sizeNewY = sizeNewX * (sizeOldY / sizeOldX);

                x = xOld + sizeOldX - sizeNewX;
                w = sizeNewX;
                h = sizeNewY;
            }
        } else if (ml == SOUTH_EAST) {
            if (!onShift) {
                pos.setLocation(pos.getX() + diffX, pos.getY() + diffY);

                sizeNewX = sizeOldX + diffX;
                sizeNewY = sizeOldY + diffY;

                w = sizeNewX;
                h = sizeNewY;
            } else {
                pos.setLocation(pos.getX() + diffX, pos.getY());

                sizeNewX = sizeOldX + diffX;
                sizeNewY = sizeNewX * (sizeOldY / sizeOldX);

                w = sizeNewX;
                h = sizeNewY;
            }
        }

        // update the rectangle
        rect.setRect(x, y, w, h);

        // System.out.println(rect);
    }

    /**
     * Moves an object to the tail of the list.
     * 
     * @param obj
     *            an objct to be moved
     * @param list
     *            the list which contains the object
     * @return true if succeeded
     */
    public static boolean moveObjectToTail(final Object obj,
            final List list) {
        return moveObjectTo(obj, list, list.size() - 1);
    }

    /**
     * Moves an object to next position of the list.
     * 
     * @param obj
     *            an objct to be moved
     * @param list
     *            the list which contains the object
     * @return true if succeeded
     */
    public static boolean moveObjectToNext(final Object obj,
            final List list) {
        List<Object> objList = new ArrayList<Object>();
        objList.add(obj);
        return moveObject(objList, list, 1);
    }

    /**
     * Moves an object to previous position of the list.
     * 
     * @param obj
     *            an objct to be moved
     * @param list
     *            the list which contains the object
     * @return true if succeeded
     */
    public static boolean moveObjectToPrevious(final Object obj,
            final List list) {
        List<Object> objList = new ArrayList<Object>();
        objList.add(obj);
        return moveObject(objList, list, -1);
    }

    /**
     * Moves an object to the head of the list.
     * 
     * @param obj
     *            an objct to be moved
     * @param list
     *            the list which contains the object
     * @return true if succeeded
     */
    public static boolean moveObjectToHead(final Object obj,
            final List list) {
        return moveObjectTo(obj, list, 0);
    }

    /**
     * Moves an object to the location of given index.
     * 
     * @param obj
     *            an object to be moved
     * @param list
     *            a list that contains the given object
     * @param index
     *            the array index
     * @return true if succeeded
     */
    public static boolean moveObjectTo(final Object obj, final List list,
            final int index) {
        if (list == null) {
            throw new IllegalArgumentException("list==null");
        }
        if (index < 0 || index >= list.size()) {
            throw new IllegalArgumentException(
                    "index < 0 || index >= list.size() : " + index);
        }
        if (list.remove(obj) == false) {
            return false;
        }
        list.add(index, obj);
        return true;
    }
    
    /**
     * Moves the objects in a list.
     * 
     * @param movedList
     *           a list of objects to be moved
     * @param list
     *           a list of objects
     * @param num
     *           the level to move the objects
     * @return true if succeeded
     */
    public static boolean moveObject(final List movedList, final List list,
            final int num) {

        if (num > 0) {
            for (int ii = movedList.size() - 1; ii >= 0; ii--) {
                Object obj = movedList.get(ii);
                final int index = list.indexOf(obj);
                if (index == -1) {
                    return false;
                }
                final int indexNew = index + num;
                if (indexNew > list.size() - 1) {
                    continue;
                }
                if (moveObjectTo(obj, list, indexNew) == false) {
                    return false;
                }
            }
        } else {
            for (int ii = 0; ii < movedList.size(); ii++) {
                Object obj = movedList.get(ii);
                final int index = list.indexOf(obj);
                if (index == -1) {
                    return false;
                }
                final int indexNew = index + num;
                if (indexNew < 0) {
                    continue;
                }
                if (moveObjectTo(obj, list, indexNew) == false) {
                    return false;
                }
            }
        }
        
        return true;
    }
        
    /**
     * 
     * @param list
     *            a list to be modified
     * @param listVisible
     *            a list of visible objects
     * @return
     */
    public static boolean setVisibleList(final List list, final List listVisible) {
        ArrayList listInvisible = new ArrayList();
        for (int ii = 0; ii < list.size(); ii++) {
            SGIVisible el = (SGIVisible) list.get(ii);
            final boolean b = listVisible.contains(el);
            el.setVisible(b);
            if (!b) {
                listInvisible.add(el);
            }
        }

        list.clear();
        for (int ii = 0; ii < listVisible.size(); ii++) {
            list.add(listVisible.get(ii));
        }
        for (int ii = 0; ii < listInvisible.size(); ii++) {
            list.add(listInvisible.get(ii));
        }

        return true;
    }

    /**
     * Returns the file name from given path name.
     * 
     * @param path
     *              path name
     * @return
     *              the file name
     */
    public static String getFileName(final String path) {
        final String fs = FILE_SEPARATOR;
        StringTokenizer stk = new StringTokenizer(path, fs);
        String sub = null;
        while (stk.hasMoreTokens()) {
            sub = stk.nextToken();
        }
        if (sub == null) {
            sub = path;
        }
        // tokenize by URL separator.
        stk = new StringTokenizer(sub, "/");
        while (stk.hasMoreTokens()) {
            sub = stk.nextToken();
        }
        return sub;
    }
    
    /**
     * Returns the simplified file name from given path name.
     * 
     * @param path
     *              path name
     * @return
     *              the simplified file name
     */
    public static String getSimpleFileName(final String path) {
        
        // get the file name
        String fileName = getFileName(path);

        String name = null;
        StringTokenizer stk = new StringTokenizer(fileName, Character.toString('.'));
        while (stk.hasMoreTokens()) {
            name = stk.nextToken();
            break;
        }
        if (name == null) {
            name = fileName;
        }

        return name;
    }

    /**
     * Creates a text string for a base of the data names from the file path.
     * 
     * @param filepath
     *             path of the data file
     * @return a text string for data name
     */
    public static String createDataNameBase(final String filepath) {
        
        // check the input value
        if (SGUtilityText.isValidString(filepath) == false) {
            return null;
        }
	
        // get the file name
        String name = getSimpleFileName(filepath);

        // add escape character
        String nameNew = addEscapeChar(name);
        
        return nameNew;
    }
    
    /**
     * Add escape characters before each characters '_' or '^';
     * 
     * @param str
     *           an input text string
     * @return
     *           modified text string
     */
    public static String addEscapeChar(final String str) {
        // if the simplified file name contains underscores, 
        // append a backslash before each of them
        char[] cArray = str.toCharArray();
        List<Character> cList = new ArrayList<Character>();
        for (int ii = 0; ii < cArray.length; ii++) {
            final char c = cArray[ii];
            if (c == '_' || c == '^') {
                cList.add('\\');
            }
            cList.add(c);
        }
        char[] cArrayNew = new char[cList.size()];
        for (int ii = 0; ii < cArrayNew.length; ii++) {
            cArrayNew[ii] = cList.get(ii);
        }
        String strNew = new String(cArrayNew);
        return strNew;
    }
    
    /**
     * Remove escape characters from a given string.
     * 
     * @param str
     *           an input text string
     * @return
     *           modified text string
     */
    public static String removeEscapeChar(final String str) {
        char[] cArray = str.toCharArray();
        List<Character> cList = new ArrayList<Character>();
        for (int ii = 0; ii < cArray.length; ii++) {
            final char c = cArray[ii];
            if (c != '\\') {
                cList.add(c);
            }
        }
        char[] cArrayNew = new char[cList.size()];
        for (int ii = 0; ii < cArrayNew.length; ii++) {
            cArrayNew[ii] = cList.get(ii);
        }
        String strNew = new String(cArrayNew);
        return strNew;
    }
    
    /**
     * Check whether two object arrays are equal. Null values are permitted.
     * 
     * @param a1
     *          the first array
     * @param a2
     *          the second array
     * @return true if two arrays are equal
     */
    public static boolean equals(Object[] a1, Object[] a2) {
        if (a1 != null) {
            if (a2 == null) {
                return false;
            }
            if (a1.length != a2.length) {
                return false;
            }
            for (int ii = 0; ii < a1.length; ii++) {
                if (a1[ii] != null) {
                    if (a1[ii].equals(a2[ii]) == false) {
                        return false;
                    }                        
                } else {
                    if (a2 != null) {
                        return false;
                    }
                }
            }
        } else {
            if (a2 != null) {
                return false;
            }
        }
        return true;
    }
    
    /**
     * Check whether two objects are equal. Null values are permitted.
     * 
     * @param obj1
     *           the first object
     * @param obj2
     *           the second object
     * @return true if two objects are equal
     */
    public static boolean equals(Object obj1, Object obj2) {
        if (obj1 != null) {
            return obj1.equals(obj2);
        } else {
            if (obj2 == null) {
                return true;
            } else {
                return false;
            }
        }
    }
    
    public static boolean isValidPropertyValue(final float value) {
    	if (Float.isNaN(value) || Float.isInfinite(value)) {
    		return false;
    	}
    	return true;
    }

    public static boolean isValidPropertyValue(final double value) {
    	if (Double.isNaN(value) || Double.isInfinite(value)) {
    		return false;
    	}
    	return true;
    }

    /**
     * Calculates a value to set to a property of various objects.
     * 
     * @param value
     *           an input value
     * @param unit
     *           the unit for the input value
     * @param convUnit
     *           the unit to convert
     * @param min
     *           the minimum value for converted result
     * @param max
     *           the minimum value for converted result
     * @param order
     *           the minimal order for converted result
     */
    public static Float calcPropertyValue(final float value, final String unit, 
            final String convUnit, final double min, final double max, final int order) {
        
    	// check the input value
    	if (isValidPropertyValue(value) == false) {
    		return null;
    	}
    	
        // convert a given value to a value in a given unit
        double vConv;
        if (unit == null || convUnit == null) {
        	if (value < min) {
                return null;
        	} else if (value > max) {
                return null;
        	} else {
                vConv = value;
        	}
        } else {
            final double conv = SGUtilityText.convert(value, unit, convUnit);
            if (conv < min) {
                return null;
            } else if (conv > max) {
                return null;
            } else {
                vConv = conv;
            }
        }
        
        // round off the value in a given unit
        final int digit = order - 1;
        final double convOff = SGUtilityNumber.roundOffNumber(vConv, digit);
        
        // convert the value
        final float vNew;
        if (convUnit == null) {
            vNew = (float) convOff;
        } else {
            vNew = (float) SGUtilityText.convertToPoint(convOff, convUnit);
        }
        return Float.valueOf(vNew);
    }

    /**
     * Returns the line width from the input value within an allowed range.
     * 
     * @param lw
     *          an input value
     * @param unit
     *          the unit for the input value
     * @return line width within an allowed range
     */
    public static final Float getLineWidth(final float lw, final String unit) {
        return calcPropertyValue(lw, unit, LINE_WIDTH_UNIT, LINE_WIDTH_MIN_VALUE, 
                LINE_WIDTH_MAX_VALUE, LINE_WIDTH_MINIMAL_ORDER);
    }
    
    /**
     * Returns the font size from the input value within an allowed range.
     * 
     * @param size
     *          an input value
     * @param unit
     *          the unit for the input value
     * @return font size within an allowed range
     */
    public static final Float getFontSize(final float size, final String unit) {
        return calcPropertyValue(size, unit, FONT_SIZE_UNIT, FONT_SIZE_MIN_VALUE, 
                FONT_SIZE_MAX_VALUE, FONT_SIZE_MINIMAL_ORDER);
    }
    
    
    // returns an integer value for axis location from a string
    public static int getAxisLocation(final String location) {
        int loc = -1;
        if (SGUtilityText.isEqualString(SGIFigureElementAxis.AXIS_BOTTOM, location)) {
            loc = SGIFigureElementAxis.AXIS_HORIZONTAL_1;
        } else if (SGUtilityText.isEqualString(SGIFigureElementAxis.AXIS_TOP, location)) {
            loc = SGIFigureElementAxis.AXIS_HORIZONTAL_2;
        } else if (SGUtilityText.isEqualString(SGIFigureElementAxis.AXIS_LEFT, location)) {
            loc = SGIFigureElementAxis.AXIS_PERPENDICULAR_1;
        } else if (SGUtilityText.isEqualString(SGIFigureElementAxis.AXIS_RIGHT, location)) {
            loc = SGIFigureElementAxis.AXIS_PERPENDICULAR_2;
        }
        return loc;
    }

    /**
     * Returns the location name of a given location ID.
     * 
     * @param id
     *           a location ID
     * @return the location name
     */
    public static String getLocationName(final int id) {
        String name = null;
        switch (id) {
        case SGIFigureElementAxis.AXIS_HORIZONTAL_1:
            name = SGIFigureElementAxis.AXIS_BOTTOM;
            break;
        case SGIFigureElementAxis.AXIS_HORIZONTAL_2:
            name = SGIFigureElementAxis.AXIS_TOP;
            break;
        case SGIFigureElementAxis.AXIS_PERPENDICULAR_1:
            name = SGIFigureElementAxis.AXIS_LEFT;
            break;
        case SGIFigureElementAxis.AXIS_PERPENDICULAR_2:
            name = SGIFigureElementAxis.AXIS_RIGHT;
            break;
        case SGIFigureElementAxis.AXIS_NORMAL:
            name = SGIFigureElementAxis.AXIS_COLOR_BAR;
            break;
        }
        return name;
    }

    /**
     * Returns the location ID of a given location name.
     * 
     * @param str
     *           a location name
     * @return the location ID
     */
    public static int getLocationInPlane(final String str) {
        int loc = -1;
        if (SGUtilityText.isEqualString(SGIFigureElementAxis.AXIS_BOTTOM, str)) {
            loc = SGIFigureElementAxis.AXIS_HORIZONTAL_1;
        } else if (SGUtilityText.isEqualString(SGIFigureElementAxis.AXIS_TOP, str)) {
            loc = SGIFigureElementAxis.AXIS_HORIZONTAL_2;
        } else if (SGUtilityText.isEqualString(SGIFigureElementAxis.AXIS_LEFT, str)) {
            loc = SGIFigureElementAxis.AXIS_PERPENDICULAR_1;
        } else if (SGUtilityText.isEqualString(SGIFigureElementAxis.AXIS_RIGHT, str)) {
            loc = SGIFigureElementAxis.AXIS_PERPENDICULAR_2;
        } else if (SGUtilityText.isEqualString(SGIFigureElementAxis.AXIS_COLOR_BAR, str)) {
            loc = SGIFigureElementAxis.AXIS_NORMAL;
        }
        return loc;
    }

    /**
     * Assigns ID number.
     * 
     * @param curIdList
     *           The list of current ID numbers. All elements must be positive.
     * @return an assigned ID number
     */
    public static int assignIdNumber(final List<Integer> curIdList) {
    	
    	// returns 1 when the given ID list is empty
    	if (curIdList.size() == 0) {
    		return 1;
    	}
    	
    	// sorts the current ID numbers
    	Integer[] array = new Integer[curIdList.size()];
    	array = curIdList.toArray(array);
    	Arrays.sort(array);
    	
    	int id;
    	final int last = array[array.length - 1];
    	if (last == Integer.MAX_VALUE) {
    		if (array.length == Integer.MAX_VALUE) {
    			// empty numbers do not exist
    			// this shouldn't happen
    			throw new Error("Failed to assign new ID number.");
    		}

        	// search an empty number
    		id = 1;
    		for (int ii = 0; ii < array.length; ii++) {
    			final int num = ii + 1;
    			if (array[ii] != num) {
    				id = num;
    				break;
    			}
    		}
    	} else {
    		id = last + 1;
    	}
    	
    	return id;
    	
/*    	
    	// initialize the returned value
    	int id = array.length + 1;
    	
    	// search an empty number
		for (int ii = 0; ii < array.length; ii++) {
			if (array[ii] <= 0) {
    			throw new IllegalArgumentException("Non-positive ID number exists: " 
    					+ array[ii]);
			}
			final int num = ii + 1;
			if (array[ii] != num) {
				id = num;
				break;
			}
		}
		return id;
*/		
    }

    /**
     * Checks whether a given array contains a given object.
     * Null objects are ignored.
     * 
     * @param array
     *           an array
     * @param obj
     *           an object
     * @return true if a given array contains a given object
     */
    public static boolean containsNullIgnored(Object[] array, Object obj) {
    	if (array == null) {
    		throw new IllegalArgumentException("array == null");
    	}
    	if (obj == null) {
    		return false;
    	}
    	for (int ii = 0; ii < array.length; ii++) {
    		if (array[ii] == null) {
    			continue;
    		}
    		if (array[ii].equals(obj)) {
    			return true;
    		}
    	}
    	return false;
    }
    
    /**
     * Checks whether a given array a1 contains at least one element of an array a2.
     * Null objects are ignored.
     * 
     * @param a1
     *           the first array
     * @param a2
     *           the second array
     * @return true if a given array a1 contains at least one element of an array a2
     */
    public static boolean containsNullIgnored(Object[] a1, Object[] a2) {
    	if (a1 == null || a2 == null) {
    		throw new IllegalArgumentException("a1 == null || a2 == null");
    	}
    	for (int ii = 0; ii < a2.length; ii++) {
    		if (containsNullIgnored(a1, a2[ii])) {
    			return true;
    		}
    	}
    	return false;
    }

    /**
     * Checks whether a given array contains a given object.
     * 
     * @param array
     *           an array
     * @param obj
     *           an object
     * @return true if a given array contains a given object
     * @throws IllegalArgumentException if array is null.
     */
    public static boolean contains(Object[] array, Object obj) {
    	if (array == null) {
    	    throw new IllegalArgumentException("array == null");
    	}
    	for (int ii = 0; ii < array.length; ii++) {
    	    if (SGUtility.equals(array[ii], obj)) {
    	        return true;
    	    }
    	}
    	return false;
    }
    
    /**
     * Checks whether a given array a1 contains at least one element of an array a2.
     * 
     * @param a1
     *           the first array
     * @param a2
     *           the second array
     * @return true if a given array a1 contains at least one element of an array a2
     */
    public static boolean contains(Object[] a1, Object[] a2) {
    	if (a1 == null || a2 == null) {
    		throw new IllegalArgumentException("a1 == null || a2 == null");
    	}
    	for (int ii = 0; ii < a2.length; ii++) {
    		if (contains(a1, a2[ii])) {
    			return true;
    		}
    	}
    	return false;
    }

    /**
     * Checks whether the first list contains at least one element in the second list.
     * 
     * @param list1
     *           the first list
     * @param list2
     *           the second list
     * @return true if the first list contains at least one element in the second list
     */
    public static boolean contains(List list1, List list2) {
    	if (list1 == null || list2 == null) {
    		throw new IllegalArgumentException("a1 == null || a2 == null");
    	}
    	for (int ii = 0; ii < list2.size(); ii++) {
    		if (list1.contains(list2.get(ii))) {
    			return true;
    		}
    	}
    	return false;
    }

    /**
     * Checks whether the first list contains all elements in the second list.
     * 
     * @param list1
     *           the first list
     * @param list2
     *           the second list
     * @return true if the first list contains all elements in the second list
     */
    public static boolean containsAll(List list1, List list2) {
    	if (list1 == null || list2 == null) {
    		throw new IllegalArgumentException("a1 == null || a2 == null");
    	}
    	for (int ii = 0; ii < list2.size(); ii++) {
    		if (!list1.contains(list2.get(ii))) {
    			return false;
    		}
    	}
    	return true;
    }

    /**
     * Read and returns the column indices.
     * 
     * @param el
     *            an Element object
     * @param key
     *            the key of index property
     * @return the indices it exists
     */
    public static Integer[] readIndices(Element el, String key) {
//        List<Integer> numList = new ArrayList<Integer>();
        String str = el.getAttribute(key);
//        if (str.length() != 0) {
//            final int start = str.indexOf('{');
//            final int end = str.lastIndexOf('}');
//            if (start == -1 || end == -1) {
//            	Integer num = SGUtilityText.getInteger(str);
//            	if (num == null) {
//            		return null;
//            	}
//            	return new Integer[] { num };
//            }
//            if (start > end) {
//                return null;
//            }
//            String sub = str.substring(start + 1, end);
//            StringTokenizer st = new StringTokenizer(sub, ",");
//            while (st.hasMoreTokens()) {
//                String token = st.nextToken();
//                Integer num = SGUtilityText.getInteger(token);
//                if (num == null) {
//                    return null;
//                }
//                numList.add(num);
//            }
//        }
//        Integer[] array = new Integer[numList.size()];
//        for (int ii = 0; ii < array.length; ii++) {
//            array[ii] = (Integer) numList.get(ii);
//        }
//        return array;
        return SGUtilityText.parseIndices(str);
    }

    /**
     * Return the string of given value.
     * @param value
     * @return
     */
    public static String getNumberName(double value) {
        boolean minus = (value < 0);
        String sval;
        value = Math.abs(value);
        if (value==0.0) {
            return "0";
        } else if (value > 10000) {
            double val2 = value;
            int count = 0;
            while (val2>=1) {
                val2 = val2 / 10.0;
                count++;
            }
            val2 = value/Math.pow(10, count-1);
            sval = String.format("%.2f", val2);
            sval = SGUtility.removeLastZero(sval);
            sval = sval+String.format("e%d", count-1);
        } else if (value>100) {
            sval = String.format("%.0f", value);
        } else if (value>1) {
            sval = String.format("%.2f", value);
            sval = SGUtility.removeLastZero(sval);
        } else if (value>0.0001) {
            sval = String.format("%.4f", value);
            sval = SGUtility.removeLastZero(sval);
        } else {
            double val2 = value;
            int count = 0;
            while (val2<10) {
                val2 = val2 * 10.0;
                count++;
            }
            val2 = value * Math.pow(10, count-1);
            sval = String.format("%.2f", val2);
            sval = SGUtility.removeLastZero(sval);
            sval = sval+String.format("e-%d", count-1);
        }
        if (minus) {
            return "-"+sval;
        } else {
            return sval;
        }
    }
    
    /**
     * Remove last zero characters in string.
     * @param str
     * @return string which has no zero at last.
     */
    private static String removeLastZero(String str) {
        while (str.substring(str.length()-1).equals("0")) {
            str = str.substring(0, str.length()-1);
        }
        if (str.substring(str.length()-1).equals(".")) {
            str = str.substring(0, str.length()-1);
        }
        return str;
    }
}
