/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * SGTextDataColumnSelectionPanel.java
 *
 * Created on 2009/08/20, 15:09:35
 */

package jp.riken.brain.ni.samuraigraph.data;

import java.awt.event.MouseEvent;

import javax.swing.JTable;

import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;


/**
 * A panel to select columns from an array data.
 *
 */
public class SGArrayDataColumnSelectionPanel extends SGDataColumnSelectionPanel {

    /**
     * Serial Version UID
     */
    private static final long serialVersionUID = 3814702849177447212L;

    /** Creates new form SGTextDataColumnSelectionPanel */
    public SGArrayDataColumnSelectionPanel() {
        super();
    }

    /**
     * Name of the column for the title of each data column.
     */
    public static final String COLUMN_NAME_TITLE = "Title";

    /**
     * Name of the column for the value type of each data column such as
     * number or text.
     */
    public static final String COLUMN_NAME_VALUE_TYPE = "Value Type";

    public static final String[] COLUMN_NAME_ARRAY = { COLUMN_NAME_NUMBER,
        COLUMN_NAME_TITLE, COLUMN_NAME_VALUE_TYPE, COLUMN_NAME_COLUMN_TYPE };

    /**
     * A constant array of column width.
     */
    private static final int[] PREFERRED_COLUMN_WIDTH_ARRAY = { 10, 80, 80, 110 };
    
    /**
     * An array of acceptable data types.
     */
    private static final String[] DATA_TYPE_ARRAY = {
            SGDataTypeConstants.SXY_DATA,
            SGDataTypeConstants.VXY_DATA,
            SGDataTypeConstants.SXYZ_DATA,
            SGDataTypeConstants.SXY_MULTIPLE_DATA,
            SGDataTypeConstants.SXY_SAMPLING_DATA,
            SGDataTypeConstants.SXY_DATE_DATA };

    protected String[] getColumnNameArray() {
        return COLUMN_NAME_ARRAY;
    }

    protected int[] getPreferredColumnWidthArray() {
        return PREFERRED_COLUMN_WIDTH_ARRAY;
    }

    protected String[] getDataTypeArray() {
        return DATA_TYPE_ARRAY;
    }

    protected DataColumnTableModel createTableModel() {
        return new TextDataColumnTableModel();
    }

    /**
     * A class of table model.
     */
    protected class TextDataColumnTableModel extends DataColumnTableModel {
        
        /**
         * 
         */
        private static final long serialVersionUID = -6117760685886862082L;

        public TextDataColumnTableModel() {
            super();
        }

        protected void addData(int index, SGDataColumnInfo data) {
            String cType = data.getColumnType();
            if (cType == null) {
                cType = "";
            }
            Object[] array = { Integer.toString(index + 1), data.getTitle(),
                    data.getValueType(), cType };
            super.addRow(array);
        }
    }

    /**
     * Checks selected items.
     * 
     * @return true if selected items are valid for the data type
     */
    public boolean checkSelectedItems() {
    	
        final int len = this.getColumnInfoList().size();
        final int colIndexTitle = this.getColumnIndex(COLUMN_NAME_TITLE);
        final int colIndexValueType = this.getColumnIndex(COLUMN_NAME_VALUE_TYPE);
        final int colIndexColumnType = this.getColumnIndex(COLUMN_NAME_COLUMN_TYPE);
        SGDataColumnInfo[] items = new SGDataColumnInfo[len];
        for (int ii = 0; ii < len; ii++) {
            String title = (String) this.mTable.getValueAt(ii, colIndexTitle);
            if (title == null) {
                title = "";
            }
            String valueType = (String) this.mTable.getValueAt(ii, colIndexValueType);
            if (valueType == null) {
            	valueType = "";
            }
            items[ii] = new SGDataColumnInfo(title, valueType);
            String columnType = (String) this.mTable.getValueAt(ii, colIndexColumnType);
            if (columnType == null) {
            	columnType = "";
            }
            items[ii].setColumnType(columnType);
        }
        
        // call a static utility method
        if (SGDataUtility.checkDataColumns(this.mDataType, items,
                this.mDataInfoMap) == false) {
            return false;
        }

        return true;
    }

    /**
     * Creates and returns a table object.
     * 
     * @return a table object
     */
    protected JTable createTableInstance() {
    	return new JTable(){
			private static final long serialVersionUID = -6879115435305260222L;
			public String getToolTipText(MouseEvent e){
				final int rowIndex = rowAtPoint(e.getPoint());
				final int colIndex = columnAtPoint(e.getPoint());
				Object obj = getModel().getValueAt(rowIndex, colIndex);
				if (obj != null && !"".equals(obj.toString())) {
					return obj.toString();
				} else {
					return null;
				}
            }
        };
    }

}
