/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * SGAttributePanel.java
 *
 * Created on 2009/08/24, 14:26:32
 */

package jp.riken.brain.ni.samuraigraph.data;

import java.util.List;

import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import javax.swing.table.TableColumnModel;
import ucar.nc2.Attribute;

/**
 * A panel to show netCDF attributes.
 *
 */
public class SGAttributePanel extends javax.swing.JPanel {

    /**
     * Serial Version UID
     */
    private static final long serialVersionUID = 8246986947494258210L;

    /** Creates new form SGAttributePanel */
    public SGAttributePanel() {
        super();
        initComponents();
        this.initProperty();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mScrollPane = new javax.swing.JScrollPane();
        mTable = new javax.swing.JTable();

        setLayout(new java.awt.BorderLayout());

        mScrollPane.setPreferredSize(new java.awt.Dimension(300, 80));
        mScrollPane.setViewportView(mTable);

        add(mScrollPane, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane mScrollPane;
    private javax.swing.JTable mTable;
    // End of variables declaration//GEN-END:variables
    
    
    /**
     * Name of the column for the sequential number.
     */
    public static final String COLUMN_NAME_NUMBER = "No.";

    /**
     * Name of the column for the name of each attribute.
     */
    public static final String COLUMN_NAME_NAME = "Name";

    /**
     * Name of the column for the value of each attribute.
     */
    public static final String COLUMN_NAME_VALUE = "Value";

    /**
     * The array of column name.
     */
    public static final String[] COLUMN_NAME_ARRAY = { COLUMN_NAME_NUMBER,
        COLUMN_NAME_NAME, COLUMN_NAME_VALUE };

    /**
     * A constant array of column width.
     */
    private static final int[] PREFERRED_COLUMN_WIDTH_ARRAY = { 10, 60, 180 };

    /**
     * The table model.
     */
    protected AttributeTableModel mTableModel = null;

    /**
     * Initialize the properties.
     *
     */
    private void initProperty() {
        // initialize the table model
        this.initTableModel();
        
        // update the table
        this.updateTable();
    }

    /**
     * Initializes the table model.
     */
    protected void initTableModel() {
        this.mTableModel = new AttributeTableModel();
        this.mTableModel.setColumnIdentifiers(this.getColumnNameArray());
        this.mTable.setModel(this.mTableModel);
    }
    
    /**
     * Update the table.
     */
    public void updateTable() {
        this.setPreferredColumnWidth();
    }

    /**
     * Set preferred size of columns.
     */
    private void setPreferredColumnWidth() {
        String[] cArray = getColumnNameArray();
        int[] pArray = getPreferredColumnWidthArray();
        for (int ii = 0; ii < cArray.length; ii++) {
            TableColumn cl = this.mTable.getColumn(cArray[ii]);
            cl.setPreferredWidth(pArray[ii]);
        }
    }

    protected String[] getColumnNameArray() {
        return COLUMN_NAME_ARRAY;
    }
    
    protected int[] getPreferredColumnWidthArray() {
        return PREFERRED_COLUMN_WIDTH_ARRAY;
    }

    /**
     * Sets the attributes.
     * 
     * @param aList
     *          the list of attributes
     * @return
     *          true if succeeded
     */
    public boolean setAttributes(List<Attribute> aList) {
        this.initTableModel();
        for (int ii = 0; ii < aList.size(); ii++) {
            Attribute attr = aList.get(ii);
            this.mTableModel.addData(ii, attr);
        }
        this.updateTable();
        return true;
    }

    /**
     * Clear all data.
     *
     */
    public void clear() {
        this.initTableModel();
    }

    /**
     * Returns the table.
     *
     * @return the table
     */
    public JTable getTable() {
        return this.mTable;
    }

    /**
     * Returns a text string about the selected row.
     *
     * @param row
     *          the row index
     * @return
     *          a text string about the row
     */
    public String getRowText(final int row) {
        TableColumnModel model = this.mTable.getColumnModel();
        final int colIndexName =  model.getColumnIndex(COLUMN_NAME_NAME);
        final int colIndexValue =  model.getColumnIndex(COLUMN_NAME_VALUE);
        Object name = this.mTable.getValueAt(row, colIndexName);
        Object value = this.mTable.getValueAt(row, colIndexValue);
        StringBuffer sb = new StringBuffer();
        if (name != null) {
            sb.append(name);
        }
        sb.append('\n');
        if (value != null) {
            sb.append(value);
        }
        return sb.toString();
    }

    /**
     * A table model class for attributes table.
     *
     */
    protected static class AttributeTableModel extends DefaultTableModel {

        /**
         * Serial Version UID
         */
        private static final long serialVersionUID = -6469371272912213709L;
        
        /**
         * The default constructor.
         *
         */
        protected AttributeTableModel() {
            super();
        }

        /**
         * Always returns false.
         */
        public boolean isCellEditable(int row, int col) {
            return false;
        }

        /**
         * Add a data.
         * 
         * @param index
         *           the array index
         * @param attr
         *           the attribute to add
         */
        protected void addData(int index, Attribute attr) {
            Object[] array = { Integer.toString(index + 1), attr.getName(),
                    attr.getValue(0) };
            super.addRow(array);
        }
    }

}
