package jp.riken.brain.ni.samuraigraph.data;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import ucar.ma2.Array;
import ucar.ma2.InvalidRangeException;
import ucar.nc2.Attribute;
import ucar.nc2.Dimension;
import ucar.nc2.Variable;

/**
 * The wrapper class for the netCDF coordinate variable that does not exist
 * in the netCDF data.
 * @author kuromaru
 *
 */
public class SGCoordinateVariable extends SGVariable {

	/**
	 * The dimension.
	 * 
	 */
	private Dimension mDimension = null;
	
	/**
	 * Builds an object with a given variable.
	 *
	 * @param var
	 *           the variable
	 */
	public SGCoordinateVariable(Variable var) {
		super(var);
		if (var.isCoordinateVariable() == false) {
			throw new IllegalArgumentException("Given variable is not a coordinate variable.");
		}
	}

	/**
	 * Builds an object with a given dimension.
	 *
	 * @param dim
	 *           the dimension
	 */
	public SGCoordinateVariable(Dimension dim) {
		super();
		if (dim == null) {
			throw new IllegalArgumentException("dim == null");
		}
		this.mDimension = dim;
	}

	/**
	 * Returns the dimension.
	 * 
	 * @return the dimension
	 */
	public Dimension getDimension() {
		return this.mDimension;
	}
	
	/**
	 * Overrode to return the name of dimension.
	 * 
	 * @return the name
	 */
	public String getName() {
		return this.mDimension.getName();
	}

	/**
	 * Returns the constant "undefined".
	 * 
	 * @return the name of this variable
	 */
	public String getNameInPriorityOrder() {
		return this.getVariableName();
	}

	public int findDimensionIndex(final String name) {
		if (this.mDimension.getName().equals(name)) {
			return 0;
		} else {
			return -1;
		}
	}

	public List<Dimension> getDimensions() {
		List<Dimension> list = new ArrayList<Dimension>();
		list.add(this.mDimension);
		return list;
	}

	public Dimension getDimension(final int index) {
		if (index == 0) {
			return this.mDimension;
		} else {
			return null;
		}
	}
	
	public boolean isCoordinateVariable() {
		// always returns true
		return true;
	}

	public Attribute findAttribute(final String name) {
		return null;
	}

	public String getUnitsString() {
		// always returns null
		return null;
	}

	public boolean isUnlimited() {
		return this.mDimension.isUnlimited();
	}

	public int getLength() {
		return this.mDimension.getLength();
	}
	
	public List<Attribute> getAttributes() {
		return new ArrayList<Attribute>();
	}

	public Array read(int[] a, int[] b) throws InvalidRangeException, IOException {
		// always returns null
		return null;
	}

	public String toString() {
		if (this.mVariable == null) {
			if (this.mDimension == null) {
				return "null";
			} else {
				return this.mDimension.toString();
			}
		} else {
			return this.mVariable.toString();
		}
	}

	/**
	 * Overrode to return the dimension name.
	 * 
	 * @return the name to identify this variable
	 */
	public String getIdentifiableNameInPriorityOrder() {
		return this.mDimension.getName();
	}

	/**
	 * Returns the constant "undefined".
	 * 
	 * @return the name of variable
	 */
	public String getVariableName() {
		// returns a constant text string
		return "undefined";
	}

}
