package jp.riken.brain.ni.samuraigraph.data;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.SwingUtilities;

import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfoSet;
import jp.riken.brain.ni.samuraigraph.base.SGDialog;

public abstract class SGDataSetupDialog extends SGDialog implements PropertyChangeListener {

    /**
     * Builds this dialog.
     * 
     * @param owner
     *           the owner of this dialog
     * @param modal
     *           true for modal dialog
     */
    public SGDataSetupDialog(Dialog owner, boolean modal) {
        super(owner, modal);
    }
    
    /**
     * Builds this dialog.
     * 
     * @param owner
     *           the owner of this dialog
     * @param modal
     *           true for modal dialog
     */
    public SGDataSetupDialog(Frame owner, boolean modal) {
        super(owner, modal);
    }

    /**
     * An array of selected data column information.
     */
    protected SGDataColumnInfo[] mDataColumnInfo = null;
    
    /**
     * Returns the OK button of this dialog.
     * 
     * @return the OK button
     */
    protected abstract JButton getOKButton();

    /**
     * Returns the Cancel button of this dialog.
     * 
     * @return the Cancel button
     */
    protected abstract JButton getCancelButton();
    
    /**
     * Returns an object that has a table to setup data.
     * 
     * @return an object that has a table to setup data
     */
    protected abstract SGIDataSetupTableHolder getTableHolder();

    /**
     * Returns an array of data column information.
     * 
     * @return an array of data column information
     */
    public SGDataColumnInfo[] getDataColumnInfo() {
        if (this.mDataColumnInfo == null) {
            return null;
        } else {
            return (SGDataColumnInfo[]) mDataColumnInfo.clone();
        }
    }

    /**
     * Called when the escape key is typed.
     * 
     */
    protected void onEscKeyTyped() {
        this.setVisible(false);
    }

    /**
     * Clear all data.
     */
    public void clear() {
        this.getTableHolder().clear();
    }

//    /**
//     * Returns selected column types.
//     * 
//     * @return selected column types
//     */
//    public SGDataColumnInfo[] getDataColumnTypes() {
//        return this.getTableHolder().getDataColumnTypes();
//    }
    
    public SGDataColumnInfoSet getDataColumnInfoSet() {
    	return this.getTableHolder().getDataColumnInfoSet();
    }

    /**
     * Called when the component is resized.
     * 
     * @param e
     *            a component event
     */
    public void componentResized(ComponentEvent e) {
        super.componentResized(e);
        this.updateTable();
    }

    /**
     * Called when the component is shown.
     * 
     * @param e
     *            a component event
     */
    public void componentShown(ComponentEvent e) {
        super.componentShown(e);
        this.updateTable();
    }

    /**
     * Update the table.
     */
    private void updateTable() {
        this.getTableHolder().updateTable();
    }

    /**
     * Called when an action event is generated.
     */
    public void actionPerformed(ActionEvent e) {
        
        Object source = e.getSource();
        String command = e.getActionCommand();
        if (OK_BUTTON_TEXT.equals(command)) {

            // get selected items and set them to the attribute
//            this.mDataColumnInfo = this.getTableHolder()
//                    .getDataColumnTypes();
        	this.mDataColumnInfo = this.getTableHolder().getDataColumnInfoSet().getDataColumnInfoArray();

            // notify to the listener
            this.notifyToListener(OK_BUTTON_TEXT);

            // hide this dialog
            this.setVisible(false);

        } else if (CANCEL_BUTTON_TEXT.equals(command)) {
            this.setVisible(false);
        } else if (source.equals(this.getTableHolder().getClearButton())) {
            this.getOKButton().setEnabled(false);
        } else if (source.equals(this.getTableHolder().getRestoreButton())) {
            this.getOKButton().setEnabled(true);
        } else if (source.equals(this.getTableHolder().getComplementButton())) {
            this.getOKButton().setEnabled(true);
        }

    }

    /**
     * Called when the table is changed.
     * 
     * @param evt
     *            an event that the property changes
     */
    public void propertyChange(PropertyChangeEvent evt) {
        Object source = evt.getSource();
        if (source.equals(this.getTableHolder().getTable())) {
            String name = evt.getPropertyName();
            if ("tableCellEditor".equals(name)) {
                SwingUtilities.invokeLater(new InputObserver());
            }
        }
    }

    /**
     * An observer to set enabled / disabled the OK button with input values.
     *
     */
    protected class InputObserver extends Thread {
    	InputObserver() {
    		super();
    	}
    	public void run() {
            final boolean b = getTableHolder().checkSelectedItems();
            getOKButton().setEnabled(b);
        }
    }

    /**
     * Overrode to pack this dialog.
     * 
     * @param visible
     *           true to set visible this dialog
     */
    public void setVisible(final boolean visible) {
    	this.pack();
    	super.setVisible(visible);
    }
}
