package jp.riken.brain.ni.samuraigraph.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jp.riken.brain.ni.samuraigraph.base.SGIData;
import jp.riken.brain.ni.samuraigraph.base.SGIDataSource;

public class SGDataSourceObserver {

    /**
	 * A map that has keys of data source and values of a list of data.
	 */
	private Map<SGIDataSource, List<SGIData>> mDataMap = new HashMap<SGIDataSource, List<SGIData>>();

    /**
	 * Builds an observer.
	 * 
	 */
    public SGDataSourceObserver() {
        super();
    }
    
    /**
     * Adds a new data to this observer.
     * 
     * @param data
     *           a data object
     */
    public void addData(SGIData data) {
        if (data == null) {
            throw new IllegalArgumentException("data == null");
        }
        
        // get the list of data
        SGIDataSource source = data.getDataSource();
        List<SGIData> dataList = this.mDataMap.get(source);

        if (dataList == null) {
            // if data list does not exist, create a new list
            List<SGIData> list = new ArrayList<SGIData>();
            list.add(data);
            this.mDataMap.put(source, list);
            dataList = list;
        } else {
            // if data list exists and it does not contain given data, add data to it
            if (!dataList.contains(data)) {
                dataList.add(data);
            }
        }
//        System.out.println("add: " + dataList.size() + " " + data);
//    	System.out.println(this.mDataMap);
    }

    /**
     * Disposes of a data source that is related to a given data object.
     * 
     * @param data
     *           a data object
     */
    public void dataDisposed(SGIData data) {

        if (data == null) {
            throw new IllegalArgumentException("data == null");
        }

        // get the list of data
        SGIDataSource source = data.getDataSource();
        List<SGIData> dataList = this.mDataMap.get(source);
        dataList.remove(data);
        
        // if the list becomes empty, disposes of the data source
        if (dataList.size() == 0) {
            this.mDataMap.remove(source);
        	source.dispose();
        }
//        System.out.println("remove: " + dataList.size() + " " + data);
//    	System.out.println(this.mDataMap);
    }

}
