package jp.riken.brain.ni.samuraigraph.data;

import jp.riken.brain.ni.samuraigraph.base.SGIData;

public interface SGISXYData extends SGIData {

    /**
     * Returns whether error bars are available.
     * 
     * @return true if error bars are available
     */
    public boolean isErrorBarAvailable();

    /**
     * Returns whether tick labels are available.
     * 
     * @return true if tick labels are available
     */
    public boolean isTickLabelAvailable();

    /**
	 * Returns whether the error bars are vertical. If this data does not have
	 * error values, returns null.
	 * 
	 * @return true if error bars are vertical, false if they are horizontal and
	 *         null if this data do not have error values
	 */
    public Boolean isErrorBarVertical();
    
    /**
	 * Returns whether the tick labels align horizontally. If this data does not have
	 * tick labels, returns null.
	 * 
	 * @return true if tick labels align horizontally, false if they do not so and
	 *         null if this data do not have tick labels
	 */
    public Boolean isTickLabelHorizontal();

    /**
     * Sets the decimal places for the tick labels.
     * 
     * @param dp
     *          a value to set to the decimal places
     */
    public void setDecimalPlaces(final int dp);

    /**
     * Sets the exponent for the tick labels.
     * 
     * @param exp
     *           a value to set to the exponent
     */
    public void setExponent(final int exp);

    /**
     * Returns the decimal places for the tick labels.
     * 
     * @return the decimal places for the tick labels
     */
    public int getDecimalPlaces();

    /**
     * Returns the exponent for tick labels.
     * 
     * @return the exponent for tick labels
     */
    public int getExponent();

}
