/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * SGNetCDFDataColumnSelectionPanel.java
 *
 * Created on 2009/08/20, 14:59:51
 */

package jp.riken.brain.ni.samuraigraph.data;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractCellEditor;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import jp.riken.brain.ni.samuraigraph.base.SGButton;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import ucar.nc2.Dimension;


/**
 *
 * @author kuromaru
 */
public class SGNetCDFDataColumnSelectionPanel extends SGDataColumnSelectionPanel
    implements MouseListener {

    /**
     * Serial Version UID
     */
    private static final long serialVersionUID = -8916914942493060714L;
    
    /** Creates new form SGNetCDFDataColumnSelectionPanel */
    public SGNetCDFDataColumnSelectionPanel() {
        super();
        
        // add a mouse listener
        this.mTable.addMouseListener(this);
    }

    /**
     * Name of the column for the name of each data column.
     */
    public static final String COLUMN_NAME_NAME = "Name";

    /**
     * Name of the column for the dimension of each data column.
     */
    public static final String COLUMN_NAME_DIMENSION = "Dimension";

    /**
     * Name of the column for the button of each data column.
     */
    public static final String COLUMN_NAME_BUTTON = "";

    /**
     * The array of column name.
     */
    public static final String[] COLUMN_NAME_ARRAY = { COLUMN_NAME_NUMBER,
        COLUMN_NAME_NAME, COLUMN_NAME_DIMENSION, COLUMN_NAME_COLUMN_TYPE,
        COLUMN_NAME_BUTTON };

    /**
     * A constant array of column width.
     */
    private static final int[] PREFERRED_COLUMN_WIDTH_ARRAY = { 10, 80, 80, 
        140, ROW_HEIGHT };

    /**
     * An array of acceptable data types.
     */
    private static final String[] DATA_TYPE_ARRAY = {
            SGDataTypeConstants.SXY_NETCDF_DATA,
            SGDataTypeConstants.SXYZ_NETCDF_DATA,
            SGDataTypeConstants.SXY_MULTIPLE_NETCDF_DATA,
            SGDataTypeConstants.SXY_MULTIPLE_NETCDF_DIMENSION_DATA,
            SGDataTypeConstants.VXY_NETCDF_DATA };

    protected String[] getColumnNameArray() {
        return COLUMN_NAME_ARRAY;
    }
    
    protected int[] getPreferredColumnWidthArray() {
        return PREFERRED_COLUMN_WIDTH_ARRAY;
    }

    protected String[] getDataTypeArray() {
        return DATA_TYPE_ARRAY;
    }

    protected DataColumnTableModel createTableModel() {
        return new NetCDFDataColumnTableModel();
    }

    /**
     * Initializes the table model.
     */
    protected void initTableModel() {
        super.initTableModel();
        
        // set up the column with buttons
        final int colIndexButton = this.getColumnIndex(COLUMN_NAME_BUTTON);
        TableColumn colButton = this.mTable.getColumnModel().getColumn(
                colIndexButton);
        ButtonColumn bcol = new ButtonColumn();
        colButton.setMinWidth(ROW_HEIGHT);
        colButton.setMaxWidth(ROW_HEIGHT);
        colButton.setResizable(false);
        colButton.setCellEditor(bcol);
        colButton.setCellRenderer(bcol);        
    }
    
    /**
     * A class of table model.
     */
    protected class NetCDFDataColumnTableModel extends DataColumnTableModel {
        
        /**
         * 
         */
        private static final long serialVersionUID = 5724156778412127794L;

        public NetCDFDataColumnTableModel() {
            super();
        }

        protected void addData(int index, SGDataColumnInfo data) {
            
            SGNetCDFDataColumnInfo nData = (SGNetCDFDataColumnInfo) data;
                
            String cType = nData.getColumnType();
            if (cType == null) {
                cType = "";
            }

            SGVariable var = nData.getVariable();
            List<Dimension> dimList = var.getDimensions();
            StringBuffer sb = new StringBuffer();
            final int len = dimList.size();
            for (int ii = 0; ii < len; ii++) {
                Dimension dim = dimList.get(ii);
                sb.append(dim.getName());
                if (ii != len - 1) {
                    sb.append(", ");
                }
            }

            // set the variable to the cell of a button
            Object[] array = { Integer.toString(index + 1), data.getTitle(),
                    sb.toString(), cType, var };
            
            super.addRow(array);
        }
    }

    
    /**
     * Cell editor class for a column with a button.
     *
     */
    private static class ButtonColumn extends AbstractCellEditor implements
            TableCellRenderer, TableCellEditor {

        private static final long serialVersionUID = 1089174278086303154L;
        private static final String BUTTON_TEXT = " ";
        private final JButton renderButton;
        private final JButton editorButton;

        public ButtonColumn() {
            super();
            renderButton = new SGButton(BUTTON_TEXT);
            editorButton = new SGButton(BUTTON_TEXT);
            editorButton.setFocusPainted(false);
            editorButton.setRolloverEnabled(false);
        }

        public Component getTableCellRendererComponent(JTable table,
                Object value, boolean isSelected, boolean hasFocus, int row,
                int column) {
            return renderButton;
        }

        public Component getTableCellEditorComponent(JTable table,
                Object value, boolean isSelected, int row, int column) {
            return editorButton;
        }

        public Object getCellEditorValue() {
            return BUTTON_TEXT;
        }
    }

    /**
     * The owner of popup dialogs.
     */
    private Window mDialogOwner = null;
    
    /**
     * Sets the owner of popup dialogs.
     * 
     * @param owner
     *           the owner of popup dialogs
     */
    public void setPopupDialogOwner(Window owner) {
        this.mDialogOwner = owner;
    }

    /**
     * A map of attribute dialogs.
     */
    private Map<String, SGAttributeDialog> mAttributeDialogMap = new HashMap<String, SGAttributeDialog>();

    /**
     * Called when an mouse event is invoked.
     * 
     * @param e
     *         a mouse event
     */
    public void mouseClicked(MouseEvent e) {
        Object source = e.getSource();
        if (source.equals(this.mTable)) {
            Point p = e.getPoint();
            final int col = this.mTable.columnAtPoint(p);
            final int row = this.mTable.rowAtPoint(p);
            if (row >= 0 && col == this.getColumnIndex(COLUMN_NAME_BUTTON)) {
                // get the variable
                Object value = this.mTable.getValueAt(row, col);
                SGVariable var = (SGVariable) value;                
                String name = var.getName();
                
                // if dialog for the clicked variable exists and is visible, do nothing
                SGAttributeDialog adg = this.mAttributeDialogMap.get(name);
                if (adg != null && adg.isVisible()) {
                    return;
                }
                
                // create a popup dialog
                SGAttributeDialog dg = null;
                if (mDialogOwner instanceof Dialog) {
                    dg = new SGAttributeDialog((Dialog) mDialogOwner, false);
                } else if (mDialogOwner instanceof Frame) {
                    dg = new SGAttributeDialog((Frame) mDialogOwner, false);
                } else {
                	throw new Error("Not supported dialog owner.");
                }
                this.mAttributeDialogMap.put(name, dg);
                
                // set the variable
                dg.setVariable(var);
                
                // shows a popup dialog
                dg.setLocation(e.getXOnScreen() + 20, e.getYOnScreen() + 20);
//                final int x = this.mDialogOwner.getX() + this.mTable.getX() + e.getX() + 20;
//                final int y = this.mDialogOwner.getY() + this.mTable.getY() + e.getY() + 20;
//                dg.setLocation(x, y);
                dg.setVisible(true);
            }
        }
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Checks selected items.
     * 
     * @return true if selected items are valid for the data type
     */
    public boolean checkSelectedItems() {

        List<SGDataColumnInfo> columnInfo = this.getColumnInfoList();
        final int len = this.getColumnInfoList().size();
        
        final int colIndexName = this.getColumnIndex(COLUMN_NAME_NAME);
        final int colIndexColumnType = this.getColumnIndex(COLUMN_NAME_COLUMN_TYPE);
        
        SGNetCDFDataColumnInfo[] items = new SGNetCDFDataColumnInfo[len];
        for (int ii = 0; ii < len; ii++) {
            String name = (String) this.mTable.getValueAt(ii, colIndexName);
            if (name == null) {
                name = "";
            }
            SGVariable var = ((SGNetCDFDataColumnInfo)columnInfo.get(ii)).getVariable();
            items[ii] = new SGNetCDFDataColumnInfo(var, name, var.getValueType());
            String columnType = (String) this.mTable.getValueAt(ii, colIndexColumnType);
            if (columnType == null) {
            	columnType = "";
            }
            items[ii].setColumnType(columnType);
        }
        
        // call a static utility method
        if (SGDataUtility.checkDataColumns(this.mDataType, items,
                this.mDataInfoMap) == false) {
            return false;
        }

        return true;
    }
    
    /**
     * Creates and returns a table object.
     * 
     * @return a table object
     */
    protected JTable createTableInstance() {
    	return new JTable(){
			private static final long serialVersionUID = -7054748048254127674L;
			public String getToolTipText(MouseEvent e){
				final int rowIndex = rowAtPoint(e.getPoint());
				final int colIndex = columnAtPoint(e.getPoint());
				if (colIndex == 4) {
					return null;
				}
				Object obj = getModel().getValueAt(rowIndex, colIndex);
                if (obj != null && !"".equals(obj.toString())) {
					return obj.toString();
				} else {
					return null;
				}
            }
        };
    }
}
