package jp.riken.brain.ni.samuraigraph.data;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import ucar.ma2.Array;
import ucar.ma2.InvalidRangeException;
import ucar.nc2.Dimension;
import ucar.nc2.Variable;

/**
 * A variable class for tick labels. A variable has only one dimension and 
 * text strings for tick labels.
 *
 */
public class SGTextVariable extends SGVariable {
	
	/**
	 * A array of text strings.
	 */
	private String[] mStringArray = null;

	/**
	 * Builds an object with text strings.
	 *
	 * @param var
	 *           the variable
	 * @param stringArray
	 *           an array of text strings
	 */
	public SGTextVariable(final Variable var, final String[] stringArray) {
		super(var);

		// check
		if (stringArray == null) {
			throw new IllegalArgumentException("stringArray == null");
		}

		// set to the attribute
		this.mStringArray = (String[]) stringArray.clone();
		for (int ii = 0; ii < this.mStringArray.length; ii++) {
			if (this.mStringArray[ii] == null) {
				this.mStringArray[ii] = "";
			}
		}
	}
	
	/**
	 * Returns the array of text strings.
	 * 
	 * @return the array of text strings
	 */
	public String[] getStringArray() {
		return (String[]) this.mStringArray.clone();
	}
	
	/**
	 * Returns the value type.
	 * 
	 * @return the value type
	 */
	public String getValueType() {
		return SGIDataColumnTypeConstants.VALUE_TYPE_TEXT;
	}

	/**
	 * Returns the list of dimension.
	 * 
	 * @return the list of dimensions
	 */
	public List<Dimension> getDimensions() {
		// returns a list that contains only the first dimension
		List<Dimension> dimList = super.getDimensions();
		return new ArrayList<Dimension>(dimList.subList(0, 1));
	}

	/**
	 * Returns the dimension of given index.
	 * 
	 * @param index
	 *           the index of dimension
     * @return the dimension of given index
	 */
	public Dimension getDimension(final int index) {
		if (index > 0) {
			return null;
		}
		return this.mVariable.getDimension(index);
	}

	public Array read(int[] a, int[] b) throws InvalidRangeException, IOException {
		// always returns null
		return null;
	}

}
