package jp.riken.brain.ni.samuraigraph.figure;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import jp.riken.brain.ni.samuraigraph.base.SGUtilityNumber;
import jp.riken.brain.ni.samuraigraph.data.SGIVXYTypeData;


/**
 * This class provides utility methods for SGFigureElement objects.
 */
public class SGUtilityForFigureElement {

    public static final int MAGNITUDE_PER_CM_MINIMAL_ORDER = 3;
    
    /**
     * Rounds the input value of the magnitude per centimeter of vector type data
     * using the maximum length of vectors.
     * 
     * @param mag
     *           the magnitude per centimeter of vector type data
     * @param data
     *           the vector type data
     * @return rounded value
     */
    public static float roundMagnitudePerCM(final float mag,
            final SGIVXYTypeData data) {
    	if (Float.isNaN(mag)) {
    		return Float.NaN;
    	}
        final double[] mArray = data.getMagnitudeArray();
        final float max = (float) SGUtilityNumber.max(mArray);
        if (Float.isNaN(max) || max <= 0.0f) {
        	return mag;
        }
        float magNew = (float) SGUtilityNumber.getNumberInNumberOrder(mag, max, 
                MAGNITUDE_PER_CM_MINIMAL_ORDER, BigDecimal.ROUND_HALF_UP);
        return magNew;
    }

    /**
     * Returns a list of element group of given class.
     * 
     * @param cl
     *           the class
     * @param dList
     *           the list of element groups
     * @return the list of element groups of given class
     */
    @SuppressWarnings("unchecked")
	public static List<SGElementGroup> getGroups(Class cl, List<SGElementGroup> dList) {
        List<SGElementGroup> list = new ArrayList<SGElementGroup>();
        for (int ii = 0; ii < dList.size(); ii++) {
            Object obj = dList.get(ii);
            if (cl.isAssignableFrom(obj.getClass())) {
                list.add((SGElementGroup) obj);
            }
        }
        return list;
    }

    /**
     * Returns an element group of given class.
     * 
     * @param cl
     *           the class
     * @param dList
     *           the list of element groups
     * @return an element group of given class
     */
    @SuppressWarnings("unchecked")
    public static SGElementGroup getGroup(Class cl, List<SGElementGroup> dList) {
        if (null==dList) {
            return null;
        }
        for (int ii = 0; ii < dList.size(); ii++) {
            Object obj = dList.get(ii);
            if (cl.isAssignableFrom(obj.getClass())) {
                return (SGElementGroup) obj;
            }
        }
        return null;
    }

}
