package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Color;
import java.util.Iterator;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGIConstants;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2f;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.data.SGIVXYTypeData;
import jp.riken.brain.ni.samuraigraph.figure.SGDrawingElementArrow;
import jp.riken.brain.ni.samuraigraph.figure.SGDrawingElementLine;
import jp.riken.brain.ni.samuraigraph.figure.SGIDataConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGIVXYDataConstants;

public class SGArrowUtility implements SGIVXYDataConstants, SGIDataConstants {

    /**
     * Sets the line width with a given unit.
     * 
     * @param lw
     *           the line width to set
     * @param unit
     *           the unit for line width
     * @param arrows
     *           a group of arrows
     * @return true if succeeded
     */
    public static boolean setLineWidth(final float lw, final String unit, 
    		final SGElementGroupArrow arrows) {
        final Float lwNew = SGUtility.getLineWidth(lw, unit);
        if (lwNew == null) {
            return false;
        }
        if (arrows.setLineWidth(lwNew) == false) {
            return false;
        }
        return true;
    }
    
    /**
     * Sets the head size with a given unit.
     * @param size
     *           the head size to set
     * @param unit
     *           the unit for a given head size
     * @param arrows
     *           a group of arrows
     * @return true if succeeded
     */
    public static boolean setHeadSize(final float size, final String unit,
    		final SGElementGroupArrow arrows) {
        final Float sNew = SGUtility.calcPropertyValue(size, unit, ARROW_HEAD_SIZE_UNIT, 
                ARROW_HEAD_SIZE_MIN, ARROW_HEAD_SIZE_MAX, ARROW_HEAD_SIZE_MINIMAL_ORDER);
        if (sNew == null) {
            return false;
        }
        if (arrows.setHeadSize(sNew) == false) {
            return false;
        }
        return true;
    }

    /**
     * Sets the head open and close angle.
     * 
     * @param openAngle
     *           the head open angle to set in units of degree
     * @param closeAngle
     *           the head close angle to set in units of degree
     * @param arrows
     *           a group of arrows
     * @return true if succeeded
     */
    public static boolean setHeadAngle(final Float openAngle, final Float closeAngle,
    		final SGElementGroupArrow arrows) {
        final Float oNew = (openAngle != null) ? SGUtility.calcPropertyValue(openAngle, 
                null, null, ARROW_HEAD_OPEN_ANGLE_MIN, ARROW_HEAD_OPEN_ANGLE_MAX, 
                ARROW_HEAD_ANGLE_MINIMAL_ORDER) : Float.valueOf(arrows.mHeadOpenAngle);
        if (oNew == null) {
            return false;
        }
        final Float cNew = (closeAngle != null) ? SGUtility.calcPropertyValue(closeAngle, 
                null, null, ARROW_HEAD_CLOSE_ANGLE_MIN, ARROW_HEAD_CLOSE_ANGLE_MAX, 
                ARROW_HEAD_ANGLE_MINIMAL_ORDER) : Float.valueOf(arrows.mHeadCloseAngle);
        if (cNew == null) {
            return false;
        }
        if (cNew <= oNew) {
            return false;
        }
        arrows.mHeadOpenAngle = oNew;
        arrows.mHeadCloseAngle = cNew;
        return true;
    }

    /**
     * Sets the head open and close angle.
     * 
     * @param openAngle
     *           the head open angle to set in units of degree
     * @param closeAngle
     *           the head close angle to set in units of degree
     * @param arrows
     *           a group of arrows
     * @return true if succeeded
     */
    public static boolean setHeadAngle(final float openAngle, final float closeAngle,
    		final SGElementGroupArrow arrows) {
        final Float oNew = SGUtility.calcPropertyValue(openAngle, null, null, 
                ARROW_HEAD_OPEN_ANGLE_MIN, ARROW_HEAD_OPEN_ANGLE_MAX, 
                ARROW_HEAD_ANGLE_MINIMAL_ORDER);
        if (oNew == null) {
            return false;
        }
        final Float cNew = SGUtility.calcPropertyValue(closeAngle, null, null, 
                ARROW_HEAD_CLOSE_ANGLE_MIN, ARROW_HEAD_CLOSE_ANGLE_MAX, 
                ARROW_HEAD_ANGLE_MINIMAL_ORDER);
        if (cNew == null) {
            return false;
        }
        if (cNew <= oNew) {
            return false;
        }
        arrows.mHeadOpenAngle = oNew;
        arrows.mHeadCloseAngle = cNew;
        return true;
    }

    protected static SGPropertyResults setProperties(SGPropertyMap map,
            SGPropertyResults iResult, SGElementGroupArrow arrows) {

        SGPropertyResults result = (SGPropertyResults) iResult.clone();

        Iterator<String> itr = map.getKeyIterator();
        while (itr.hasNext()) {
            String key = itr.next();
            String value = map.getValueString(key);
            
            if (COM_DATA_ARROW_LINE_WIDTH.equalsIgnoreCase(key)) {
                StringBuffer unit = new StringBuffer();
                Number num = SGUtilityText.getNumber(value, unit);
                if (num == null) {
                    result.putResult(COM_DATA_ARROW_LINE_WIDTH, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (arrows.setLineWidth(num.floatValue(), unit
                        .toString()) == false) {
                    result.putResult(COM_DATA_ARROW_LINE_WIDTH, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_ARROW_LINE_WIDTH, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_ARROW_LINE_TYPE.equalsIgnoreCase(key)) {
                final Integer type = SGDrawingElementLine.getLineTypeFromName(value);
                if (type == null) {
                    result.putResult(COM_DATA_ARROW_LINE_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (arrows.setLineType(type) == false) {
                    result.putResult(COM_DATA_ARROW_LINE_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_ARROW_LINE_TYPE, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_ARROW_HEAD_SIZE.equalsIgnoreCase(key)) {
                StringBuffer unit = new StringBuffer();
                Number num = SGUtilityText.getNumber(value, unit);
                if (num == null) {
                    result.putResult(COM_DATA_ARROW_HEAD_SIZE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (arrows.setHeadSize(num.floatValue(), unit
                        .toString()) == false) {
                    result.putResult(COM_DATA_ARROW_HEAD_SIZE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_ARROW_HEAD_SIZE, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_ARROW_COLOR.equalsIgnoreCase(key)) {
                Color cl = SGUtilityText.getColor(value);
                if (cl != null) {
                    if (arrows.setColor(cl) == false) {
                        result.putResult(COM_DATA_ARROW_COLOR, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                } else {
                	cl = SGUtilityText.parseColor(value);
					if (cl == null) {
						result.putResult(COM_DATA_ARROW_COLOR,
								SGPropertyResults.INVALID_INPUT_VALUE);
						continue;
					}
					if (arrows.setColor(cl) == false) {
						result.putResult(COM_DATA_ARROW_COLOR,
								SGPropertyResults.INVALID_INPUT_VALUE);
						continue;
					}
                }
                result.putResult(COM_DATA_ARROW_COLOR, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_ARROW_START_TYPE.equalsIgnoreCase(key)) {
                final Integer type = SGDrawingElementArrow.getArrowHeadTypeFromName(value);
                if (type == null) {
                    result.putResult(COM_DATA_ARROW_START_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (arrows.setStartHeadType(type) == false) {
                    result.putResult(COM_DATA_ARROW_START_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_ARROW_START_TYPE, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_ARROW_END_TYPE.equalsIgnoreCase(key)) {
                final Integer type = SGDrawingElementArrow.getArrowHeadTypeFromName(value);
                if (type == null) {
                    result.putResult(COM_DATA_ARROW_END_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (arrows.setEndHeadType(type) == false) {
                    result.putResult(COM_DATA_ARROW_END_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_ARROW_END_TYPE, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_ARROW_HEAD_OPEN_ANGLE.equalsIgnoreCase(key)) {
                Float openAngle = SGUtilityText.getFloat(value);
                if (openAngle == null) {
                    result.putResult(COM_DATA_ARROW_HEAD_OPEN_ANGLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (setHeadAngle(openAngle, null, arrows) == false) {
                    result.putResult(COM_DATA_ARROW_HEAD_OPEN_ANGLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_ARROW_HEAD_OPEN_ANGLE, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_ARROW_HEAD_CLOSE_ANGLE.equalsIgnoreCase(key)) {
                Float closeAngle = SGUtilityText.getFloat(value);
                if (closeAngle == null) {
                    result.putResult(COM_DATA_ARROW_HEAD_CLOSE_ANGLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (setHeadAngle(null, closeAngle, arrows) == false) {
                    result.putResult(COM_DATA_ARROW_HEAD_CLOSE_ANGLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_ARROW_HEAD_CLOSE_ANGLE, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_ARROW_HEAD_ANGLE.equalsIgnoreCase(key)) {
            	float[] angles = SGUtilityText.getFloatArray(value);
            	if (angles == null) {
                    result.putResult(COM_DATA_ARROW_HEAD_ANGLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
            	if (angles.length != 2) {
                    result.putResult(COM_DATA_ARROW_HEAD_ANGLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
                if (arrows.setHeadAngle(angles[0], angles[1]) == false) {
                    result.putResult(COM_DATA_ARROW_HEAD_ANGLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_ARROW_HEAD_ANGLE, SGPropertyResults.SUCCEEDED);
            }
        }            

        return result;
    }

    
    static boolean calcEndLocation(SGIVXYTypeData dataVXY, SGFigureElementGraph graph, 
    		SGElementGroupSetInGraph groupSet, final float magPerCM, final boolean invariant,
    		float[] startXArray, float[] startYArray, SGTuple2f[] startArray,
    		SGTuple2f[] endArray) {

    	final SGAxis xAxis = groupSet.getXAxis();
    	final SGAxis yAxis = groupSet.getYAxis();
		final float mag = groupSet.getMagnification();
    	
        // calculate the location of the end points and set to the groups
        final double[] xComponentArray = dataVXY.getXComponentArray();
        final double[] yComponentArray = dataVXY.getYComponentArray();
        final double[] magnitudeArray = dataVXY.getMagnitudeArray();
        final float ratio = 1.0f / (SGIConstants.CM_POINT_RATIO * magPerCM);
        
        double xyRatio = 0.0;
        if (!invariant) {
            final double xRange = xAxis.getMaxValue() - xAxis.getMinValue();
            final double yRange = yAxis.getMaxValue() - yAxis.getMinValue();
            xyRatio = (graph.getGraphRectHeight() / yRange)
                    / (graph.getGraphRectWidth() / xRange);
        }
        
        final float factor = mag * ratio;
        if (invariant) {
        	if (startArray != null) {
                for (int ii = 0; ii < startArray.length; ii++) {
        			final float endX;
        			final float endY;
                	if (Double.isNaN(xComponentArray[ii]) || Double.isNaN(yComponentArray[ii])) {
                		endX = Float.NaN;
                		endY = Float.NaN;
                	} else {
                        endX = startArray[ii].x + factor * (float) (xComponentArray[ii]);
                        endY = startArray[ii].y - factor * (float) (yComponentArray[ii]);
                	}
                    endArray[ii].setValues(endX, endY);
                }
        	} else {
        		final int xNum = startXArray.length;
        		final int yNum = startYArray.length;
        		for (int yy = 0; yy < yNum; yy++) {
                	if (Double.isNaN(startYArray[yy])) {
                		for (int xx = 0; xx < xNum; xx++) {
                			final int index = yy * xNum + xx;
                    		endArray[index].setValues(Float.NaN, Float.NaN);
                		}
                	} else {
                		for (int xx = 0; xx < xNum; xx++) {
                			final int index = yy * xNum + xx;
                			final float endX;
                			final float endY;
                        	if (Double.isNaN(startXArray[xx])) {
                        		endX = Float.NaN;
                        		endY = Float.NaN;
                        	} else {
                                endX = startXArray[xx] + factor * (float) (xComponentArray[index]);
                                endY = startYArray[yy] - factor * (float) (yComponentArray[index]);
                        	}
                            endArray[index].setValues(endX, endY);
                		}
                	}
        		}
        	}
        } else {
        	if (startArray != null) {
                for (int ii = 0; ii < startArray.length; ii++) {
                	if (Double.isNaN(xComponentArray[ii]) || Double.isNaN(yComponentArray[ii])) {
                		endArray[ii].setValues(Float.NaN, Float.NaN);
                		continue;
                	}
                	if (Double.isNaN(magnitudeArray[ii]) || magnitudeArray[ii] < 0.0) {
                		endArray[ii].setValues(Float.NaN, Float.NaN);
                		continue;
                	}
            		calcEnd(startArray[ii].x, startArray[ii].y,
							xComponentArray[ii], yComponentArray[ii],
							magnitudeArray[ii], factor, xyRatio, endArray[ii]);
                }
        	} else {
        		final int xNum = startXArray.length;
        		final int yNum = startYArray.length;
        		for (int yy = 0; yy < yNum; yy++) {
                	if (Double.isNaN(startYArray[yy])) {
                		for (int xx = 0; xx < xNum; xx++) {
                			final int index = yy * xNum + xx;
                    		endArray[index].setValues(Float.NaN, Float.NaN);
                		}
                	} else {
                		for (int xx = 0; xx < xNum; xx++) {
                			final int index = yy * xNum + xx;
                        	if (Double.isNaN(startXArray[xx])) {
                                endArray[index].setValues(Float.NaN, Float.NaN);
                        	} else {
                        		calcEnd(startXArray[xx], startYArray[yy],
										xComponentArray[index],
										yComponentArray[index],
										magnitudeArray[index], factor, xyRatio,
										endArray[index]);
                        	}
                		}
                	}
        		}
        	}
        }

        return true;
    }
    
    private static void calcEnd(final float startX, final float startY,
			final double xComponent, final double yComponent,
			final double magnitude, final float factor, final double xyRatio,
			final SGTuple2f end) {
    	if (Double.isNaN(magnitude) || magnitude < 0.0) {
    		end.setValues(Float.NaN, Float.NaN);
    		return;
    	}
		final double tanValue = xyRatio * yComponent / xComponent;
		double angle = Math.atan(tanValue);
		if (xComponent < 0.0) {
			angle += Math.PI;
		}
		final double cosValue = Math.cos(angle);
		final double sinValue = Math.sin(angle);
		final float vx = factor * (float) (magnitude * cosValue);
		final float vy = -factor * (float) (magnitude * sinValue);
		final float endX = startX + vx;
		final float endY = startY + vy;
		end.setValues(endX, endY);
	}

}
