package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.BasicStroke;
import java.awt.Color;
import java.util.Iterator;

import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.data.SGISXYData;
import jp.riken.brain.ni.samuraigraph.figure.SGDrawingElementLine;
import jp.riken.brain.ni.samuraigraph.figure.SGIDataConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGISXYDataConstants;

public abstract class SGElementGroupLineForData extends SGElementGroupLine
        implements SGISXYDataConstants, SGIDataConstants {

    /**
     * Create a group of lines.
     */
    public SGElementGroupLineForData(SGISXYData data) {
        super();
        
        // set default properties
        this.setVisible(DEFAULT_LINE_VISIBLE);
        this.setLineWidth(DEFAULT_LINE_WIDTH, LINE_WIDTH_UNIT);
        this.setLineType(DEFAULT_LINE_TYPE);
        this.setColor(DEFAULT_LINE_COLOR);

        // setup the stroke object
        this.mStroke.setEndCap(BasicStroke.CAP_BUTT);
        this.mStroke.setJoin(BasicStroke.JOIN_ROUND);
        
        // set default shift
        this.setLineShiftX(DEFAULT_LINE_SHIFT_X);
        this.setLineShiftY(DEFAULT_LINE_SHIFT_Y);
    }

    /**
     * Set the line width in a given unit.
     * 
     * @param lw
     *            the line width to set
     * @param unit
     *            the unit for the given line width
     * @return true if succeeded
     */
    public boolean setLineWidth(final float lw, final String unit) {
        final Float lwNew = SGUtility.getLineWidth(lw, unit);
        if (lwNew == null) {
            return false;
        }
        if (this.setLineWidth(lwNew) == false) {
            return false;
        }
        return true;
    }

    
    protected SGPropertyResults setProperties(SGPropertyMap map,
            SGPropertyResults iResult) {

        SGPropertyResults result = (SGPropertyResults) iResult.clone();

        Iterator<String> itr = map.getKeyIterator();
        while (itr.hasNext()) {
            String key = itr.next();
            String value = map.getValueString(key);
            
            if (COM_DATA_LINE_WIDTH.equalsIgnoreCase(key)) {
                StringBuffer unit = new StringBuffer();
                Number num = SGUtilityText.getNumber(value, unit);
                if (num == null) {
                    result.putResult(COM_DATA_LINE_WIDTH, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setLineWidth(num.floatValue(), unit
                        .toString()) == false) {
                    result.putResult(COM_DATA_LINE_WIDTH, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_LINE_WIDTH, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_LINE_TYPE.equalsIgnoreCase(key)) {
                final Integer type = SGDrawingElementLine.getLineTypeFromName(value);
                if (type == null) {
                    result.putResult(COM_DATA_LINE_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setLineType(type) == false) {
                    result.putResult(COM_DATA_LINE_TYPE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_LINE_TYPE, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_LINE_COLOR.equalsIgnoreCase(key)) {
                Color cl = SGUtilityText.getColor(value);
                if (cl != null) {
                    if (this.setColor(cl) == false) {
                        result.putResult(COM_DATA_LINE_COLOR, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                } else {
                	cl = SGUtilityText.parseColor(value);
					if (cl == null) {
						result.putResult(COM_DATA_LINE_COLOR,
								SGPropertyResults.INVALID_INPUT_VALUE);
						continue;
					}
					if (this.setColor(cl) == false) {
						result.putResult(COM_DATA_LINE_COLOR,
								SGPropertyResults.INVALID_INPUT_VALUE);
						continue;
					}
                }
                result.putResult(COM_DATA_LINE_COLOR, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_LINE_SHIFTX.equalsIgnoreCase(key)) {
                Number num = SGUtilityText.getDouble(value);
                if (num == null) {
                    result.putResult(COM_DATA_LINE_SHIFTX, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setLineShiftX(num.doubleValue()) == false) {
                    result.putResult(COM_DATA_LINE_SHIFTX, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                
                SGElementGroupSymbol symbol = getElementGroupSymbol();
                if (null==symbol || symbol.setShiftX(num.doubleValue())==false) {
                    result.putResult(COM_DATA_LINE_SHIFTX, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_LINE_SHIFTX, SGPropertyResults.SUCCEEDED);
            } else if (COM_DATA_LINE_SHIFTY.equalsIgnoreCase(key)) {
                Number num = SGUtilityText.getDouble(value);
                if (num == null) {
                    result.putResult(COM_DATA_LINE_SHIFTY, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setLineShiftY(num.doubleValue()) == false) {
                    result.putResult(COM_DATA_LINE_SHIFTY, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                
                SGElementGroupSymbol symbol = getElementGroupSymbol();
                if (null==symbol || symbol.setShiftY(num.doubleValue())==false) {
                    result.putResult(COM_DATA_LINE_SHIFTY, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_DATA_LINE_SHIFTY, SGPropertyResults.SUCCEEDED);
            } else if (COM_LINE_CONNECT_ALL.equalsIgnoreCase(key)) {
            	Boolean b = SGUtilityText.getBoolean(value);
            	if (b == null) {
                    result.putResult(COM_LINE_CONNECT_ALL, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
            	if (this.setLineConnectingAll(b.booleanValue()) == false) {
                    result.putResult(COM_LINE_CONNECT_ALL, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
                result.putResult(COM_LINE_CONNECT_ALL, SGPropertyResults.SUCCEEDED);
            }
        }

        return result;
    }
    
    protected abstract SGElementGroupSymbol getElementGroupSymbol();

}
