package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;

import jp.riken.brain.ni.samuraigraph.base.SGTuple2f;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.data.SGISXYData;
import jp.riken.brain.ni.samuraigraph.figure.SGElementGroup;
import jp.riken.brain.ni.samuraigraph.figure.SGISXYDataConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGISymbolConstants;

import org.w3c.dom.Element;

/**
 * A group of lines in the graph object.
 * 
 * @author kuromaru
 * 
 */
public class SGElementGroupLineInGraph extends SGElementGroupLineForData implements
        SGIElementGroupInGraph, SGIStrokeEditDialogObserver,
        SGISXYDataConstants {

    /**
     * Maximum number of anchors.
     */
    public static final int MAX_NUMBER_OF_ANCHORS = 8;

    private SGFigureElementGraph mGraph = null;

    // 
    private SGElementGroupSetInGraph mGroupSet = null;

    public boolean setElementGroupSet(SGElementGroupSetInGraph gs) {
        this.mGroupSet = gs;
        return true;
    }

    private boolean mFocusedFlag = false;

    public void setFocused(final boolean b) {
        this.mFocusedFlag = b;
    }

    public boolean isFocused() {
        return this.mFocusedFlag;
    }

    /**
     * 
     */
    public SGElementGroupLineInGraph(SGISXYData data, SGFigureElementGraph graph) {
        super(data);
        this.mGraph = graph;
    }

    /**
     * 
     */
    public boolean writeProperty(final Element el) {
        if (super.writeProperty(el) == false) {
            return false;
        }
        el.setAttribute(KEY_VISIBLE, Boolean.toString(this.mVisibleFlag));
        el.setAttribute(KEY_LINE_CONNECT_ALL_EFFECTIVE_POINTS, Boolean.toString(this.mConnectingAllFlag));
        
        el.setAttribute(KEY_LINE_SHIFTX, Double.toString(this.getLineShiftX()));
        el.setAttribute(KEY_LINE_SHIFTY, Double.toString(this.getLineShiftY()));
        
        return true;
    }

    /**
     * 
     * @param el
     * @return
     */
    public boolean readProperty(final Element el) {
        if (super.readProperty(el) == false) {
            return false;
        }
        
        String str = null;
        Number num = null;
        
        // shiftx
        str = el.getAttribute(KEY_LINE_SHIFTX);
        if (str.length() != 0) {
            num = SGUtilityText.getDouble(str);
            if (num == null) {
                return false;
            }
            final double lineShiftX = num.doubleValue();
            if (this.setLineShiftX(lineShiftX) == false) {
                return false;
            }
            
            SGElementGroupSymbol symbol = getElementGroupSymbol();
            if (null==symbol || symbol.setShiftX(lineShiftX)==false) {
                return false;
            }
        }
        
        // shifty
        str = el.getAttribute(KEY_LINE_SHIFTY);
        if (str.length() != 0) {
            num = SGUtilityText.getDouble(str);
            if (num == null) {
                return false;
            }
            final double lineShiftY = num.doubleValue();
            if (this.setLineShiftY(lineShiftY) == false) {
                return false;
            }
            
            SGElementGroupSymbol symbol = getElementGroupSymbol();
            if (null==symbol || symbol.setShiftY(lineShiftY)==false) {
                return false;
            }
        }

        return this.mGraph.readProperty(this, el);
    }
    
    protected SGElementGroupSymbol getElementGroupSymbol() {
        List<SGElementGroup> list = this.mGroupSet.getElementGroupList();
        for (int i = 0; i < list.size(); i++) {
            if (list.get(i).getTagName().equals(SGISymbolConstants.TAG_NAME_SYMBOL)) {
                SGElementGroup group = list.get(i);
                if (group instanceof SGElementGroupSymbol) {
                    return (SGElementGroupSymbol)group;
                }
            }
        }
        
        return null;
    }

    /**
     * 
     */
    public boolean setLocation(final SGTuple2f[] pointArray) {
        final int mode = this.mGraph.getMode();
        if (mode == MODE_EXPORT_AS_IMAGE) {
            this.mMode = MODE_ALL;
        } else if (mode == MODE_DISPLAY) {
            this.mMode = MODE_OMIT;
        }
        return this.setLocationSub(pointArray);
    }
    
//    @Override
//    protected boolean setLocationSub(final SGTuple2f[] pointArray) {
//        SGTuple2f[] shiftedPointArray = pointArray;
//        final float shiftx = getShiftXInGraph();
//        final float shifty = getShiftYInGraph();
//        if (shiftx!=0.0f || shifty!=0.0f) {
//            shiftedPointArray = new SGTuple2f[pointArray.length];
//            for (int i = 0; i < pointArray.length; i++) {
//                shiftedPointArray[i] = new SGTuple2f(pointArray[i].x+shiftx, pointArray[i].y+shifty);
//            }
//        }
//        
//        return super.setLocationSub(shiftedPointArray);
//    }

    /**
     * 
     */
    public boolean paintElement(final Graphics2D g2d, final Rectangle2D rect) {
        super.paintElement(g2d, rect);

        SGElementGroupSetInGraphSXY groupSet = (SGElementGroupSetInGraphSXY) this.mGroupSet;

        if (this.isFocused()
                && this.mGraph.isSymbolsVisibleAroundFocusedObjects()) {
            if (!groupSet.isSymbolVisible() && !groupSet.isBarVisible()) {
                List<GeneralPath> pathList = this.mConnectedPathList;
                for (int ii = 0; ii < pathList.size(); ii++) {
                    GeneralPath gp = (GeneralPath) pathList.get(ii);

                    int num = 0;
                    ArrayList list = new ArrayList();

                    PathIterator itr = gp
                            .getPathIterator(new AffineTransform());
                    final float[] array = new float[6];
                    itr.currentSegment(array);
                    Point2D prev = new Point2D.Float(array[0], array[1]);
                    itr.next();

                    while (!itr.isDone()) {
                        itr.currentSegment(array);
                        Point2D pos = new Point2D.Float(array[0], array[1]);
                        pos.setLocation((prev.getX() + pos.getX()) / 2.0, (prev
                                .getY() + pos.getY()) / 2.0);
                        prev.setLocation(array[0], array[1]);
                        list.add(pos);
                        itr.next();
                        num++;
                    }

                    if (num <= MAX_NUMBER_OF_ANCHORS) {
                        for (int jj = 0; jj < num; jj++) {
                            Point2D pos = (Point2D) list.get(jj);
                            SGUtilityForFigureElementJava2D
                                    .drawAnchorAsFocusedObject(pos, g2d);
                        }
                    } else {
                        int div = num / MAX_NUMBER_OF_ANCHORS;
                        int cnt = 0;
                        while (true) {
                            Point2D pos = (Point2D) list.get(cnt);
                            SGUtilityForFigureElementJava2D
                                    .drawAnchorAsFocusedObject(pos, g2d);
                            cnt += div;
                            if (cnt >= num) {
                                break;
                            }
                        }

                    }

                }

            }

        }

        return true;
    }

    /**
     * Returns whether this group set contains the given point.
     * 
     * @param x
     *          the x coordinate
     * @param y
     *          the y coordinate
     * @return
     *          true if this element group contains the given point
     */
    public boolean contains(final int x, final int y) {
	// if a given point is out of the graph rectangle, returns false;
	Rectangle2D gRect = this.mGraph.getGraphRect();
	if (!gRect.contains(x, y)) {
	    return false;
	}
	return super.contains(x, y);
    }

    /**
     * Returns the location of the end point at a given index.
     * 
     * @param index
     *           the index of line
     * @return the location
     */
    public SGTuple2f getEnd(final int index) {
        if (this.mGroupSet instanceof SGElementGroupSetInGraphSXY) {
            SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
            final int n = this.getEndIndex(index);
            SGTuple2f tuple = gs.getLineLocation(n);
//            int sx = (int)getShiftXInGraph();
//            int sy = (int)getShiftYInGraph();
//            return new SGTuple2f(tuple.x+sx, tuple.y+sy);
            return new SGTuple2f(tuple);
        } else {
            throw new Error("Not supported.");
        }
    }

    /**
     * Returns the location of the start point at a given index.
     * 
     * @param index
     *           the index of line
     * @return the location
     */
    public SGTuple2f getStart(int index) {
        if (this.mGroupSet instanceof SGElementGroupSetInGraphSXY) {
            SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
            final int n = this.getStartIndex(index);
            SGTuple2f tuple = gs.getLineLocation(n);
//            int sx = (int)getShiftXInGraph();
//            int sy = (int)getShiftYInGraph();
//            return new SGTuple2f(tuple.x+sx, tuple.y+sy);
            return new SGTuple2f(tuple);
        } else {
            throw new Error("Not supported.");
        }
    }
    
//    protected float getShiftXInGraph() {
//        final double shift = this.getLineShiftX();
//        if (shift==0.0f) {
//            return 0.0f;
//        }
//        
//        final SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//        return gs.getShiftXLength(shift);
//    }
//    
//    protected float getShiftYInGraph() {
//        final double shift = this.getLineShiftY();
//        if (shift==0.0f) {
//            return 0.0f;
//        }
//        
//        final SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//        return - gs.getShiftYLength(shift);
//    }

}
