package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGColorBarModel;
import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.base.SGIData;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElement;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2f;
import jp.riken.brain.ni.samuraigraph.data.SGDataUtility;
import jp.riken.brain.ni.samuraigraph.data.SGISXYZTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGNetCDFData;
import jp.riken.brain.ni.samuraigraph.figure.SGElementGroup;
import jp.riken.brain.ni.samuraigraph.figure.SGUtilityForFigureElement;

import org.w3c.dom.Element;

/**
 * A class for the scalar XYZ-type data object in graph.
 * @author kuromaru
 *
 */
public class SGElementGroupSetInGraphSXYZ extends SGElementGroupSetInGraph
    implements SGIElementGroupSetSXYZ, SGISXYZDataDialogObserver {

    /**
     * An array of coordinates.
     */
    private SGTuple2f[] mPointsArray = null;

    /**
     * The array of x-coordinates.
     */
    private float[] mXArray = null;
    
    /**
     * The array of y-coordinates.
     */
    private float[] mYArray = null;

    /**
     * Builds a object.
     * 
     * @param data
     *           the data
     * @param graph
     *           the graph that this object belongs.
     */
    public SGElementGroupSetInGraphSXYZ(SGData data, SGFigureElementGraph graph) {
        super(data, graph);
    }
    
    /**
     * Initialize the points.
     * 
     * @param num
     *          the number of points
     */
    protected void initPointsArray(final int num) {
        SGTuple2f[] pointsArray = new SGTuple2f[num];
        for (int ii = 0; ii < num; ii++) {
            pointsArray[ii] = new SGTuple2f();
        }
        this.mPointsArray = pointsArray;
    }
    
    /**
     * Initialize the points.
     * 
     * @param numX
     *          the number of x-coordinates
     * @param numY
     *          the number of y-coordinates
     */
    protected void initPointsArray(final int numX, final int numY) {
        this.mXArray = new float[numX];
        this.mYArray = new float[numY];
    }

    /**
     * Dispose this object.
     */
    public void dispose() {
        super.dispose();
        this.mPointsArray = null;
        this.mXArray = null;
        this.mYArray = null;
    }


    /**
     * Called when the location of data points is changed.
     * 
     * @param data
     *           a data object
     * @return true if succeeded
     */
    public boolean updateDrawingElementsLocation(final SGIData data) {

        if ((data instanceof SGISXYZTypeData) == false) {
            return false;
        }
        SGISXYZTypeData dataSXYZ = (SGISXYZTypeData) data;

        if (this.isGridMode((SGData) data)) {
        	
        	// get x and y-values
            final double[] xValues = dataSXYZ.getXValueArray();
            final double[] yValues = dataSXYZ.getYValueArray();

            // initialize the points
            this.initPointsArray(xValues.length, yValues.length);

            // set the location of the data points
            if (this.mGraph.calcLocationOfPoints(xValues, 
                    this.getXAxis(), true, this.mXArray) == false) {
                return false;
            }
            if (this.mGraph.calcLocationOfPoints(yValues, 
                    this.getYAxis(), false, this.mYArray) == false) {
                return false;
            }
            
            // get z-values
            final double[] zValues = dataSXYZ.getZValueArray();
           
            List<SGElementGroup> gList = this.mDrawingElementGroupList;
            for (int ii = 0; ii < gList.size(); ii++) {
                SGElementGroup group = (SGElementGroup) gList.get(ii);
                if (group.isVisible()) {
                    if (group instanceof SGElementGroupColorMap) {
                    	SGElementGroupColorMap colorMap = (SGElementGroupColorMap) group;
                        if (colorMap.setLocation(this.mXArray, this.mYArray) == false) {
                            return false;
                        }
                        if (colorMap.setZValues(zValues) == false) {
                            return false;
                        }
                    }
                }
            }

        } else {

            final int num = dataSXYZ.getPointsNumber();
            this.initPointsArray(num);

            // set the location of the data points
            if (this.mGraph.calcLocationOfPoints(dataSXYZ.getXYValueArray(), 
                    this.getXAxis(), this.getYAxis(), this.mPointsArray) == false) {
                return false;
            }

            // get z-values
            final double[] zValues = dataSXYZ.getZValueArray();
            
            List<SGElementGroup> gList = this.mDrawingElementGroupList;
            for (int ii = 0; ii < gList.size(); ii++) {
                SGElementGroup group = (SGElementGroup) gList.get(ii);
                if (group.isVisible()) {
                    if (group instanceof SGElementGroupColorMap) {
                        SGElementGroupColorMap colorMap = (SGElementGroupColorMap) group;
                        
                        // set the location
                        if (colorMap.setLocation(this.mPointsArray) == false) {
                            return false;
                        }
                        
                        // set the z-values
                        colorMap.setZValues(zValues);
                    }
                }
            }
        }

        return true;
    }

    /**
     * Add an element group.
     */
    public boolean addDrawingElementGroup(int type) {
        SGElementGroupColorMap colorMap = null;
        if (type == SGIElementGroupConstants.RECTANGLE_GROUP) {
            colorMap = new SGElementGroupColorMapInGraph(this.mGraph);
            colorMap.setColorBarModel(this.mGraph.getColorBarModel());
            colorMap.setGridMode(this.isGridMode());
        } else {
            return false;
        }

        // add a group
        if (this.addElementGroup(colorMap) == false) {
        	return false;
        }

        if (!this.isGridMode()) {
            // set the colors
            SGISXYZTypeData data = (SGISXYZTypeData) this.mData;
            final double[] zArray = data.getZValueArray();
            colorMap.setColors(zArray);
        }
        
        return true;
    }
    
    private boolean addElementGroup(final SGElementGroupColorMap group) {

        // set the group set
        SGIElementGroupInGraph iGroup = (SGIElementGroupInGraph) group;
        iGroup.setElementGroupSet(this);

        // create drawing elements
        if (group.initDrawingElement(this.mXArray, this.mYArray) == false) {
        	return false;
        }
        if (group.initDrawingElement(this.mPointsArray) == false) {
        	return false;
        }

        // add a group to the list
        this.mDrawingElementGroupList.add(group);

        return true;
    }
    
    /**
     * Paint the objects.
     * 
     * @param g2d
     *           the graphic context
     */
    public void paintGraphics2D(Graphics2D g2d) {
        Rectangle2D gRect = null;
        if (!this.getClipFlag()) {
            gRect = this.mGraph.getGraphRect();
        }
        
        // color map
        final SGElementGroupColorMapInGraph colorMap = this.mGraph.getColorMap(this);
        if (colorMap != null) {
            if (colorMap.isVisible()) {
                colorMap.paintElement(g2d, gRect);
            }
        }
    }

    public boolean prepare() {
        this.mTemporaryProperties = this.getProperties();
        return true;
    }

    /**
     * Returns a text string denoting this class.
     */
    public String getClassDescription() {
        // returns an empty string
        return "";
    }

    public boolean getLegendVisibleFlag() {
        return this.isVisibleInLegend();
    }

    /**
     * Returns a color map which is the first element of an array.
     * 
     * @return the first element of an array of color maps, or null when this
     *         group set does not have any color maps
     */
    public SGElementGroupColorMap getColorMap() {
        return this.getColorMapGroup();
    }
    
    /**
     * Sets the color bar model.
     * 
     * @param model
     *           the color bar model
     * @return true if succeeded
     */
    public boolean setColorBarModel(final SGColorBarModel model) {
        List<SGElementGroupColorMap> groups = this.getColorMapGroups();
        for (int ii = 0; ii < groups.size(); ii++) {
            SGElementGroupColorMap colorMap = groups.get(ii);
            colorMap.setColorBarModel(model);
        }
        this.mZAxis = model.getAxis();
        return true;
    }


    public double getRectangleWidthValue() {
        return this.getColorMap().getWidthValue();
    }
    
    public double getRectangleHeightValue() {
        return this.getColorMap().getHeightValue();
    }

    public boolean setRectangleWidthValue(final double value) {
        return this.getColorMap().setWidthValue(value);
    }

    public boolean setRectangleHeightValue(final double value) {
        return this.getColorMap().setHeightValue(value);
    }

    public int getXAxisLocation() {
        return this.mGraph.getAxisElement().getLocationInPlane(this.getXAxis());
    }

    public int getYAxisLocation() {
        return this.mGraph.getAxisElement().getLocationInPlane(this.getYAxis());
    }
    
    public boolean hasValidRectangleWidthValue(final int config,
            final Number value) {
        final SGAxis axis = (config == -1) ? this.mXAxis : this.mGraph
                .getAxisElement().getAxisInPlane(config);
        final double v = (value != null) ? value.doubleValue() : this
                .getRectangleWidthValue();
        return axis.isValidValue(v);
    }

    public boolean hasValidRectangleHeightValue(final int config,
            final Number value) {
        final SGAxis axis = (config == -1) ? this.mYAxis : this.mGraph
                .getAxisElement().getAxisInPlane(config);
        final double v = (value != null) ? value.doubleValue() : this
                .getRectangleHeightValue();
        return axis.isValidValue(v);
    }

    /**
     * Write properties of this object to the Element.
     * 
     * @param el
     *            the Element object
     * @param type
     *            type of the method to save properties
     * @return true if succeeded
     */
    public boolean writeProperty(final Element el, final int type) {
        if (super.writeProperty(el, type) == false) {
            return false;
        }
        final String strX = this.mGraph.getAxisElement().getLocationName(
                this.getXAxis());
        final String strY = this.mGraph.getAxisElement().getLocationName(
                this.getYAxis());
        el.setAttribute(SGIFigureElement.KEY_X_AXIS_POSITION, strX);
        el.setAttribute(SGIFigureElement.KEY_Y_AXIS_POSITION, strY);
        return true;
    }

    /**
     * Retruns the location at a given index.
     * 
     * @param index
     *           the index
     * @return the location
     */
    public SGTuple2f getLocation(final int index) {
        if (this.mPointsArray == null) {
            return null;
        }
        if (index < 0 || index >= this.mPointsArray.length) {
            throw new IllegalArgumentException("Index out of bounds: " + index);
        }
        return this.mPointsArray[index];
    }

    protected SGPropertyResults setProperties(SGPropertyMap map,
            SGPropertyResults iResult) {

        SGPropertyResults result = (SGPropertyResults) iResult.clone();
        result = super.setProperties(map, result);
        if (result == null) {
            return null;
        }
        
        List<SGElementGroupColorMap> colorMaps = this.getColorMapGroups();
        for (int ii = 0; ii < colorMaps.size(); ii++) {
            SGElementGroupColorMapForData colorMap = (SGElementGroupColorMapForData) colorMaps.get(ii);
            result = colorMap.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }
        
        return result;
    }

	/**
	 * Returns a list of color map groups.
	 * 
	 * @return a list of color map groups
	 */
	public List<SGElementGroupColorMap> getColorMapGroups() {
		List<SGElementGroupColorMap> retList = new ArrayList<SGElementGroupColorMap>();
		List<SGElementGroup> list = SGUtilityForFigureElement.getGroups(
				SGElementGroupColorMap.class, this.mDrawingElementGroupList);
		for (int ii = 0; ii < list.size(); ii++) {
			retList.add((SGElementGroupColorMap) list.get(ii));
		}
		return retList;
	}

	/**
	 * Returns a color map group which is the first element of an array.
	 * 
	 * @return the first element of an array of arrow groups, or null when this
	 *         group set does not have any arrow groups
	 */
	public SGElementGroupColorMap getColorMapGroup() {
		return (SGElementGroupColorMap) SGUtilityForFigureElement
				.getGroup(SGElementGroupColorMap.class,
						this.mDrawingElementGroupList);
	}

    /**
     * Sets the properties of element groups.
     * 
     * @param elementGroupPropertiesList
     * @return true if succeeded
     * 
     */
    protected boolean setElementGroupProperties(List elementGroupPropertiesList) {
        for (int ii = 0; ii < elementGroupPropertiesList.size(); ii++) {
            SGProperties gp = (SGProperties) elementGroupPropertiesList.get(ii);
            SGElementGroup group = null;
            if (gp instanceof SGElementGroupColorMap.ColorMapProperties) {
                group = this.getColorMapGroup();
            } else {
                throw new Error("Illegal group property: " + gp);
            }
            if (group == null) {
                continue;
            }
            if (group.setProperties(gp) == false) {
                return false;
            }
        }
        return true;
    }
    
    /**
     * Returns true if this group set is under the grid mode.
     * 
     * @return true if this group set is under the grid mode
     */
    public boolean isGridMode() {
    	return this.isGridMode(this.getData());
    }

    /**
     * Returns true if a given data is under the grid mode.
     * 
     * @param data
     *           the data
     * @return true if a given data is under the grid mode
     */
    public boolean isGridMode(SGData data) {
    	if (SGDataUtility.isNetCDFData(data)) {
    		SGNetCDFData nData = (SGNetCDFData) data;
    		return !nData.isSerialNumberAvailable();
    	} else {
    		return false;
    	}
    }
    
    /**
     * Sets the information of data columns.
     * @param columns
     *                information of data columns
     */
    public boolean setColumnInfo(SGDataColumnInfo[] columns, String message) {
    	if (super.setColumnInfo(columns, message) == false) {
    		return false;
    	}

    	// initialize the points
    	SGISXYZTypeData data = (SGISXYZTypeData) this.getData();
    	this.initPointsArray(data.getPointsNumber());
    	
    	// update the grid mode
    	if (this.updateGridMode() == false) {
    		return false;
    	}
    	
    	return true;
    }

    private boolean updateGridMode() {
    	final boolean b = this.isGridMode();
    	List<SGElementGroupColorMap> maps = this.getColorMapGroups();
    	for (int ii = 0; ii < maps.size(); ii++) {
    		SGElementGroupColorMap map = maps.get(ii);
    		if (map.setGridMode(b) == false) {
    			return false;
    		}
    	}
    	return true;
    }

    /**
     * Sets the data.
     * 
     * @param data
     *            a data object
     * @return true if succeeded
     */
    public boolean setData(SGData data) {
    	if (!super.setData(data)) {
    		return false;
    	}
    	if (data == null) {
        	this.initPointsArray(0);
    		return true;
    	} else {
        	if (!(data instanceof SGISXYZTypeData)) {
        		return false;
        	}
        	SGISXYZTypeData dataSXYZ = (SGISXYZTypeData) data;
        	this.initPointsArray(dataSXYZ.getPointsNumber());
        	if (!this.updateDrawingElementsLocation(dataSXYZ)) {
        		return false;
        	}
    	}
    	return true;
    }

    /**
     * Updates the size of color map.
     * 
     * @return true if succeeded
     */
    public boolean updateColorMapSize() {
    	
    	SGISXYZTypeData data = (SGISXYZTypeData) this.getData();
    	
    	SGAxis axisX = this.getXAxis();
    	final double[] xArray = data.getXValueArray();
    	final double widthValue = SGUtilityForFigureElementJava2D.calcSizeValue(
    			xArray, axisX, true);
    	
    	SGAxis axisY = this.getYAxis();
    	final double[] yArray = data.getYValueArray();
    	final double heightValue = SGUtilityForFigureElementJava2D.calcSizeValue(
    			yArray, axisY, false);
    	
        SGElementGroupColorMap colorMap = (SGElementGroupColorMap) this.getColorMap();
        colorMap.setWidthValue(widthValue);
        colorMap.setHeightValue(heightValue);
        
        return true;
    }

    /**
     * Overrode to update the grid mode.
     * 
     */
    public boolean setProperties(SGProperties p ) {
    	if (super.setProperties(p) == false) {
    		return false;
    	}
    	this.updateGridMode();
    	return true;
    }
}
