package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;
import jp.riken.brain.ni.samuraigraph.base.SGIData;
import jp.riken.brain.ni.samuraigraph.base.SGIDataSource;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElement;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementAxis;
import jp.riken.brain.ni.samuraigraph.base.SGIPaint;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyDialog;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2f;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.data.SGDataUtility;
import jp.riken.brain.ni.samuraigraph.data.SGISXYData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYTypeMultipleData;
import jp.riken.brain.ni.samuraigraph.data.SGSXYMultipleNetCDFData;
import jp.riken.brain.ni.samuraigraph.figure.SGElementGroup;
import jp.riken.brain.ni.samuraigraph.figure.SGUtilityForFigureElement;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 */
public class SGElementGroupSetMultipleGraphSXY extends
        SGElementGroupSetMultipleInGraph implements
        SGIElementGroupSetMultipleSXY, SGIElementGroupSetInGraphSXY,
        SGISXYDataDialogObserver, SGISXYAxisShiftEnabled {

    /**
     * 
     */
    protected SGElementGroupSetMultipleGraphSXY(SGData data, SGFigureElementGraph graph) {
        super(data, graph);
    }

    /**
     * 
     */
    public boolean addDrawingElementGroup(final int type) {

        SGISXYData data = (SGISXYData) this.mData;
        SGElementGroupSXY group = null;
        if (type == SGIElementGroupConstants.POLYLINE_GROUP) {
            group = new SGElementGroupLineInGraph(data, this.mGraph);
        } else if (type == SGIElementGroupConstants.RECTANGLE_GROUP) {
            group = new SGElementGroupBarInGraph(data, this.mGraph);
        } else if (type == SGIElementGroupConstants.SYMBOL_GROUP) {
            group = new SGElementGroupSymbolInGraph(data, this.mGraph);
        } else {
            return false;
        }
        
        // set the group set
        SGIElementGroupInGraph iGroup = (SGIElementGroupInGraph) group;
        iGroup.setElementGroupSet(this);

        // only one element
        if (group.initDrawingElement(1) == false) {
        	return false;
        }
        
//        if (group.setPropertiesOfDrawingElements() == false) {
//            return false;
//        }
        
        // add a group to the list
        this.mDrawingElementGroupList.add(group);

        return true;
    }
    
    /**
     * 
     * @return
     */
    public String getClassDescription() {
        return "";
    }

    /**
     * Updates the location of drawing elements.
     * This method is overridden to update the child data objects.
     * 
     * @param data
     *            a data object
     * @return true if succeeded
     */
    public boolean updateDrawingElementsLocation(final SGIData data) {
        if ((data instanceof SGISXYTypeMultipleData) == false) {
            return false;
        }
        SGISXYTypeMultipleData dataMult = (SGISXYTypeMultipleData) data;
        SGISXYTypeData[] dataSXYArray = dataMult.getSXYDataArray();
        
        try {
    		// updates the location
            for (int ii = 0; ii < this.mElementGroupSetList.size(); ii++) {
                SGElementGroupSetInGraph gs = (SGElementGroupSetInGraph) this.mElementGroupSetList
                        .get(ii);
                if (ii < dataSXYArray.length) {
                    if (gs.updateDrawingElementsLocation(dataSXYArray[ii]) == false) {
                        return false;
                    }
                }
            }
        } finally {
        	if (dataSXYArray != null) {
            	// disposes of data objects
            	for (int ii = 0; ii < dataSXYArray.length; ii++) {
            		dataSXYArray[ii].dispose();
            	}    	
        	}
        }

        return true;
    }

    /**
     * 
     */
    public Rectangle2D getTickLabelsBoundingBox(final SGIData data) {
        if ((data instanceof SGISXYTypeMultipleData) == false) {
            throw new IllegalArgumentException("(data instanceof SGISXYTypeMultipleData) == false");
        }
        ArrayList<Rectangle2D> rectList = new ArrayList<Rectangle2D>();
        for (int ii = 0; ii < this.mElementGroupSetList.size(); ii++) {
        	SGIElementGroupSetInFigureElement groupSet = this.mElementGroupSetList.get(ii);
        	SGIElementGroupSetInGraphSXY gs = (SGIElementGroupSetInGraphSXY) this.mElementGroupSetList.get(ii);
        	Rectangle2D rect = gs.getTickLabelsBoundingBox(groupSet.getData());
        	if (rect == null) {
        		continue;
        	}
        	rectList.add(rect);
        }
        Rectangle2D uniRect = SGUtility.createUnion(rectList);
        if (uniRect == null) {
        	return null;
        }
        return uniRect;
    }

    /**
     * Paints tick labels.
     * 
     * @param g2d
     *            the graphics context
     * @return true if succeeded
     */
    public boolean paintDataString(final Graphics2D g2d) {
    	
    	// Paints tick labels of child SXY data sets.
    	// Checks overlapping of tick label resources.
    	List<SGISXYTypeData> dataList = new ArrayList<SGISXYTypeData>();
        List<Integer> indexList = new ArrayList<Integer>();
        for (int ii = 0; ii < this.mElementGroupSetList.size(); ii++) {
            SGIElementGroupSetInFigureElement gs = this.mElementGroupSetList.get(ii);
            SGISXYTypeData data = (SGISXYTypeData) gs.getData();
        	if (!data.isTickLabelAvailable()) {
        		continue;
        	}
        	boolean available = true;
        	for (int jj = 0; jj < dataList.size(); jj++) {
        		SGISXYTypeData avData = dataList.get(jj);
        		if (data.hasEqualTickLabelResource(avData)) {
        			available = false;
        			break;
        		}
        	}
        	if (available) {
        		dataList.add(data);
        		indexList.add(ii);
        	}
        }
        
        for (int ii = 0; ii < indexList.size(); ii++) {
			int index = indexList.get(ii);
			SGIElementGroupSetInGraphSXY groupSet = (SGIElementGroupSetInGraphSXY) this.mElementGroupSetList
					.get(index);
			groupSet.paintDataString(g2d);
        }

        return true;
    }

    /**
     * 
     */
    public boolean prepare() {
        this.mTemporaryProperties = this.getProperties();
        return true;
    }

    /**
     * 
     */
    public boolean setXAxisLocation(final int location) {
        if (location != SGIFigureElementAxis.AXIS_HORIZONTAL_1
                && location != SGIFigureElementAxis.AXIS_HORIZONTAL_2) {
            return false;
        }
        this.setXAxis(this.getAxis(location));
        return true;
    }

    /**
     * 
     */
    public boolean setYAxisLocation(final int location) {
        if (location != SGIFigureElementAxis.AXIS_PERPENDICULAR_1
                && location != SGIFigureElementAxis.AXIS_PERPENDICULAR_2) {
            return false;
        }
        this.setYAxis(this.getAxis(location));
        return true;
    }

    private SGAxis getAxis(final int location) {
        return this.mGraph.getAxisElement().getAxisInPlane(location);
    }

    public boolean setXAxis(SGAxis axis) {
        super.setXAxis(axis);
        ArrayList list = this.mElementGroupSetList;
        for (int ii = 0; ii < list.size(); ii++) {
            SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            groupSet.setXAxis(axis);
        }
        return true;
    }

    public boolean setYAxis(SGAxis axis) {
        super.setYAxis(axis);
        ArrayList list = this.mElementGroupSetList;
        for (int ii = 0; ii < list.size(); ii++) {
            SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            groupSet.setYAxis(axis);
        }
        return true;
    }

    /**
     * 
     * @param document
     * @return
     */
    public Element createElement(final Document document, final int type) {
        Element el = document.createElement(this.getTagName());
        if (this.writeProperty(el, type) == false) {
            return null;
        }

        // get the first group set
        SGElementGroupSetInGraphSXY groupSet = (SGElementGroupSetInGraphSXY) this.getFirst();

        // drawing elements
        ArrayList list = groupSet.getElementGroupList();
        for (int ii = 0; ii < list.size(); ii++) {
            SGElementGroup group = (SGElementGroup) list.get(ii);
            Element element = group.createElement(document);
            if (element == null) {
                return null;
            }
            el.appendChild(element);
        }

        return el;
    }

    /**
     * Write properties of this object to the Element.
     * 
     * @param el
     *            the Element object
     * @param type
     *            type of the method to save properties
     * @return true if succeeded
     */
    public boolean writeProperty(final Element el, final int type) {
        // call the method in the super class
        if (super.writeProperty(el, type) == false) {
            return false;
        }

        // get the first group set
        SGElementGroupSetInGraphSXY groupSet = (SGElementGroupSetInGraphSXY) this.getFirst();

        // axes
        final String strX = this.mGraph.getAxisElement().getLocationName(
                groupSet.getXAxis());
        final String strY = this.mGraph.getAxisElement().getLocationName(
                groupSet.getYAxis());
        el.setAttribute(SGIFigureElement.KEY_X_AXIS_POSITION, strX);
        el.setAttribute(SGIFigureElement.KEY_Y_AXIS_POSITION, strY);

        return true;
    }

    // Line
    public List<SGElementGroupLine> getLineGroups() {
    	return SGMultipleSXYUtility.getLineGroups(this);
    }

    public List<SGElementGroupLine> getLineGroupsIgnoreNull() {
    	return SGMultipleSXYUtility.getLineGroupsIgnoreNull(this);
    }

    public SGElementGroupLine getLineGroup() {
		return (SGElementGroupLine) SGUtilityForFigureElement.getGroup(
				SGElementGroupLine.class, this.mDrawingElementGroupList);
	}

    public boolean isLineVisible() {
        return this.getLineGroup().isVisible();
    }

    public float getLineWidth() {
        return this.getLineGroup().getLineWidth();
    }

    public float getLineWidth(final String unit) {
        return (float) SGUtilityText
                .convertFromPoint(this.getLineWidth(), unit);
    }

    public int getLineType() {
        return this.getLineGroup().getLineType();
    }

    public Color getLineColor() {
        return this.getLineGroup().getColor();
    }

    /**
     * Returns whether the lines connect all effective points.
     * 
     * @return true if connecting all effective points
     */
    public boolean isLineConnectingAll() {
        return this.getLineGroup().isLineConnectingAll();
    }
    
    public boolean setLineVisible(final boolean b) {
    	return SGMultipleSXYUtility.setLineVisible(this.getLineGroupsIgnoreNull(), b);
    }

    public boolean setLineWidth(final float width) {
    	return SGMultipleSXYUtility.setLineWidth(this.getLineGroupsIgnoreNull(), width);
    }

    public boolean setLineWidth(final float width, final String unit) {
    	return SGMultipleSXYUtility.setLineWidth(this.getLineGroupsIgnoreNull(), width, unit);
    }

    public boolean setLineType(final int type) {
    	return SGMultipleSXYUtility.setLineType(this.getLineGroupsIgnoreNull(), type);
    }

    public boolean setLineColor(final Color cl) {
    	return SGMultipleSXYUtility.setLineColor(this.getLineGroupsIgnoreNull(), cl);
    }

    /**
     * Sets whether the lines connect all effective points.
     * 
     * @return true if succeeded
     */
    public boolean setLineConnectingAll(final boolean b) {
    	return SGMultipleSXYUtility.setLineConnectingAll(this.getLineGroups(), b);
    }
    
    // Symbol
    public List<SGElementGroupSymbol> getSymbolGroups() {
    	return SGMultipleSXYUtility.getSymbolGroups(this);
    }

    public List<SGElementGroupSymbol> getSymbolGroupsIgnoreNull() {
    	return SGMultipleSXYUtility.getSymbolGroupsIgnoreNull(this);
    }

    public SGElementGroupSymbol getSymbolGroup() {
		return (SGElementGroupSymbol) SGUtilityForFigureElement.getGroup(
				SGElementGroupSymbol.class, this.mDrawingElementGroupList);
	}

    public boolean isSymbolVisible() {
        return this.getSymbolGroup().isVisible();
    }

    public int getSymbolType() {
        return this.getSymbolGroup().getType();
    }

    public float getSymbolSize() {
        return this.getSymbolGroup().getSize();
    }

    public float getSymbolSize(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this.getSymbolSize(),
                unit);
    }

    public float getSymbolLineWidth() {
        return this.getSymbolGroup().getLineWidth();
    }

    public float getSymbolLineWidth(final String unit) {
        return (float) SGUtilityText.convertFromPoint(
                this.getSymbolLineWidth(), unit);
    }

    @Override
    public SGIPaint getSymbolInnerPaint() {
        return this.getSymbolGroup().getInnerPaint();
    }

    public Color getSymbolLineColor() {
        return this.getSymbolGroup().getLineColor();
    }

    @Override
    public boolean isSymbolLineVisible() {
        return this.getSymbolGroup().isLineVisible();
    }

    public boolean setSymbolVisible(final boolean b) {
    	return SGMultipleSXYUtility.setSymbolVisible(this.getSymbolGroupsIgnoreNull(), b);
    }

    public boolean setSymbolType(final int type) {
    	return SGMultipleSXYUtility.setSymbolType(this.getSymbolGroupsIgnoreNull(), type);
    }

    public boolean setSymbolSize(final float size) {
    	return SGMultipleSXYUtility.setSymbolSize(this.getSymbolGroupsIgnoreNull(), size);
    }

    public boolean setSymbolSize(final float size, final String unit) {
    	return SGMultipleSXYUtility.setSymbolSize(this.getSymbolGroupsIgnoreNull(), size, unit);
    }

    public boolean setSymbolLineWidth(final float width) {
    	return SGMultipleSXYUtility.setSymbolLineWidth(this.getSymbolGroupsIgnoreNull(), width);
    }

    public boolean setSymbolLineWidth(final float width, final String unit) {
    	return SGMultipleSXYUtility.setSymbolLineWidth(this.getSymbolGroupsIgnoreNull(), width, unit);
    }

    @Override
    public boolean setSymbolInnerPaint(final SGIPaint paint) {
    	return SGMultipleSXYUtility.setSymbolInnerPaint(this.getSymbolGroupsIgnoreNull(), paint);
    }

    public boolean setSymbolLineColor(final Color cl) {
    	return SGMultipleSXYUtility.setSymbolLineColor(this.getSymbolGroupsIgnoreNull(), cl);
    }

    @Override
    public boolean setSymbolLineVisible(final boolean b) {
        return SGMultipleSXYUtility.setSymbolLineVisible(this.getSymbolGroupsIgnoreNull(), b);
    }

    // Bar
    public List<SGElementGroupBar> getBarGroups() {
    	return SGMultipleSXYUtility.getBarGroups(this);
    }

    public List<SGElementGroupBar> getBarGroupsIgnoreNull() {
    	return SGMultipleSXYUtility.getBarGroupsIgnoreNull(this);
    }

    public SGElementGroupBar getBarGroup() {
        return (SGElementGroupBar) SGUtilityForFigureElement.getGroup(
                SGElementGroupBar.class, this.mDrawingElementGroupList);
    }

    public boolean isBarVisible() {
        if (null==this.getBarGroup()) {
            return false;
        }
        return this.getBarGroup().isVisible();
    }

    public double getBarBaselineValue() {
        return this.getBarGroup().getBaselineValue();
    }

    public float getBarWidth() {
        return this.getBarGroup().getRectangleWidth();
    }

    public double getBarWidthValue() {
        return this.getBarGroup().getWidthValue();
    }

    public float getBarEdgeLineWidth() {
        return this.getBarGroup().getEdgeLineWidth();
    }

    public float getBarEdgeLineWidth(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this
                .getBarEdgeLineWidth(), unit);
    }

    @Override
    public SGIPaint getBarInnerPaint() {
        return this.getBarGroup().getInnerPaint();
    }

    @Override
    public Color getBarEdgeLineColor() {
        return this.getBarGroup().getEdgeLineColor();
    }

    @Override
    public boolean isBarEdgeLineVisible() {
        return this.getBarGroup().isEdgeLineVisible();
    }

    @Override
    public boolean isBarVertical() {
        return this.getBarGroup().isVertical();
    }

    @Override
    public double getBarInterval() {
        return this.getBarGroup().getInterval();
    }

    public boolean setBarVisible(final boolean b) {
    	return SGMultipleSXYUtility.setBarVisible(this.getBarGroupsIgnoreNull(), b);
    }

    public boolean setBarBaselineValue(final double value) {
    	return SGMultipleSXYUtility.setBarBaselineValue(this.getBarGroupsIgnoreNull(), value);
    }

    public boolean setBarWidth(final float width) {
    	return SGMultipleSXYUtility.setBarWidth(this.getBarGroupsIgnoreNull(), width);
    }

    public boolean setBarWidthValue(final double value) {
    	return SGMultipleSXYUtility.setBarWidthValue(this.getBarGroupsIgnoreNull(), value);
    }

    public boolean setBarEdgeLineWidth(final float width) {
    	return SGMultipleSXYUtility.setBarEdgeLineWidth(this.getBarGroupsIgnoreNull(), width);
    }

    public boolean setBarEdgeLineWidth(final float width, final String unit) {
    	return SGMultipleSXYUtility.setBarEdgeLineWidth(this.getBarGroupsIgnoreNull(), width, unit);
    }

    @Override
    public boolean setBarInnerPaint(final SGIPaint paint) {
        return SGMultipleSXYUtility.setBarInnerPaint(this.getBarGroupsIgnoreNull(), paint);
    }

    @Override
    public boolean setBarEdgeLineColor(final Color cl) {
    	return SGMultipleSXYUtility.setBarEdgeLineColor(this.getBarGroupsIgnoreNull(), cl);
    }

    @Override
    public boolean setBarEdgeLineVisible(final boolean b) {
        return SGMultipleSXYUtility.setBarEdgeLineVisible(this.getBarGroupsIgnoreNull(), b);
    }

    @Override
    public boolean setBarVertical(final boolean b) {
        return SGMultipleSXYUtility.setBarVertical(this.getBarGroupsIgnoreNull(), b);
    }

    public boolean hasValidBaselineValue(final int location, final Number value) {
        return ((SGElementGroupSetInGraphSXY) this.getFirst()).hasValidBaselineValue(location, value);
    }

    @Override
    public boolean setBarInterval(final double interval) {
        return SGMultipleSXYUtility.setBarInterval(this.getBarGroupsIgnoreNull(), interval);
    }

    // Error Bar
    public boolean isErrorBarAvailable() {
    	SGISXYTypeMultipleData data = (SGISXYTypeMultipleData) this.mData;
        return data.isErrorBarAvailable();
    }
    
    public List<SGElementGroupErrorBar> getErrorBarGroups() {
    	return SGMultipleSXYUtility.getErrorBarGroups(this);
    }

    public List<SGElementGroupErrorBar> getErrorBarGroupsIgnoreNull() {
    	return SGMultipleSXYUtility.getErrorBarGroupsIgnoreNull(this);
    }

    public SGElementGroupErrorBar getErrorBarGroup() {
		return (SGElementGroupErrorBar) SGUtilityForFigureElement.getGroup(
				SGElementGroupErrorBar.class, this.mDrawingElementGroupList);
	}

    public boolean isErrorBarVisible() {
        return this.getErrorBarGroup().isVisible();
    }

	public boolean isErrorBarVertical() {
		return this.getErrorBarGroup().isVertical();
	}

    public int getErrorBarHeadType() {
        return this.getErrorBarGroup().getHeadType();
    }

    public float getErrorBarHeadSize() {
        return this.getErrorBarGroup().getHeadSize();
    }

    public float getErrorBarHeadSize(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this
                .getErrorBarHeadSize(), unit);
    }

    public Color getErrorBarColor() {
        return this.getErrorBarGroup().getColor();
    }

    public float getErrorBarLineWidth() {
        return this.getErrorBarGroup().getLineWidth();
    }

    public float getErrorBarLineWidth(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this
                .getErrorBarLineWidth(), unit);
    }

    public int getErrorBarStyle() {
        return this.getErrorBarGroup().getErrorBarStyle();
    }

    @Override
    public boolean isErrorBarOnLinePosition() {
        return this.getErrorBarGroup().isPositionOnLine();
    }

    public boolean setErrorBarVisible(final boolean b) {
    	return SGMultipleSXYUtility.setErrorBarVisible(this.getErrorBarGroupsIgnoreNull(), b);
    }

    public boolean setErrorBarHeadType(final int type) {
    	return SGMultipleSXYUtility.setErrorBarHeadType(this.getErrorBarGroupsIgnoreNull(), type);
    }

    public boolean setErrorBarHeadSize(final float size) {
    	return SGMultipleSXYUtility.setErrorBarHeadSize(this.getErrorBarGroupsIgnoreNull(), size);
    }

    public boolean setErrorBarHeadSize(final float size, final String unit) {
    	return SGMultipleSXYUtility.setErrorBarHeadSize(this.getErrorBarGroupsIgnoreNull(), size, unit);
    }

    public boolean setErrorBarColor(final Color cl) {
    	return SGMultipleSXYUtility.setErrorBarColor(this.getErrorBarGroupsIgnoreNull(), cl);
    }

    public boolean setErrorBarLineWidth(final float width) {
    	return SGMultipleSXYUtility.setErrorBarLineWidth(this.getErrorBarGroupsIgnoreNull(), width);
    }

    public boolean setErrorBarLineWidth(final float width, final String unit) {
    	return SGMultipleSXYUtility.setErrorBarLineWidth(this.getErrorBarGroupsIgnoreNull(), width, unit);
    }

    public boolean setErrorBarStyle(final int style) {
    	return SGMultipleSXYUtility.setErrorBarStyle(this.getErrorBarGroupsIgnoreNull(), style);
    }

    public boolean setErrorBarVertical(final boolean b) {
        return SGMultipleSXYUtility.setErrorBarVertical(this.getErrorBarGroupsIgnoreNull(), b);
    }

    @Override
    public boolean setErrorBarOnLinePosition(boolean b) {
        return SGMultipleSXYUtility.setErrorBarOnLinePosition(this.getErrorBarGroupsIgnoreNull(), b);
    }


    // Tick Label
    public boolean isTickLabelAvailable() {
    	SGISXYTypeMultipleData data = (SGISXYTypeMultipleData) this.mData;
        return data.isTickLabelAvailable();
    }
    
    public List<SGElementGroupTickLabel> getTickLabelGroups() {
    	return SGMultipleSXYUtility.getTickLabelGroups(this);
    }

    public List<SGElementGroupTickLabel> getTickLabelGroupsIgnoreNull() {
    	return SGMultipleSXYUtility.getTickLabelGroupsIgnoreNull(this);
    }

    public SGElementGroupTickLabel getTickLabelGroup() {
		return (SGElementGroupTickLabel) SGUtilityForFigureElement.getGroup(
				SGElementGroupTickLabel.class, this.mDrawingElementGroupList);
	}

    public boolean isTickLabelVisible() {
        return this.getTickLabelGroup().isVisible();
    }

    public String getTickLabelFontName() {
        return this.getTickLabelGroup().getFontName();
    }

    public int getTickLabelFontStyle() {
        return this.getTickLabelGroup().getFontStyle();
    }

    public float getTickLabelFontSize() {
        return this.getTickLabelGroup().getFontSize();
    }

    public float getTickLabelFontSize(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this
                .getTickLabelFontSize(), unit);
    }

    public Color getTickLabelColor() {
        return this.getTickLabelGroup().getColor();
    }

    public float getTickLabelAngle() {
        return this.getTickLabelGroup().getAngle();
    }

    public int getTickLabelDecimalPlaces() {
        return this.getTickLabelGroup().getDecimalPlaces();
    }
    
    public int getTickLabelExponent() {
        return this.getTickLabelGroup().getExponent();
    }

	public boolean hasTickLabelHorizontalAlignment() {
		return this.getTickLabelGroup().hasHorizontalAlignment();
	}

    public boolean setTickLabelVisible(final boolean b) {
        return SGMultipleSXYUtility.setTickLabelVisible(this.getTickLabelGroupsIgnoreNull(), b);
    }

    public boolean setTickLabelFontName(final String name) {
        return SGMultipleSXYUtility.setTickLabelFontName(this.getTickLabelGroupsIgnoreNull(), name);
    }

    public boolean setTickLabelFontStyle(final int style) {
        return SGMultipleSXYUtility.setTickLabelFontStyle(this.getTickLabelGroupsIgnoreNull(), style);
    }

    public boolean setTickLabelFontSize(final float size) {
        return SGMultipleSXYUtility.setTickLabelFontSize(this.getTickLabelGroupsIgnoreNull(), size);
    }

    public boolean setTickLabelFontSize(final float size, final String unit) {
        return SGMultipleSXYUtility.setTickLabelFontSize(this.getTickLabelGroupsIgnoreNull(), size, unit);
    }

    public boolean setTickLabelColor(final Color cl) {
        return SGMultipleSXYUtility.setTickLabelColor(this.getTickLabelGroupsIgnoreNull(), cl);
    }

    public boolean setTickLabelAngle(final float angle) {
        return SGMultipleSXYUtility.setTickLabelAngle(this.getTickLabelGroupsIgnoreNull(), angle);
    }

    public boolean setTickLabelDecimalPlaces(final int dp) {
        return SGMultipleSXYUtility.setTickLabelDecimalPlaces(this.getTickLabelGroupsIgnoreNull(), dp);
    }
    
    public boolean setTickLabelExponent(final int exp) {
        return SGMultipleSXYUtility.setTickLabelExponent(this.getTickLabelGroupsIgnoreNull(), exp);
    }

	public boolean setTickLabelHorizontalAlignment(final boolean b) {
		return SGMultipleSXYUtility.setTickLabelHorizontalAlignment(
				this.getTickLabelGroupsIgnoreNull(), b);
	}

    public int getSelectedTabIndex() {
        return this.mSelectedTabIndex;
    }

    private int mSelectedTabIndex = -1;

    /**
     * 
     * @param e
     * @param groupSet
     * @return
     */
    protected boolean onMouseClicked(final MouseEvent e) {
        if (super.onMouseClicked(e) == false) {
            return false;
        }

        SGElementGroup group = this.getElementGroupAt(e.getX(), e.getY());

        int index = -1;
		if (group instanceof SGElementGroupLineInGraph) {
			index = 0;
		} else if (group instanceof SGElementGroupSymbolInGraph) {
			index = 1;
		} else if (group instanceof SGElementGroupBarInGraph) {
			index = 2;
		} else if (group instanceof SGElementGroupErrorBar) {
			index = 3;
		} else if (group instanceof SGElementGroupTickLabel) {
			index = 4;
		}
        this.mSelectedTabIndex = index;

        return true;
    }

// /**
// *
// * @return
// */
//    private SGElementGroupSetInGraphSXY getFirst() {
//        SGElementGroupSetInGraphSXY groupSet = (SGElementGroupSetInGraphSXY) this.mElementGroupSetList
//                .get(0);
//        return groupSet;
//    }

    /**
     * 
     */
    public int getXAxisLocation() {
//        return ((SGElementGroupSetInGraphSXY) this.getFirst()).getXAxisLocation();
		return this.mGraph.getAxisElement().getLocationInPlane(this.getXAxis());
	}

    /**
     * 
     */
    public int getYAxisLocation() {
//        return ((SGElementGroupSetInGraphSXY) this.getFirst()).getYAxisLocation();
		return this.mGraph.getAxisElement().getLocationInPlane(this.getYAxis());
    }

    /**
     * 
     */
    public boolean getLegendVisibleFlag() {
        return this.isVisibleInLegend();
    }

//    /**
//     * 
//     */
//    public boolean synchronizeDrawingElement(ArrayList sList) {
//        boolean diffFlag = false;
//
//        ArrayList gList = this.mElementGroupSetList;
//        for (int ii = 0; ii < gList.size(); ii++) {
//            SGElementGroupSetInGraphSXY groupSet = (SGElementGroupSetInGraphSXY) gList
//                    .get(ii);
//            final boolean b = groupSet.synchronizeDrawingElement(sList);
//            if (b) {
//                diffFlag = true;
//            }
//        }
//
//        return diffFlag;
//    }
    
//    /**
//     * Returns properties of this group set.
//     * 
//     * @return properties of this group set
//     */
//    public SGProperties getProperties() {
//        ElementGroupSetMultipleSXYDataProperties ep = new ElementGroupSetMultipleSXYDataProperties();
//        if (this.getProperties(ep) == false) {
//            return null;
//        }
//        return ep;
//    }
//    
//    /**
//     * 
//     */
//    public boolean getProperties(final SGProperties p) {
//        if ((p instanceof ElementGroupSetMultipleSXYDataProperties) == false) {
//            return false;
//        }
//        if (super.getProperties(p) == false) {
//            return false;
//        }
//        ElementGroupSetMultipleSXYDataProperties ep = (ElementGroupSetMultipleSXYDataProperties) p;
//        SGData data = this.getData();
//        SGSXYMultipleData mData = (SGSXYMultipleData) data;
//        ep.mXIndex = mData.getXIndex();
//        ep.mYIndices = mData.getYIndices();
//        return true;
//    }
//
//    /**
//     * 
//     */
//    public boolean setProperties(final SGProperties p) {
//        if ((p instanceof ElementGroupSetMultipleSXYDataProperties) == false) {
//            return false;
//        }
//        if (super.setProperties(p) == false) {
//            return false;
//        }
//        ElementGroupSetMultipleSXYDataProperties ep = (ElementGroupSetMultipleSXYDataProperties) p;
//        SGData data = this.getData();
//        SGSXYMultipleData mData = (SGSXYMultipleData) data;
//        mData.setXIndex(ep.mXIndex);
//        mData.setYIndices(ep.mYIndices);
//        return true;
//    }
//
//    
//    /**
//     * A class of properties of a group set for multiple SXY data.
//     */
//    public static class ElementGroupSetMultipleSXYDataProperties extends
//            ElementGroupSetPropertiesInFigureElement {
//    
//        // indices of data columns
//        int mXIndex = -1;
//    
//        int[] mYIndices = null;
//        
//        /**
//         * The default constructor.
//         */
//        public ElementGroupSetMultipleSXYDataProperties() {
//            super();
//        }
//    
//        /**
//         * Returns whether this object is equal to given object.
//         * @param obj
//         *            an object to be compared
//         * @return
//         *            true if two objects are equal
//         */
//        public boolean equals(final Object obj) {
//            if ((obj instanceof ElementGroupSetMultipleSXYDataProperties) == false) {
//                return false;
//            }
//            if (super.equals(obj) == false) {
//                return false;
//            }
//    
//            ElementGroupSetMultipleSXYDataProperties p = (ElementGroupSetMultipleSXYDataProperties) obj;
//            if (p.mXIndex != this.mXIndex) {
//                return false;
//            }
//            for (int ii = 0; ii < this.mYIndices.length; ii++) {
//                if (p.mYIndices[ii] != this.mYIndices[ii]) {
//                    return false;
//                }
//            }
//    
//            return true;
//        }
//    
//        /**
//         * Returns string representation of this object.
//         * @return
//         *        string representation of this object
//         */
//        public String toString() {
//            String str = super.toString();
//            StringBuffer sb = new StringBuffer();
//            sb.append("[");
//            sb.append(str);
//            sb.append(", XIndex=");
//            sb.append(this.mXIndex);
//            sb.append(", YIndices={");
//            for (int ii = 0; ii < this.mYIndices.length; ii++) {
//                sb.append(this.mYIndices[ii]);
//                if (ii != this.mYIndices.length - 1) {
//                    sb.append(", ");
//                }
//            }
//            sb.append("}]");
//            return sb.toString();
//        }
//    }
 
    public void addChildGroupSet(SGIElementGroupSetInFigureElement gs) {
        this.mElementGroupSetList.add(gs);
    }

    public SGIElementGroupSetInFigureElement[] getChildGroupSetArray() {
        SGIElementGroupSetInFigureElement[] array = new SGIElementGroupSetInFigureElement[this.mElementGroupSetList.size()];
        this.mElementGroupSetList.toArray(array);
        return array;
    }

    /**
     * Returns the location of a line at a given index.
     * 
     * @param index
     *           the index
     * @return the location
     */
    public SGTuple2f getLineLocation(final int groupIndex, final int index) {
        if (groupIndex < 0 || groupIndex >= this.mElementGroupSetList.size()) {
            throw new IllegalArgumentException("Index out of bounds: " + index);
        }
        SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mElementGroupSetList.get(groupIndex);
        return gs.getLineLocation(index);
    }

    /**
     * Returns the location of a symbol at a given index.
     * 
     * @param index
     *           the index
     * @return the location
     */
    public SGTuple2f getSymbolLocation(final int groupIndex, final int index) {
        if (groupIndex < 0 || groupIndex >= this.mElementGroupSetList.size()) {
            throw new IllegalArgumentException("Index out of bounds: " + index);
        }
        SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mElementGroupSetList.get(groupIndex);
        return gs.getSymbolLocation(index);
    }

    /**
     * Returns the location of a bar at a given index.
     * 
     * @param index
     *           the index
     * @return the location
     */
    public SGTuple2f getBarLocation(final int groupIndex, final int index) {
        if (groupIndex < 0 || groupIndex >= this.mElementGroupSetList.size()) {
            throw new IllegalArgumentException("Index out of bounds: " + index);
        }
        SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mElementGroupSetList.get(groupIndex);
        return gs.getBarLocation(index);
    }

    protected SGPropertyResults setProperties(SGPropertyMap map,
            SGPropertyResults iResult) {

        SGPropertyResults result = (SGPropertyResults) iResult.clone();
        result = super.setProperties(map, result);
        if (result == null) {
            return null;
        }
        
        result = SGUtilityForFigureElementJava2D
				.checkDataElementGroupVisibility(this, map, result);
		if (result == null) {
			return null;
		}

        List<SGElementGroupLine> lineGroups = this.getLineGroups();
        for (int ii = 0; ii < lineGroups.size(); ii++) {
            SGElementGroupLineForData lines = (SGElementGroupLineForData) lineGroups.get(ii);
            result = lines.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }
        
        List<SGElementGroupSymbol> symbolGroups = this.getSymbolGroups();
        for (int ii = 0; ii < symbolGroups.size(); ii++) {
            SGElementGroupSymbolForData symbols = (SGElementGroupSymbolForData) symbolGroups.get(ii);
            result = symbols.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }

        List<SGElementGroupBar> barGroups = this.getBarGroups();
        for (int ii = 0; ii < barGroups.size(); ii++) {
            SGElementGroupBarForData bar = (SGElementGroupBarForData) barGroups.get(ii);
            result = bar.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }
        
        List<SGElementGroupErrorBar> errorBarGroups = this.getErrorBarGroups();
        for (int ii = 0; ii < errorBarGroups.size(); ii++) {
            SGElementGroupErrorBarForData errorBar = (SGElementGroupErrorBarForData) errorBarGroups.get(ii);
            result = errorBar.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }
        
        List<SGElementGroupTickLabel> tickLabelGroups = this.getTickLabelGroups();
        for (int ii = 0; ii < tickLabelGroups.size(); ii++) {
            SGElementGroupTickLabelForData tickLabel = (SGElementGroupTickLabelForData) tickLabelGroups.get(ii);
            result = tickLabel.setProperties(map, result);
            if (result == null) {
                return null;
            }
        }

        return result;
    }

    /**
     * Creates drawing elements of error bars.
     * 
     * @param dataXY
     *             XY type data
     * @return the group of error bars
     */
	public SGElementGroupErrorBar createErrorBars(SGISXYTypeMultipleData dataXY) {
        final SGElementGroupErrorBarInGraph group = new SGElementGroupErrorBarInGraph(
                dataXY, this.mGraph);
        this.mDrawingElementGroupList.add(group);
        group.mGroupSet = this;
        return group;
	}

    /**
     * Creates drawing elements of tick labels.
     * 
     * @param dataXY
     *             XY type data
     * @return the group of tick labels
     */
	public SGElementGroupTickLabel createTickLabels(SGISXYTypeMultipleData dataXY) {
        final SGElementGroupTickLabelInGraphSXY group = new SGElementGroupTickLabelInGraphSXY(
                dataXY, this.mGraph);
        this.mDrawingElementGroupList.add(group);
        group.mGroupSet = this;
        return group;
	}
	
    /**
     * Sets the information of data columns.
     * This method is overridden to update the child data objects.
     * 
     * @param columns
     *                information of data columns
     * @return true if succeeded
     */
    public boolean setColumnInfo(SGDataColumnInfo[] columns, String message) {
        this.mGraph.undoDataConversion();
        
        boolean pickupInBefore = SGDataUtility.isPickupColumnContained(this.getDataColumnInfoArray());
        boolean pickupInAfter = SGDataUtility.isPickupColumnContained(columns);
        if (this.isSelected()==false) {
            this.setSelected(true);
            this.mGraph.notifyDataSelection();
        }
        
        if (pickupInBefore!=pickupInAfter) {
            if (this.mData instanceof SGSXYMultipleNetCDFData) {
                this.mGraph.notifyDataConversion(this.getData(), this.getName(), columns, message);
                return true;
            }
            if (SGPropertyDialog.NOTIFY_ACTION_PREVIEW.equals(message)) {
                return false;
            }
        }
        
    	if (!super.setColumnInfo(columns, message)) {
    		return false;
    	}
    	if (SGMultipleSXYUtility.setColumnInfo(this, this.mGraph) == false) {
    		return false;
    	}
    	return true;
    }
    
    @Override
    public boolean cancel() {
        this.mGraph.undoDataConversion();
        return super.cancel();
    }
    
    /**
     * Update drawing elements with related data object.
     * @return
     *         true if succeeded
     */
    public boolean updateWithData() {
        this.updateTickLabelStrings();
        return super.updateWithData();
    }
    
    /**
     * Update the text strings of tick labels.
     */
    public void updateTickLabelStrings() {
        SGData data = this.getData();
        SGISXYTypeMultipleData dataSXY = (SGISXYTypeMultipleData) data;
        if (dataSXY.isTickLabelAvailable()) {
            SGElementGroupTickLabel tg = this.getTickLabelGroup();

            // set decimal places and exponent
            final int dp = tg.getDecimalPlaces();
            final int exp = tg.getExponent();
            
        	// set the decimal places and the exponent to the data object
            dataSXY.setDecimalPlaces(dp);
            dataSXY.setExponent(exp);
            for (int ii = 0; ii < this.mElementGroupSetList.size(); ii++) {
            	SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mElementGroupSetList.get(ii);
            	gs.setTickLabelDecimalPlaces(dp);
            	gs.setTickLabelExponent(exp);
            	gs.updateTickLabelStrings();
            }
        }
    }

    /**
     * Sets the properties of element groups.
     * 
     * @param elementGroupPropertiesList
     * @return true if succeeded
     * 
     */
    protected boolean setElementGroupProperties(List elementGroupPropertiesList) {
        for (int ii = 0; ii < elementGroupPropertiesList.size(); ii++) {
            SGProperties gp = (SGProperties) elementGroupPropertiesList.get(ii);
            SGElementGroup group = null;
            if (gp instanceof SGElementGroupLine.LineProperties) {
                group = this.getLineGroup();
            } else if (gp instanceof SGElementGroupSymbol.SymbolProperties) {
                group = this.getSymbolGroup();
            } else if (gp instanceof SGElementGroupBar.BarProperties) {
                group = this.getBarGroup();
            } else if (gp instanceof SGElementGroupErrorBar.ErrorBarProperties) {
                group = this.getErrorBarGroup();
            } else if (gp instanceof SGElementGroupString.StringProperties) {
                group = this.getTickLabelGroup();
            } else {
                throw new Error("Illegal group property: " + gp);
            }
            if (group == null) {
                continue;
            }
            if (group.setProperties(gp) == false) {
                return false;
            }
        }
        return true;
    }

    /**
     * Overrode to add a command to split data.
     */
    @Override
    protected List<PopupMenuItem> getOptionalCommandList() {
        List<PopupMenuItem> list = super.getOptionalCommandList();
        
        List<SGData> dataList = this.mGraph.getFocusedDataList();
        if (dataList.size()>1) {
            for (PopupMenuItem item : list) {
                if (item.mItemString.equals(MENUCMD_TRANSFORM_DATA)) {
                    item.setEnabled(false);
                    break;
                }
            }
        }
        
        boolean isSplitEnabled = false;
        if (this.mData instanceof SGISXYTypeMultipleData) {
            if (((SGISXYTypeMultipleData)this.mData).isSplitEnabled()) {
                isSplitEnabled = true;
            }
        }
        list.add(new PopupMenuItem(MENUCMD_SPLIT_DATA, isSplitEnabled));
        list.add(new PopupMenuItem(MENUCMD_MERGE_DATA, this.isMergeEnabled()));
        return list;
    }
    
    /**
     * Returns true if this groupSet enables to be merged.
     * 
     * @return true if this groupSet enables to be merged
     */
    private boolean isMergeEnabled() {
        List<SGData> dataList = this.mGraph.getFocusedDataList();
        SGIDataSource dataSource = this.mData.getDataSource();
        int count = 0;
        for (int i = 0; i < dataList.size(); i++) {
            if (dataSource.equals(dataList.get(i).getDataSource())) {
                count++;
                if (count > 1) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Overrode for a command to split data.
     */
    public void actionPerformed(final ActionEvent e) {
        super.actionPerformed(e);
    }

    @Override
    public int getMultipleDimensionStart() {
        SGData data = this.getData();
        if (data instanceof SGSXYMultipleNetCDFData) {
            SGSXYMultipleNetCDFData nData = (SGSXYMultipleNetCDFData) data;
            return nData.getDimensionStart();
        } else {
            return -1;
        }
    }

    @Override
    public boolean setMultipleDimensionStart(final int start) {
        SGData data = this.getData();
        if (data instanceof SGSXYMultipleNetCDFData) {
            SGSXYMultipleNetCDFData nData = (SGSXYMultipleNetCDFData) data;
            return nData.setDimensionStart(start);
        } else {
            return false;
        }
    }

    @Override
    public int getMultipleDimensionEnd() {
        SGData data = this.getData();
        if (data instanceof SGSXYMultipleNetCDFData) {
            SGSXYMultipleNetCDFData nData = (SGSXYMultipleNetCDFData) data;
            return nData.getDimensionEnd();
        } else {
            return -1;
        }
    }

    @Override
    public boolean setMultipleDimensionEnd(final int end) {
        SGData data = this.getData();
        if (data instanceof SGSXYMultipleNetCDFData) {
            SGSXYMultipleNetCDFData nData = (SGSXYMultipleNetCDFData) data;
            return nData.setDimensionEnd(end);
        } else {
            return false;
        }
    }
    
    @Override
    public int getMultipleDimensionStep() {
        SGData data = this.getData();
        if (data instanceof SGSXYMultipleNetCDFData) {
            SGSXYMultipleNetCDFData nData = (SGSXYMultipleNetCDFData) data;
            return nData.getDimensionStep();
        } else {
            return -1;
        }
    }
    
    @Override
    public boolean setMultipleDimensionStep(final int step) {
        SGData data = this.getData();
        if (data instanceof SGSXYMultipleNetCDFData) {
            SGSXYMultipleNetCDFData nData = (SGSXYMultipleNetCDFData) data;
            return nData.setDimensionStep(step);
        } else {
            return false;
        }
    }

    @Override
    public double getLineShiftX() {
        return this.getLineGroup().getLineShiftX();
    }

    @Override
    public double getLineShiftY() {
        return this.getLineGroup().getLineShiftY();
    }

    @Override
    public boolean setLineShiftX(final double shift) {
        return SGMultipleSXYUtility.setLineShiftX(this.getLineGroupsIgnoreNull(), shift);
    }

    @Override
    public boolean setLineShiftY(final double shift) {
        return SGMultipleSXYUtility.setLineShiftY(this.getLineGroupsIgnoreNull(), shift);
    }

    @Override
    public double getBarShiftX() {
        return this.getBarGroup().getShiftX();
    }

    @Override
    public double getBarShiftY() {
        return this.getBarGroup().getShiftY();
    }

    @Override
    public boolean setBarShiftX(final double shift) {
        return SGMultipleSXYUtility.setBarShiftX(this.getBarGroupsIgnoreNull(), shift);
    }

    @Override
    public boolean setBarShiftY(final double shift) {
        return SGMultipleSXYUtility.setBarShiftY(this.getBarGroupsIgnoreNull(), shift);
    }

    @Override
    public double getSymbolShiftX() {
        return this.getSymbolGroup().getShiftX();
    }

    @Override
    public double getSymbolShiftY() {
        return this.getSymbolGroup().getShiftY();
    }

    @Override
    public boolean setSymbolShiftX(final double shift) {
        return SGMultipleSXYUtility.setSymbolShiftX(this.getSymbolGroupsIgnoreNull(), shift);
    }

    @Override
    public boolean setSymbolShiftY(final double shift) {
        return SGMultipleSXYUtility.setSymbolShiftY(this.getSymbolGroupsIgnoreNull(), shift);
    }

//    /**
//     * 
//     * @param shift
//     * @return
//     */
//    public float getShiftXLength(final float shift) {
//        if (shift==0.0f) {
//            return 0.0f;
//        }
//        
//        final SGAxis axis = this.getXAxis();
//        final double minValue = axis.getMinValue();
//        final double maxValue = axis.getMaxValue();
//        final double diff = maxValue - minValue;
//        if (diff==0.0) {
//            return 0.0f;
//        }
//        return this.mGraph.getGraphRectWidth() * (float) (shift / diff);
//    }
//    
//    /**
//     * 
//     * @param shift
//     * @return
//     */
//    public float getShiftYLength(final float shift) {
//        if (shift==0.0f) {
//            return 0.0f;
//        }
//        
//        final SGAxis axis = this.getYAxis();
//        final double minValue = axis.getMinValue();
//        final double maxValue = axis.getMaxValue();
//        final double diff = maxValue - minValue;
//        if (diff==0.0) {
//            return 0.0f;
//        }
//        return this.mGraph.getGraphRectHeight() * (float) (shift / diff);
//    }

    @Override
    public Double getShiftMinValue(SGAxis axis) {
        Double shift = null;
        if (axis.equals(this.getXAxis())) {
            if (this.isLineVisible() || this.isSymbolVisible()) {
                shift = Double.valueOf(this.getLineShiftX());
            }
            if (this.isBarVisible()) {
                // minimum left shift size of bars.
                // If this groupset has multiple bar series, minimum shift means shiftx point minus
                // bar widths of series which are enabled to be visible.
                double barLeftShift = this.getBarShiftX();
                if (this.isBarVertical()) {
                    int numberSeries = this.mGraph.getNumberOfVisibleBarSeries(this);
                    barLeftShift = barLeftShift -this.getBarInterval()*(numberSeries*0.5)-this.getBarWidthValue();
                }
                if (null==shift || shift.doubleValue()>barLeftShift) {
                    shift = Double.valueOf(barLeftShift);
                }
            }
        } else if (axis.equals(this.getYAxis())) {
            if (this.isLineVisible() || this.isSymbolVisible()) {
                shift = Double.valueOf(this.getLineShiftY());
            }
            if (this.isBarVisible()) {
                double barBottomShift = this.getBarShiftY();
                if (this.isBarVertical()==false) {
                    int numberSeries = this.mGraph.getNumberOfVisibleBarSeries(this);
                    barBottomShift = barBottomShift-this.getBarInterval()*(numberSeries*0.5)-this.getBarWidthValue();
                }
                if (null==shift || shift.doubleValue()>barBottomShift) {
                    shift = Double.valueOf(barBottomShift);
                }
            }
        }
        return shift;
    }

    @Override
    public Double getShiftMaxValue(SGAxis axis) {
        Double shift = null;
        if (axis.equals(this.getXAxis())) {
            if (this.isLineVisible() || this.isSymbolVisible()) {
                shift = Double.valueOf(this.getLineShiftX());
            }
            if (this.isBarVisible()) {
                // maximum right shift size of bars.
                // If this groupset has multiple bar series, maximum shift means shiftx point plus
                // bar widths of series which are enabled to be visible.
                double barRightShift = this.getBarShiftX();
                if (this.isBarVertical()) {
                    int numberSeries = this.mGraph.getNumberOfVisibleBarSeries(this);
                    barRightShift = barRightShift + this.getBarInterval()*(numberSeries*0.5)+this.getBarWidthValue();
                }
                if (null==shift || shift.doubleValue()<barRightShift) {
                    shift = Double.valueOf(barRightShift);
                }
            }
        } else if (axis.equals(this.getYAxis())) {
            if (this.isLineVisible() || this.isSymbolVisible()) {
                shift = Double.valueOf(this.getLineShiftY());
            }
            if (this.isBarVisible()) {
                double barTopShift = this.getBarShiftY();
                if (this.isBarVertical()==false) { 
                    int numberSeries = this.mGraph.getNumberOfVisibleBarSeries(this);
                    barTopShift = barTopShift+this.getBarInterval()*(numberSeries*0.5)+this.getBarWidthValue();
                }
                if (null==shift || shift.doubleValue()<barTopShift) {
                    shift = Double.valueOf(barTopShift);
                }
            }
        }
        return shift;
    }
    
    @Override
    public Double getBarBaselineForAxis(final SGAxis axis) {
        if (this.isBarVisible()==false) {
            return null;
        }
        if (axis.equals(this.getXAxis())) {
            if (this.isBarVertical()) {
                return null;
            } else {
                return Double.valueOf(this.getBarBaselineValue());
            }
        } else if (axis.equals(this.getYAxis())) {
            if (this.isBarVertical()) {
                return Double.valueOf(this.getBarBaselineValue());
            } else {
                return null;
            }
        }
        return null;
    }
    
	/**
	 * Sets the direction of error bars.
	 * 
	 * @param vertical
	 *           true to set vertical
	 * @return true if succeeded
	 */
	public boolean setErrorBarDirection(final boolean vertical) {
    	SGIElementGroupSetInFigureElement[] childArray = this.getChildGroupSetArray();
    	for (int ii = 0; ii < childArray.length; ii++) {
    		SGIElementGroupSetSXY gs = (SGIElementGroupSetSXY) childArray[ii];
    		gs.setErrorBarDirection(vertical);
    	}
    	List<SGElementGroupErrorBar> groupList = this.getErrorBarGroups();
    	for (int ii = 0; ii < groupList.size(); ii++) {
    		SGElementGroupErrorBar group = groupList.get(ii);
    		group.setVertical(vertical);
    	}
		return true;
	}

	/**
	 * Sets the alignment of tick label.
	 * 
	 * @param horizontal
	 *           true to align horizontally
	 * @return true if succeeded
	 */
	public boolean setTickLabelAlignment(final boolean horizontal) {
    	SGIElementGroupSetInFigureElement[] childArray = this.getChildGroupSetArray();
    	for (int ii = 0; ii < childArray.length; ii++) {
    		SGIElementGroupSetSXY gs = (SGIElementGroupSetSXY) childArray[ii];
    		gs.setTickLabelAlignment(horizontal);
    	}
    	List<SGElementGroupTickLabel> groupList = this.getTickLabelGroups();
    	for (int ii = 0; ii < groupList.size(); ii++) {
    		SGElementGroupTickLabel group = groupList.get(ii);
    		group.setHorizontalAlignment(horizontal);
    	}
		return true;
	}

}
