package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.List;

import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGISelectable;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.figure.SGElementGroup;
import jp.riken.brain.ni.samuraigraph.figure.SGElementGroupSet;

/**
 * A element group set for multiple data.
 */
public abstract class SGElementGroupSetMultipleInGraph extends
        SGElementGroupSetInGraph implements SGIElementGroupSetMultipleSXY {

    /**
     * The list of element group sets.
     */
    protected ArrayList<SGIElementGroupSetInFigureElement> mElementGroupSetList = new ArrayList<SGIElementGroupSetInFigureElement>();

    /**
     * Adds a group set.
     * 
     * @param gs
     *           the group set to be add
     */
    public void addChildGroupSet(SGIElementGroupSetInFigureElement gs) {
        this.mElementGroupSetList.add(gs);
    }

    /**
     * Removes a group set.
     * 
     * @param gs
     *           the group set to be removed
     */
    public void removeChildGroupSet(SGIElementGroupSetInFigureElement gs) {
        this.mElementGroupSetList.remove(gs);
    }

    /**
     * The constructor of element group set for multiple data.
     * 
     */
    protected SGElementGroupSetMultipleInGraph(SGData data, SGFigureElementGraph graph) {
        super(data, graph);
    }

    /**
     * Disposes this object.
     * 
     */
    public void dispose() {
        super.dispose();
        ArrayList list = this.mElementGroupSetList;
        for (int ii = 0; ii < list.size(); ii++) {
            SGElementGroupSet gs = (SGElementGroupSet) list.get(ii);
            gs.dispose();
        }
        this.mElementGroupSetList.clear();
        this.mElementGroupSetList = null;
    }

    /**
     * 
     */
    public List<Boolean> getVisibleFlagList() {
        if (this.mElementGroupSetList.size() == 0) {
            return new ArrayList<Boolean>();
        }
        SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) this.mElementGroupSetList
                .get(0);
        List<Boolean> list = groupSet.getVisibleFlagList();
        return list;
    }

    /**
     * 
     */
    public void setSelected(final boolean b) {
        this.mSelectedFlag = b;
        ArrayList list = this.mElementGroupSetList;
        for (int ii = list.size() - 1; ii >= 0; ii--) {
            SGISelectable groupSet = (SGISelectable) list.get(ii);
            groupSet.setSelected(b);
        }
    }

    /**
     * 
     * @param b
     * @return
     */
    public void setVisible(final boolean b) {
        super.setVisible(b);
        ArrayList list = this.mElementGroupSetList;
        for (int ii = list.size() - 1; ii >= 0; ii--) {
            SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            groupSet.setVisible(b);
        }
    }

    /**
     * 
     */
    public boolean setName(final String name) {
        super.setName(name);
        ArrayList list = this.mElementGroupSetList;
        for (int ii = list.size() - 1; ii >= 0; ii--) {
            SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            groupSet.setName(name);
        }
        return true;
    }

    /**
     * 
     * @param b
     * @return
     */
    public boolean setVisibleInLegend(final boolean b) {
        super.setVisibleInLegend(b);
        ArrayList list = this.mElementGroupSetList;
        for (int ii = list.size() - 1; ii >= 0; ii--) {
            SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            groupSet.setVisibleInLegend(b);
        }
        return true;
    }

    /**
     * 
     */
    public boolean contains(final int x, final int y) {
        ArrayList list = this.mElementGroupSetList;
        for (int ii = list.size() - 1; ii >= 0; ii--) {
            SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            if (!groupSet.isVisible()) {
            	continue;
            }
            if (groupSet.contains(x, y)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 
     * @param x
     * @param y
     * @return
     */
    public SGElementGroup getElementGroupAt(final int x, final int y) {
        ArrayList list = this.mElementGroupSetList;
        for (int ii = list.size() - 1; ii >= 0; ii--) {
            SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            if (groupSet.getData() == null) {
            	continue;
            }
            SGElementGroup group = groupSet.getElementGroupAt(x, y);
            if (group != null) {
                return group;
            }

        }

        return null;
    }

    /**
     * 
     */
    public boolean setMagnification(final float mag) {
        super.setMagnification(mag);
        for (int ii = 0; ii < this.mElementGroupSetList.size(); ii++) {
            SGElementGroupSet groupSet = (SGElementGroupSet) this.mElementGroupSetList
                    .get(ii);
            groupSet.setMagnification(mag);
        }

        return true;
    }

    /**
     * 
     */
    public boolean onDrawingElement(final int x, final int y) {

        ArrayList list = this.mElementGroupSetList;
        for (int ii = list.size() - 1; ii >= 0; ii--) {
            SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            boolean flag = groupSet.onDrawingElement(x, y);
            if (flag) {
                return true;
            }
        }

        return false;
    }

    /**
     * 
     */
    public void paintGraphics2D(final Graphics2D g2d) {
        ArrayList list = this.mElementGroupSetList;
        for (int ii = list.size() - 1; ii >= 0; ii--) {
            SGElementGroupSetInGraph groupSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            if (groupSet.isVisible()) {
                groupSet.paintGraphics2D(g2d);
            }
        }
    }

    /**
     * 
     */
    public void setClipFlag(final boolean b) {
        super.setClipFlag(b);
        ArrayList list = this.mElementGroupSetList;
        for (int ii = 0; ii < list.size(); ii++) {
            SGElementGroupSetInGraph grouptSet = (SGElementGroupSetInGraph) list
                    .get(ii);
            grouptSet.setClipFlag(b);
        }
    }
    
    /**
     * Returns the first element group set.
     * @return
     *         the first element group set
     */
    public SGIElementGroupSetInFigureElement getFirst() {
        if (this.mElementGroupSetList.size() == 0) {
            return null;
        } else {
            return (SGElementGroupSetInFigureElement) this.mElementGroupSetList.get(0);
        }
    }


    public SGProperties getProperties() {
        MultipleElementGroupSetProperties p = new MultipleElementGroupSetProperties();
        if (this.getProperties(p) == false) {
            return null;
        }
        return p;
    }
    
    public boolean getProperties(SGProperties p) {
        if ((p instanceof MultipleElementGroupSetProperties) == false) {
            return false;
        }
        if (super.getProperties(p) == false) {
            return false;
        }
        MultipleElementGroupSetProperties mp = (MultipleElementGroupSetProperties) p;
//        SGIElementGroupSetInFigureElement first = this.getFirst();
//        if (first == null) {
//            return false;
//        }
//        ElementGroupSetProperties pFirst = (ElementGroupSetProperties) first.getProperties();
//        if (pFirst == null) {
//            return false;
//        }
//        mp.setFirstGroupSetProperties(pFirst);
        for (int ii = 0; ii < this.mElementGroupSetList.size(); ii++) {
            SGIElementGroupSetInFigureElement gs = this.mElementGroupSetList.get(ii);
            mp.mChildPropertyList.add(gs.getProperties());
        }
        return true;
    }

    /**
     * 
     */
    public boolean setProperties(SGProperties p) {
        if ((p instanceof MultipleElementGroupSetProperties) == false) {
            return false;
        }
        if (super.setProperties(p) == false) {
            return false;
        }
        
    	// disposes of old data
    	if (SGMultipleSXYUtility.removeAllChildData(this) == false) {
    	    return false;
    	}

    	// synchronize the child group set
    	if (this.mGraph.updateChildGroupSet(this) == false) {
    	    return false;
    	}

        MultipleElementGroupSetProperties mp = (MultipleElementGroupSetProperties) p;
        int count = 0;
        for (int ii = 0; ii < this.mElementGroupSetList.size(); ii++) {
            SGIElementGroupSetInFigureElement gs = this.mElementGroupSetList.get(ii);
            if (!gs.setProperties(mp.mChildPropertyList.get(count))) {
                return false;
            }
            count++;
            if (count >= mp.mChildPropertyList.size()) {
                count = mp.mChildPropertyList.size()-1;
            }
        }
        return true;
    }

//    /**
//     * Overrode to set the properties to all element groups.
//     * 
//     * @param elementGroupPropertiesList
//     *           properties for element groups
//     * @return true if succeeded
//     */
//    protected boolean setElementGroupProperties(List elementGroupPropertiesList) {
//        
//        for (int ii = 0; ii < elementGroupPropertiesList.size(); ii++) {
//            SGProperties gp = (SGProperties) elementGroupPropertiesList.get(ii);
//            List groupList = null;
//            if (gp instanceof SGElementGroupLine.LineProperties) {
//            	groupList = this.getLineGroups();
//            } else if (gp instanceof SGElementGroupSymbol.SymbolProperties) {
//            	groupList = this.getSymbolGroups();
//            } else if (gp instanceof SGElementGroupBar.BarProperties) {
//            	groupList = this.getBarGroups();
//            } else if (gp instanceof SGElementGroupErrorBar.ErrorBarProperties) {
//            	groupList = this.getErrorBarGroups();
//            } else if (gp instanceof SGElementGroupString.StringProperties) {
//            	groupList = this.getTickLabelGroups();
//            } else if (gp instanceof SGElementGroupArrow.ArrowProperties) {
//            	groupList = this.getArrowGroups();
//            } else if (gp instanceof SGElementGroupColorMap.ColorMapProperties) {
//            	groupList = this.getColorMapGroups();
//            } else if (gp instanceof SGElementGroupGridColorMap.GridColorMapProperties) {
//            	groupList = this.getGridColorMapGroups();
//            } else {
//                throw new Error("Illegal group property: " + gp);
//            }
//            if (groupList == null) {
//                continue;
//            }
//            for (int jj = 0; jj < groupList.size(); jj++) {
//            	SGElementGroup group = (SGElementGroup) groupList.get(jj);
//                if (group.setProperties(gp) == false) {
//                    return false;
//                }
//            }
//        }
//        return true;
//    }

    /**
     * Returns whether this group set "contains" the given group set.
     * 
     * @param gs
     *           the group set
     * @return true if this group set "contains" the given group set
     */
    public boolean contains(SGElementGroupSetInFigureElement gs) {
    	if (super.contains(gs)) {
    		return true;
    	}
    	for (int ii = 0; ii < this.mElementGroupSetList.size(); ii++) {
    		SGIElementGroupSetInFigureElement groupSet = this.mElementGroupSetList.get(ii);
    		if (groupSet.equals(gs)) {
    			return true;
    		}
    	}
    	return false;
    }

    /**
     * Sets the data.
     * 
     * @param data
     *            a data object
     * @return true if succeeded
     */
    public boolean setData(SGData data) {
    	if (super.setData(data) == false) {
    		return false;
    	}
    	if (data == null) {
        	for (int ii = 0; ii < this.mElementGroupSetList.size(); ii++) {
        		SGIElementGroupSetInFigureElement gs = this.mElementGroupSetList.get(ii);
        		
        		// disposes of the data of child group set
        		SGData d = gs.getData();
        		if (d != null) {
            		d.dispose();
        		}
        		
        		// set null
        		gs.setData(null);
        	}
    	}
    	return true;
    }

}
