package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

import jp.riken.brain.ni.samuraigraph.base.SGDrawingElement;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2f;
import jp.riken.brain.ni.samuraigraph.data.SGISXYData;
import jp.riken.brain.ni.samuraigraph.figure.SGIFigureDrawingElementConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGISXYDataConstants;

import org.w3c.dom.Element;

/**
 * @author okumura
 */
public class SGElementGroupSymbolInGraph extends SGElementGroupSymbolForData implements
        SGIElementGroupInGraph, SGISXYDataConstants,
        SGIFigureDrawingElementConstants {

    /**
     * Maximum number of anchors.
     */
    public static final int MAX_NUMBER_OF_ANCHORS = 8;

    private SGFigureElementGraph mGraph = null;

    protected SGElementGroupSetInGraph mGroupSet = null;

    public boolean setElementGroupSet(SGElementGroupSetInGraph gs) {
        this.mGroupSet = gs;
        return true;
    }

    private boolean mFocusedFlag = false;

    public void setFocused(final boolean b) {
        this.mFocusedFlag = b;
    }

    public boolean isFocused() {
        return this.mFocusedFlag;
    }

    /**
     * 
     */
    public SGElementGroupSymbolInGraph(SGISXYData data, SGFigureElementGraph graph) {
        super(data);
        this.mGraph = graph;
    }

//    @Override
//    public boolean setLocation(final SGTuple2f[] pointArray) {
//        SGTuple2f[] shiftedPointArray = pointArray;
//        final float shiftx = this.getShiftXInGraph();
//        final float shifty = this.getShiftYInGraph();
//        if (shiftx!=0.0f || shifty!=0.0f) {
//            shiftedPointArray = new SGTuple2f[pointArray.length];
//            for (int i = 0; i < pointArray.length; i++) {
//                shiftedPointArray[i] = new SGTuple2f(pointArray[i].x+shiftx, pointArray[i].y+shifty);
//            }
//        }
//        
//        return super.setLocation(shiftedPointArray);
//    }
    
//    public boolean setLocation(final SGTuple2f[] pointArray) {
//        // There is nothing to do because each symbol has an SGTuple2f object
//        // that is shared with the group set
//        return true;
//    }

    /**
     * 
     */
    public boolean writeProperty(final Element el) {
        if (super.writeProperty(el) == false) {
            return false;
        }
        el.setAttribute(KEY_VISIBLE, Boolean.toString(this.mVisibleFlag));
        return true;
    }

    /**
     * 
     * @param el
     * @return
     */
    public boolean readProperty(final Element el) {
        if (super.readProperty(el) == false) {
            return false;
        }

        return this.mGraph.readProperty(this, el);
    }

    // /**
    // *
    // */
    // private boolean drawRectangle( final Shape shape, final Graphics2D g2d )
    // {
    //
    // final Rectangle2D rect = shape.getBounds2D();
    //
    // g2d.setPaint(Color.black);
    // g2d.setStroke( new BasicStroke(1) );
    //
    //
    // final int x = (int)rect.getX();
    // final int y = (int)rect.getY();
    // final int w = (int)rect.getWidth();
    // final int h = (int)rect.getHeight();
    //		
    // g2d.drawRect( x,y,w,h );
    //
    // return true;
    //
    // }

    /**
     * 
     */
    public boolean paintElement(final Graphics2D g2d, final Rectangle2D rect) {
        super.paintElement(g2d, rect);

        SGElementGroupSetInGraphSXY groupSet = (SGElementGroupSetInGraphSXY) this.mGroupSet;
        SGDrawingElement[] array = this.mDrawingElementArray;

        if (this.isFocused() && this.mGraph.isSymbolsVisibleAroundFocusedObjects()) {
            if (!groupSet.isBarVisible()) {
                final int num = array.length;
                if (num <= MAX_NUMBER_OF_ANCHORS) {
                    for (int ii = 0; ii < num; ii++) {
                        SGDrawingElementSymbol2D symbol = (SGDrawingElementSymbol2D) array[ii];
                        emphasisSymbol(symbol, g2d);
                    }
                } else {
                    int div = num / MAX_NUMBER_OF_ANCHORS;
                    int cnt = 0;
                    while (true) {
                        SGDrawingElementSymbol2D symbol = (SGDrawingElementSymbol2D) array[cnt];
                        emphasisSymbol(symbol, g2d);
                        cnt += div;
                        if (cnt >= num) {
                            break;
                        }
                    }
                }
            }

        }

        return true;
    }

    /**
     * 
     * @param g2d
     * @param symbol
     * @return
     */
    private boolean emphasisSymbol(final SGDrawingElementSymbol2D symbol,
            final Graphics2D g2d) {
        SGUtilityForFigureElementJava2D.drawAnchorsOnRectangle(symbol
                .getElementBounds(), g2d);
        return true;
    }

    /**
     * Returns whether this group set contains the given point.
     * 
     * @param x
     *          the x coordinate
     * @param y
     *          the y coordinate
     * @return
     *          true if this element group contains the given point
     */
    public boolean contains(final int x, final int y) {
	// if a given point is out of the graph rectangle, returns false;
	Rectangle2D gRect = this.mGraph.getGraphRect();
	if (!gRect.contains(x, y)) {
	    return false;
	}
	return super.contains(x, y);
    }

    /**
     * Returns the location at a given index.
     * 
     * @param index
     *           the index
     * @return the location
     */
    public SGTuple2f getLocation(int index) {
        if (this.mGroupSet instanceof SGElementGroupSetInGraphSXY) {
            SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//            final float shiftx = this.getShiftXInGraph();
//            final float shifty = this.getShiftYInGraph();
            SGTuple2f location = gs.getSymbolLocation(index);
//            return new SGTuple2f(location.x+shiftx, location.y+shifty);
            return new SGTuple2f(location.x, location.y);
        } else {
            throw new Error("Not supported.");
        }
    }

//    protected float getShiftXInGraph() {
//        final double shift = this.getShiftX();
//        if (shift==0.0f) {
//            return 0.0f;
//        }
//        
//        final SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//        return gs.getShiftXLength(shift);
//    }
//
//    protected float getShiftYInGraph() {
//        final double shift = this.getShiftY();
//        if (shift==0.0f) {
//            return 0.0f;
//        }
//        
//        final SGElementGroupSetInGraphSXY gs = (SGElementGroupSetInGraphSXY) this.mGroupSet;
//        return - gs.getShiftYLength(shift);
//    }
    
}
