package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElement;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityNumber;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.figure.SGFigureElement;
import jp.riken.brain.ni.samuraigraph.figure.SGIDataConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGIFigureElementConstants;

/**
 * This class has utility methods for the subclasses of SGFigureElement in this
 * package.
 * 
 */
public class SGUtilityForFigureElementJava2D implements
        SGIFigureElementConstants, SGIDataConstants {

    /**
     * Draw an anchor around a focused object.
     * 
     * @param pos
     *            location of an anchor
     * @param g2d
     *            graphic context
     */
    public static void drawAnchorAsFocusedObject(final Point2D pos,
            final Graphics2D g2d) {
        drawAnchorAsFocusedObject((float) pos.getX(), (float) pos.getY(), g2d);
    }

    /**
     * Draw an anchor around a focused object.
     * 
     * @param pos
     *            location of an anchor
     * @param g2d
     *            graphic context
     */
    public static void drawAnchorAsFocusedObject(final float x, final float y,
            final Graphics2D g2d) {
        SGUtilityJava2D.drawCircle(x, y, ANCHOR_SIZE_FOR_FOCUSED_OBJECTS,
                ANCHOR_EDGE_LINE_WIDTH, ANCHOR_EDGE_LINE_COLOR,
                ANCHOR_INNER_COLOR, g2d);
    }

    /**
     * Draw anchors around focused objects.
     * 
     * @param pointList
     *            list of the location of anchors
     * @param g2d
     *            graphic context
     */
    public static void drawAnchorAsFocusedObject(final List pointList,
            final Graphics2D g2d) {
        for (int ii = 0; ii < pointList.size(); ii++) {
            Point2D pos = (Point2D) pointList.get(ii);
            drawAnchorAsFocusedObject(pos, g2d);
        }
    }
    
    /**
     * Draw anchors around focused objects which are anchored.
     * 
     * @param pointList
     *            list of the location of anchors
     * @param g2d
     *            graphic context
     */
    public static void drawAnchorPointsAsAnchoredForcusObject(
            final List<Point2D> pointList, final Graphics2D g2d) {
        for (int ii = 0; ii < pointList.size(); ii++) {
            Point2D pos = (Point2D) pointList.get(ii);
            SGUtilityJava2D.drawTriangle(pos,
                    ANCHOR_SIZE_FOR_FOCUSED_OBJECTS,
                    ANCHOR_EDGE_LINE_WIDTH,
                    ANCHOR_EDGE_LINE_COLOR,
                    ANCHOR_INNER_COLOR,
                    g2d);
        }
    }

    /**
     * Draw an anchor around child objects.
     * 
     * @param pos
     *            location of an anchor
     * @param g2d
     *            graphic context
     */
    public static void drawAnchorAsChildObject(final Point2D pos,
            final Graphics2D g2d) {
        SGUtilityJava2D.drawSquare(pos, ANCHOR_SIZE_FOR_CHILD_OBJECTS,
                ANCHOR_EDGE_LINE_WIDTH, ANCHOR_EDGE_LINE_COLOR,
                ANCHOR_INNER_COLOR_FOR_CHILD_OBJECTS, g2d);
    }

    /**
     * Draw anchors around child objects.
     * 
     * @param pointList
     *            list of the location of anchors
     * @param g2d
     *            graphic context
     */
    public static void drawAnchorAsChildObject(final List pointList,
            final Graphics2D g2d) {
        for (int ii = 0; ii < pointList.size(); ii++) {
            Point2D pos = (Point2D) pointList.get(ii);
            drawAnchorAsChildObject(pos, g2d);
        }
    }
    
    /**
     * Draw anchors around child objects which are anchored.
     * 
     * @param pointList
     *            list of the location of anchors
     * @param g2d
     *            graphic context
     */
    public static void drawAnchorPointsAsAnchoredChildObject(
            final List<Point2D> pointList, final Graphics2D g2d) {
        for (int ii = 0; ii < pointList.size(); ii++) {
            Point2D pos = (Point2D) pointList.get(ii);
            SGUtilityJava2D.drawTriangle(pos,
                    ANCHOR_SIZE_FOR_FOCUSED_OBJECTS,
                    ANCHOR_EDGE_LINE_WIDTH,
                    ANCHOR_EDGE_LINE_COLOR,
                    ANCHOR_INNER_COLOR_FOR_CHILD_OBJECTS,
                    g2d);
        }
    }

    /**
     * Clip the graph rectangle of figure element.
     * 
     * @param el -
     *            figure element
     * @param g2d -
     *            graphic context to be clipped
     * @return
     */
    public static boolean clipGraphRect(final SGFigureElement el,
            final Graphics2D g2d) {
        Rectangle2D vBounds = el.getViewBounds();
        if (vBounds == null) {
            return false;
        }
        final Rectangle2D rectGraph = el.getGraphRect();
        final Rectangle2D rectCommon = new Rectangle2D.Float();
        Rectangle2D.intersect(rectGraph, vBounds, rectCommon);
        g2d.setClip(rectCommon);
        return true;
    }

    /**
     * 
     * @param g2d
     * @param symbol
     * @return
     */
    public static boolean drawAnchorsOnRectangle(final Rectangle2D rect,
            final Graphics2D g2d) {
        final float x = (float) rect.getX();
        final float y = (float) rect.getY();
        final float w = (float) rect.getWidth();
        final float h = (float) rect.getHeight();
        Point2D pos = new Point2D.Float();
        pos.setLocation(x, y);
        drawAnchorAsFocusedObject(pos, g2d);
        pos.setLocation(x + w, y);
        drawAnchorAsFocusedObject(pos, g2d);
        pos.setLocation(x, y + h);
        drawAnchorAsFocusedObject(pos, g2d);
        pos.setLocation(x + w, y + h);
        drawAnchorAsFocusedObject(pos, g2d);
        return true;
    }
    
    
    static SGPropertyResults checkDataElementGroupVisibility(SGISXYDataDialogObserver obs,
    		SGPropertyMap map, SGPropertyResults iResult) {
    	
        SGPropertyResults result = (SGPropertyResults) iResult.clone();

        Boolean lineVisible = null;
        Boolean symbolVisible = null;
        Boolean barVisible = null;

        Iterator<String> itr = map.getKeyIterator();
        while (itr.hasNext()) {
            String key = itr.next();
            String value = map.getValueString(key);
            if (SGUtilityText.isEqualString(COM_DATA_LINE_VISIBLE, key)) {
                final Boolean b = SGUtilityText.getBoolean(value);
                if (b == null) {
                    result.putResult(COM_DATA_LINE_VISIBLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                lineVisible = b;
            } else if (SGUtilityText.isEqualString(COM_DATA_SYMBOL_VISIBLE, key)) {
                final Boolean b = SGUtilityText.getBoolean(value);
                if (b == null) {
                    result.putResult(COM_DATA_SYMBOL_VISIBLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
            	symbolVisible = b;
            } else if (SGUtilityText.isEqualString(COM_DATA_BAR_VISIBLE, key)) {
                final Boolean b = SGUtilityText.getBoolean(value);
                if (b == null) {
                    result.putResult(COM_DATA_BAR_VISIBLE, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
            	barVisible = b;
            }
        }
        
        // current values
        boolean curLineVisible = obs.isLineVisible();
        boolean curSymbolVisible = obs.isSymbolVisible();
        boolean curBarVisible = obs.isBarVisible();

        // new values
        boolean lineVisibleNew = (lineVisible != null) ? lineVisible.booleanValue() : curLineVisible;
        boolean symbolVisibleNew = (symbolVisible != null) ? symbolVisible.booleanValue() : curSymbolVisible;
        boolean barVisibleNew = (barVisible != null) ? barVisible.booleanValue() : curBarVisible;

        if (!lineVisibleNew && !symbolVisibleNew && !barVisibleNew) {
        	// if all elements are set invisible, returns error status
        	if (lineVisible != null) {
                result.putResult(COM_DATA_LINE_VISIBLE, SGPropertyResults.INVALID_INPUT_VALUE);
        	}
        	if (symbolVisible != null) {
                result.putResult(COM_DATA_SYMBOL_VISIBLE, SGPropertyResults.INVALID_INPUT_VALUE);
        	}
        	if (barVisible != null) {
                result.putResult(COM_DATA_BAR_VISIBLE, SGPropertyResults.INVALID_INPUT_VALUE);
        	}
        } else {
        	if (lineVisible != null) {
                result.putResult(COM_DATA_LINE_VISIBLE, SGPropertyResults.SUCCEEDED);
                obs.setLineVisible(lineVisibleNew);
        	}
        	if (symbolVisible != null) {
                result.putResult(COM_DATA_SYMBOL_VISIBLE, SGPropertyResults.SUCCEEDED);
                obs.setSymbolVisible(symbolVisibleNew);
        	}
        	if (barVisible != null) {
                result.putResult(COM_DATA_BAR_VISIBLE, SGPropertyResults.SUCCEEDED);
                obs.setBarVisible(barVisibleNew);
        	}
        }
        
    	return result;
    }
    
    static double calcSizeValue(final double[] valueArray, SGAxis axis, 
    		final boolean horizontal) {
    	
        final double sizeMinFactor = 0.010;
        final double minValue = axis.getMinValue();
        final double maxValue = axis.getMaxValue();
        final double minWidth = (maxValue - minValue) * sizeMinFactor;
        
    	final double[] values = valueArray.clone();
        Arrays.sort(values);
        double minDiff = Double.MAX_VALUE;
        for (int ii = 0; ii < values.length - 1; ii++) {
            final double diff = Math.abs(values[ii + 1] - values[ii]);
            if (diff < minWidth) {
                continue;
            }
            if (diff < minDiff) {
                minDiff = diff;
            }
        }
        
        final double size;
        if (minDiff == Double.MAX_VALUE) {
            size = 1.0;
        } else {
            minDiff = SGUtilityNumber.getNumberInRangeOrder(minDiff, axis, 
            		SGIFigureElement.AXIS_SCALE_EFFECTIVE_DIGIT, BigDecimal.ROUND_HALF_UP);
            size = minDiff;
        }
        
        return size;
    }

}
