package jp.riken.brain.ni.samuraigraph.data;

import jp.riken.brain.ni.samuraigraph.base.SGDataColumnInfo;

/**
 * A class of data column for netCDF data.
 *
 */
public class SGNetCDFDataColumnInfo extends SGDataColumnInfo {

    /**
     * The variable.
     */
    private SGVariable mVar = null;
    
    /**
     * The origin of data column.
     */
    private int mOrigin = 0;
    
    /**
     * Builds a data column object with a given title and the value type.
     * 
     * @param title
     *            a string to set to the title of this column
     * @param valueType
     *            a value to set to the type of value of this column
     */
    public SGNetCDFDataColumnInfo(final SGVariable var, final String title, final String valueType) {
        super(title, valueType);
        this.mVar = var;
    }

    /**
     * Builds a data column object with a given title, value type and the origin.
     * 
     * @param var
     *            a variable
     * @param title
     *            a string to set to the title of this column
     * @param valueType
     *            a value to set to the type of value of this column
     * @param origin
     *            the origin
     */
    public SGNetCDFDataColumnInfo(final SGVariable var, final String title, final String valueType, 
    		final int origin) {
        this(var, title, valueType);
        this.mOrigin = origin;
    }
    
    /**
     * Sets the origin.
     * 
     * @param origin
     *           the value to set to the origin
     */
    public void setOrigin(final int origin) {
        this.mOrigin = origin;
    }
    
    /**
     * Returns the origin.
     * 
     * @return the origin
     */
    public int getOrigin() {
        return this.mOrigin;
    }

    /**
     * Returns the variable.
     * 
     * @return the variable
     */
    public SGVariable getVariable() {
        return this.mVar;
    }

    /**
     * Returns the name of variable.
     * 
     * @return the name variable
     */
    public String getVariableName() {
        return this.mVar.getName();
    }

    /**
     * Returns a text string for this object.
     * 
     * @return a text string for this object
     */
    protected String paramString() {
        StringBuffer sb = new StringBuffer();
        sb.append(super.paramString());
        sb.append(", origin=");
        sb.append(this.getOrigin());
        return sb.toString();
    }

}
