package jp.riken.brain.ni.samuraigraph.figure.java2d;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import jp.riken.brain.ni.samuraigraph.base.SGAxis;
import jp.riken.brain.ni.samuraigraph.base.SGColorBarModel;
import jp.riken.brain.ni.samuraigraph.base.SGData;
import jp.riken.brain.ni.samuraigraph.base.SGDataAxisInfo;
import jp.riken.brain.ni.samuraigraph.base.SGDrawingElement;
import jp.riken.brain.ni.samuraigraph.base.SGIChildObject;
import jp.riken.brain.ni.samuraigraph.base.SGIColorBar;
import jp.riken.brain.ni.samuraigraph.base.SGIConstants;
import jp.riken.brain.ni.samuraigraph.base.SGIDisposable;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureConstants;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElement;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementAxis;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementAxisBreak;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementForData;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementGraph;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementGrid;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementLegend;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementShape;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementSignificantDifference;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementString;
import jp.riken.brain.ni.samuraigraph.base.SGIFigureElementTimingLine;
import jp.riken.brain.ni.samuraigraph.base.SGIMovable;
import jp.riken.brain.ni.samuraigraph.base.SGISelectable;
import jp.riken.brain.ni.samuraigraph.base.SGIUndoable;
import jp.riken.brain.ni.samuraigraph.base.SGProperties;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyDialog;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyMap;
import jp.riken.brain.ni.samuraigraph.base.SGPropertyResults;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2d;
import jp.riken.brain.ni.samuraigraph.base.SGTuple2f;
import jp.riken.brain.ni.samuraigraph.base.SGUndoManager;
import jp.riken.brain.ni.samuraigraph.base.SGUtility;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityNumber;
import jp.riken.brain.ni.samuraigraph.base.SGUtilityText;
import jp.riken.brain.ni.samuraigraph.base.SGValueRange;
import jp.riken.brain.ni.samuraigraph.data.SGDataTypeConstants;
import jp.riken.brain.ni.samuraigraph.data.SGDataUtility;
import jp.riken.brain.ni.samuraigraph.data.SGISXYData;
import jp.riken.brain.ni.samuraigraph.data.SGISXYZTypeData;
import jp.riken.brain.ni.samuraigraph.data.SGIThreeDimensionalData;
import jp.riken.brain.ni.samuraigraph.data.SGITwoDimensionalData;
import jp.riken.brain.ni.samuraigraph.data.SGIVXYTypeData;
import jp.riken.brain.ni.samuraigraph.figure.SGColorBarDialog;
import jp.riken.brain.ni.samuraigraph.figure.SGColorBarStyleManager;
import jp.riken.brain.ni.samuraigraph.figure.SGDrawingElementLine;
import jp.riken.brain.ni.samuraigraph.figure.SGDrawingElementString;
import jp.riken.brain.ni.samuraigraph.figure.SGIAxisConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGIAxisPanelObserver;
import jp.riken.brain.ni.samuraigraph.figure.SGIColorBarConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGIColorBarDialogObserver;
import jp.riken.brain.ni.samuraigraph.figure.SGIStringConstants;
import jp.riken.brain.ni.samuraigraph.figure.SGStroke;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * A class managing axes.
 */
public class SGFigureElementAxis extends SGFigureElement2D implements
        SGIFigureElementAxis, SGIStringConstants, SGIAxisDialogObserver,
        CaretListener, DocumentListener, ActionListener, KeyListener, 
        SGIAxisConstants, SGIColorBarConstants {

    public static final int AXIS_NORMAL_HORIZONTAL_LOWER = 10;  // horizontal lower
    
    public static final int AXIS_NORMAL_HORIZONTAL_UPPER = 11;  // horizontal upper
    
    public static final int AXIS_NORMAL_PERPENDICULAR_LEFT = 12;  // perpendicular left

    public static final int AXIS_NORMAL_PERPENDICULAR_RIGHT = 13;  // perpendicular right
    
    /**
     * The list of axes.
     */
    private ArrayList<ElementsGroupOfAxis> mElementsGroupList = new ArrayList<ElementsGroupOfAxis>();
    
    /**
     * The z-axis.
     */
    private ElementsGroupColorBar mZAxisElementsGroup = null;
    
    /**
     * The flag whether the color bar is available.
     */
    private boolean mColorBarAvailableFlag = false;

    /**
     * Color of lines.
     */
    private Color mLineColor = DEFAULT_LINE_COLOR;

//    /**
//     * Width of axis lines.
//     */
//    private float mAxisLineWidth = DEFAULT_AXIS_LINE_WIDTH;

//    /**
//     * Width of scale lines.
//     */
//    private float mScaleLineWidth = DEFAULT_TICK_MARK_LINE_WIDTH;

    /**
     * Color of the title and scale strings.
     */
    private Color mAxisFontColor = DEFAULT_FONT_COLOR;

    /**
     * Font name of the title.
     */
    private String mTitleFontName = DEFAULT_FONT_NAME;

    /**
     * Font size of the title.
     */
    private float mTitleFontSize = DEFAULT_FONT_SIZE;

    /**
     * Font style of the title.
     */
    private int mTitleFontStyle = -1;

    /**
     * Font name of scale.
     */
    private String mScaleFontName = DEFAULT_FONT_NAME;

    /**
     * Font size of the scale.
     */
    private float mScaleFontSize = DEFAULT_FONT_SIZE;

    /**
     * Font size of the scale.
     */
    private int mScaleFontStyle = -1;

//    /**
//     * Line width of tick mark
//     */
//    private float mTickMarkWidth = DEFAULT_TICK_MARK_LINE_WIDTH;

    /**
     * Length of tick mark.
     */
    private float mTickMarkLength = DEFAULT_TICK_MARK_LENGTH;
    
    /**
     * Space between an axis line and scale numbers.
     */
    private float mSpaceAxisLineAndNumber;

    /**
     * Space between scale numbers and the title.
     */
    private float mSpaceNumberAndTitle;

    /**
     * A flag whether the frame lines are visible.
     */
    private boolean mFrameLineVisibleFlag = true;

    /**
     * Color of frame lines.
     */
    private Color mFrameLineColor = DEFAULT_FRAME_LINE_COLOR;

//    /**
//     * Line width of frame lines.
//     */
//    private float mFrameLineWidth = DEFAULT_FRAME_LINE_WIDTH;

    /**
     * A drawing element pressed by the mouse such as a scale number or a tick
     * mark.
     */
    private SGDrawingElement mPressedElement = null;

    /**
     * 
     */
    private Point mPressedElementOrigin = null;

    /**
     * Property dialog for axes.
     */
    private SGPropertyDialog mPropertyDialog = null;

    /**
     * Property dialog for the color bar.
     */
    private SGPropertyDialog mColorBarPropertyDialog = null;

    /**
     * 
     */
    private static boolean mNotifyChangeOnDraggingFlag = true;

    /**
     * Used to edit the title.
     */
    private JTextField mTextField = new JTextField();

    // used only on the start-up
    private boolean mStartFlag = true;

    /**
     * The color bar style manager.
     */
    private SGColorBarStyleManager mColorBarStyleManager = new SGColorBarStyleManager();
    
    /**
     * The color bar model.
     */
    private SGColorBarModel mColorBarModel = null;

    /**
     * The line stroke for the axis lines.
     */
    private SGStroke mAxisLineStroke = new SGStroke();

    /**
     * The line stroke for the tick marks.
     */
    private SGStroke mTickMarkStroke = new SGStroke();
    
    /**
     * The line stroke for the frame lines.
     */
    private SGStroke mFrameLineStroke = new SGStroke();

    /**
     * The default constructor.
     */
    public SGFigureElementAxis() {
        super();
        this.initEditField();
        this.init();
    }

    // initialization
    private void init() {

        // lines
        this.setAxisLineWidth(DEFAULT_AXIS_LINE_WIDTH, LINE_WIDTH_UNIT);
        this.setTickMarkWidth(DEFAULT_TICK_MARK_LINE_WIDTH, LINE_WIDTH_UNIT);
        this.setTickMarkLength(DEFAULT_TICK_MARK_LENGTH, AXIS_TICK_LENGTH_UNIT);
        this.setLineColor(DEFAULT_LINE_COLOR);
        this.setFrameVisible(DEFAULT_FRAME_LINE_VISIBLE);
        this.setFrameLineWidth(DEFAULT_FRAME_LINE_WIDTH, LINE_WIDTH_UNIT);
        this.setFrameLineColor(DEFAULT_FRAME_LINE_COLOR);

        // strings
        this.setFontColor(DEFAULT_FONT_COLOR);
        this.setTitleFontName(DEFAULT_FONT_NAME);
        this.setTitleFontSize(DEFAULT_FONT_SIZE, FONT_SIZE_UNIT);
        this.setTitleFontStyle(DEFAULT_FONT_STYLE);
        this.setScaleFontName(DEFAULT_FONT_NAME);
        this.setScaleFontSize(DEFAULT_FONT_SIZE, FONT_SIZE_UNIT);
        this.setScaleFontStyle(DEFAULT_FONT_STYLE);

        // space
        this.setSpaceAxisLineAndNumber(
                SGIFigureConstants.DEFAULT_SPACE_AXIS_LINE_AND_NUMBER,
                SGIFigureConstants.FIGURE_SPACE_UNIT);
        this.setSpaceNumberAndTitle(
                SGIFigureConstants.DEFAULT_SPACE_NUMBER_AND_TITLE,
                SGIFigureConstants.FIGURE_SPACE_UNIT);
        
        // set the color bar
        String colorBarName = SGColorBarStyleManager.COLOR_BAR_NAME_GRADIENT_COLORS;
        this.mColorBarModel = this.mColorBarStyleManager.getColorBarModel(colorBarName);
        
        // creates the X and Y-axes
        this.createXYAxisGroups(
                new SGTuple2d(DEFAULT_X_MIN_VALUE, DEFAULT_X_MAX_VALUE),
                new SGTuple2d(DEFAULT_Y_MIN_VALUE, DEFAULT_Y_MAX_VALUE), 
                DEFAULT_HORIZONTAL_AXIS_TITLE, DEFAULT_PERPENDICULAR_AXIS_TITLE);
        
        // creates the Z-axis
        SGAxis zAxis = this.mColorBarModel.getAxis();
        final ElementsGroupColorBar group = new ElementsGroupColorBar();
        group.setLocationInPlane(AXIS_NORMAL);
        group.mAxis = zAxis;
        group.setAxisVisible(false);
        group.setTitle(DEFAULT_NORMAL_AXIS_TITLE);
        this.mZAxisElementsGroup = group;
        
        // create all drawing elements
        this.createAllDrawingElements();
    }

    // initialization of the text field
    private void initEditField() {
        this.mTextField.setVisible(false);
        this.mTextField.addActionListener(this);
        this.mTextField.addCaretListener(this);
        this.mTextField.getDocument().addDocumentListener(this);
        this.mTextField.addKeyListener(this);
    }

    /**
     * Initialize the history of properties.
     * 
     * @return true if succeeded
     */
    public boolean initPropertiesHistory() {
        if (super.initPropertiesHistory() == false) {
            return false;
        }
        if (this.mZAxisElementsGroup.initPropertiesHistory() == false) {
            return false;
        }
        
        // clear the start up flag on initialization
        this.mStartFlag = false;
        return true;
    }

    /**
     * Dispose all objects in this class.
     */
    public void dispose() {

        super.dispose();

        this.mColorBarModel = null;
        this.mColorBarStyleManager = null;

        // dispose the property dialog
        if (this.mPropertyDialog != null) {
            this.mPropertyDialog.dispose();
            this.mPropertyDialog = null;
        }
        
        this.mEditingStringElement = null;
        this.mFrameLineColor = null;
        this.mLineColor = null;
        this.mPressedElement = null;
        this.mPressedElementOrigin = null;
        this.mPressedPoint = null;
        this.mTemporaryProperties = null;
        this.mTextField = null;
        this.mAxisFontColor = null;
        this.mTitleFontName = null;
        this.mScaleFontName = null;

        for (int ii = 0; ii < this.mElementsGroupList.size(); ii++) {
            ElementsGroupOfAxis group = (ElementsGroupOfAxis) this.mElementsGroupList
                    .get(ii);
            group.dispose();
        }
        this.mElementsGroupList.clear();
        this.mElementsGroupList = null;
        this.mZAxisElementsGroup.dispose();
        this.mZAxisElementsGroup = null;
    }

    /**
     * 
     */
    public void setComponent(JComponent com) {
        super.setComponent(com);
        com.add(this.mTextField);
    }

    /**
     * 
     */
    public void actionPerformed(final ActionEvent e) {
        // String command = e.getActionCommand();
        Object source = e.getSource();

        if (source.equals(this.mTextField)) {
            this.closeTextField();
            return;
        }
    }

    /**
     * Overrode for the text field for axis title.
     * 
     * @return true if a text field is shown
     */
    public boolean closeTextField() {
        this.commitEdit();
        this.hideEditField();
        return true;
    }

    // commit the change of the text in the text field
    private boolean commitEdit() {
        String str = this.mTextField.getText();

        if (!SGUtilityText.isValidString(str)) {
            return false;
        }
        
        String before = this.mEditingStringElement.getString();
        String after = str;
        this.mEditingStringElement.setString(after);

        // update the history
        if (before.equals(after) == false) {
            if (this.mEditingStringElement.equals(this.mZAxisElementsGroup.mTitle)) {
                // if the title of the color bar is changed,
                // set changed the color bar
                this.mZAxisElementsGroup.setChanged(true);
            } else {
                // otherwise, set changed this figure element
                this.setChanged(true);
            }
        }

        this.createAllDrawingElements();

        this.repaint();

        notifyChange();
        this.notifyToRoot();

        return true;
    }

    // clear and hide the text field
    private boolean hideEditField() {
        this.mTextField.setText("");
        this.mTextField.setVisible(false);
        return true;
    }

    /**
     * Returns the font name of the title.
     * 
     * @return font name of the title
     */
    public String getTitleFontName() {
        return this.mTitleFontName;
    }

    /**
     * Returns the font style of the title.
     * 
     * @return font style of the title
     */
    public int getTitleFontStyle() {
        return this.mTitleFontStyle;
    }

    /**
     * Returns the font size of the title.
     * 
     * @return font size of the title
     */
    public float getTitleFontSize() {
        return this.mTitleFontSize;
    }

    /**
     * Returns the font size of the title in given unit.
     * 
     * @param unit
     *            unit for the font size
     * @return font size of the title in given unit
     */
    public float getTitleFontSize(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this.getTitleFontSize(),
                unit);
    }

    /**
     * Returns the font name of the scale.
     * 
     * @return font name of the scale.
     */
    public String getScaleFontName() {
        return this.mScaleFontName;
    }

    /**
     * Returns the font style of the scale.
     * 
     * @return font style of the scale
     */
    public int getScaleFontStyle() {
        return this.mScaleFontStyle;
    }

    /**
     * Returns the font size of the scale.
     * 
     * @return font size of the scale
     */
    public float getScaleFontSize() {
        return this.mScaleFontSize;
    }

    /**
     * Returns the font size of the scale in given unit.
     * 
     * @param unit
     *            unit for the font size
     * @return font size of the scale in given unit
     */
    public float getScaleFontSize(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this.getScaleFontSize(),
                unit);
    }

    /**
     * Returns the color of title and scale strings.
     * 
     * @return color of strings
     */
    public Color getStringColor() {
        return this.mAxisFontColor;
    }

    /**
     * Returns whether the frame line is visible.
     * 
     * @return whether the frame line is visible
     */
    public boolean isFrameLineVisible() {
        return this.mFrameLineVisibleFlag;
    }

    /**
     * Returns width of frame lines.
     * 
     * @return width of frame lines
     */
    public float getFrameLineWidth() {
//        return this.mFrameLineWidth;
        return this.mFrameLineStroke.getLineWidth();
    }

    /**
     * Returns width of frame lines in given unit.
     * 
     * @param unit
     *            unit of length
     * @return width of frame lines in given unit
     */
    public float getFrameLineWidth(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this.getFrameLineWidth(),
                unit);
    }

    /**
     * 
     * @return
     */
    public Color getFrameLineColor() {
        return this.mFrameLineColor;
    }

    /**
     * 
     * @return
     */
    public float getAxisLineWidth() {
//        return this.mAxisLineWidth;
        return this.mAxisLineStroke.getLineWidth();
    }

    /**
     * 
     */
    public float getAxisLineWidth(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this.getAxisLineWidth(),
                unit);
    }

    /**
     * 
     * @return
     */
    public float getTickMarkWidth() {
//        return this.mTickMarkWidth;
        return this.mTickMarkStroke.getLineWidth();
    }

    /**
     * 
     */
    public float getTickMarkWidth(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this.getTickMarkWidth(),
                unit);
    }

    /**
     * 
     * @return
     */
    public float getTickMarkLength() {
        return this.mTickMarkLength;
    }

    /**
     * 
     */
    public float getTickMarkLength(final String unit) {
        return (float) SGUtilityText.convertFromPoint(this.getTickMarkLength(),
                unit);
    }

    /**
     * 
     */
    public Color getLineColor() {
        return this.mLineColor;
    }

    /**
     * Sets the visibility of the frame.
     * 
     * @param b
     *          true to set visible
     * @return true if succeeded
     */
    public boolean setFrameVisible(final boolean b) {
        this.mFrameLineVisibleFlag = b;
        return true;
    }

    /**
     * Sets the tick mark width.
     * 
     * @param lw
     *           the tick mark width
     * @return true if succeeded
     */
    public boolean setTickMarkWidth(final float lw) {
        if (lw < 0.0f) {
            throw new IllegalArgumentException("lw < 0.0f");
        }
        this.mTickMarkStroke.setLineWidth(lw);
        return true;
    }

    /**
     * Sets the tick mark width in a given unit.
     * 
     * @param lw
     *          the tick mark width in a given unit
     * @param unit
     *          the unit of length
     * @return true if succeeded
     */
    public boolean setTickMarkWidth(final float lw, final String unit) {
        final Float lwNew = SGUtility.getLineWidth(lw, unit);
        if (lwNew == null) {
            return false;
        }
        return this.setTickMarkWidth(lwNew);
    }

    /**
     * Sets the tick mark length.
     * 
     * @param len
     *           the tick mark length
     * @return true if succeeded
     */
    public boolean setTickMarkLength(final float len) {
        if (len < 0.0f) {
            throw new IllegalArgumentException("len < 0.0f");
        }
        this.mTickMarkLength = len;
        return true;
    }

    /**
     * Sets the tick mark length in a given unit.
     * 
     * @param len
     *           the tick mark length in a given unit
     * @param unit
     *           the unit of length
     * @return true if succeeded
     */
    public boolean setTickMarkLength(final float len, final String unit) {
        final Float lNew = SGUtility.calcPropertyValue(len, unit, 
                AXIS_TICK_LENGTH_UNIT, AXIS_TICK_LENGTH_MIN, AXIS_TICK_LENGTH_MAX, 
                TICK_MARK_LENGTH_MINIMAL_ORDER);
        if (lNew == null) {
            return false;
        }
        return this.setTickMarkLength(lNew);
    }

    /**
     * Sets the line color.
     * 
     * @param cl
     *          the line color to set
     * @return true if succeeded
     */
    public boolean setLineColor(final Color cl) {
        if (cl == null) {
            throw new IllegalArgumentException("cl == null");
        }
        this.mLineColor = cl;
        return true;
    }

    /**
     * Returns the axis location of the selected tab.
     * 
     * @return
     */
    public int getAxisLocationForSelectedTab() {
        return this.mTempAxisLocation;
    }

    private int mTempAxisLocation = -1;

    /**
     * Show the property dialog.
     * 
     */
    private void showPropertyDialog() {
        // before showing dialog
        this.mCurrentPropertyDialog = this.mPropertyDialog;
        
        // show the dialog
        this.setPropertiesOfSelectedObjects();

        // after hid the dialog
        this.mCurrentPropertyDialog = null;
    }
    
    
    /**
     * Synchronize with other figure element.
     */
    public boolean synchronize(final SGIFigureElement element, String msg) {

        boolean flag = true;
        if (element instanceof SGIFigureElementGraph) {
            SGIFigureElementGraph gElement = (SGIFigureElementGraph) element;
            this.synchronizeColorBar(gElement, msg);
            this.synchronizeDataColumnChange(gElement, msg);
        } else if (element instanceof SGIFigureElementString) {

        } else if (element instanceof SGIFigureElementLegend) {
            SGIFigureElementLegend lElement = (SGIFigureElementLegend) element;
            this.synchronizeColorBar(lElement, msg);
            this.synchronizeDataColumnChange(lElement, msg);
        } else if (element instanceof SGIFigureElementAxis) {

        } else if (element instanceof SGIFigureElementAxisBreak) {

        } else if (element instanceof SGIFigureElementSignificantDifference) {

        } else if (element instanceof SGIFigureElementTimingLine) {

        } else if (element instanceof SGIFigureElementGrid) {

        } else if (element instanceof SGIFigureElementShape) {

        } else {
            flag = element.synchronizeArgument(this, msg);
        }

        return flag;
    }
    
    /**
     * Synchronizes the color bar.
     * 
     * @param element
     *           figure element as an event source
     * @param msg
     *           the message
     */
    private void synchronizeColorBar(final SGIFigureElementForData element, 
            final String msg) {
        if (SGIFigureElement.NOTIFY_DATA_HIDDEN.equals(msg)
                || SGIFigureElement.NOTIFY_CHANGE_ON_UNDO.equals(msg)) {
            // if there are no SXYZ type of data exist, hide the color bar
            List<SGData> dataList = this.getDataList();
            boolean sxyzVisible = false;
            for (int ii = 0; ii < dataList.size(); ii++) {
                SGData data = dataList.get(ii);
                if (data instanceof SGISXYZTypeData) {
                    final boolean b = element.isDataVisible(data);
                    if (b) {
                        sxyzVisible = true;
                        break;
                    }
                }
            }
            this.mColorBarAvailableFlag = sxyzVisible;
        }
    }
    
    /**
     * Synchronizes the element given by the argument.
     * 
     * @param element
     *            An object to be synchronized.
     */
    public boolean synchronizeArgument(final SGIFigureElement element,
            String msg) {
        // this shouldn't happen
        throw new Error();
    }

    /**
     * Sets the magnification.
     * 
     * @param mag
     *           the magnification to set
     * @return true if succeeded
     */
    public boolean setMagnification(final float mag) {
        super.setMagnification(mag);
        List gList = this.getAllAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            group.setMagnification(mag);
        }
        this.mFrameLineStroke.setMagnification(mag);
        this.mAxisLineStroke.setMagnification(mag);
        this.mTickMarkStroke.setMagnification(mag);
        return true;
    }

    /**
     * Called when the mouse is clicked.
     */
    public boolean onMouseClicked(final MouseEvent e) {
        if (this.mZAxisElementsGroup.isAxisVisible()) {
            if (this.mZAxisElementsGroup.onMouseClicked(e)) {
                return true;
            }
        }
        List gList = this.getAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            final boolean flag = group.onMouseClicked(e);
            if (flag) {
                return true;
            }
        }
        if (this.mTextField.isVisible()) {
            this.closeTextField();
        } else {
            this.hideEditField();
        }
        return false;
    }

    /**
     * Called when the mouse is pressed.
     */
    public boolean onMousePressed(final MouseEvent e) {
        if (this.mZAxisElementsGroup.isAxisVisible()) {
            if (this.mZAxisElementsGroup.onMousePressed(e)) {
                this.mPressedPoint = e.getPoint();
                return true;
            }
        }
        List gList = this.getAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            final boolean flag = group.onMousePressed(e);
            if (flag) {
                this.mPressedPoint = e.getPoint();
                return true;
            }
        }
        return false;
    }

    /**
     * Called when the mouse is dragged.
     */
    public boolean onMouseDragged(final MouseEvent e) {
        if (this.mZAxisElementsGroup.isAxisVisible()) {
            if (this.mZAxisElementsGroup.onMouseDragged(e)) {
                return true;
            }
        }
        List gList = this.getAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            final boolean flag = group.onMouseDragged(e);
            if (flag) {
                return true;
            }
        }
        return false;
    }

    /**
     * Called when the mouse is released.
     */
    public boolean onMouseReleased(final MouseEvent e) {

        boolean notifyFlag = false;
        
        if (this.mZAxisElementsGroup.isAxisVisible()) {
            if (this.mZAxisElementsGroup.onMouseReleased(e)) {
                if (this.mZAxisElementsGroup.isChanged()) {
                    notifyFlag = true;
                }
            }
        }
        
        List gList = this.getAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            if (group.onMouseReleased(e)) {
                if (this.isChanged()) {
                    notifyFlag = true;
                }
                break;
            }
        }

        this.initializePressedElement();

        if (notifyFlag) {
            this.notifyChange();
            this.notifyToRoot();
        }
        
        this.repaint();

        return true;
    }

    /**
     * Called when the mouse is on drawing elements.
     * 
     * @param x
     *            x coordinate
     * @param y
     *            y coordinate
     * @return true when the mouse is on something
     */
    public boolean onDrawingElement(final int x, final int y) {
        List gList = this.getAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            if (group.onDrawingElement(x, y)) {
                return true;
            }
        }
        if (this.mZAxisElementsGroup.isAxisVisible()) {
            if (this.mZAxisElementsGroup.onDrawingElement(x, y)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 
     * @return
     */
    public boolean setTemporaryPropertiesOfFocusedObjects() {
        return true;
    }

    /**
     * 
     * @return
     */
    public boolean setChangedFocusedObjects() {
        return true;
    }

    private void initializePressedElement() {
        this.mPressedElement = null;
        this.mPressedElementOrigin = null;
    }

    private SGProperties mTemporaryProperties = null;

    /**
     * 
     * @return
     */
    public boolean prepare() {
        this.mTemporaryProperties = this.getProperties();
        return true;
    }

    // calculate appropriate axis range
    private SGValueRange calcRange(final double min, final double max,
            final double factor) {
        final double aValue = ALTERNATIVE_AXIS_VALUE;
        final double diff = max - min;

        double minNew = min;
        double maxNew = max;
        if (Math.abs(max - min) < Double.MIN_VALUE) {
            minNew = min - aValue;
            maxNew = max + aValue;
        } else {
            final double margin = factor * diff;
            minNew = min - margin;
            maxNew = max + margin;

            minNew = SGUtilityNumber.getNumberInRangeOrder(minNew, min, max,
                    AXIS_SCALE_EFFECTIVE_DIGIT, BigDecimal.ROUND_HALF_UP);
            maxNew = SGUtilityNumber.getNumberInRangeOrder(maxNew, min, max,
                    AXIS_SCALE_EFFECTIVE_DIGIT, BigDecimal.ROUND_HALF_UP);
        }

        SGValueRange range = new SGValueRange(minNew, maxNew);
        return range;
    }
    
    /**
     * Creates a text string for the title.
     * 
     * @param ori
     *           the original text string
     * @param location
     *           the axis location
     * @return a text string for the title
     */
    private String createTitleString(final String ori, final int location) {
        String title = null;
        if (ori == null || "".equals(ori)) {
            switch (location) {
            case SGIFigureElementAxis.AXIS_HORIZONTAL_1:
                title = DEFAULT_BOTTOM_AXIS_TITLE;
                break;
            case SGIFigureElementAxis.AXIS_HORIZONTAL_2:
                title = DEFAULT_TOP_AXIS_TITLE;
                break;
            case SGIFigureElementAxis.AXIS_PERPENDICULAR_1:
                title = DEFAULT_LEFT_AXIS_TITLE;
                break;
            case SGIFigureElementAxis.AXIS_PERPENDICULAR_2:
                title = DEFAULT_RIGHT_AXIS_TITLE;
                break;
            case SGIFigureElementAxis.AXIS_NORMAL:
                title = DEFAULT_Z_AXIS_TITLE;
                break;
            default:
                break;
            }
        } else {
            title = SGUtility.addEscapeChar(ori);
        }
        return title;
    }
    
    /**
     * Creates X and Y-axes.
     */
    private void createXYAxisGroups(final SGTuple2d xRange, final SGTuple2d yRange, 
            final String xTitle, final String yTitle) {
        // bottom
        this.createAxisGroup(xRange, AXIS_HORIZONTAL_1, xTitle, DEFAULT_BOTTOM_AXIS_VISIBLE);
        
        // top
        this.createAxisGroup(xRange, AXIS_HORIZONTAL_2, xTitle, DEFAULT_TOP_AXIS_VISIBLE);

        // left
        this.createAxisGroup(yRange, AXIS_PERPENDICULAR_1, yTitle, DEFAULT_LEFT_AXIS_VISIBLE);

        // right
        this.createAxisGroup(yRange, AXIS_PERPENDICULAR_2, yTitle, DEFAULT_RIGHT_AXIS_VISIBLE);
    }

    /**
     * Creates a single axis group.
     * 
     */
    private ElementsGroupOfAxis createAxisGroup(final SGTuple2d range,
            final int location, final String title, final boolean visible) {
        final SGAxis axis = new SGAxis(range);
        final ElementsGroupOfAxis group = new ElementsGroupOfAxis();
        group.setLocationInPlane(location);
        group.mAxis = axis;
        group.setAxisVisible(visible);
        group.setTitle(title);
        this.mElementsGroupList.add(group);
        return group;
    }

    /**
     * Sets up X and Y-axes.
     */
    private void setupXYAxisGroups(final String xTitle, final String yTitle) {
        ElementsGroupOfAxis groupX1 = this.getAxisGroup(AXIS_HORIZONTAL_1);
        ElementsGroupOfAxis groupX2 = this.getAxisGroup(AXIS_HORIZONTAL_2);
        ElementsGroupOfAxis groupY1 = this.getAxisGroup(AXIS_PERPENDICULAR_1);
        ElementsGroupOfAxis groupY2 = this.getAxisGroup(AXIS_PERPENDICULAR_2);
        
        // set magnification
        final float mag = this.getMagnification();
        groupX1.setMagnification(mag);
        groupX2.setMagnification(mag);
        groupY1.setMagnification(mag);
        groupY2.setMagnification(mag);
        
        // set the title
        groupX1.setTitle(xTitle);
        groupX2.setTitle(xTitle);
        groupY1.setTitle(yTitle);
        groupY2.setTitle(yTitle);

    }

    /**
     * Add data.
     * 
     * @param data
     *            data added
     * @param name
     *            name of data
     * @return true if succeeded
     */
    public boolean addData(final SGData data, final String name) {
        if (super.addData(data, name) == false) {
            return false;
        }
        SGData[] dataArray = new SGData[] { data };
        if (this.setupAxes(dataArray) == false) {
        	return false;
        }
        return true;
    }
    
    private boolean setupAxes(SGData[] dataArray) {
    	
        // setup axes only once at start up
        if (this.mStartFlag) {

        	/*
            // get XY range
            List<Double> xMinList = new ArrayList<Double>();
            List<Double> xMaxList = new ArrayList<Double>();
            List<Double> yMinList = new ArrayList<Double>();
            List<Double> yMaxList = new ArrayList<Double>();
            for (int ii = 0; ii < dataArray.length; ii++) {
                if (!(dataArray[ii] instanceof SGITwoDimensionalData)) {
                    continue;
                }
                SGITwoDimensionalData tData = (SGITwoDimensionalData) dataArray[ii];
                final SGValueRange xBounds = tData.getBoundsX();
                if (xBounds == null) {
                    return false;
                }
                if (xBounds.isMinValid()) {
                    xMinList.add(xBounds.getMinValue());
                }
                if (xBounds.isMaxValid()) {
                    xMaxList.add(xBounds.getMaxValue());
                }
                final SGValueRange yBounds = tData.getBoundsY();
                if (yBounds == null) {
                    return false;
                }
                if (yBounds.isMinValid()) {
                    yMinList.add(yBounds.getMinValue());
                }
                if (yBounds.isMaxValid()) {
                    yMaxList.add(yBounds.getMaxValue());
                }
            }
    		
            // x range
            SGValueRange xRange = this.getRange(xMinList, xMaxList);
            double xMin = xRange.getMinValue();
            double xMax = xRange.getMaxValue();

            // y range
            SGValueRange yRange = this.getRange(yMinList, yMaxList);
            double yMin = yRange.getMinValue();
            double yMax = yRange.getMaxValue();

            boolean allSXY = true;
            boolean allVXY = true;
            for (int ii = 0; ii < dataArray.length; ii++) {
                if (!(dataArray[ii] instanceof SGISXYData)) {
                    allSXY = false;
                }
                if (!(dataArray[ii] instanceof SGIVXYTypeData)) {
                    allVXY = false;
                }
            }
            if (allSXY) {
                xRange = this.calcRange(xMin, xMax, 0.0);
                xMin = xRange.getMinValue();
                xMax = xRange.getMaxValue();

                yRange = this.calcRange(yMin, yMax, 0.050);
                yMin = yRange.getMinValue();
                yMax = yRange.getMaxValue();

            } else if (allVXY) {
                xRange = this.calcRange(xMin, xMax, 0.10);
                yRange = this.calcRange(yMin, yMax, 0.10);

                final double min = Math.min(xRange.getMinValue(), yRange.getMinValue());
                final double max = Math.max(xRange.getMaxValue(), yRange.getMaxValue());
                xMin = min;
                xMax = max;
                yMin = min;
                yMax = max;
            }
            */
        	
            // determine title strings
            String xt = "";
            String yt = "";
            if (dataArray.length > 0 && dataArray[0] instanceof SGITwoDimensionalData) {
            	SGITwoDimensionalData tData0 = (SGITwoDimensionalData) dataArray[0];
            	xt = tData0.getTitleX();
            	if (xt == null) {
            		xt = "";
            	}
            	yt = tData0.getTitleY();
            	if (yt == null) {
            		yt = "";
            	}
            	if (dataArray.length > 1) {
                    for (int ii = 1; ii < dataArray.length; ii++) {
                    	if (dataArray[ii] instanceof SGITwoDimensionalData) {
                        	SGITwoDimensionalData tData = (SGITwoDimensionalData) dataArray[ii];
                        	String xtCur = tData.getTitleX();
                        	if (xtCur == null) {
                        		xtCur = "";
                        	}
                        	if (!xtCur.equals(xt)) {
                        		xt = "";
                        		break;
                        	}
                    	}
                    }
                    for (int ii = 1; ii < dataArray.length; ii++) {
                    	if (dataArray[ii] instanceof SGITwoDimensionalData) {
                        	SGITwoDimensionalData tData = (SGITwoDimensionalData) dataArray[ii];
                        	String ytCur = tData.getTitleY();
                        	if (ytCur == null) {
                        		ytCur = "";
                        	}
                        	if (!ytCur.equals(yt)) {
                        		yt = "";
                        		break;
                        	}
                    	}
                    }
            	}
            }
            final String xTitle = this.createTitleString(xt,
                    SGIFigureElementAxis.AXIS_HORIZONTAL_1);
            final String yTitle = this.createTitleString(yt,
                    SGIFigureElementAxis.AXIS_PERPENDICULAR_1);

            // setup X and Y-axes
            this.setupXYAxisGroups(xTitle, yTitle);

            // for data with tick labels
    		if (dataArray.length == 1 && dataArray[0] instanceof SGISXYData) {
    			SGISXYData sxy = (SGISXYData) dataArray[0];
                if (sxy.isTickLabelAvailable()) {
                    ElementsGroupOfAxis group;
                	if (sxy.isTickLabelHorizontal()) {
                        group = this.getAxisGroup(AXIS_HORIZONTAL_1);
                	} else {
                        group = this.getAxisGroup(AXIS_PERPENDICULAR_1);
                	}
                    group.setNumbersVisible(false);
                    group.setTickMarksVisible(false);
                }
            }

            // creates the map of lists of axis information
            ElementsGroupOfAxis groupHorizontal1 = this.getAxisGroup(AXIS_HORIZONTAL_1);
            ElementsGroupOfAxis groupHorizontal2 = this.getAxisGroup(AXIS_HORIZONTAL_2);
            ElementsGroupOfAxis groupPerpendicular1 = this.getAxisGroup(AXIS_PERPENDICULAR_1);
            ElementsGroupOfAxis groupPerpendicular2 = this.getAxisGroup(AXIS_PERPENDICULAR_2);
        	Map<SGData, List<SGDataAxisInfo>> axisInfoListMap = new HashMap<SGData, List<SGDataAxisInfo>>();
        	for (int ii = 0; ii < dataArray.length; ii++) {
                if (!(dataArray[ii] instanceof SGITwoDimensionalData)) {
                    continue;
                }
                SGITwoDimensionalData tData = (SGITwoDimensionalData) dataArray[ii];
                
                // gets bounds
                SGValueRange xBounds = tData.getBoundsX();
                SGValueRange yBounds = tData.getBoundsY();
                
        		List<SGDataAxisInfo> axisInfoList = new ArrayList<SGDataAxisInfo>();
        		SGDataAxisInfo axisInfoHorizontal = new SGDataAxisInfo(dataArray[ii], groupHorizontal1.mAxis, 
        				xBounds, xTitle, AXIS_HORIZONTAL_1);
        		SGDataAxisInfo axisInfoPerpendicular = new SGDataAxisInfo(dataArray[ii], groupPerpendicular1.mAxis, 
        				yBounds, yTitle, AXIS_PERPENDICULAR_1);
        		axisInfoList.add(axisInfoHorizontal);
        		axisInfoList.add(axisInfoPerpendicular);
        		axisInfoListMap.put(dataArray[ii], axisInfoList);
        	}
        	
    		// fits axis range
        	this.fitAxes(dataArray, axisInfoListMap, new HashMap<SGData, SGISXYAxisShiftEnabled>());
        	
        	// copies the same axis range
        	groupHorizontal2.mAxis.setScale(groupHorizontal1.getRange());
        	groupPerpendicular2.mAxis.setScale(groupPerpendicular1.getRange());

            // creates all drawing elements
            this.createAllDrawingElements();
        }
        
        // if the color bar is not available and SXYZ type of data is added, 
        // setup the color bar
        if (!this.mColorBarAvailableFlag && dataArray.length > 0) {

        	SGData data = dataArray[0];
        	
        	if (data instanceof SGIThreeDimensionalData) {
        		
	            // in case of three-dimensional data
	            SGIThreeDimensionalData tData = (SGIThreeDimensionalData) data;
	            
	            final SGValueRange zBounds = tData.getBoundsZ();
	            if (zBounds == null) {
	                return false;
	            }
	            double zMin, zMax;
	    		if (zBounds.isMinValid()) {
	    			zMin = zBounds.getMinValue();
	    		} else {
	    			zMin = - ALTERNATIVE_AXIS_VALUE;
	    		}
	    		if (zBounds.isMaxValid()) {
	    			zMax = zBounds.getMaxValue();
	    		} else {
	    			zMax = - ALTERNATIVE_AXIS_VALUE;
	    		}
	    		if (zMin == zMax) {
	    			zMin = - ALTERNATIVE_AXIS_VALUE;
	    			zMax = ALTERNATIVE_AXIS_VALUE;
	    		}
	    		SGValueRange zRange = this.calcRange(zMin, zMax, 0.0);
	            
	            String zTitle = null;
	            String dataType = data.getDataType();
	            if (dataType.equals(SGDataTypeConstants.SXYZ_DATA)
	                    || dataType.equals(SGDataTypeConstants.SXYZ_NETCDF_DATA)) {
	                zTitle = this.createTitleString(tData.getTitleZ(), AXIS_NORMAL);
	                
	            } else {
	                throw new Error("Unsupported data type: " + data.getDataType());
	            }
	
	            // setup the z-axis
	            SGColorBarModel model = this.getColorBarModel();
	            model.setValueRange(zRange.getMinValue(), zRange.getMaxValue());
	            this.mZAxisElementsGroup.setTitle(zTitle);
	            this.mZAxisElementsGroup.setMagnification(this.getMagnification());
	
	            // set the default location and size
	            final float gx = this.mGraphRectX;
	            final float gy = this.mGraphRectY;
	            final float gw = this.mGraphRectWidth;
	            final float gh = this.mGraphRectHeight;
	            this.mZAxisElementsGroup.setDirection(SGIColorBarConstants.DIRECTION_HORIZONTAL_LOWER);
	            this.mZAxisElementsGroup.setLocation(gx, gy + 1.20f * gh);
	            this.mZAxisElementsGroup.setSize(gw / this.mMagnification, 
	                    DEFAULT_COLOR_BAR_WIDTH / CM_POINT_RATIO);
	            
	            this.mZAxisElementsGroup.setAxisVisible(true);
	            
	            // create all drawing elements
	            this.createAllDrawingElements();
	
	            // initialize the property history
	            if (this.mZAxisElementsGroup.initPropertiesHistory() == false) {
	                return false;
	            }
	            
	            // set the visibility flag to true
	            this.mColorBarAvailableFlag = true;
        	}
        }

        return true;
    }
    
    private SGValueRange getRange(List<Double> minList, List<Double> maxList) {
		double min, max;
		if (minList.size() != 0) {
			min = SGDataUtility.getMinValue(SGUtilityNumber.toArray(minList));
		} else {
			min = - ALTERNATIVE_AXIS_VALUE;
		}
		if (maxList.size() != 0) {
			max = SGDataUtility.getMaxValue(SGUtilityNumber.toArray(maxList));
		} else {
			max = ALTERNATIVE_AXIS_VALUE;
		}
		if (min >= max) {
			min = - ALTERNATIVE_AXIS_VALUE;
			max = ALTERNATIVE_AXIS_VALUE;
		}
		return new SGValueRange(min, max);
    }

    /**
     * Adds data objects with given name.
     * 
     * @param data
     *            an array of data objects
     * @param name
     *            an array of names of data objects
     * @param infoMap
     *            the information map of data object
     * @return true if succeeded
     */
    @Override
    public boolean addData(final SGData[] data, final String[] name, final Map<String, Object> infoMap) {
    	if (data.length != name.length) {
    		return false;
    	}
    	for (int ii = 0; ii < data.length; ii++) {
    		if (super.addData(data[ii], name[ii]) == false) {
    			return false;
    		}
    	}
    	if (this.setupAxes(data) == false) {
    		return false;
    	}
    	return true;
    }

    /**
     * Add a data object with a set of properties.
     * 
     * @param data
     *             added data.
     * @param name
     *             the name set to the data
     * @param p
     *             properties set to be data.
     * @return
     *             true if succeeded
     */
    public boolean addData(final SGData data, final String name,
            final SGProperties p) {
        if (super.addData(data, name, p) == false) {
            return false;
        }
        if (data instanceof SGIThreeDimensionalData) {
            if (data instanceof SGISXYZTypeData) {
                // set the visibility flag to true
                this.mColorBarAvailableFlag = true;
            }
        }
        return true;
    }
    
    /**
     * 
     */
    public boolean setGraphRect(final float x, final float y,
            final float width, final float height) {

        super.setGraphRect(x, y, width, height);
        
        if (this.mTextField.isVisible()) {
            if (this.closeTextField() == false) {
                return false;
            }
        }

        this.setLocationOfAllDrawingElements();

        return true;
    }

    /**
     * 
     */
    public boolean getMarginAroundGraphRect(final SGTuple2f topAndBottom,
            final SGTuple2f leftAndRight) {

        if (super.getMarginAroundGraphRect(topAndBottom, leftAndRight) == false) {
            return false;
        }

        // groups
        final ElementsGroupOfAxis groupTop = this
                .getAxisGroup(AXIS_HORIZONTAL_2);
        final ElementsGroupOfAxis groupBottom = this
                .getAxisGroup(AXIS_HORIZONTAL_1);
        final ElementsGroupOfAxis groupLeft = this
                .getAxisGroup(AXIS_PERPENDICULAR_1);
        final ElementsGroupOfAxis groupRight = this
                .getAxisGroup(AXIS_PERPENDICULAR_2);
        if (groupTop == null || groupBottom == null || groupLeft == null
                || groupRight == null) {
            return false;
        }

        //
        // final float space =
        // this.getSpaceAxisLineAndNumber()*this.mMagnification;

        // get a bounding box which contains all drawing elements
        Rectangle2D rectAllTop = groupTop.getBoundingBox();
        Rectangle2D rectAllBottom = groupBottom.getBoundingBox();
        Rectangle2D rectAllLeft = groupLeft.getBoundingBox();
        Rectangle2D rectAllRight = groupRight.getBoundingBox();

        Rectangle2D gRect = this.getGraphRect();

        ArrayList rectList = new ArrayList();
        rectList.add(gRect);

        if (rectAllTop != null) {
            rectList.add(rectAllTop);
        }
        if (rectAllBottom != null) {
            rectList.add(rectAllBottom);
        }
        if (rectAllLeft != null) {
            rectList.add(rectAllLeft);
        }
        if (rectAllRight != null) {
            rectList.add(rectAllRight);
        }
        
        if (this.mZAxisElementsGroup.isAxisVisible()) {
            rectList.add(this.mZAxisElementsGroup.getBoundingBox());
        }

        Rectangle2D rectAll = SGUtility.createUnion(rectList);

        final float top = (float) (gRect.getY() - rectAll.getY());
        final float bottom = (float) ((rectAll.getY() + rectAll.getHeight()) - (gRect
                .getY() + gRect.getHeight()));
        final float left = (float) (gRect.getX() - rectAll.getX());
        final float right = (float) ((rectAll.getX() + rectAll.getWidth()) - (gRect
                .getX() + gRect.getWidth()));

        topAndBottom.x += top;
        topAndBottom.y += bottom;
        leftAndRight.x += left;
        leftAndRight.y += right;

        return true;

    }

    /**
     * Sets the font name of the title.
     * 
     * @param name
     *            font name of the title to set
     * @return true if succeeded
     */
    public boolean setTitleFontName(final String name) {
        this.mTitleFontName = name;

        // TODO
        // temporary code
        this.mScaleFontName = name;

        return true;
    }

    /**
     * Set font style of the title.
     * 
     * @param style
     *            font style of the title
     * @return
     */
    public boolean setTitleFontStyle(final int style) {
        if (SGUtilityText.isValidFontStyle(style) == false) {
            return false;
        }
        this.mTitleFontStyle = style;
        // TODO
        // temporary code
        this.mScaleFontStyle = style;

        return true;
    }

    /**
     * Sets the font size of the title.
     * 
     * @param size
     *            font size of the title
     * @return true if succeeded
     */
    public boolean setTitleFontSize(final float size) {
        if (size < 0) {
            throw new IllegalArgumentException("size < 0");
        }
        
        this.mTitleFontSize = size;

        // TODO
        // temporary code
        this.mScaleFontSize = size;

        return true;
    }

    /**
     * Sets the axis line width.
     * 
     * @param lw
     *           the line width to set
     * @return true if succeeded
     */
    public boolean setAxisLineWidth(final float lw) {
        if (lw < 0.0f) {
            throw new IllegalArgumentException("lw < 0.0f");
        }
        this.mAxisLineStroke.setLineWidth(lw);
        return true;
    }

    /**
     * Sets the axis line width in a given unit.
     * 
     * @param lw
     *          the line width to set in a given unit
     * @param unit
     *          the unit of length
     * @return true if succeeded
     */
    public boolean setAxisLineWidth(final float lw, final String unit) {
        final Float lwNew = SGUtility.getLineWidth(lw, unit);
        if (lwNew == null) {
            return false;
        }
        return this.setAxisLineWidth(lwNew);
    }

    /**
     * Sets the font color of axes.
     * 
     * @param color
     *           the font color to set
     * @return true if succeeded
     */
    public boolean setFontColor(final Color color) {
        if (color == null) {
            throw new IllegalArgumentException("color == null");
        }
        this.mAxisFontColor = color;
        return true;
    }

    /**
     * Sets the title font size in a given unit.
     * 
     * @param size
     *           the font size to set in a given unit
     * @param unit
     *           the unit of length
     * @return true if succeeded
     */
    public boolean setTitleFontSize(final float size, final String unit) {
        final Float sNew = SGUtility.getFontSize(size, unit);
        if (sNew == null) {
            return false;
        }
        return this.setTitleFontSize(sNew);
    }

    /**
     * Sets the scale font size in a given unit.
     * 
     * @param size
     *           the font size to set in a given unit
     * @param unit
     *           the unit of length
     * @return true if succeeded
     */
    public boolean setScaleFontSize(final float size, final String unit) {
        final Float sNew = SGUtility.getFontSize(size, unit);
        if (sNew == null) {
            return false;
        }
        return this.setScaleFontSize(sNew);
    }

    /**
     * Sets the scale font size.
     * 
     * @param size
     *           the font size to set
     * @return true if succeeded
     */
    public boolean setScaleFontSize(final float size) {
        if (size < 0) {
            throw new IllegalArgumentException("size < 0");
        }
        this.mScaleFontSize = size;
        return true;
    }

    /**
     * Sets the scale font style.
     * 
     * @param style
     *           the font style to set
     * @return true if succeeded
     */
    public boolean setScaleFontStyle(final int style) {
        if (SGUtilityText.isValidFontStyle(style) == false) {
            return false;
        }
        this.mScaleFontStyle = style;
        return true;
    }

    /**
     * 
     * @return
     */
    public boolean setScaleFontName(final String name) {
        this.mScaleFontName = name;
        return true;
    }

    /**
     * Sets the frame line width.
     * 
     * @param lw
     *          the line width to set
     * @return true if succeeded
     */
    public boolean setFrameLineWidth(final float lw) {
        this.mFrameLineStroke.setLineWidth(lw);
        return true;
    }

    /**
     * Sets the frame line width in a given unit.
     * 
     * @param lw
     *          the line width to set in a given unit
     * @param unit
     *          the unit of length
     * @return true if succeeded
     */
    public boolean setFrameLineWidth(final float lw, final String unit) {
        final Float lwNew = SGUtility.getLineWidth(lw, unit);
        if (lwNew == null) {
            return false;
        }
        return this.setFrameLineWidth(lwNew);
    }

    /**
     * Sets the frame line color.
     * 
     * @param cl
     *          the frame line color to set
     * @return true if succeeded
     */
    public boolean setFrameLineColor(final Color cl) {
        if (cl == null) {
            throw new IllegalArgumentException("cl == null");
        }
        this.mFrameLineColor = cl;
        return true;
    }

    /**
     * 
     * @return
     */
    public boolean setSpaceAxisLineAndNumber(final float space) {
        this.mSpaceAxisLineAndNumber = space;
        return true;
    }

    /**
     * 
     * @return
     */
    public boolean setSpaceAxisLineAndNumber(final float space,
            final String unit) {
        final double conv = SGUtilityText.convert(space, unit,
                SGIFigureConstants.FIGURE_SPACE_UNIT);
        if (conv < SGIFigureConstants.FIGURE_SPACE_TO_SCALE_MIN) {
            return false;
        }
        if (conv > SGIFigureConstants.FIGURE_SPACE_TO_SCALE_MAX) {
            return false;
        }
        return this.setSpaceAxisLineAndNumber((float) SGUtilityText
                .convertToPoint(space, unit));
    }

    /**
     * 
     * @return
     */
    public boolean setSpaceNumberAndTitle(final float space) {
        this.mSpaceNumberAndTitle = space;
        return true;
    }

    /**
     * 
     * @return
     */
    public boolean setSpaceNumberAndTitle(final float space, final String unit) {
        final double conv = SGUtilityText.convert(space, unit,
                SGIFigureConstants.FIGURE_SPACE_UNIT);
        if (conv < SGIFigureConstants.FIGURE_SPACE_TO_SCALE_MIN) {
            return false;
        }
        if (conv > SGIFigureConstants.FIGURE_SPACE_TO_SCALE_MAX) {
            return false;
        }
        return this.setSpaceNumberAndTitle((float) SGUtilityText
                .convertToPoint(space, unit));
    }

    /**
     * 
     */
    public float getSpaceAxisLineAndNumber() {
        return this.mSpaceAxisLineAndNumber;
    }

    /**
     * 
     */
    public float getSpaceNumberAndTitle() {
        return this.mSpaceNumberAndTitle;
    }

    /**
     * Returns the list of focused objects.
     * 
     * @param list
     *           the list of focused objects
     * @return true if succeeded
     */
    public boolean getFocusedObjectsList(List<SGISelectable> list) {
        if (this.mZAxisElementsGroup.isAxisVisible()) {
            if (this.mZAxisElementsGroup.isSelected()) {
                list.add(this.mZAxisElementsGroup);
            }
        }
        return true;
    }

    /**
     * 
     * 
     */
    public boolean hideSelectedObject(SGISelectable s) {
        return true;
    }

    /**
     * 
     * @param loc
     * @param x
     * @param y
     * @return
     */
    public double getValue(final int loc, final int x, final int y) {
        final ElementsGroupOfAxis group = this.getAxisGroup(loc);
        final SGAxis axis = group.mAxis;
        double value = 0.0;
        if (loc == SGIFigureElementAxis.AXIS_HORIZONTAL_1
                || loc == SGIFigureElementAxis.AXIS_HORIZONTAL_2) {
            value = calcValue(x, axis, true);
        } else if (loc == SGIFigureElementAxis.AXIS_PERPENDICULAR_1
                || loc == SGIFigureElementAxis.AXIS_PERPENDICULAR_2) {
            value = calcValue(y, axis, false);
        } else {
            throw new Error();
        }
        value = this.getNumberInRangeOrder(value, axis);

        return value;
    }

//    /**
//     * Add a new axis group.
//     */
//    private ElementsGroupOfAxis addAxisGroup(
//    // final int locInCube,
//            final int loc, final SGTuple2d range, final int scaleType) {
//        final ElementsGroupOfAxis group = new ElementsGroupOfAxis();
//        final SGAxis axis = new SGAxis(range, scaleType);
//        group.mAxis = axis;
//        group.setLocationInPlane(loc);
//        this.mElementsGroupList.add(group);
//
//        return group;
//    }

//    /**
//     * Add a new axis group.
//     */
//    // private ElementsGroupOfAxis addAxisGroup(final int locInCube) {
//    // return this.addAxisGroup(locInCube, new SGTuple2d(0.0, 1.0),
//    // SGAxis.LINEAR_SCALE);
//    // }
//    private ElementsGroupOfAxis addAxisGroup(final int loc) {
//        return this.addAxisGroup(loc, new SGTuple2d(0.0, 1.0),
//                SGAxis.LINEAR_SCALE);
//    }

    /**
     * Paint axes.
     */
    public void paintGraphics(Graphics g, boolean clip) {
        final Graphics2D g2d = (Graphics2D) g;

        // draw frame lines
        if (this.mFrameLineVisibleFlag) {
            this.drawGraphAreaBoundsLines(g2d);
        }

        // draw axes
        List gList = this.getAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            if (group.isAxisVisible() == false) {
                continue;
            }
            group.paintGraphics2D(g2d);
        }
        
        // draw color bar
        if (this.isColorBarAvailable() && this.isColorBarVisible()) {
            this.mZAxisElementsGroup.paintGraphics2D(g2d);
        }

    }

    // paint frame lines
    private boolean drawGraphAreaBoundsLines(final Graphics2D g2d) {
        if (g2d == null) {
            return false;
        }
        g2d.setPaint(this.mFrameLineColor);
        g2d.setStroke(new BasicStroke(this.getFrameLineWidth()));
        g2d.drawRect((int) (this.mGraphRectX), (int) (this.mGraphRectY),
                (int) (this.mGraphRectWidth), (int) (this.mGraphRectHeight));
        return true;
    }

    // draw a bounding box for a string element
    private boolean drawStringBounds(final SGDrawingElementString2D element,
            final Graphics2D g2d) {

        if (element == null || g2d == null) {
            return false;
        }

        final double strRectLineWidth = 1.0;
        final Color strRectLineColor = Color.BLACK;
        g2d.setPaint(strRectLineColor);
        g2d.setStroke(new BasicStroke((float) strRectLineWidth));

        final Rectangle2D rect = element.getElementBounds();

        g2d.drawRect((int) rect.getX(), (int) rect.getY(), (int) rect
                .getWidth(), (int) rect.getHeight());

        return true;

    }

    // draw a string element
    private boolean drawString(final SGDrawingElementString2D element,
            final Graphics2D g2d) {

        if (element == null || g2d == null) {
            return false;
        }

        element.paint(g2d);

        return true;
    }

    // draw a line
    private boolean drawLine(final SGDrawingElementLine2D element,
            final Graphics2D g2d, final BasicStroke stroke) {
        if (element == null || g2d == null) {
            return false;
        }

        g2d.setPaint(element.getColor());
        g2d.setStroke(stroke);

        SGTuple2f start = element.getStart();
        SGTuple2f end = element.getEnd();
        final Line2D line = new Line2D.Float(start.x, start.y, end.x, end.y);
        g2d.draw(line);

        return true;
    }

    /**
     * Returns whether given axis is horizontal.
     * 
     * @param axis
     *            an axis
     * @return true when given axis is horizontal
     */
    public boolean isHorizontal(final SGAxis axis) {
        return this.getHorizontalAxisList().contains(axis);
    }

    /**
     * Returns whether given axis is perpendicular.
     * 
     * @param axis
     *            an axis
     * @return true when given axis is perpendicular
     */
    public boolean isPerpendicular(final SGAxis axis) {
        return this.getPerpendicularAxisList().contains(axis);
    }

    /**
     * Returns whether a given axis is normal.
     * 
     * @param axis
     *           an axis
     * @return true if a given axis is normal
     */
    public boolean isNormal(final SGAxis axis) {
        if (this.mZAxisElementsGroup.mAxis.equals(axis)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Returns a list which contains all axes.
     * 
     * @return a list of all axes
     */
    public ArrayList getAxisList() {
        final ArrayList list = new ArrayList();
        List gList = this.getAllAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            list.add(group.mAxis);
        }
        return list;
    }

    /**
     * Returns a list of horizontal axes.
     * 
     * @return a list of horizontal axes
     */
    public ArrayList getHorizontalAxisList() {
        final ArrayList list = new ArrayList();
        for (int ii = 0; ii < this.mElementsGroupList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) this.mElementsGroupList
                    .get(ii);
            final int loc = group.mLocationInPlane;
            if (loc == AXIS_HORIZONTAL_1 || loc == AXIS_HORIZONTAL_2) {
                list.add(group.mAxis);
            }
        }
        return list;
    }

    /**
     * 
     */
    public ArrayList getPerpendicularAxisList() {
        final ArrayList list = new ArrayList();
        for (int ii = 0; ii < this.mElementsGroupList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) this.mElementsGroupList
                    .get(ii);
            final int loc = group.mLocationInPlane;
            if (loc == AXIS_PERPENDICULAR_1 || loc == AXIS_PERPENDICULAR_2) {
                list.add(group.mAxis);
            }
        }
        return list;
    }

    /**
     * Returns the list of normal axes.
     * 
     * @return the list of normal axes
     */
    public ArrayList getNormalAxisList() {
        ArrayList list = new ArrayList();
        list.add(this.mZAxisElementsGroup.mAxis);
        return list;
    }
    

    /**
     * 
     */
    public SGAxis getAxisInPlane(final int locInPlane) {
        final ElementsGroupOfAxis group = this.getAxisGroup(locInPlane);
        if (group == null) {
            return null;
        }
        return group.mAxis;
    }

    /**
     * Returns an observer of the axis panel of given location.
     * 
     * @param loc
     *            location of the axis in a plane
     * @return the observer
     */
    public SGIAxisPanelObserver getAxisPanelObserverInPlane(final int loc) {
        return this.getAxisGroup(loc);
    }

    /**
     * Returns an axis group of given location.
     * 
     * @param loc
     *            location of the axis in a plane
     * @return the axis group
     */
    private ElementsGroupOfAxis getAxisGroup(final int loc) {
        List gList = this.getAllAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            if (group.getLocation() == loc) {
                return group;
            }
        }
        return null;
    }

    /**
     * Returns a name of axis location in a plane.
     * 
     * @param axis
     * @return
     */
    public String getLocationName(final SGAxis axis) {
        final int loc = this.getLocationInPlane(axis);
        return this.getLocationName(loc);
    }

    /**
     * Returns the location name of a given location ID.
     * 
     * @param id
     *           a location ID
     * @return the location name
     */
    public String getLocationName(final int id) {
        return SGUtility.getLocationName(id);
    }

    /**
     * Returns the location ID of a given location name.
     * 
     * @param name
     *           a location name
     * @return the location ID
     */
    public int getLocationInPlane(final String name) {
        return SGUtility.getLocationInPlane(name);
    }

    /**
     * Returns a text string for the axis location in a plane.
     * 
     * @param axis
     * @return
     */
    public String getLocationStringInPlane(final SGAxis axis) {
        final int loc = this.getLocationInPlane(axis);
        return this.getLocationName(loc);
    }

    /**
     * Returns a location in a plane from the location on the cube and the view
     * direction.
     * 
     * @param axis -
     *            an axis
     * @return The location in the plane such as AXIS_HORIZONTAL_1.
     */
    public int getLocationInPlane(final SGAxis axis) {
        if (axis == null) {
            throw new IllegalArgumentException("axis==null");
        }

        // final int locationInCube = this.getLocationInCube(axis);
        // return this.getLocationInPlane(locationInCube);

        for (int ii = 0; ii < this.mElementsGroupList.size(); ii++) {
            ElementsGroupOfAxis group = (ElementsGroupOfAxis) this.mElementsGroupList
                    .get(ii);
            if (axis.equals(group.mAxis)) {
                return group.mLocationInPlane;
            }
        }

        return -1;
    }

    /**
     * 
     * @param str
     * @return
     */
    public SGAxis getAxis(final String str) {
        // final int loc = this.getLocationInCube(str);
        // final SGAxis axis = this.getAxisInCube(loc);
        final SGAxis axis = this.getAxisGroup(str).mAxis;
        return axis;
    }

    /**
     * 
     * @return
     */
    public String getClassDescription() {
        return "Axes";
    }

    private SGPropertyDialog mCurrentPropertyDialog = null;

    /**
     * 
     * @return
     */
    public ArrayList getPropertyDialogObserverList() {
        ArrayList list = new ArrayList();
        if (this.mPropertyDialog.equals(this.mCurrentPropertyDialog)) {
            list.add(this);
        } else if (this.mZAxisElementsGroup.isAxisVisible()) {
            if (this.mZAxisElementsGroup.isSelected()) {
                list.add(this.mZAxisElementsGroup);
            }
        }
        return list;
    }

    /**
     * Returns a list of child nodes.
     * 
     * @return a list of chid nodes
     */
    public ArrayList getChildNodes() {
        return new ArrayList();
    }

    /**
     * Returns a property dialog.
     * 
     * @return
     *         a property dialog
     */
    public SGPropertyDialog getPropertyDialog() {
//        SGPropertyDialog dg = null;
//        if (this.mPropertyDialog != null) {
//            dg = this.mPropertyDialog;
//        } else {
//            dg = new SGAxisDialog(mDialogOwner, true,
//                    new int[] { SGIFigureElementAxis.AXIS_HORIZONTAL_1,
//                            SGIFigureElementAxis.AXIS_HORIZONTAL_2,
//                            SGIFigureElementAxis.AXIS_PERPENDICULAR_1,
//                            SGIFigureElementAxis.AXIS_PERPENDICULAR_2 });
//            this.mPropertyDialog = dg;
//        }
//        return dg;
        return this.mPropertyDialog;
    }

    // edited string
    private SGDrawingElementString2D mEditingStringElement = null;

    // returns editing now
    private boolean isEditing() {
        return this.mTextField.isVisible();
    }

//    /**
//     * 
//     */
//    private boolean showEditField(SGDrawingElementString2D el) {
//        JTextField tf = this.mTextField;
//        this.mEditingStringElement = el;
//
//        final Rectangle2D sRect = el.getStringRect();
//        final Rectangle2D eRect = el.getElementBounds();
//        final float fontSize = this.getMagnification()
//                * this.getTitleFontSize();
//
//        final int x = (int) (eRect.getX() - tf.getInsets().left);
//        final int y = (int) (eRect.getY() - fontSize / 2.0f);
//        final int w = (int) (sRect.getWidth() + fontSize);
//        final int h = (int) (sRect.getHeight() + fontSize);
//        tf.setLocation(x, y);
//        tf.setSize(w, h);
//
//        Font font = new Font(this.getTitleFontName(), this.getTitleFontStyle(),
//                (int) fontSize);
//        tf.setFont(font);
//        tf.setForeground(this.mStringColor);
//        tf.setText(el.getString());
//
//        // show the text field
//        tf.setVisible(true);
//        tf.requestFocus();
//        tf.setCaretPosition(0);
//
//        return true;
//    }

    /**
     * 
     */
    public boolean commit() {
        SGProperties pTemp = this.mTemporaryProperties;
        SGProperties pPresent = this.getProperties();
        if (pTemp.equals(pPresent) == false) {
            this.setChanged(true);
        }

        this.mTemporaryProperties = null;

        this.createAllDrawingElements();
        this.repaint();

        this.notifyChange();

        return true;
    }

    /**
     * 
     */
    public boolean cancel() {

        if (this.setProperties(this.mTemporaryProperties) == false) {
            return false;
        }

        this.mTemporaryProperties = null;
//        this.mDialog.setVisible(false);

        this.createAllDrawingElements();
        this.repaint();

        this.notifyChange();

        return true;
    }

    /**
     * 
     */
    public boolean preview() {

        if (this.createAllDrawingElements() == false) {
            return false;
        }
        this.repaint();

        this.notifyChange();

        return true;
    }

    /**
     * 
     */
    public boolean setMementoBackward() {
        boolean flag = super.setMementoBackward();
        if (!flag) {
            return false;
        }
        this.createAllDrawingElements();
        this.notifyChangeOnUndo();
        return true;
    }

    /**
     * 
     */
    public boolean setMementoForward() {
        boolean flag = super.setMementoForward();
        if (!flag) {
            return false;
        }
        this.createAllDrawingElements();
        this.notifyChangeOnUndo();
        return true;
    }

    /**
     * 
     */
    private float getMaxLengthOfScaleNumbers(final ElementsGroupOfAxis group) {

        final ArrayList numberList = group.mNumberList;
        double maxLength = Double.MIN_VALUE;
        for (int ii = 0; ii < numberList.size(); ii++) {
            final SGDrawingElementString2D str = (SGDrawingElementString2D) numberList
                    .get(ii);
            final Rectangle2D rect = str.getElementBounds();
            final double width = rect.getWidth();
            // System.out.println(ii+" "+width);
            if (width > maxLength) {
                maxLength = width;
            }
        }

        return (float) maxLength;
    }

    /**
     * 
     */
    private boolean createAllDrawingElements() {
        List gList = this.getAllAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            if (group.createDrawingElements() == false) {
                return false;
            }
        }
        return true;
    }

    /**
     * 
     */
    private boolean setLocationOfAllDrawingElements() {
        List gList = this.getAllAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            if (group.setLocationOfDrawingElements() == false) {
                return false;
            }
        }
        return true;
    }

    /**
     * 
     * @return
     */
    public String getTagName() {
        return SGIFigureElementAxis.TAG_NAME_AXES;
    }

    /**
     * Creates an array of Element objects.
     * 
     * @param document
     *           an Document objects to append elements
     * @return an array of Element objects
     */
    public Element[] createElement(final Document document) {
        // create an Element object
        Element el = this.createThisElement(document);
        if (el == null) {
            return null;
        }

        // each axes
        List gList = this.getAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            final ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            Element elAxis = group.createElement(document);
            if (elAxis == null) {
                return null;
            }
            el.appendChild(elAxis);
        }
        
        // color bar
        Element elColorBar = this.mZAxisElementsGroup.createElement(document);
        if (elColorBar == null) {
        	return null;
        }
        
        return new Element[] { el, elColorBar };
    }

    /**
     * 
     */
    public boolean createDataObject(final Element el, final SGData data, final boolean readDataProperty) {
        if (super.createDataObject(el, data, readDataProperty) == false) {
            return false;
        }
        
        if (data instanceof SGIThreeDimensionalData) {
            if (data instanceof SGISXYZTypeData) {
                this.mColorBarAvailableFlag = true;
            }
        }

        // set false the flag
        this.mStartFlag = false;

        return true;
    }

    /**
     * 
     */
    public boolean writeProperty(final Element el) {
	
        final int digitLineWidth = SGIConstants.LINE_WIDTH_MINIMAL_ORDER - 1;
		final int digitFontSize = FONT_SIZE_MINIMAL_ORDER - 1;
		final int digitTickMarkLength = TICK_MARK_LENGTH_MINIMAL_ORDER - 1;

		final float axisLineWidth = (float) SGUtilityNumber.roundOffNumber(this
				.getAxisLineWidth(), digitLineWidth);
		final float tickMarkWidth = (float) SGUtilityNumber.roundOffNumber(this
				.getTickMarkWidth(), digitLineWidth);
		final float tickMarkLength = (float) SGUtilityNumber.roundOffNumber(
				this.mTickMarkLength * SGIConstants.CM_POINT_RATIO,
				digitTickMarkLength);
		final float fontSize = (float) SGUtilityNumber.roundOffNumber(
				this.mTitleFontSize, digitFontSize);
		final float frameLineWidth = (float) SGUtilityNumber.roundOffNumber(
				this.getFrameLineWidth(), digitLineWidth);
        
        el.setAttribute(KEY_AXIS_LINE_WIDTH, Float.toString(axisLineWidth) + SGIConstants.pt);
        el.setAttribute(KEY_TICK_MARK_WIDTH, Float.toString(tickMarkWidth) + SGIConstants.pt);
        el.setAttribute(KEY_TICK_MARK_LENGTH, Float.toString(tickMarkLength) + SGIConstants.cm);
        el.setAttribute(KEY_LINE_COLOR, SGUtilityText.getColorString(this.mLineColor));
        el.setAttribute(KEY_AXIS_STRING_COLOR, SGUtilityText.getColorString(this.mAxisFontColor));

        // TODO
        // temporary code
        el.setAttribute(KEY_FONT_NAME, this.mTitleFontName);
        el.setAttribute(KEY_FONT_SIZE, Float.toString(fontSize) + SGIConstants.pt);
        el.setAttribute(KEY_FONT_STYLE, SGUtilityText.getFontStyleName(this.mTitleFontStyle));
        
        el.setAttribute(KEY_FRAME_LINE_VISIBLE, Boolean.toString(this.mFrameLineVisibleFlag));
        el.setAttribute(KEY_FRAME_LINE_WIDTH, Float.toString(frameLineWidth) + SGIConstants.pt);
        el.setAttribute(KEY_FRAME_LINE_COLOR, SGUtilityText.getColorString(this.mFrameLineColor));

        return true;
    }

    /**
     * Read properties from the Element object.
     * 
     * @param element
     *            An Element object which has properties.
     * @return
     */
    public boolean readProperty(final Element element) {

//        // add axis groups
//        ElementsGroupOfAxis group = null;
//        group = this.addAxisGroup(SGIFigureElementAxis.AXIS_HORIZONTAL_1);
//        if (group == null) {
//            return false;
//        }
//        group = this.addAxisGroup(SGIFigureElementAxis.AXIS_HORIZONTAL_2);
//        if (group == null) {
//            return false;
//        }
//        group = this.addAxisGroup(SGIFigureElementAxis.AXIS_PERPENDICULAR_1);
//        if (group == null) {
//            return false;
//        }
//        group = this.addAxisGroup(SGIFigureElementAxis.AXIS_PERPENDICULAR_2);
//        if (group == null) {
//            return false;
//        }

        // set common properties
        if (this.setCommonProperties(element) == false) {
            return false;
        }

        // set false the flag
        this.mStartFlag = false;

        NodeList nList = element
                .getElementsByTagName(SGIFigureElementAxis.TAG_NAME_AXIS);
        for (int ii = 0; ii < nList.getLength(); ii++) {
            Node node = nList.item(ii);
            if (node instanceof Element) {
                Element el = (Element) node;

                // location
                String str = el.getAttribute(SGIFigureElementAxis.KEY_POSITION);
                if (str.length() == 0) {
                    return false;
                }
                final int locInPlane = this.getLocationInPlane(str);
                if (locInPlane == -1) {
                    return false;
                }

                // get the axis-group
                ElementsGroupOfAxis group = this.getAxisGroup(locInPlane);
                if (group.readProperties(el) == false) {
                    return false;
                }
                if (group.createDrawingElements() == false) {
                    return false;
                }
            }
        }

        // create the color bar
        Node parentNode = element.getParentNode();
        Element parentElement = (Element) parentNode;
        NodeList cbnList = parentElement
                .getElementsByTagName(SGIColorBarConstants.TAG_NAME_COLOR_BAR);
        if (cbnList.getLength() > 0) {

//            // create an instance
//            this.mZAxisElementsGroup = new ElementsGroupColorBar();
//            this.mZAxisElementsGroup.setLocationInPlane(SGIFigureElementAxis.AXIS_NORMAL);
//            this.mZAxisElementsGroup.mAxis = new SGAxis(new SGTuple2d(0.0, 1.0), SGAxis.LINEAR_SCALE);;
            
            // set properties
            Node node = cbnList.item(0);
            if (node instanceof Element) {
                Element el = (Element) node;
                if (this.mZAxisElementsGroup.readProperties(el) == false) {
                    return false;
                }
            }
        }

        // create all drawing elements
        if (this.createAllDrawingElements() == false) {
        	return false;
        }

        return true;
    }

    /**
     * Set common properties from Element object.
     * 
     * @param element
     *            The Element object which has common axis properties.
     * @return
     */
    private boolean setCommonProperties(final Element element) {

        String str = null;
        Number num = null;
        Color cl = null;
        Boolean b = null;

        // axis line width
        str = element.getAttribute(SGIFigureElementAxis.KEY_AXIS_LINE_WIDTH);
        if (str.length() == 0) {
            return false;
        }
        StringBuffer uAxisLineWidth = new StringBuffer();
        num = SGUtilityText.getNumber(str, uAxisLineWidth);
        if (num == null) {
            return false;
        }
        final float axisLineWidth = num.floatValue();

        // tick mark width
        str = element.getAttribute(SGIFigureElementAxis.KEY_TICK_MARK_WIDTH);
        if (str.length() == 0) {
            return false;
        }
        StringBuffer uTickMarkWidth = new StringBuffer();
        num = SGUtilityText.getNumber(str, uTickMarkWidth);
        if (num == null) {
            return false;
        }
        final float tickMarkWidth = num.floatValue();

        // tick mark length
        str = element.getAttribute(SGIFigureElementAxis.KEY_TICK_MARK_LENGTH);
        if (str.length() == 0) {
            return false;
        }
        StringBuffer uTickMarkLength = new StringBuffer();
        num = SGUtilityText.getNumber(str, uTickMarkLength);
        if (num == null) {
            return false;
        }
        final float tickMarkLength = num.floatValue();

        // line color
        str = element.getAttribute(SGIFigureElementAxis.KEY_LINE_COLOR);
        if (str.length() == 0) {
            return false;
        }
        final Color lineColor = SGUtilityText.parseColor(str);

        //
        // font name
        //

        str = element.getAttribute(KEY_FONT_NAME);
        final String fontName = str;

        // title font name
        str = element.getAttribute(KEY_AXIS_TITLE_FONT_NAME);
        final String titleFontName = str;

        // scale font name
        str = element.getAttribute(KEY_AXIS_SCALE_FONT_NAME);
        final String scaleFontName = str;

        // check whether the same font name is set to title and scale
        boolean sameFontNameFlag;
        if (fontName.length() != 0) {
            sameFontNameFlag = true;
        } else if (titleFontName.length() != 0 && scaleFontName.length() != 0) {
            sameFontNameFlag = false;
        } else {
            return false;
        }

        //
        // font style
        //

        str = element.getAttribute(KEY_FONT_STYLE);
        String fontStyleStr = str;

        // title font style
        str = element.getAttribute(KEY_AXIS_TITLE_FONT_STYLE);
        String titleFontStyleStr = str;

        // scale font style
        str = element.getAttribute(KEY_AXIS_SCALE_FONT_STYLE);
        String scaleFontStyleStr = str;

        // check whether the same font style is set to title and scale
        boolean sameFontStyleFlag;
        if (fontStyleStr.length() != 0) {
            sameFontStyleFlag = true;
        } else if (titleFontStyleStr.length() != 0
                && scaleFontStyleStr.length() != 0) {
            sameFontStyleFlag = false;
        } else {
            return false;
        }

        Integer fontStyle = null;
        Integer titleFontStyle = null;
        Integer scaleFontStyle = null;
        if (sameFontStyleFlag) {
            fontStyle = SGUtilityText.getFontStyle(fontStyleStr);
            if (fontStyle == null) {
                return false;
            }
        } else {
            titleFontStyle = SGUtilityText.getFontStyle(titleFontStyleStr);
            if (titleFontStyle == null) {
                return false;
            }
            scaleFontStyle = SGUtilityText.getFontStyle(scaleFontStyleStr);
            if (scaleFontStyle == null) {
                return false;
            }
        }

        //
        // font size
        //

        str = element.getAttribute(KEY_FONT_SIZE);
        String fontSizeStr = str;

        // title font size
        str = element.getAttribute(KEY_AXIS_TITLE_FONT_SIZE);
        String titleFontSizeStr = str;

        // scale font size
        str = element.getAttribute(KEY_AXIS_SCALE_FONT_SIZE);
        String scaleFontSizeStr = str;

        // check whether the same font size is set to title and scale
        boolean sameFontSizeFlag;
        if (fontSizeStr.length() != 0) {
            sameFontSizeFlag = true;
        } else if (titleFontSizeStr.length() != 0
                && scaleFontSizeStr.length() != 0) {
            sameFontSizeFlag = false;
        } else {
            return false;
        }

        float fontSize = 0.0f;
        float titleFontSize = 0.0f;
        float scaleFontSize = 0.0f;
        StringBuffer uFontSize = new StringBuffer();
        StringBuffer uTitleFontSize = new StringBuffer();
        StringBuffer uScaleFontSize = new StringBuffer();
        if (sameFontSizeFlag) {
            num = SGUtilityText.getNumber(fontSizeStr, uFontSize);
            if (num == null) {
                return false;
            }
            fontSize = num.floatValue();
        } else {
            num = SGUtilityText.getNumber(titleFontSizeStr, uTitleFontSize);
            if (num == null) {
                return false;
            }
            titleFontSize = num.floatValue();

            num = SGUtilityText.getNumber(scaleFontSizeStr, uScaleFontSize);
            if (num == null) {
                return false;
            }
            scaleFontSize = num.floatValue();
        }

        // string color
        str = element.getAttribute(KEY_AXIS_STRING_COLOR);
        if (str.length() == 0) {
            return false;
        }
        cl = SGUtilityText.parseColor(str);
        if (cl == null) {
            return false;
        }
        final Color stringColor = cl;

        // frame line visible
        str = element.getAttribute(SGIFigureElementAxis.KEY_FRAME_LINE_VISIBLE);
        if (str.length() == 0) {
            return false;
        }
        b = SGUtilityText.getBoolean(str);
        if (b == null) {
            return false;
        }
        final boolean frameLineVisible = b.booleanValue();

        // frame line width
        str = element.getAttribute(SGIFigureElementAxis.KEY_FRAME_LINE_WIDTH);
        if (str.length() == 0) {
            return false;
        }
        StringBuffer uFrameLineWidth = new StringBuffer();
        num = SGUtilityText.getNumber(str, uFrameLineWidth);
        if (num == null) {
            return false;
        }
        final float frameLineWidth = num.floatValue();

        // frame line color
        str = element.getAttribute(SGIFigureElementAxis.KEY_FRAME_LINE_COLOR);
        if (str.length() == 0) {
            return false;
        }
        cl = SGUtilityText.parseColor(str);
        if (cl == null) {
            return false;
        }
        final Color frameLineColor = cl;

        //
        // set properties
        //

        if (this.setAxisLineWidth(axisLineWidth, uAxisLineWidth.toString()) == false) {
            return false;
        }
        if (this.setTickMarkWidth(tickMarkWidth, uTickMarkWidth.toString()) == false) {
            return false;
        }
        if (this.setTickMarkLength(tickMarkLength, uTickMarkLength.toString()) == false) {
            return false;
        }
        if (this.setLineColor(lineColor) == false) {
            return false;
        }
        if (this.setFontColor(stringColor) == false) {
            return false;
        }
        if (sameFontNameFlag) {
            if (this.setTitleFontName(fontName) == false) {
                return false;
            }
            if (this.setScaleFontName(fontName) == false) {
                return false;
            }
        } else {
            if (this.setTitleFontName(titleFontName) == false) {
                return false;
            }
            if (this.setScaleFontName(scaleFontName) == false) {
                return false;
            }
        }
        if (sameFontStyleFlag) {
            if (this.setTitleFontStyle(fontStyle.intValue()) == false) {
                return false;
            }
            if (this.setScaleFontStyle(fontStyle.intValue()) == false) {
                return false;
            }
        } else {
            if (this.setTitleFontStyle(titleFontStyle) == false) {
                return false;
            }
            if (this.setScaleFontStyle(scaleFontStyle) == false) {
                return false;
            }
        }
        if (sameFontSizeFlag) {
            if (this.setTitleFontSize(fontSize, uFontSize.toString()) == false) {
                return false;
            }
            if (this.setScaleFontSize(fontSize, uFontSize.toString()) == false) {
                return false;
            }
        } else {
            if (this.setTitleFontSize(titleFontSize, uTitleFontSize.toString()) == false) {
                return false;
            }
            if (this.setScaleFontSize(scaleFontSize, uScaleFontSize.toString()) == false) {
                return false;
            }
        }
        if (this.setFrameVisible(frameLineVisible) == false) {
            return false;
        }
        if (this.setFrameLineWidth(frameLineWidth, uFrameLineWidth.toString()) == false) {
            return false;
        }
        if (this.setFrameLineColor(frameLineColor) == false) {
            return false;
        }

        return true;
    }
    
    //
    private ElementsGroupOfAxis getAxisGroup(final String location) {
        int loc = -1;
        if (SGUtilityText.isEqualString(AXIS_BOTTOM, location)) {
            loc = AXIS_HORIZONTAL_1;
        } else if (SGUtilityText.isEqualString(AXIS_TOP, location)) {
            loc = AXIS_HORIZONTAL_2;
        } else if (SGUtilityText.isEqualString(AXIS_LEFT, location)) {
            loc = AXIS_PERPENDICULAR_1;
        } else if (SGUtilityText.isEqualString(AXIS_RIGHT, location)) {
            loc = AXIS_PERPENDICULAR_2;
        } else if (SGUtilityText.isEqualString(AXIS_COLOR_BAR, location)) {
            loc = AXIS_NORMAL;
        }
        return this.getAxisGroup(loc);
    }


    // axis line
    private static class ElementLineOfAxis extends SGSimpleLine2D {
        ElementLineOfAxis() {
            super();
        }
    }

    // axis title
    private static class ElementStringOfTitle extends
            SGDrawingElementString2DExtended {
        ElementStringOfTitle() {
            super();
        }
    }

    // line for tick marks
    private class ElementLineOfScale extends SGDrawingElementLine2D {
        
        private SGTuple2f mStart = new SGTuple2f();
        
        private SGTuple2f mEnd = new SGTuple2f();
        
        // value in scale
        private double mValue;

        ElementLineOfScale(final SGTuple2f start, final SGTuple2f end) {
            super();
            this.mStart.setValues(start);
            this.mEnd.setValues(end);
        }

        ElementLineOfScale(final ElementLineOfScale el) {
            super();
            this.mValue = el.mValue;
        }

        @Override
        public Color getColor() {
            return SGFigureElementAxis.this.getLineColor();
        }

        @Override
        public SGTuple2f getEnd() {
            return this.mEnd;
        }

        @Override
        public SGTuple2f getStart() {
            return this.mStart;
        }

        @Override
        protected SGStroke getStroke() {
            return SGFigureElementAxis.this.mTickMarkStroke;
        }

        @Override
        public boolean setColor(Color cl) {
            // do nothing
            return true;
        }

        @Override
        public boolean setLineType(int type) {
            // do nothing
            return true;
        }

        @Override
        public boolean setLineWidth(float width) {
            // do nothing
            return true;
        }

        @Override
        public boolean setTermPoints(SGTuple2f start, SGTuple2f end) {
            this.mStart.setValues(start);
            this.mEnd.setValues(end);
            return true;
        }

        @Override
        public float getMagnification() {
            return SGFigureElementAxis.this.getMagnification();
        }

        @Override
        public boolean setMagnification(float mag) {
            // do nothing
            return true;
        }
    }

    // string for scale numbers
    private static class ElementStringOfScale extends
            SGDrawingElementString2DExtended {
        // value in scale
        private double mValue;
        private final int mLocationInPlane;

        ElementStringOfScale(final String str, final String fontName,
                final int fontStyle, final float fontSize,
                final Color fontColor, final float mag, final float angle,
                final int locationInPlane) {
            super(str, fontName, fontStyle, fontSize, fontColor, mag, angle);
            this.mLocationInPlane = locationInPlane;
        }

        ElementStringOfScale(final ElementStringOfScale el) {
            super(el);
            this.mValue = el.mValue;
            this.mLocationInPlane = el.mLocationInPlane;
        }
        
        private float[] movePositionForRotation() {
            float thisx = this.getX();
            float thisy = this.getY();
            float angle = this.getAngle();
            Rectangle2D rect = super.getElementBounds();
            if (thisy > rect.getY()) {
                float diffy = thisy-(float)rect.getY();
                this.setY(thisy+diffy);
            }
            if (angle>90 && angle<=120) {
                float diffx = (float)rect.getWidth();
                this.setX(thisx+diffx/30.0f*(angle-90.0f));
            } else if (angle>120) {
                float diffx = (float)rect.getWidth();
                this.setX(thisx+diffx);
            } else if (angle<0.0 && angle>=-90.0f) {
                float diffx = (float)rect.getWidth();
                this.setX(thisx+diffx/90.0f*(-angle));
            } else if (angle<-90.0f) {
                float diffx = (float)rect.getWidth();
                this.setX(thisx+diffx);
            }
                
            if (this.mLocationInPlane==AXIS_PERPENDICULAR_1 ||
                    mLocationInPlane==AXIS_NORMAL_PERPENDICULAR_LEFT) {
                Rectangle2D replacedRect = super.getElementBounds();
                float diffx = (float)(replacedRect.getX()+replacedRect.getWidth()-thisx-rect.getWidth());
                if (diffx>0) {
                    this.setX(this.getX()-diffx);
                }
            } else if (this.mLocationInPlane==AXIS_PERPENDICULAR_2 ||
                    mLocationInPlane==AXIS_NORMAL_PERPENDICULAR_RIGHT) {
                Rectangle2D replacedRect = super.getElementBounds();
                float diffx = (float)(replacedRect.getX()-thisx);
                if (diffx>0) {
                    this.setX(this.getX()-diffx);
                }
            }
            
            return new float[] { thisx, thisy };
        }
        
        @Override
        public void paint(final Graphics2D g2d) {
            float[] originalPosition = movePositionForRotation();
            
            super.paint(g2d);
            
            this.setX(originalPosition[0]);
            this.setY(originalPosition[1]);
        }
        
        @Override
        public Rectangle2D getElementBounds() {
            float[] originalPosition = movePositionForRotation();
            
            Rectangle2D rect = super.getElementBounds();
            
            this.setX(originalPosition[0]);
            this.setY(originalPosition[1]);
            return rect;
        }
    }

    // a class for single axis
    /**
     * @author okumura
     */
    private class ElementsGroupOfAxis implements ActionListener,
            SGIAxisPanelObserver, SGIDisposable {

        /**
         * The location of axis in the plane.
         */
        private int mLocationInPlane = -1;

        // private int mLocationInCube = -1;

        /**
         * An axis.
         */
        protected SGAxis mAxis = null;

        /**
         * Line element of axis.
         */
        protected ElementLineOfAxis[] mAxisLines = null;

        /**
         * String element of the title.
         */
        protected ElementStringOfTitle mTitle = new ElementStringOfTitle();

        /**
         * 
         */
        protected final ArrayList mNumberList = new ArrayList();

        /**
         * 
         */
        protected final ArrayList mTickMarksList = new ArrayList();

        /**
         * 
         */
        protected boolean mVisibleFlag = true;

        /**
         * 
         */
        // private boolean mAxisLineVisibleFlag = true;
        /**
         * 
         */
        protected boolean mTitleVisibleFlag = true;

        /**
         * 
         */
        protected boolean mNumbersVisibleFlag = true;

        /**
         * 
         */
        protected boolean mTickMarksVisibleFlag = true;

        /**
         * 
         */
        protected boolean mTickMarksInsideFlag = true;

        /**
         * 
         */
        protected double[] mAxisValueArray = null;

        /**
         * 
         */
        protected boolean mAutoCalcFlag = true;

        /**
         * 
         */
        protected double mStepValue = 1.0;

        /**
         * 
         */
        protected double mBaselineValue = 0.0;

        /**
         * 
         */
        protected boolean mScaleNumbersIntegerFlag = false;

        /**
         * 
         */
        protected boolean mExponentFlag = false;

        /**
         * 
         */
        protected int mExponentValue = 0;
        
        /**
         * 
         */
        protected float mNumbersAngle = 0.0f;

        /**
         * 
         */
        protected SGDrawingElementString2DExtended mExponentDrawingElement = null;

        /**
         * 
         */
        protected SGDrawingElement mDraggingElement = null;

        /**
         * The pop-up menu.
         */
        protected JPopupMenu mPopupMenu = null;

        /**
         * Builds an axis.
         */
        protected ElementsGroupOfAxis() {
            super();
            this.mAxisLines = this.createAxisLines();
            this.setNumbersVisible(DEFAULT_NUMBER_VISIBLE);
            this.setNumbersInteger(DEFAULT_NUMBER_INTEGER);
            this.setTickMarksVisible(DEFAULT_SCALE_LINE_VISIBLE);
            this.setTickMarksInside(DEFAULT_SCALE_LINE_INNER);
            this.setNumbersAngle(DEFAULT_NUMBER_ANGLE);
        }
        
        public boolean prepare() {
            return SGFigureElementAxis.this.prepare();
        }

        public boolean commit() {
            return SGFigureElementAxis.this.commit();
        }

        /**
         * Creates axis lines.
         * 
         * @return axis lines
         */
        protected ElementLineOfAxis[] createAxisLines() {
            ElementLineOfAxis[] array = new ElementLineOfAxis[1];
            array[0] = new ElementLineOfAxis();
            return array;
        }

        /**
         * Disposes of this object.
         */
        public void dispose() {
        	this.mDisposed = true;
        	
            this.mAxis = null;
            this.mAxisValueArray = null;

            for (int ii = 0; ii < this.mAxisLines.length; ii++) {
                this.mAxisLines[ii].dispose();
            }
            this.mAxisLines = null;

            for (int ii = 0; ii < this.mNumberList.size(); ii++) {
                SGDrawingElementString el = (SGDrawingElementString) this.mNumberList.get(ii);
                el.dispose();
            }
            this.mNumberList.clear();

            for (int ii = 0; ii < this.mTickMarksList.size(); ii++) {
                SGDrawingElementLine el = (SGDrawingElementLine) this.mTickMarksList.get(ii);
                el.dispose();
            }
            this.mTickMarksList.clear();

            this.mTitle.dispose();
            this.mTitle = null;

            this.mPopupMenu = null;
        }

        // The flag whether this object is already disposed of.
        private boolean mDisposed = false;

        /**
         * Returns whether this object is already disposed of.
         * 
         * @return true if this object is already disposed of
         */
        public boolean isDisposed() {
        	return this.mDisposed;
        }
        
        public SGAxis getAxis() {
            return this.mAxis;
        }

        public boolean setScale(final double minValue, final double maxValue) {
            return this.getAxis().setScale(minValue, maxValue);
        }
        
        /**
         * 
         * @return
         */
        public boolean isAxisVisible() {
            return this.mVisibleFlag;
        }

        /**
         * 
         * @return
         */
        public boolean isTitleVisible() {
            return this.mTitleVisibleFlag;
        }

        /**
         * 
         * @return
         */
        public String getTitleString() {
            return this.mTitle.getString();
        }

        /**
         * 
         * @return
         */
        public double getMinValue() {
            return this.mAxis.getMinValue();
        }

        /**
         * 
         * @return
         */
        public double getMaxValue() {
            return this.mAxis.getMaxValue();
        }
        
        public SGTuple2d getRange() {
        	return this.mAxis.getRange();
        }

        /**
         * 
         * @return
         */
        public boolean isInvertCoordinates() {
            return this.mAxis.isInvertCoordinates();
        }

        /**
         * 
         * @return
         */
        public int getScaleType() {
            return this.mAxis.getScaleType();
        }

        /**
         * 
         * @return
         */
        public boolean isCalculateAutomatically() {
            return this.mAutoCalcFlag;
        }

        /**
         * 
         * @return
         */
        public double getStepValue() {
            return this.mStepValue;
        }

        /**
         * 
         * @return
         */
        public double getBaselineValue() {
            return this.mBaselineValue;
        }

        /**
         * 
         * @return
         */
        public boolean isTickMarksVisible() {
            return this.mTickMarksVisibleFlag;
        }

        /**
         * 
         * @return
         */
        public boolean isTickMarksInside() {
            return this.mTickMarksInsideFlag;
        }

        /**
         * 
         * @return
         */
        public boolean isNumbersVisible() {
            return this.mNumbersVisibleFlag;
        }

        /**
         * 
         * @return
         */
        public boolean isNumbersInteger() {
            return this.mScaleNumbersIntegerFlag;
        }

        /**
         * 
         * @return
         */
        public boolean getExponentFlag() {
            return this.mExponentFlag;
        }

        /**
         * 
         * @return
         */
        public int getExponentValue() {
            return this.mExponentValue;
        }
        
        /**
         * 
         * @return
         */
        @Override
        public float getNumbersAngle() {
            return this.mNumbersAngle;
        }

        /**
         * Sets the visibility of the axis.
         * 
         * @param b
         *          true to set visible
         * @return true if succeeded
         */
        public boolean setAxisVisible(final boolean b) {
            this.mVisibleFlag = b;
            return true;
        }

//        /**
//         * 
//         * @param value
//         * @return
//         */
//        public boolean setAxisVisible(final String value) {
//            Boolean b = SGUtilityText.getBoolean(value);
//            if (b == null) {
//                return false;
//            }
//            return this.setAxisVisible(b.booleanValue());
//        }

        /**
         * Sets the visibility of the title.
         * 
         * @param b
         *          true to set visible
         * @return true if succeeded
         */
        public boolean setTitleVisible(final boolean b) {
            this.mTitleVisibleFlag = b;
            return true;
        }

        /**
         * Sets the title.
         * 
         * @param str
         *           a text string to set to the title
         * @return true if succeeded
         */
        public boolean setTitle(final String str) {
            if (SGUtilityText.isValidString(str) == false) {
                return false;
            }
            this.mTitle.setString(str);
            return true;
        }

        /**
         * Sets whether the axis scale is inverted.
         * 
         * @param b
         *          true to be inverted
         * @return true if succeeded
         */
        public boolean setInvertCoordinates(final boolean b) {
            return this.mAxis.setInvertCoordinates(b);
        }

        /**
         * Sets the scale range and the scale type.
         * 
         * @param minValue
         *           the minimum value to set
         * @param maxValue
         *           the maximum value to set
         * @param scaleType
         *           the scale type to set
         * @return true if succeeded
         */
        public boolean setScale(final Number minValue, final Number maxValue, final Integer scaleType) {
            final double min = (minValue != null) ? minValue.doubleValue() : this.getMinValue();
            final double max = (maxValue != null) ? maxValue.doubleValue() : this.getMaxValue();
            final int type = (scaleType != null) ? scaleType.intValue() : this.getScaleType();
            if (max <= min) {
                return false;
            }
            if (!SGAxis.isValidValue(min, type) || !SGAxis.isValidValue(max, type)) {
                return false;
            }
            if (this.mAxis.setScale(min, max, type) == false) {
            	return false;
            }
            return true;
        }

        /**
         * Sets the scale range with current scale type.
         * 
         * @param minValue
         *           the minimum value to set
         * @param maxValue
         *           the maximum value to set
         * @return true if succeeded
         */
        public boolean setScale(final Number minValue, final Number maxValue) {
            return this.setScale(minValue, maxValue, this.mAxis.getScaleType());
        }

        /**
         * Sets the scale type.
         * 
         * @param type
         *           the scale type to set
         * @return true if succeeded
         */
        public boolean setScaleType(final int type) {
            if (SGAxis.isValidScaleType(type) == false) {
                return false;
            }
            final double min = this.mAxis.getMinValue();
            final double max = this.mAxis.getMaxValue();
            if (!SGAxis.isValidValue(min, type) || !SGAxis.isValidValue(max, type)) {
                return false;
            }
            return this.mAxis.setScale(min, max, type);
        }
        
        /**
         * Sets the flag whether the tick marks are calculated automatically.
         * @param b
         *          true to calculate automatically
         * @return true if succeeded
         */
        public boolean setCalculateAutomatically(final boolean b) {
            this.mAutoCalcFlag = b;
            return true;
        }

        /**
         * Sets the step value.
         * 
         * @param value
         *           the step value to set
         * @return true if succeeded
         */
        public boolean setStepValue(final double value) {
            if (value == 0.0) {
                return false;
            }
            this.mStepValue = value;
            return true;
        }

        /**
         * Sets the baseline value.
         * 
         * @param value
         *           the baseline value to set
         * @return true if succeeded
         */
        public boolean setBaselineValue(final double value) {
            this.mBaselineValue = value;
            return true;
        }

        /**
         * Sets the visibility of the tick marks.
         * 
         * @param b
         *          true to set visible
         * @return true if succeeded
         */
        public boolean setTickMarksVisible(final boolean b) {
            this.mTickMarksVisibleFlag = b;
            return true;
        }

        /**
         * Sets the direction of tick marks.
         * 
         * @param b
         *          true for inner direction
         * @return true if succeeded
         */
        public boolean setTickMarksInside(final boolean b) {
            this.mTickMarksInsideFlag = b;
            return true;
        }

        /**
         * Sets the visibility of the numbers.
         * 
         * @param b
         *          true to set visible
         * @return true if succeeded
         */
        public boolean setNumbersVisible(final boolean b) {
            this.mNumbersVisibleFlag = b;
            return true;
        }

        /**
         * Sets the numbers to be integer.
         * 
         * @param b
         *          true fot integer
         * @return true if succeeded
         */
        public boolean setNumbersInteger(final boolean b) {
            this.mScaleNumbersIntegerFlag = b;
            return true;
        }

        /**
         * Sets whether to use the exponent symbol.
         * 
         * @param b
         *          true to use the exponent symbol
         * @return true if succeeded
         */
        public boolean setExponentFlag(final boolean b) {
            this.mExponentFlag = b;
            return true;
        }

        /**
         * Sets the exponent value.
         * 
         * @param value
         *           the exponent value to set
         * @return true if succeeded
         */
        public boolean setExponentValue(final int value) {
            final int vNew;
            if (value < AXIS_EXPONENT_MIN) {
                return false;
            } else if (value > AXIS_EXPONENT_MAX) {
                return false;
            } else {
                vNew = value;
            }
            this.mExponentValue = vNew;
            return true;
        }
        
        @Override
        public boolean setNumbersAngle(final float angle) {
            final Float aNew = SGUtility.calcPropertyValue(angle, null, null, 
                    AXIS_NUMBER_ANGLE_MIN, AXIS_NUMBER_ANGLE_MAX,
                    AXIS_NUMBER_ANGLE_MINIMAL_ORDER);
            if (aNew == null) {
                return false;
            }
            this.mNumbersAngle = aNew.floatValue();
            return true;
        }

        /**
         * 
         * @return
         */
        public boolean hasValidAxisValues(final Number minValue,
                final Number maxValue, final Integer scaleType,
                final Number baseValue, final Number stepValue) {
            final double min = (minValue != null) ? minValue.doubleValue()
                    : this.getMinValue();
            final double max = (maxValue != null) ? maxValue.doubleValue()
                    : this.getMaxValue();
            final int type = (scaleType != null) ? scaleType.intValue() : this
                    .getScaleType();
            // final double base = (baseValue!=null) ? baseValue.doubleValue() :
            // this.getBaselineValue();
            final double step = (stepValue != null) ? stepValue.doubleValue()
                    : this.getStepValue();

            // min and max values
            if (min >= max) {
                return false;
            }

            // scale type
            if (type == SGAxis.LOG_SCALE) {
                if (min <= 0.0) {
                    return false;
                }
            }

            // step value
            if (step == 0.0) {
                return false;
            }

            return true;
        }

        /**
         * 
         * @param minValue
         * @param maxValue
         * @param scaleType
         * @return
         */
        public boolean hasValidAxisRange(final Number minValue,
                final Number maxValue, final Integer scaleType) {
            final int type = (scaleType != null) ? scaleType.intValue() : this
                    .getScaleType();
            final double min = (minValue != null) ? minValue.doubleValue()
                    : this.getMinValue();
            final double max = (maxValue != null) ? maxValue.doubleValue()
                    : this.getMaxValue();

            if (min >= max) {
                return false;
            }

            if (type == SGAxis.LOG_SCALE) {
                if (min <= 0.0) {
                    return false;
                }
            }

            return true;
        }

        /**
         * 
         * @param baseValue
         * @param stepValue
         * @param scaleType
         * @return
         */
        public boolean hasValidAxisValues(final Number baseValue,
                final Number stepValue, final Integer scaleType) {
            // final int type = (scaleType!=null) ? scaleType.intValue() :
            // this.getScaleType();
            // final double base = (baseValue!=null) ? baseValue.doubleValue() :
            // this.getBaselineValue();
            final double step = (stepValue != null) ? stepValue.doubleValue()
                    : this.getStepValue();

            if (step == 0.0) {
                return false;
            }

            return true;
        }

        /**
         * Returns a pop-up menu.
         * @return
         *         a pop-up menu
         */
        public JPopupMenu getPopupMenu() {
            JPopupMenu p = null;
            if (this.mPopupMenu != null) {
                p = this.mPopupMenu;
            } else {
                p = this.createPopupMenu();
                this.mPopupMenu = p;
            }
            
            JCheckBoxMenuItem item;
            Component[] array = p.getComponents();
            for (int ii = 0; ii < array.length; ii++) {
                if (array[ii] instanceof JCheckBoxMenuItem) {
                    item = (JCheckBoxMenuItem) array[ii];
                    String com = item.getActionCommand();
                    if (MENUCMD_DRAW_LATER.equals(com)) {
                        item.setSelected(!SGFigureElementAxis.mNotifyChangeOnDraggingFlag);
                    } else if (MENUCMD_SHOW_AXIS.equals(com)) {
                        item.setSelected(this.isAxisVisible());
                    }
                }
            }
            
            return p;
        }
        
        /**
         * Create a pop-up menu.
         * @return
         *         a pop-up menu
         */
        protected JPopupMenu createPopupMenu() {
            JPopupMenu p = new JPopupMenu();
            
            p.setBounds(0, 0, 100, 100);

            StringBuffer sb = new StringBuffer();
            sb.append("  -- Axis: ");
            sb.append(getLocationName(this.mAxis));
            sb.append(" --");
            
            p.add(new JLabel(sb.toString()));
            p.addSeparator();

            SGUtility.addCheckBoxItem(p, this, MENUCMD_SHOW_AXIS);

            p.addSeparator();

            SGUtility.addCheckBoxItem(p, this, MENUCMD_DRAW_LATER);

            p.addSeparator();

            SGUtility.addItem(p, this, MENUCMD_PROPERTY);

            return p;
        }

        /**
         * 
         */
        public void actionPerformed(ActionEvent e) {
            // Object source = e.getSource();
            String command = e.getActionCommand();

            if (command.equals(MENUCMD_SHOW_AXIS)) {
                this.setAxisVisible(!this.isAxisVisible());
                repaint();
                setChanged(true);
                notifyToRoot();
                return;
            } else if (command.equals(MENUCMD_PROPERTY)) {
                this.showPropertyDialog();
            } else if (command.equals(MENUCMD_DRAW_LATER)) {
                mNotifyChangeOnDraggingFlag = !mNotifyChangeOnDraggingFlag;
            }

        }

        /**
         * Shows the property dialog.
         * 
         */
        protected void showPropertyDialog() {

            // before showing dialog
            SGFigureElementAxis.this.mTempAxisLocation = this.getLocationInPlane();
            
            // show the dialog
            SGFigureElementAxis.this.showPropertyDialog();
            
            // after hid the dialog
            SGFigureElementAxis.this.mTempAxisLocation = -1;
        }
        
        /**
         * Shows the popup menu.
         * 
         * @param x
         *          the x-coordinate
         * @param y
         *          the y-coordinate
         */
        protected void showPopupMenu(final int x, final int y) {
            SGFigureElementAxis.this.showPopupMenu(this.getPopupMenu(), x, y);
        }

        /**
         * Draw all objects.
         * 
         * @param g2d
         *           the graphic context
         */
        public void paintGraphics2D(final Graphics2D g2d) {

            if (this.isAxisVisible()) {
                
                // axis line
                this.drawAxisLine(g2d);

                // title
                if (this.isTitleVisible()) {
                    this.drawTitle(g2d);
                }

                // scale numbers
                if (this.isNumbersVisible()) {
                    this.drawScaleNumbers(g2d);
                }

                // scale lines
                if (this.isTickMarksVisible()) {
                    this.drawTickMarks(g2d);
                }
            }
        }
        
        protected void drawAxisLine(final Graphics2D g2d) {
            for (int ii = 0; ii < this.mAxisLines.length; ii++) {
                ElementLineOfAxis line = this.mAxisLines[ii];
                final float width = SGFigureElementAxis.this.mMagnification
                        * line.getLineWidth();
                BasicStroke stroke = new BasicStroke(width,
                        BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER);
                drawLine(line, g2d, stroke);
            }
        }
        
        protected void drawTitle(final Graphics2D g2d) {
            drawString(this.mTitle, g2d);
            if (this.mTitle
                    .equals(SGFigureElementAxis.this.mPressedElement)) {
                drawStringBounds(this.mTitle, g2d);
            }
        }
        
        protected void drawScaleNumbers(final Graphics2D g2d) {
            final ArrayList nList = this.mNumberList;
            for (int jj = 0; jj < nList.size(); jj++) {
                final ElementStringOfScale el = (ElementStringOfScale) nList
                        .get(jj);
                drawString(el, g2d);
            }

            // draw temporary number
            if (this.mDraggingElement != null) {
                if (this.mDraggingElement instanceof ElementStringOfScale) {
                    ElementStringOfScale el = (ElementStringOfScale) this.mDraggingElement;
                    drawString(el, g2d);
                    drawStringBounds(el, g2d);
                }
            }

            // exponent
            if (this.getExponentFlag()) {
                drawString(this.mExponentDrawingElement, g2d);
            }
        }
        
        protected void drawTickMarks(final Graphics2D g2d) {
            final ArrayList lList = this.mTickMarksList;
            final boolean lineDragFlag = (this.mDraggingElement != null)
                    && (this.mDraggingElement instanceof ElementLineOfScale);
//            final float width = SGFigureElementAxis.this.mMagnification
//                    * SGFigureElementAxis.this.mScaleLineWidth;
            for (int ii = 0; ii < lList.size(); ii++) {
                final ElementLineOfScale el = (ElementLineOfScale) lList
                        .get(ii);
                final float width = mMagnification * el.getLineWidth();
                final float width_ = lineDragFlag ? width * 2.0f : width;
                BasicStroke stroke = new BasicStroke(width_,
                        BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
                if (lineDragFlag) {
                    el.setLineWidth(width_);
                }
                drawLine(el, g2d, stroke);
                if (lineDragFlag) {
                    el.setLineWidth(width);
                }
            }
        }

        /**
         * 
         */
        private double getTitleHeight() {
            Rectangle2D rect = this.mTitle.getStringRect();
            return rect.getHeight();
        }

        /**
         * 
         */
        protected double getScaleHeight() {
            double height = 0.0;
            if (this.mNumberList.size() == 0) {
                final Font font = new Font(
                        SGFigureElementAxis.this.mScaleFontName,
                        SGFigureElementAxis.this.mScaleFontStyle,
                        (int) (SGFigureElementAxis.this.mMagnification * SGFigureElementAxis.this.mScaleFontSize));
                
                // create font render context
                final FontRenderContext frc = new FontRenderContext(null, false, false);

                // create text layout
                final TextLayout layout = new TextLayout("0", font, frc);

                // get a visual bounds rectangle from Font object
                final Rectangle2D rect = layout.getOutline(new AffineTransform()).getBounds2D();
                height = rect.getHeight();
                
            } else {
                SGDrawingElementString2D el = (SGDrawingElementString2D) this.mNumberList
                        .get(0);
                Rectangle2D rect = el.getStringRect();
                height = rect.getHeight();
            }
            return height;
        }

        protected int getLocationInPlane() {
            return this.mLocationInPlane;
        }

        protected boolean setLocationInPlane(final int locationInPlane) {
            this.mLocationInPlane = locationInPlane;
            return true;
        }
        
        protected int getLocation() {
        	return this.getLocationInPlane();
        }

        /**
         * Sets the magnification.
         * 
         * @param mag
         *           the magnification
         * @return true if succeeded
         */
        public boolean setMagnification(final float mag) {
//            for (int ii = 0; ii < this.mAxisLines.length; ii++) {
//                this.mAxisLines[ii].setMagnification(mag);
//            }

            this.mTitle.setMagnification(mag);

            ArrayList nList = this.mNumberList;
            for (int ii = 0; ii < nList.size(); ii++) {
                ElementStringOfScale el = (ElementStringOfScale) nList.get(ii);
                el.setMagnification(mag);
            }

//            ArrayList tList = this.mTickMarksList;
//            for (int ii = 0; ii < tList.size(); ii++) {
//                ElementLineOfScale el = (ElementLineOfScale) tList.get(ii);
//                el.setMagnification(mag);
//            }

            if (this.mExponentDrawingElement != null) {
                this.mExponentDrawingElement.setMagnification(mag);
            }

            return true;
        }

        /**
         * 
         * @return
         */
        protected Rectangle2D getBoundingBox() {
            if (this.isAxisVisible() == false) {
                return null;
            }

            Rectangle2D rect = new Rectangle2D.Float();

            ArrayList list = new ArrayList();

            for (int ii = 0; ii < this.mAxisLines.length; ii++) {
                list.add(this.mAxisLines[ii]);
            }

            if (this.isTitleVisible()) {
                list.add(this.mTitle);
            }

            if (this.isNumbersVisible()) {
                ArrayList nList = this.mNumberList;
                for (int ii = 0; ii < nList.size(); ii++) {
                    ElementStringOfScale el = (ElementStringOfScale) nList
                            .get(ii);
                    if (el.isVisible()) {
                        list.add(el);
                    }
                }

                if (this.getExponentFlag()) {
                    list.add(this.mExponentDrawingElement);
                }
            }

            if (list.size() != 0) {
                rect.setRect(SGUtilityJava2D.getBoundingBox(list));
            }

            return rect;
        }

        /**
         * 
         */
        public boolean onMouseClicked(final MouseEvent e) {
            int x = e.getX();
            int y = e.getY();

            boolean flag;

            // check axis line even when axis is invisible
            for (int ii = 0; ii < this.mAxisLines.length; ii++) {
                flag = this.mAxisLines[ii].contains(x, y);
                if (flag) {
                    return this.clicked(e);
                }
            }

            // return if axis is invisible
            if (!this.isAxisVisible()) {
                return false;
            }

            // title
            if (this.isTitleVisible()) {
                flag = this.mTitle.contains(x, y);
                if (flag) {
                    if (SGFigureElementAxis.this.isEditing()) {
                        SGFigureElementAxis.this.closeTextField();
                    }

                    if ((SwingUtilities.isLeftMouseButton(e))
                            && (e.getClickCount() == 1)) {
                        // if( SGAxisElement.this.isEditing() )
                        // {
                        // SGAxisElement.this.terminateEditField();
                        // }
                        // else
                        {
//                            SGFigureElementAxis.this.showEditField(this.mTitle);
                            SGFigureElementAxis.this.mEditingStringElement = this.mTitle;
                            
                            final int tx = mPressedPoint.x - (int) this.mTitle.getX();
                            final int ty = mPressedPoint.y - (int) this.mTitle.getY();                            
                            SGFigureElementAxis.this.showEditField(mTextField, this.mTitle, tx, ty);
                        }

                        return true;
                    }
                    return this.clicked(e);
                }
            }

            // numbers of axis
            if (this.isNumbersVisible()) {
                ArrayList list = this.mNumberList;
                for (int ii = 0; ii < list.size(); ii++) {
                    ElementStringOfScale el = (ElementStringOfScale) list
                            .get(ii);
                    flag = el.contains(x, y);
                    if (flag) {
                        return this.clicked(e);
                    }
                }
            }

            // exponent
            if (this.mExponentDrawingElement != null) {
                if (this.mExponentDrawingElement.isVisible()) {
                    flag = this.mExponentDrawingElement.contains(x, y);
                    if (flag) {
                        return this.clicked(e);
                    }
                }
            }

            // scale lines of axis
            if (this.isTickMarksVisible()) {
                ArrayList list = this.mTickMarksList;
                for (int ii = 0; ii < list.size(); ii++) {
                    ElementLineOfScale el = (ElementLineOfScale) list.get(ii);
                    flag = el.contains(x, y);
                    if (flag) {
                        return this.clicked(e);
                    }
                }
            }

            return false;
        }

        /**
         * 
         */
        protected boolean clicked(final MouseEvent e) {
            final int x = e.getX();
            final int y = e.getY();
            final int cnt = e.getClickCount();

            if ((SwingUtilities.isRightMouseButton(e)) && (cnt == 1)) {
//                SGFigureElementAxis.this.showPopupMenu(
//                        this.getPopupMenu(), x, y);
                this.showPopupMenu(x, y);
            } else if ((SwingUtilities.isLeftMouseButton(e)) && (cnt == 2)) {
                this.showPropertyDialog();
            }

            return true;
        }

        /**
         * Called when the mouse is pressed.
         */
        public boolean onMousePressed(final MouseEvent e) {
            // System.out.println("$$ onMousePressed $$");

            // If this axis-group is invisible, return false
            if (!this.isAxisVisible()) {
                return false;
            }

            final int x = e.getX();
            final int y = e.getY();

            boolean flag;

            // axis line
            for (int ii = 0; ii < this.mAxisLines.length; ii++) {
                flag = this.mAxisLines[ii].contains(x, y);
                if (flag) {
                    SGFigureElementAxis.this.prepare();
                    return true;
                }
            }

            // title
            if (this.isTitleVisible()) {
                flag = this.mTitle.contains(x, y);
                if (flag) {
                    SGFigureElementAxis.this.mPressedElement = this.mTitle;
                    SGFigureElementAxis.this.mPressedElementOrigin = new Point(
                            (int) (e.getX() - this.mTitle.getX()), 
                            (int) (e.getY() - this.mTitle.getY()));
                    this.prepare();
                    return true;
                }
            }
            
            // exponent
            if (this.mExponentDrawingElement != null) {
                if (this.mExponentDrawingElement.isVisible()) {
                    flag = this.mExponentDrawingElement.contains(x, y);
                    if (flag) {
                        SGFigureElementAxis.this.mPressedElement
                            = this.mExponentDrawingElement;
                        SGFigureElementAxis.this.mPressedElementOrigin = new Point(
                                (int) (e.getX() - this.mExponentDrawingElement.getX()), 
                                (int) (e.getY() - this.mExponentDrawingElement.getY()));
                        this.prepare();
                        return true;
                    }
                }
            }

            // numbers for axis
            if (this.isNumbersVisible()) {
                ArrayList nList = this.mNumberList;
                for (int ii = 0; ii < nList.size(); ii++) {
                    ElementStringOfScale el = (ElementStringOfScale) nList
                            .get(ii);
                    if (el.isVisible() == false) {
                        continue;
                    }
                    if (el.contains(x, y)) {
                        SGFigureElementAxis.this.mPressedElement = el;
                        SGFigureElementAxis.this.mPressedElementOrigin = new Point(
                                (int) (e.getX() - el.getX()),
                                (int) (e.getY() - el.getY()));
                        this.mTempRange = this.mAxis.getRange();
                        // this.mTempRectangle = new Rectangle();
                        this.mDraggingElement = new ElementStringOfScale(el);
                        this.mDraggingElement.setMagnification(el
                                .getMagnification());
                        this.prepare();
                        return true;
                    }
                }
            }

            // scale lines for axis
            if (this.isTickMarksVisible()) {
                ArrayList lList = this.mTickMarksList;
                for (int ii = 0; ii < lList.size(); ii++) {
                    ElementLineOfScale el = (ElementLineOfScale) lList.get(ii);
                    if (el.isVisible() == false) {
                        continue;
                    }
                    if (el.contains(x, y)) {
                        SGFigureElementAxis.this.mPressedElement = el;
                        SGFigureElementAxis.this.mPressedElementOrigin = e
                                .getPoint();
                        this.mTempRange = this.mAxis.getRange();
                        // this.mTempRectangle = new Rectangle();
                        this.mDraggingElement = this.createScaleLineInstance(el);
                        this.mDraggingElement.setMagnification(el
                                .getMagnification());
                        this.prepare();
                        return true;
                    }
                }
            }

            return false;
        }
        
        protected ElementLineOfScale createScaleLineInstance(ElementLineOfScale el) {
        	return new ElementLineOfScale(el);
        }

        // temporary range for mouse drag
        protected SGTuple2d mTempRange = null;

        /**
         * Called when the mouse is dragged.
         */
        public boolean onMouseDragged(final MouseEvent e) {
            // System.out.println("<< ElementGroupOfAxis::onMouseDragged >>");

            if (this.mDraggingElement == null || this.mTempRange == null) {
                return false;
            }

            final int loc = this.getLocationInPlane();
            final int h1 = AXIS_HORIZONTAL_1;
            final int h2 = AXIS_HORIZONTAL_2;
            final int p1 = AXIS_PERPENDICULAR_1;
            final int p2 = AXIS_PERPENDICULAR_2;
            final int n1 = AXIS_NORMAL_HORIZONTAL_LOWER;
            final int n2 = AXIS_NORMAL_HORIZONTAL_UPPER;
            final int n3 = AXIS_NORMAL_PERPENDICULAR_LEFT;
            final int n4 = AXIS_NORMAL_PERPENDICULAR_RIGHT;

            final boolean invcoord = this.mAxis.isInvertCoordinates();

            final int scaleType = this.mAxis.getScaleType();

            final Rectangle2D gRect = SGFigureElementAxis.this.getGraphRect();
            final int gx = (int) gRect.getX();
            final int gy = (int) gRect.getY();
            final int gw = (int) gRect.getWidth();
            final int gh = (int) gRect.getHeight();

            // rubber band
            int rbx = gx;
            int rby = gy;
            int rbw = gw;
            int rbh = gh;

            if (this.mDraggingElement instanceof ElementStringOfScale) {
        	//
        	// ElementStringOfScale
        	//

                final ElementStringOfScale el = (ElementStringOfScale) this.mDraggingElement;
                final Rectangle2D rect = el.getElementBounds();

                double minValueTempInScale;
                double maxValueTempInScale;
                double oldValueInScale;
                if (scaleType == SGAxis.LINEAR_SCALE) {
                    minValueTempInScale = this.mTempRange.x;
                    maxValueTempInScale = this.mTempRange.y;
                    oldValueInScale = el.mValue;
                } else if (scaleType == SGAxis.LOG_SCALE) {
                    minValueTempInScale = Math.log10(this.mTempRange.x);
                    maxValueTempInScale = Math.log10(this.mTempRange.y);
                    oldValueInScale = Math.log10(el.mValue);
                } else {
                    throw new Error();
                }

                double minValueInScale;
                double maxValueInScale;

                if ((loc == h1) || (loc == h2) || (loc == n1) || (loc == n2)) {
                    // horizontal axes

                    double draggedCoordinate = e.getX()
                            - SGFigureElementAxis.this.mPressedElementOrigin.x
                            + rect.getWidth() / 2.0;
                    final float minCoordinate = this.mAxisLines[0].getStart().x;
                    final float maxCoordinate = this.mAxisLines[0].getEnd().x;
                    // final float width = maxCoordinate - minCoordinate;

                    if (invcoord) {
                        if (draggedCoordinate < minCoordinate) {
                            draggedCoordinate = minCoordinate;
                        }
                        if (draggedCoordinate >= maxCoordinate) {
                            return false;
                        }
                    } else {
                        if (draggedCoordinate > maxCoordinate) {
                            draggedCoordinate = maxCoordinate;
                        }
                        if (draggedCoordinate <= minCoordinate) {
                            return false;
                        }
                    }

                    double valueInScale;
                    if (invcoord) {
                        valueInScale = minValueTempInScale
                        + (maxValueTempInScale - minValueTempInScale)
                        * (maxCoordinate - draggedCoordinate)
                        / (maxCoordinate - minCoordinate);
                    } else {
                        valueInScale = minValueTempInScale
                        + (maxValueTempInScale - minValueTempInScale)
                        * (draggedCoordinate - minCoordinate)
                        / (maxCoordinate - minCoordinate);
                    }

                    // get new range
                    minValueInScale = minValueTempInScale;
                    maxValueInScale = minValueTempInScale
                            + (maxValueTempInScale - minValueTempInScale)
                            * (oldValueInScale - minValueTempInScale)
                            / (valueInScale - minValueTempInScale);

//                    if (scaleType == SGAxis.LINEAR_SCALE) {
//                        maxValueInScale = getNumberInRangeOrder(
//                                maxValueInScale, this.mAxis);
//                    }
                    
                    minValueInScale = getNumberInRangeOrder(minValueInScale, this.mAxis);
                    maxValueInScale = getNumberInRangeOrder(maxValueInScale, this.mAxis);

                    final float x = (float) (draggedCoordinate - rect
                            .getWidth() / 2.0);
                    el.setX(x);

                    final float ratio = (float) ((maxValueInScale - minValueInScale) / (maxValueTempInScale - minValueTempInScale));
                    if (ratio < 0.05) {
                        return false;
                    }

                    // rubber band
                    rbw = (int) (gw / ratio);

                } else if ((loc == p1) || (loc == p2) || (loc == n3) || (loc == n4)) {
                    // perpendicular axes

                    float draggedCoordinate = e.getY()
                            - SGFigureElementAxis.this.mPressedElementOrigin.y
                            + (float) rect.getHeight() / 2.0f;
                    final float minCoordinate = this.mAxisLines[0].getEnd().y;
                    final float maxCoordinate = this.mAxisLines[0].getStart().y;
                    // final float width = maxCoordinate - minCoordinate;

                    if (invcoord) {
                        if (draggedCoordinate <= minCoordinate) {
                            return false;
                        }
                        if (draggedCoordinate > maxCoordinate) {
                            draggedCoordinate = maxCoordinate;
                        }
                    } else {
                        if (draggedCoordinate >= maxCoordinate) {
                            return false;
                        }
                        if (draggedCoordinate < minCoordinate) {
                            draggedCoordinate = minCoordinate;
                        }
                    }

                    double valueInScale;
                    if (invcoord) {
                        valueInScale = minValueTempInScale
                        + (maxValueTempInScale - minValueTempInScale)
                        * (1.0 - (maxCoordinate - draggedCoordinate)
                                / (maxCoordinate - minCoordinate));
                    } else {
                        valueInScale = minValueTempInScale
                        + (maxValueTempInScale - minValueTempInScale)
                        * (1.0 - (draggedCoordinate - minCoordinate)
                                / (maxCoordinate - minCoordinate));
                    }

                    // get new range
                    minValueInScale = minValueTempInScale;
                    maxValueInScale = minValueTempInScale
                            + (maxValueTempInScale - minValueTempInScale)
                            * (oldValueInScale - minValueTempInScale)
                            / (valueInScale - minValueTempInScale);

//                    if (scaleType == SGAxis.LINEAR_SCALE) {
//                        maxValueInScale = getNumberInRangeOrder(
//                                maxValueInScale, this.mAxis);
//                    }
                    minValueInScale = getNumberInRangeOrder(minValueInScale, this.mAxis);
                    maxValueInScale = getNumberInRangeOrder(maxValueInScale, this.mAxis);

                    final float y = (float) (draggedCoordinate - rect
                            .getHeight() / 2.0);
                    el.setY(y);

                    final float ratio = (float) ((maxValueInScale - minValueInScale) / (maxValueTempInScale - minValueTempInScale));
                    if (ratio < 0.05) {
                        return false;
                    }

                    // rubber band
                    rbh = (int) (gh / ratio);
                    rby = gy + gh - rbh;

                } else {
                    throw new Error();
                }

                // set the range to the axis
                double minValue;
                double maxValue;
                if (scaleType == SGAxis.LINEAR_SCALE) {
                    minValue = minValueInScale;
                    maxValue = maxValueInScale;
                } else if (scaleType == SGAxis.LOG_SCALE) {
                    minValue = Math.pow(10.0, minValueInScale);
                    maxValue = Math.pow(10.0, maxValueInScale);
                } else {
                    throw new Error();
                }

                this.mAxis.setScale(minValue, maxValue, scaleType);

            } else if (this.mDraggingElement instanceof ElementLineOfScale) {
        	//
                // ElementLineOfScale
        	//

                final ElementLineOfScale el = (ElementLineOfScale) this.mDraggingElement;

                double minValueTempInScale;
                double maxValueTempInScale;
                double oldValueInScale;
                if (scaleType == SGAxis.LINEAR_SCALE) {
                    minValueTempInScale = this.mTempRange.x;
                    maxValueTempInScale = this.mTempRange.y;
                    oldValueInScale = el.mValue;
                } else if (scaleType == SGAxis.LOG_SCALE) {
                    minValueTempInScale = Math.log(this.mTempRange.x)
                            / Math.log(10.0);
                    maxValueTempInScale = Math.log(this.mTempRange.y)
                            / Math.log(10.0);
                    oldValueInScale = Math.log(el.mValue) / Math.log(10.0);
                } else {
                    throw new Error();
                }

                double minValueInScale;
                double maxValueInScale;

                // horizontal axes
                if ((loc == h1) || (loc == h2) || (loc == n1) || (loc == n2)) {

                    // calculate the location of the point at which the mouse button is dragged
                    float draggedCoordinate = e.getX();
                    if (SGFigureElementAxis.this.mPressedElementOrigin.x == draggedCoordinate) {
                        return false;
                    }

                    float minCoordinate = this.mAxisLines[0].getStart().x;
                    float maxCoordinate = this.mAxisLines[0].getEnd().x;

                    if (draggedCoordinate > maxCoordinate) {
                        draggedCoordinate = maxCoordinate;
                    }
                    if (draggedCoordinate < minCoordinate) {
                        draggedCoordinate = minCoordinate;
                    }

                    double draggedValue;
                    if (invcoord) {
                        draggedValue = minValueTempInScale
                        + (maxValueTempInScale - minValueTempInScale)
                        * (maxCoordinate - draggedCoordinate)
                        / (maxCoordinate - minCoordinate);
                    } else {
                        draggedValue = minValueTempInScale
                        + (maxValueTempInScale - minValueTempInScale)
                        * (draggedCoordinate - minCoordinate)
                        / (maxCoordinate - minCoordinate);
                    }
                    double diff = Math.abs(draggedValue - oldValueInScale);

                    // get new range
                    if (draggedValue > oldValueInScale) {
                        minValueInScale = minValueTempInScale - diff;
                        maxValueInScale = maxValueTempInScale - diff;
                    } else {
                        minValueInScale = minValueTempInScale + diff;
                        maxValueInScale = maxValueTempInScale + diff;
                    }

//                    if (scaleType == SGAxis.LINEAR_SCALE) {
                        minValueInScale = getNumberInRangeOrder(
                                minValueInScale, this.mAxis);
                        maxValueInScale = getNumberInRangeOrder(
                                maxValueInScale, this.mAxis);
//                    }

                } else if ((loc == p1) || (loc == p2) || (loc == n3) || (loc == n4)) {
                    // perpendicular

                    // calculate the location of the point at which the mouse button is dragged
                    float draggedCoordinate = e.getY();
                    if (SGFigureElementAxis.this.mPressedElementOrigin.y == draggedCoordinate) {
                        return false;
                    }

                    float minCoordinate = this.mAxisLines[0].getEnd().y;
                    float maxCoordinate = this.mAxisLines[0].getStart().y;
                    if (draggedCoordinate > maxCoordinate) {
                        draggedCoordinate = maxCoordinate;
                    }
                    if (draggedCoordinate < minCoordinate) {
                        draggedCoordinate = minCoordinate;
                    }
                    double draggedValue;
                    if (invcoord) {
                        draggedValue = minValueTempInScale
                        + (maxValueTempInScale - minValueTempInScale)
                        * (1.0 - (maxCoordinate - draggedCoordinate)
                                / (maxCoordinate - minCoordinate));
                    } else {
                        draggedValue = minValueTempInScale
                        + (maxValueTempInScale - minValueTempInScale)
                        * (1.0 - (draggedCoordinate - minCoordinate)
                                / (maxCoordinate - minCoordinate));
                    }
                    double diff = Math.abs(draggedValue - oldValueInScale);

                    // get new range
                    // double min;
                    // double max;
                    if (draggedValue > oldValueInScale) {
                        minValueInScale = minValueTempInScale - diff;
                        maxValueInScale = maxValueTempInScale - diff;
                    } else {
                        minValueInScale = minValueTempInScale + diff;
                        maxValueInScale = maxValueTempInScale + diff;
                    }

//                    if (scaleType == SGAxis.LINEAR_SCALE) {
                        minValueInScale = getNumberInRangeOrder(
                                minValueInScale, this.mAxis);
                        maxValueInScale = getNumberInRangeOrder(
                                maxValueInScale, this.mAxis);
//                    }

                } else {
                    throw new Error();
                }

                // set the range to the axis
                double minValue;
                double maxValue;
                if (scaleType == SGAxis.LINEAR_SCALE) {
                    minValue = minValueInScale;
                    maxValue = maxValueInScale;
                } else if (scaleType == SGAxis.LOG_SCALE) {
                    minValue = Math.pow(10.0, minValueInScale);
                    maxValue = Math.pow(10.0, maxValueInScale);
                } else {
                    throw new Error();
                }

                this.mAxis.setScale(minValue, maxValue, scaleType);
            }

            // create drawing elements
            if (this.createDrawingElements() == false) {
                return false;
            }

            // set the rubber band
            final Rectangle rubberband = new Rectangle();
            rubberband.setRect(rbx, rby, rbw, rbh);
            // this.mTempRectangle = rubberband;

            // notify to listeners
            if (mNotifyChangeOnDraggingFlag) {
                notifyChange();
            }

            return true;
        }

        /**
         * 
         */
        public boolean onMouseReleased(final MouseEvent e) {

            if (this.mTempRange != null || this.mDraggingElement != null) {
                this.mTempRange = null;
                this.mDraggingElement = null;

                if (SGFigureElementAxis.this.createAllDrawingElements() == false) {
                    return false;
                }
                SGFigureElementAxis.this.initializePressedElement();

                SGProperties pTemp = SGFigureElementAxis.this.mTemporaryProperties;
                SGProperties pPresent = SGFigureElementAxis.this.getProperties();
                if (pTemp.equals(pPresent) == false) {
                    SGFigureElementAxis.this.setChanged(true);
                }
                SGFigureElementAxis.this.mTemporaryProperties = null;

                return true;
            }

            return false;
        }

        /**
         * 
         */
        public boolean onDrawingElement(final int x, final int y) {
            boolean flag = false;
            if (this.isAxisVisible()) {
                flag = this.contains(x, y);
            } else {
                for (int ii = 0; ii < this.mAxisLines.length; ii++) {
                    flag = this.mAxisLines[ii].contains(x, y);
                    if (flag) {
                        break;
                    }
                }
            }

            if (flag) {
                SGFigureElementAxis.this.setMouseCursor(Cursor.HAND_CURSOR);
                return true;
            }

            return false;
        }

        /**
         * 
         * @param x
         * @param y
         * @return
         */
        public boolean contains(final int x, final int y) {
            // axis line
            for (int ii = 0; ii < this.mAxisLines.length; ii++) {
                if (this.mAxisLines[ii].contains(x, y)) {
                    return true;
                }
            }

            // title
            if (this.isTitleVisible()) {
                if (this.mTitle.contains(x, y)) {
                    return true;
                }
            }

            // numbers of axis
            if (this.isNumbersVisible()) {
                final ArrayList nList = this.mNumberList;
                for (int ii = 0; ii < nList.size(); ii++) {
                    final ElementStringOfScale el = (ElementStringOfScale) nList
                            .get(ii);
                    if (el.contains(x, y)) {
                        return true;
                    }
                }
            }

            // scale lines of axis
            if (this.isTickMarksVisible()) {
                final ArrayList sList = this.mTickMarksList;
                for (int ii = 0; ii < sList.size(); ii++) {
                    final ElementLineOfScale el = (ElementLineOfScale) sList
                            .get(ii);
                    if (el.contains(x, y)) {
                        return true;
                    }
                }
            }

            // exponent
            if (this.getExponentFlag()) {
                if (this.mExponentDrawingElement.contains(x, y)) {
                    return true;
                }
            }

            return false;
        }

        /**
         * Creates all drawing elements.
         */
        protected boolean createDrawingElements() {

            // The order of calling these methods are important.

            // axis line
            if (this.setAxisLineProperties() == false) {
                return false;
            }
            if (this.setLocationOfAxisLines() == false) {
                return false;
            }
            // System.out.println("axis line");

            // numbers and scale lines
            if (this.createScaleDrawingElements() == false) {
                return false;
            }
            // System.out.println("numbers and scale lines");

            // title
            if (this.setTitleProperties() == false) {
                return false;
            }
            if (this.setLocationOfTitle() == false) {
                return false;
            }
            // System.out.println("title");

            return true;
        }

        /**
         * 
         * @return
         */
        private boolean setLocationOfDrawingElements() {

            // axis line
            if (this.setLocationOfAxisLines() == false) {
                return false;
            }

            // scale numbers
            if (this.isNumbersVisible()) {
                if (this.setLocationOfScaleNumbers() == false) {
                    return false;
                }
            }

            // scale lines
            if (this.isTickMarksVisible()) {
                if (this.createTickMarks() == false) {
                    return false;
                }
            }

            // title
            if (this.isTitleVisible()) {
                if (this.setLocationOfTitle() == false) {
                    return false;
                }
            }

            return true;
        }

        // create drawing elements of scale lines and numbers
        private boolean createScaleDrawingElements() {

            // calculate scale numbers
            if (this.calcValueArrayInScale() == false) {
                return false;
            }
            // System.out.println("calcValueArrayInScale");

            // create instances of numbers
            if (this.createScaleNumberInstances() == false) {
                return false;
            }
            // System.out.println("createScaleNumberInstances");

            // determine the location of numbers
            if (this.setLocationOfScaleNumbers() == false) {
                return false;
            }
            // System.out.println("createScaleLines");
            // System.out.println(this.mNumberList);

            // create scale lines
            if (this.createTickMarks() == false) {
                return false;
            }
            // System.out.println("createScaleLines");

            return true;
        }
        
        /**
         * Sets the location of the axis line.
         * 
         * @return true if succeeded
         */
        protected boolean setLocationOfAxisLines() {
            float x1;
            float y1;
            float x2;
            float y2;
            final float gx = SGFigureElementAxis.this.mGraphRectX;
            final float gy = SGFigureElementAxis.this.mGraphRectY;
            final float gw = SGFigureElementAxis.this.mGraphRectWidth;
            final float gh = SGFigureElementAxis.this.mGraphRectHeight;

            switch (this.getLocationInPlane()) {
            case AXIS_HORIZONTAL_1: {
                x1 = gx;
                y1 = gy + gh;
                x2 = gx + gw;
                y2 = y1;
                break;
            }
            case AXIS_HORIZONTAL_2: {
                x1 = gx;
                y1 = gy;
                x2 = gx + gw;
                y2 = y1;
                break;
            }
            case AXIS_PERPENDICULAR_1: {
                x1 = gx;
                y1 = gy + gh;
                x2 = x1;
                y2 = gy;
                break;
            }
            case AXIS_PERPENDICULAR_2: {
                x1 = gx + gw;
                y1 = gy + gh;
                x2 = x1;
                y2 = gy;
                break;
            }
            default: {
                throw new Error();
            }
            }
            this.mAxisLines[0].setTermPoints(x1, y1, x2, y2);
            return true;
        }

        /**
         * Set properties to axis lines.
         * 
         * @return true if succeeded
         */
        protected boolean setAxisLineProperties() {
            for (int ii = 0; ii < this.mAxisLines.length; ii++) {
                this.mAxisLines[ii].setLineWidth(getAxisLineWidth());
                this.mAxisLines[ii].setColor(mLineColor);
            }
            return true;
        }

        /**
         * Sets the location of the title.
         * 
         * @return true if succeeded
         */
        protected boolean setLocationOfTitle() {
            final float gx = SGFigureElementAxis.this.mGraphRectX;
            final float gy = SGFigureElementAxis.this.mGraphRectY;
            final float gw = SGFigureElementAxis.this.mGraphRectWidth;
            final float gh = SGFigureElementAxis.this.mGraphRectHeight;

            final float mag = SGFigureElementAxis.this.mMagnification;

            final float spaceLN = getSpaceAxisLineAndNumber() * mag;
            final float spaceNT = getSpaceNumberAndTitle() * mag;

            float scaleNumberHeight = 0.0f;
            float maxWidth = 0.0f;
            // if( this.mScaleNumbersVisibleFlag )
            {
                scaleNumberHeight = (float) getScaleHeight();
                maxWidth = getMaxLengthOfScaleNumbers(this);
            }

            switch (this.getLocationInPlane()) {
            case SGIFigureElementAxis.AXIS_HORIZONTAL_1: {
                Rectangle2D rectTitle = this.mTitle.getElementBounds();
                this.mTitle.setLocation(gx
                        + (gw - (float) rectTitle.getWidth()) / 2.0f, gy + gh
                        + scaleNumberHeight + (spaceLN + spaceNT));

                break;
            }
            case SGIFigureElementAxis.AXIS_HORIZONTAL_2: {
                Rectangle2D rectTitle = this.mTitle.getElementBounds();
                this.mTitle.setLocation(gx
                        + (gw - (float) rectTitle.getWidth()) / 2.0f, gy
                        - scaleNumberHeight - (float) rectTitle.getHeight()
                        - (spaceLN + spaceNT));
                break;
            }
            case SGIFigureElementAxis.AXIS_PERPENDICULAR_1: {
                this.mTitle.setAngle(90.0f); // rotation of 90 degree
                Rectangle2D rectTitle = this.mTitle.getElementBounds();
                this.mTitle.setLocation(gx - maxWidth
                        - (float) rectTitle.getWidth() - (spaceLN + spaceNT),
                        gy + (gh + (float) rectTitle.getHeight()) / 2.0f);
                break;
            }
            case SGIFigureElementAxis.AXIS_PERPENDICULAR_2: {
                this.mTitle.setAngle(90.0f); // rotation of 90 degree
                Rectangle2D rectTitle = this.mTitle.getElementBounds();
                this.mTitle.setLocation(gx + gw + maxWidth
                        + (spaceLN + spaceNT), gy
                        + (gh + (float) rectTitle.getHeight()) / 2.0f);
                break;
            }
            default: {
                throw new Error();
            }
            }
            return true;
        }

        /**
         * Set properties of the title.
         * 
         * @return true if succeeded
         */
        protected boolean setTitleProperties() {
            this.mTitle.setColor(SGFigureElementAxis.this.mAxisFontColor);
            this.mTitle.setFont(SGFigureElementAxis.this.mTitleFontName,
                    SGFigureElementAxis.this.mTitleFontStyle,
                    SGFigureElementAxis.this.mTitleFontSize);
            return true;
        }

        /**
         * 
         * @return
         */
        private boolean createScaleNumberInstances() {

            // clear
            this.mNumberList.clear();
            this.mExponentDrawingElement = null;

            // scale type
            final int scaleType = this.mAxis.getScaleType();

            final double[] vArray = this.mAxisValueArray;

            // exponent
            double[] valueArray = null;
            final int eValue = this.getExponentValue();
            if (this.getExponentFlag()) {
                valueArray = new double[this.mAxisValueArray.length];
                for (int ii = 0; ii < valueArray.length; ii++) {
                    BigDecimal db = new BigDecimal(Double.toString(vArray[ii]));
                    db = db.movePointLeft(eValue);
                    valueArray[ii] = db.doubleValue();
                }
            } else {
                valueArray = vArray;
            }

            // create string array
            String[] strArray = new String[valueArray.length];
            if (scaleType == SGAxis.LINEAR_SCALE) {
                if (this.isNumbersInteger()) {
                    for (int ii = 0; ii < strArray.length; ii++) {
                        strArray[ii] = Integer.toString((int) valueArray[ii]);
                    }
                } else {
                    for (int ii = 0; ii < strArray.length; ii++) {
                        strArray[ii] = Double.toString(valueArray[ii]);
                    }
                }
            } else if (scaleType == SGAxis.LOG_SCALE) {
                for (int ii = 0; ii < strArray.length; ii++) {
                    final int order = SGUtilityNumber.getOrder(valueArray[ii]);
                    strArray[ii] = SGUtilityText.getSuperscriptString("10",
                            Integer.toString(order));
                }
            }

            // create drawing elements of the scale numbers
            final float mag = SGFigureElementAxis.this.mMagnification;
//            final Color cl = SGFigureElementAxis.this.mStringColor;
//            final String name = SGFigureElementAxis.this.mScaleFontName;
//            final int style = SGFigureElementAxis.this.mScaleFontStyle;
//            final float size = SGFigureElementAxis.this.mScaleFontSize;
//            final float angle = 0.0f;
            for (int ii = 0; ii < vArray.length; ii++) {
                ElementStringOfScale el = this.createSingleTickMarkInstance(
                        strArray[ii]);
//                ElementStringOfScale el = new ElementStringOfScale(
//                        strArray[ii], name, style, size, cl, mag, angle);
                el.mValue = vArray[ii];
                if (this.mAxis.insideRange(el.mValue) == false) {
                    el.setVisible(false);
                }
                el.setMagnification(mag);
                this.mNumberList.add(el);
            }

            // create the exponent drawing element
            if (this.getExponentFlag()) {
                this.createExponentDrawingElement();
            }

            return true;
        }
        
        /**
         * Creates a drawing element of a scale number.
         */
        protected ElementStringOfScale createSingleTickMarkInstance(
                final String str) {
            ElementStringOfScale el = new ElementStringOfScale(
                    str, SGFigureElementAxis.this.mScaleFontName, 
                    SGFigureElementAxis.this.mScaleFontStyle, 
                    SGFigureElementAxis.this.mScaleFontSize, 
                    SGFigureElementAxis.this.mAxisFontColor, 
                    SGFigureElementAxis.this.mMagnification,
                    this.mNumbersAngle,
                    this.mLocationInPlane);
            return el;
        }

        /**
         * 
         * @return
         */
        private boolean createExponentDrawingElement() {
            // craete an instance
            String str = multiply + "10";
            str = SGUtilityText.getSuperscriptString(str, Integer.toString(this
                    .getExponentValue()));
//            final float angle = 0.0f;
//            SGDrawingElementString2DExtended el = new SGDrawingElementString2DExtended(
//                    str, SGFigureElementAxis.this.mScaleFontName,
//                    SGFigureElementAxis.this.mScaleFontStyle,
//                    SGFigureElementAxis.this.mScaleFontSize,
//                    SGFigureElementAxis.this.mStringColor,
//                    SGFigureElementAxis.this.mMagnification, angle);

            // set to the attribute
//            this.mExponentDrawingElement = el;
            this.mExponentDrawingElement = this.createExponentStringElement(str);

            // set the location
            this.setLocationOfExponentDrawingElement();

            return true;
        }
        
        /**
         * Creates a string element for the exponent.
         */
        protected SGDrawingElementString2DExtended createExponentStringElement(
                final String str) {
            SGDrawingElementString2DExtended el = new SGDrawingElementString2DExtended(
                    str, SGFigureElementAxis.this.mScaleFontName,
                    SGFigureElementAxis.this.mScaleFontStyle,
                    SGFigureElementAxis.this.mScaleFontSize,
                    SGFigureElementAxis.this.mAxisFontColor,
                    SGFigureElementAxis.this.mMagnification, 0.0f);
            return el;
        }

        /**
         * Sets the location of exponent object.
         * 
         * @return true if succeeded
         */
        protected boolean setLocationOfExponentDrawingElement() {
            SGDrawingElementString2D el = this.mExponentDrawingElement;

            final Rectangle2D rect = el.getElementBounds();

            final float gx = SGFigureElementAxis.this.mGraphRectX;
            final float gy = SGFigureElementAxis.this.mGraphRectY;
            final float gw = SGFigureElementAxis.this.mGraphRectWidth;
            final float gh = SGFigureElementAxis.this.mGraphRectHeight;

            final float mag = SGFigureElementAxis.this.mMagnification;

            final float spaceLN = getSpaceAxisLineAndNumber() * mag;
            final float spaceNT = getSpaceNumberAndTitle() * mag;

            float x = 0.0f;
            float y = 0.0f;
            switch (this.getLocationInPlane()) {

            case AXIS_HORIZONTAL_1: {
                x = gx + gw - 0 * (float) rect.getWidth();
                y = gy + gh + (float) (spaceLN + getScaleHeight() + spaceNT);
                break;
            }

            case AXIS_HORIZONTAL_2: {
                x = gx + gw - 0 * (float) rect.getWidth();
                y = gy
                        - (float) (spaceLN + getScaleHeight() + spaceNT + getTitleHeight());
                break;
            }

            case AXIS_PERPENDICULAR_1: {
                x = gx - (float) (spaceLN + getMaxLengthOfScaleNumbers(this)
                                + spaceNT + rect.getWidth());
                y = gy - (float) rect.getHeight();
                break;
            }

            case AXIS_PERPENDICULAR_2: {
                x = gx + gw + spaceLN + getMaxLengthOfScaleNumbers(this)
                        + spaceNT;
                y = gy - (float) rect.getHeight();
                break;
            }
            
            default: {
                throw new Error();
            }

            }

            el.setLocation(x, y);

            return true;
        }

        /**
         * Sets the location of scale numbers.
         * 
         * @return true if succeeded
         */
        protected boolean setLocationOfScaleNumbers() {

            final int loc = this.getLocationInPlane();
            final boolean invcoord = this.isInvertCoordinates();

            final float gx = SGFigureElementAxis.this.mGraphRectX;
            final float gy = SGFigureElementAxis.this.mGraphRectY;
            final float gw = SGFigureElementAxis.this.mGraphRectWidth;
            final float gh = SGFigureElementAxis.this.mGraphRectHeight;

            int scaleType = this.mAxis.getScaleType();

            double[] valueArrayInScale = null;
            if (scaleType == SGAxis.LINEAR_SCALE) {
                valueArrayInScale = this.mAxisValueArray;
            } else if (scaleType == SGAxis.LOG_SCALE) {
                valueArrayInScale = new double[this.mAxisValueArray.length];
                for (int ii = 0; ii < valueArrayInScale.length; ii++) {
                    valueArrayInScale[ii] = Math.log(this.mAxisValueArray[ii]);
                }
            }

            double axisMinInScale = 0.0;
            double axisMaxInScale = 0.0;
            if (scaleType == SGAxis.LINEAR_SCALE) {
                axisMinInScale = this.mAxis.getMinValue();
                axisMaxInScale = this.mAxis.getMaxValue();
            } else if (scaleType == SGAxis.LOG_SCALE) {
                axisMinInScale = Math.log(this.mAxis.getMinValue());
                axisMaxInScale = Math.log(this.mAxis.getMaxValue());
            }

            final float mag = SGFigureElementAxis.this.mMagnification;

            // final float spaceNT = getSpaceNumberAndTitle()*mag;
            final float spaceLN = getSpaceAxisLineAndNumber() * mag;

            for (int ii = 0; ii < this.mNumberList.size(); ii++) {
                ElementStringOfScale el = (ElementStringOfScale) this.mNumberList
                        .get(ii);

                Rectangle2D rect = el.getElementBounds();
                final float width = (float) rect.getWidth();
                final float height = (float) rect.getHeight();

                switch (loc) {
                case AXIS_HORIZONTAL_1: // bottom
                {
                    final float x;
                    if (invcoord) {
                        x = (float) (gx + gw
                                * (axisMaxInScale - valueArrayInScale[ii])
                                / (axisMaxInScale - axisMinInScale));
                    } else {
                        x = (float) (gx + gw
                                * (valueArrayInScale[ii] - axisMinInScale)
                                / (axisMaxInScale - axisMinInScale));
                    }
                    final float y = gy + gh + spaceLN;
                    el.setLocation((x - 0.50f * width), y);
                    break;
                }

                case AXIS_HORIZONTAL_2: // top
                {
                    final float x;
                    if (invcoord) {
                        x = (float) (gx + gw
                                * (axisMaxInScale - valueArrayInScale[ii])
                                / (axisMaxInScale - axisMinInScale));
                    } else {
                        x = (float) (gx + gw
                                * (valueArrayInScale[ii] - axisMinInScale)
                                / (axisMaxInScale - axisMinInScale));
                    }
                    final float y = (float) (gy - spaceLN - rect.getHeight());
                    el.setLocation((x - 0.50f * width), y);
                    break;
                }

                case AXIS_PERPENDICULAR_1: // left
                {
                    final float y;
                    if (invcoord) {
                        y = (float) (gy + gh
                                * (1.0 - (axisMaxInScale - valueArrayInScale[ii])
                                        / (axisMaxInScale - axisMinInScale)));
                    } else {
                        y = (float) (gy + gh
                                * (1.0 - (valueArrayInScale[ii] - axisMinInScale)
                                        / (axisMaxInScale - axisMinInScale)));
                    }
                    final float x = (float) (gx - spaceLN - rect.getWidth());
                    el.setLocation(x, (y - 0.5f * height));
                    break;
                }

                case AXIS_PERPENDICULAR_2: // right
                {
                    final float y;
                    if (invcoord) {
                        y = (float) (gy + gh
                                * (1.0 - (axisMaxInScale - valueArrayInScale[ii])
                                        / (axisMaxInScale - axisMinInScale)));
                    } else {
                        y = (float) (gy + gh
                                * (1.0 - (valueArrayInScale[ii] - axisMinInScale)
                                        / (axisMaxInScale - axisMinInScale)));
                    }
                    final float x = gx + gw + spaceLN;
                    el.setLocation(x, (y - 0.5f * height));
                    break;
                }
                
                default: {
                    throw new Error();
                }
                }
            }

            // the exponent drawing element
            if (this.getExponentFlag()) {
                this.setLocationOfExponentDrawingElement();
            }

            return true;
        }

        /**
         * Create the scale lines.
         * 
         * @return true if succeeded
         */
        protected boolean createTickMarks() {
            // clear the list
            this.mTickMarksList.clear();

            // create
            boolean flag = true;
            switch (this.mAxis.getScaleType()) {
            case SGAxis.LINEAR_SCALE:
                flag = this.createTickMarksInLinearScale(getTickMarkWidth(), mTickMarkLength,
                        mLineColor);
                break;
            case SGAxis.LOG_SCALE:
                flag = this.createTickMarksInLogScale(getTickMarkWidth(), mTickMarkLength,
                        mLineColor);
                break;
            default:
                throw new Error();
            }

            return flag;
        }

        /**
         * Creates scale lines in the linear scale.
         * 
         * @return true if succeeded
         */
        protected boolean createTickMarksInLinearScale(final float width,
                final float length, final Color cl) {

            final float len = mMagnification * length;
            final double[] valueArray = this.mAxisValueArray;
            
            for (int ii = 0; ii < valueArray.length; ii++) {
                final double value = valueArray[ii];
                this.createTickMarkPair(value, width, len, cl);
            }

            // create scale lines between scale numbers
            for (int ii = 0; ii < valueArray.length; ii++) {
                if (ii == valueArray.length - 1) {
                    break;
                }
                final double value = (valueArray[ii] + valueArray[ii + 1]) / 2.0;
                this.createTickMarkPair(value, width, len, cl);
            }
            
            return true;
        }

        // create a pari of tick marks
        private void createTickMarkPair(final double value, 
                final float width, final float len, final Color cl) {
            ElementLineOfScale el = this.createSingleTickMark(value,
                    width, len);
            if (el != null) {
                this.setupCreatedScaleLineProperties(el, value, cl);
            }
            ElementLineOfScale elc = this.createSingleTickMarkCounter(value,
                    width, len);
            if (elc != null) {
                this.setupCreatedScaleLineProperties(elc, value, cl);
            }
        }

        /**
         * Creates scale lines in the log scale.
         * 
         * @return true if succeeded
         */
        protected boolean createTickMarksInLogScale(final float width,
                final float length, final Color cl) {
            // create the main scale lines
            final float len = mMagnification * length;
            for (int ii = 0; ii < this.mAxisValueArray.length; ii++) {
                final double value = this.mAxisValueArray[ii];
                ElementLineOfScale el = this.createSingleTickMark(value,
                        width, len);
                if (el != null) {
                    this.setupCreatedScaleLineProperties(el, value, cl);
                }
                ElementLineOfScale elc = this.createSingleTickMarkCounter(value,
                        width, len);
                if (elc != null) {
                    this.setupCreatedScaleLineProperties(elc, value, cl);
                }
            }

            // create the sub scale lines
            final float ratio = 1.0f;
            for (int ii = 0; ii < this.mAxisValueArray.length; ii++) {
                for (int jj = 2; jj < 10; jj++) {
                    final double value = this.mAxisValueArray[ii] * 0.10 * jj;
                    ElementLineOfScale el = this.createSingleTickMark(value,
                            width, ratio * len);
                    if (el != null) {
                        this.setupCreatedScaleLineProperties(el, value, cl);
                    }
                    ElementLineOfScale elc = this.createSingleTickMarkCounter(value,
                            width, len);
                    if (elc != null) {
                        this.setupCreatedScaleLineProperties(elc, value, cl);
                    }
                }
            }
            if (this.mAxisValueArray.length >= 1) {
                for (int jj = 2; jj < 10; jj++) {
                    final double value = this.mAxisValueArray[this.mAxisValueArray.length - 1]
                            * jj;
                    ElementLineOfScale el = this.createSingleTickMark(value,
                            width, ratio * len);
                    if (el != null) {
                        this.setupCreatedScaleLineProperties(el, value, cl);
                    }
                    ElementLineOfScale elc = this.createSingleTickMarkCounter(value,
                            width, len);
                    if (elc != null) {
                        this.setupCreatedScaleLineProperties(elc, value, cl);
                    }
                }
            }

            return true;
        }
        
        /**
         * Sets up the properties of created scale line.
         * 
         * @param el
         *           a scale line
         * @param value
         *           axis value set to the scale line
         * @param cl
         *           the color of line
         */
        protected void setupCreatedScaleLineProperties(final ElementLineOfScale el, 
                final double value, final Color cl) {
            el.mValue = value;
            el.setMagnification(mMagnification);
            el.setColor(cl);
            this.mTickMarksList.add(el);
        }

        /**
         * Creates a scale line.
         * 
         * @param value
         *            axis value of the scale line
         * @param length
         *            the length of the scale line
         * @return an instance of the scale line
         */
        protected ElementLineOfScale createSingleTickMark(final double value,
                final float width, final float length) {
            // System.out.println("<< createSingleScaleLine >>");
            if (this.mAxis.insideRange(value) == false) {
                return null;
            }

            final float gx = SGFigureElementAxis.this.mGraphRectX;
            final float gy = SGFigureElementAxis.this.mGraphRectY;
            final float gw = SGFigureElementAxis.this.mGraphRectWidth;
            final float gh = SGFigureElementAxis.this.mGraphRectHeight;

            final float mag = SGFigureElementAxis.this.mMagnification;
            final boolean inside = this.isTickMarksInside();
//            final float width = SGFigureElementAxis.this.mScaleLineWidth;

            final boolean invcoord = this.isInvertCoordinates();

            double min;
            double max;
            double valueInScale;
            switch (this.mAxis.getScaleType()) {
            case SGAxis.LINEAR_SCALE: {
                min = this.mAxis.getMinValue();
                max = this.mAxis.getMaxValue();
                valueInScale = value;
                break;
            }

            case SGAxis.LOG_SCALE: {
                min = Math.log(this.mAxis.getMinValue());
                max = Math.log(this.mAxis.getMaxValue());
                valueInScale = Math.log(value);
                break;
            }

            default: {
                throw new Error();
            }
            }

            ElementLineOfScale el = null;
            final SGTuple2f start;
            final SGTuple2f end;
            
            switch (this.getLocationInPlane()) {
            case AXIS_HORIZONTAL_1: // bottom
            {
                final double x;
                if (invcoord) {
                    x = gx + gw * (max - valueInScale) / (max - min);
                } else {
                    x = gx + gw * (valueInScale - min) / (max - min);
                }
                final double yStart = gy + gh;
                final double yEnd;
                if (inside) {
                    yEnd = yStart - length;
                } else {
                    yEnd = yStart + length;
                }
                start = new SGTuple2f((float) x, (float) yStart);
                end = new SGTuple2f((float) x, (float) yEnd);
                break;
            }

            case AXIS_HORIZONTAL_2: // top
            {
                final double x;
                if (invcoord) {
                    x = gx + gw * (max - valueInScale) / (max - min);
                } else {
                    x = gx + gw * (valueInScale - min) / (max - min);
                }
                final double yStart = gy;
                final double yEnd;
                if (inside) {
                    yEnd = yStart + length;
                } else {
                    yEnd = yStart - length;
                }
                start = new SGTuple2f((float) x, (float) yStart);
                end = new SGTuple2f((float) x, (float) yEnd);
                break;
            }

            case AXIS_PERPENDICULAR_1: // left
            {
                final double y;
                if (invcoord) {
                    y = gy + gh * (1.0 - (max - valueInScale) / (max - min));
                } else {
                    y = gy + gh * (1.0 - (valueInScale - min) / (max - min));
                }
                final double xStart = gx;
                final double xEnd;
                if (inside) {
                    xEnd = xStart + length;
                } else {
                    xEnd = xStart - length;
                }
                start = new SGTuple2f((float) xStart, (float) y);
                end = new SGTuple2f((float) xEnd, (float) y);
                break;
            }

            case AXIS_PERPENDICULAR_2: // right
            {
                final double y;
                if (invcoord) {
                    y = gy + gh * (1.0 - (max - valueInScale) / (max - min));
                } else {
                    y = gy + gh * (1.0 - (valueInScale - min) / (max - min));
                }
                final double xStart = gx + gw;
                final double xEnd;
                if (inside) {
                    xEnd = xStart - length;
                } else {
                    xEnd = xStart + length;
                }
                start = new SGTuple2f((float) xStart, (float) y);
                end = new SGTuple2f((float) xEnd, (float) y);
                break;
            }

            default: {
                throw new Error();
            }

            }

            el = new ElementLineOfScale(start, end);
            el.setLineWidth(width);
            el.setMagnification(mag);
            
            return el;
        }

        /**
         * Creates a scale line at the counter position.
         * 
         * @param value
         *            axis value of the scale line
         * @param length
         *            the length of the scale line
         * @return an instance of the scale line
         */
        protected ElementLineOfScale createSingleTickMarkCounter(final double value,
                final float width, final float length) {
            // returns null
            return null;
        }
        
        /**
         * Create an array of numbers for current scale type.
         */
        private boolean calcValueArrayInScale() {

            double[] array = null;
            switch (this.mAxis.getScaleType()) {
            case SGAxis.LINEAR_SCALE: // linear scale
                array = this.calcScaleValuesInLinearScale();
                break;

            case SGAxis.LOG_SCALE: // log scale
                array = this.calcScaleValuesInLogScale();
                break;

            default:
            }

            // set to an attribute
            this.mAxisValueArray = array;

            return true;
        }

        /**
         * Calculate the numbers of scale in for the linear scale.
         * 
         * @return
         */
        private double[] calcScaleValuesInLinearScale() {
            
            SGAxis axis = this.mAxis;
            final double baseline;
            final double step;
            if (this.isCalculateAutomatically()) {
                baseline = 0.0;
                step = calcStepValue(axis);
                this.mBaselineValue = baseline;
                this.mStepValue = step;
            } else {
                baseline = this.getBaselineValue();
                step = Math.abs(this.getStepValue());
            }

            // get an array of numbers
            double[] axisValueArray = SGUtilityNumber.calcStepValueSorted(
            		axis.getMinValue(), axis.getMaxValue(), baseline, step, 
            		AXIS_SCALE_EFFECTIVE_DIGIT);
            
            // set scale numbers integer when all numbers
            // can be replaced with an integer
            if (SGFigureElementAxis.this.mStartFlag) {
                boolean flag = true;
                for (int ii = 0; ii < axisValueArray.length; ii++) {
                    final double value = axisValueArray[ii];
                    final long round = Math.round(value);
                    final double diff = Math.abs(value - round);
                    if (diff != 0.0) {
                        flag = false;
                        break;
                    }
                }
                this.setNumbersInteger(flag);
            }

            // When the scale numbers are set to be integer,
            // cast values to integer
            if (this.isNumbersInteger()) {

                final List<Integer> numList = new ArrayList<Integer>();
                for (int ii = 0; ii < axisValueArray.length; ii++) {
                    double value;
                    if (this.getExponentFlag()) {
                        BigDecimal db = new BigDecimal(Double
                                .toString(axisValueArray[ii]));
                        db = db.movePointLeft(this.getExponentValue());
                        value = db.doubleValue();
                    } else {
                        value = axisValueArray[ii];
                    }
                    final int num = (int) value;
                    if (Math.abs(num - value) < Double.MIN_VALUE) {
                        numList.add(Integer.valueOf(num));
                    }
                }

                // remove the same values
                for (int ii = numList.size() - 1; ii >= 1; ii--) {
                    final Integer n1 = numList.get(ii);
                    for (int jj = ii - 1; jj >= 0; jj--) {
                        final Integer n2 = numList.get(jj);
                        if (n2.intValue() == n1.intValue()) {
                            numList.remove(ii);
                            break;
                        }
                    }
                }

                final double[] valueArray = new double[numList.size()];
                for (int ii = 0; ii < valueArray.length; ii++) {
                    double value = ((Integer) numList.get(ii)).doubleValue();
                    if (this.getExponentFlag()) {
                        BigDecimal db = new BigDecimal(Double.toString(value));
                        db = db.movePointRight(this.getExponentValue());
                        value = db.doubleValue();
                    }
                    valueArray[ii] = value;
                }
                axisValueArray = valueArray;
            }

            // this returned value reflects the integer flag,
            // but is not influenced by the exponent flag
            return axisValueArray;
        }
        
        /**
         * Calculate the numbers of scale in for the linear scale.
         * 
         * @return
         */
        private double[] calcScaleValuesInLogScale() {
            // System.out.println("<< calcScaleValuesInLogScale >>");

            // minimum and maximum values of axis range
            final SGTuple2d range = this.mAxis.getRange();
            final double min = range.x;
            final double max = range.y;

            // array of values for scale
            double[] axisValueArray = null;

            final int minOrder = SGUtilityNumber.getOrder(min);
            final int maxOrder = SGUtilityNumber.getOrder(max);
            // System.out.println(min+" "+minOrder);
            // System.out.println(max+" "+maxOrder);

            // final double diff = max - min;
            // final int diffOrder = SGUtility.getOrder( (1.0-1.0E-14)*diff );

            ArrayList list = new ArrayList();
            for (int ii = minOrder; ii <= maxOrder; ii++) {
                list.add(Integer.valueOf(ii));
            }
            // System.out.println(list);

            axisValueArray = new double[list.size()];
            for (int ii = 0; ii < axisValueArray.length; ii++) {
                final int num = ((Integer) list.get(ii)).intValue();
                axisValueArray[ii] = SGUtilityNumber.getPowersOfTen(num);
            }
            // System.out.println();

            return axisValueArray;
        }

        /**
         * Returns the tag name.
         * 
         * @return the tag name
         */
        public String getTagName() {
            return SGIFigureElementAxis.TAG_NAME_AXIS;
        }

        /**
         * Create an Element object.
         * 
         * @param document
         *           a Document object to append the Element object
         * @return an Element object
         */
        public Element createElement(final Document document) {
            Element element = document.createElement(this.getTagName());
            if (this.writeProperty(element) == false) {
                return null;
            }
            return element;
        }

        /**
         * Write the properties to an given Element object.
         * 
         * @param el
         *          an Element object
         * @return true if succeeded
         */
        public boolean writeProperty(final Element el) {
            final int loc = this.getLocationInPlane();
            final String str = getLocationName(loc);
            el.setAttribute(KEY_POSITION, str);
            el.setAttribute(KEY_AXIS_VISIBLE, Boolean.toString(this
                    .isAxisVisible()));
            if (this.writeCommonProperties(el) == false) {
                return false;
            }
            return true;
        }
        
        protected boolean writeCommonProperties(final Element el) {
            final SGTuple2d range = this.mAxis.getRange();

            el.setAttribute(KEY_AXIS_SCALE_TYPE, SGUtilityText
                    .getScaleTypeName(this.mAxis.getScaleType()));
            el.setAttribute(KEY_AXIS_MIN_VALUE, Double.toString(range.x));
            el.setAttribute(KEY_AXIS_MAX_VALUE, Double.toString(range.y));
            el.setAttribute(KEY_AXIS_INVERT_COORDINATES, Boolean.toString(this
                    .isInvertCoordinates()));

            el.setAttribute(KEY_AUTO_CALC_NUMBER, Boolean.toString(this
                    .isCalculateAutomatically()));
            el.setAttribute(KEY_STEP_VALUE, Double
                    .toString(this.getStepValue()));
            el.setAttribute(KEY_BASELINE_VALUE, Double.toString(this
                    .getBaselineValue()));

            el.setAttribute(KEY_TITLE_VISIBLE, Boolean.toString(this
                    .isTitleVisible()));
            el.setAttribute(KEY_TITLE_TEXT, this.getTitleString());

            el.setAttribute(KEY_NUMBER_VISIBLE, Boolean.toString(this
                    .isNumbersVisible()));
            el.setAttribute(KEY_NUMBER_INTEGER, Boolean.toString(this
                    .isNumbersInteger()));

            el.setAttribute(KEY_TICK_MARK_VISIBLE, Boolean.toString(this
                    .isTickMarksVisible()));
            el.setAttribute(KEY_TICK_MARK_INSIDE, Boolean.toString(this
                    .isTickMarksInside()));

            el.setAttribute(KEY_EXPONENT_VISIBLE, Boolean.toString(this
                    .getExponentFlag()));
            el.setAttribute(KEY_EXPONENT_VALUE, Integer.toString(this
                    .getExponentValue()));
            
            el.setAttribute(KEY_NUMBER_ANGLE, Float.toString(this.getNumbersAngle()));

            return true;
        }

        protected boolean readCommonProperties(final Element element) {

            String str = null;
            Number num = null;
            Boolean b = null;

            // scale type
            str = element
                    .getAttribute(SGIFigureElementAxis.KEY_AXIS_SCALE_TYPE);
            if (str.length() == 0) {
                return false;
            }
            final int scaleType = SGUtilityText.getScaleType(str);
            if (scaleType == -1) {
                return false;
            }

            // min value
            str = element.getAttribute(SGIFigureElementAxis.KEY_AXIS_MIN_VALUE);
            if (str.length() == 0) {
                return false;
            }
            num = SGUtilityText.getDouble(str);
            if (num == null) {
                return false;
            }
            final double minValue = num.doubleValue();

            // max value
            str = element.getAttribute(SGIFigureElementAxis.KEY_AXIS_MAX_VALUE);
            if (str.length() == 0) {
                return false;
            }
            num = SGUtilityText.getDouble(str);
            if (num == null) {
                return false;
            }
            final double maxValue = num.doubleValue();

            // invert coordinates
            str = element
                    .getAttribute(SGIFigureElementAxis.KEY_AXIS_INVERT_COORDINATES);
            if (str.length() == 0) {
                b = Boolean.FALSE; // version >= 1.0.4
            } else {
                b = SGUtilityText.getBoolean(str); // version > 1.0.5
            }
            final boolean invCoord = b.booleanValue();

            // auto calc
            str = element
                    .getAttribute(SGIFigureElementAxis.KEY_AUTO_CALC_NUMBER);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean autoCalc = b.booleanValue();

            // step value
            str = element.getAttribute(SGIFigureElementAxis.KEY_STEP_VALUE);
            if (str.length() == 0) {
                return false;
            }
            num = SGUtilityText.getDouble(str);
            if (num == null) {
                return false;
            }
            final double stepValue = num.doubleValue();

            // baseline value
            str = element.getAttribute(SGIFigureElementAxis.KEY_BASELINE_VALUE);
            if (str.length() == 0) {
                return false;
            }
            num = SGUtilityText.getDouble(str);
            if (num == null) {
                return false;
            }
            final double baselineValue = num.doubleValue();

            // title visible
            str = element.getAttribute(SGIFigureElementAxis.KEY_TITLE_VISIBLE);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean titleVisible = b.booleanValue();

            // title string
            str = element.getAttribute(SGIFigureElementAxis.KEY_TITLE_TEXT);
            // null string is ok
            // if( str.length()==0 )
            // {
            // return false;
            // }
            final String titleString = str;

            // scale number visible
            str = element.getAttribute(SGIFigureElementAxis.KEY_NUMBER_VISIBLE);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean numberVisible = b.booleanValue();

            // number integer
            str = element.getAttribute(SGIFigureElementAxis.KEY_NUMBER_INTEGER);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean numberInteger = b.booleanValue();

            // tick mark visible
            str = element
                    .getAttribute(SGIFigureElementAxis.KEY_TICK_MARK_VISIBLE);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean tickMarkVisible = b.booleanValue();

            // tick mark inside
            str = element
                    .getAttribute(SGIFigureElementAxis.KEY_TICK_MARK_INSIDE);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean tickMarkInside = b.booleanValue();

            // exponent visible
            str = element
                    .getAttribute(SGIFigureElementAxis.KEY_EXPONENT_VISIBLE);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean exponentVisible = b.booleanValue();

            // exponent value
            str = element.getAttribute(SGIFigureElementAxis.KEY_EXPONENT_VALUE);
            if (str.length() == 0) {
                return false;
            }
            num = SGUtilityText.getInteger(str);
            if (num == null) {
                return false;
            }
            final int exponentValue = num.intValue();

            // format angle
            float formatAngle = DEFAULT_NUMBER_ANGLE;
            str = element.getAttribute(SGIFigureElementAxis.KEY_NUMBER_ANGLE);
            if (str.length() != 0) {
                num = SGUtilityText.getFloat(str);
                if (num == null) {
                    return false;
                }
                formatAngle = num.floatValue();
            }

            //
            // set properties
            //

            if (this.setScale(minValue, maxValue, scaleType) == false) {
                return false;
            }

            if (this.setInvertCoordinates(invCoord) == false) {
                return false;
            }

            if (this.setCalculateAutomatically(autoCalc) == false) {
                return false;
            }

            if (this.setStepValue(stepValue) == false) {
                return false;
            }

            if (this.setBaselineValue(baselineValue) == false) {
                return false;
            }

            if (this.setTitleVisible(titleVisible) == false) {
                return false;
            }

            if (this.setTitle(titleString) == false) {
                return false;
            }

            if (this.setNumbersVisible(numberVisible) == false) {
                return false;
            }

            if (this.setNumbersInteger(numberInteger) == false) {
                return false;
            }

            if (this.setTickMarksVisible(tickMarkVisible) == false) {
                return false;
            }

            if (this.setTickMarksInside(tickMarkInside) == false) {
                return false;
            }

            if (this.setExponentFlag(exponentVisible) == false) {
                return false;
            }

            if (this.setExponentValue(exponentValue) == false) {
                return false;
            }

            if (this.setNumbersAngle(formatAngle) == false) {
                return false;
            }
            
            return true;
        }
        
        /**
         * Read properties from a given Element object.
         * 
         * @param element
         *           an Element object
         * @return true if succeeded
         */
        protected boolean readProperties(final Element element) {
            
            if (this.readCommonProperties(element) == false) {
                return false;
            }
            
            String str = null;
            Boolean b = null;

            // axis visible
            str = element.getAttribute(SGIFigureElementAxis.KEY_AXIS_VISIBLE);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean axisVisible = b.booleanValue();
            
            if (this.setAxisVisible(axisVisible) == false) {
                return false;
            }

            return true;
        }

        public SGProperties getProperties() {
            AxisIndividualProperties p = new AxisIndividualProperties();
            if (this.getProperties(p) == false) {
                return null;
            }
            return p;
        }
        
        public boolean getProperties(SGProperties p) {
            AxisIndividualProperties ap = (AxisIndividualProperties) p;
            ap.location = this.getLocationInPlane();
            ap.scaleType = this.mAxis.getScaleType();
            SGTuple2d range = this.mAxis.getRange();
            ap.minValue = range.x;
            ap.maxValue = range.y;
            ap.invertCoordinatesFlag = this.isInvertCoordinates();
            ap.autoCalcAxisNumberFlag = this.isCalculateAutomatically();
            ap.incrementValue = this.getStepValue();
            ap.baseTickValue = this.getBaselineValue();
            ap.titleString = this.mTitle.getString();
            ap.axisVisibleFlag = this.isAxisVisible();
            ap.titleVisibleFlag = this.isTitleVisible();
            ap.scaleLinesVisibleFlag = this.isTickMarksVisible();
            ap.scaleNumbersVisibleFlag = this.isNumbersVisible();
            ap.scaleLinesInsideFlag = this.isTickMarksInside();
            ap.scaleNumbersIntegerFlag = this.isNumbersInteger();
            ap.exponentFlag = this.getExponentFlag();
            ap.exponentValue = this.getExponentValue();
            ap.scaleNumbersAngle = this.getNumbersAngle();

            return true;
        }

        /**
         * 
         */
        public boolean setProperties(SGProperties p) {
            AxisIndividualProperties ap = (AxisIndividualProperties) p;
            
            this.mLocationInPlane = ap.location;
            this.mAxis.setScale(ap.minValue, ap.maxValue, ap.scaleType);
            this.setInvertCoordinates(ap.invertCoordinatesFlag);
            this.setCalculateAutomatically(ap.autoCalcAxisNumberFlag);
            this.setStepValue(ap.incrementValue);
            this.setBaselineValue(ap.baseTickValue);
            this.setTitle(ap.titleString);
            this.setAxisVisible(ap.axisVisibleFlag);
            this.setTitleVisible(ap.titleVisibleFlag);
            this.setTickMarksVisible(ap.scaleLinesVisibleFlag);
            this.setNumbersVisible(ap.scaleNumbersVisibleFlag);
            this.setTickMarksInside(ap.scaleLinesInsideFlag);
            this.setNumbersInteger(ap.scaleNumbersIntegerFlag);
            this.setExponentFlag(ap.exponentFlag);
            this.setExponentValue(ap.exponentValue);
            this.setNumbersAngle(ap.scaleNumbersAngle);

            return true;
        }

        /**
		 * Sets the properties of this axis.
		 * 
		 * @param map
		 *            a map of properties
		 * @return the result of setting properties
		 */
        public SGPropertyResults setProperties(SGPropertyMap map) {
            SGPropertyResults result = new SGPropertyResults();
            
            // prepare
            if (this.prepare() == false) {
                return null;
            }
            
            result = this.setProperties(map, result);
            if (result == null) {
            	return null;
            }
            
            // commit the changes
            if (this.commit() == false) {
                return null;
            }
            SGFigureElementAxis.this.notifyToRoot();
            SGFigureElementAxis.this.notifyChange();
            SGFigureElementAxis.this.repaint();
            
            return result;
        }

        /**
		 * Sets the properties of this axis.
		 * 
		 * @param map
		 *            a map of properties
		 * @param iResult
		 *            the input result
		 * @return the updated result of setting properties
		 */
        protected SGPropertyResults setProperties(SGPropertyMap map,
        		SGPropertyResults iResult) {
            SGPropertyResults result = (SGPropertyResults) iResult.clone();
            
            Iterator<String> itr = map.getKeyIterator();
            while (itr.hasNext()) {
                String key = itr.next();
                String value = map.getValueString(key);
                
                if (COM_AXIS_VISIBLE.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_AXIS_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setAxisVisible(b.booleanValue()) == false) {
                        result.putResult(COM_AXIS_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_VISIBLE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_TITLE_VISIBLE.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_AXIS_TITLE_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setTitleVisible(b.booleanValue()) == false) {
                        result.putResult(COM_AXIS_TITLE_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_TITLE_VISIBLE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_TITLE.equalsIgnoreCase(key)) {
                	if (map.isDoubleQuoted(key) == false) {
                        result.putResult(COM_AXIS_TITLE, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}
                    if (this.setTitle(value) == false) {
                        result.putResult(COM_AXIS_TITLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_TITLE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_SCALE_MIN_VALUE.equalsIgnoreCase(key)) {
                    Double num = SGUtilityText.getDouble(value);
                    if (num == null) {
                        result.putResult(COM_AXIS_SCALE_MIN_VALUE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
                        result.putResult(COM_AXIS_SCALE_MIN_VALUE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setScale(num, null) == false) {
                        result.putResult(COM_AXIS_SCALE_MIN_VALUE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_SCALE_MIN_VALUE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_SCALE_MAX_VALUE.equalsIgnoreCase(key)) {
                    Double num = SGUtilityText.getDouble(value);
                    if (num == null) {
                        result.putResult(COM_AXIS_SCALE_MAX_VALUE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
                        result.putResult(COM_AXIS_SCALE_MAX_VALUE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setScale(null, num) == false) {
                        result.putResult(COM_AXIS_SCALE_MAX_VALUE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_SCALE_MAX_VALUE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_SCALE_RANGE.equalsIgnoreCase(key)) {
                    String[] strArray = SGUtilityText.getStringsInBracket(value);
                    if (strArray == null) {
                        result.putResult(COM_AXIS_SCALE_RANGE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    Double min = null;
                    Double max = null;
                    if (strArray.length >= 2) {
                        min = SGUtilityText.getDouble(strArray[0]);
                        max = SGUtilityText.getDouble(strArray[1]);
                    }
                    if (min == null || max == null) {
                        result.putResult(COM_AXIS_SCALE_RANGE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (SGUtility.isValidPropertyValue(min.doubleValue()) == false) {
                        result.putResult(COM_AXIS_SCALE_RANGE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (SGUtility.isValidPropertyValue(max.doubleValue()) == false) {
                        result.putResult(COM_AXIS_SCALE_RANGE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (strArray.length == 2) {
                        if (this.setScale(min, max) == false) {
                            result.putResult(COM_AXIS_SCALE_RANGE,
                                    SGPropertyResults.INVALID_INPUT_VALUE);
                            continue;
                        }
                    } else if (strArray.length == 3) {
                        final int scaleType = SGUtilityText.getScaleType(strArray[2]);
                        if (scaleType == -1) {
                            result.putResult(COM_AXIS_SCALE_RANGE,
                                    SGPropertyResults.INVALID_INPUT_VALUE);
                            continue;
                        }
                        if (this.setScale(min, max, scaleType) == false) {
                            result.putResult(COM_AXIS_SCALE_RANGE,
                                    SGPropertyResults.INVALID_INPUT_VALUE);
                            continue;
                        }
                    } else {
                        result.putResult(COM_AXIS_SCALE_RANGE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_SCALE_RANGE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_SCALE_TYPE.equalsIgnoreCase(key)) {
                    final int scaleType = SGUtilityText.getScaleType(value);
                    if (scaleType == -1) {
                        result.putResult(COM_AXIS_SCALE_TYPE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setScaleType(scaleType) == false) {
                        result.putResult(COM_AXIS_SCALE_TYPE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_SCALE_TYPE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_INVERT_COORDINATES.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_AXIS_INVERT_COORDINATES,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setInvertCoordinates(b.booleanValue()) == false) {
                        result.putResult(COM_AXIS_INVERT_COORDINATES,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_INVERT_COORDINATES,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_TICK_MARK_VISIBLE.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_AXIS_TICK_MARK_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setTickMarksVisible(b.booleanValue()) == false) {
                        result.putResult(COM_AXIS_TICK_MARK_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_TICK_MARK_VISIBLE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_TICK_MARK_AUTO.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_AXIS_TICK_MARK_AUTO,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setCalculateAutomatically(b.booleanValue()) == false) {
                        result.putResult(COM_AXIS_TICK_MARK_AUTO,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_TICK_MARK_AUTO,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_TICK_MARK_STEP.equalsIgnoreCase(key)) {
                    Double num = SGUtilityText.getDouble(value);
                    if (num == null) {
                        result.putResult(COM_AXIS_TICK_MARK_STEP,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
                        result.putResult(COM_AXIS_TICK_MARK_STEP,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setStepValue(num.doubleValue()) == false) {
                        result.putResult(COM_AXIS_TICK_MARK_STEP,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_TICK_MARK_STEP,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_TICK_MARK_BASE.equalsIgnoreCase(key)) {
                    Double num = SGUtilityText.getDouble(value);
                    if (num == null) {
                        result.putResult(COM_AXIS_TICK_MARK_BASE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
                        result.putResult(COM_AXIS_TICK_MARK_BASE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setBaselineValue(num.doubleValue()) == false) {
                        result.putResult(COM_AXIS_TICK_MARK_BASE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_TICK_MARK_BASE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_TICK_MARK_INNER.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_AXIS_TICK_MARK_INNER,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setTickMarksInside(b.booleanValue()) == false) {
                        result.putResult(COM_AXIS_TICK_MARK_INNER,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_TICK_MARK_INNER,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_NUMBER_VISIBLE.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_AXIS_NUMBER_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setNumbersVisible(b.booleanValue()) == false) {
                        result.putResult(COM_AXIS_NUMBER_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_NUMBER_VISIBLE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_NUMBER_INTEGER.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_AXIS_NUMBER_INTEGER,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setNumbersInteger(b.booleanValue()) == false) {
                        result.putResult(COM_AXIS_NUMBER_INTEGER,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_NUMBER_INTEGER,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_NUMBER_FORMAT_EXPONENT_VISIBLE.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_AXIS_NUMBER_FORMAT_EXPONENT_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setExponentFlag(b.booleanValue()) == false) {
                        result.putResult(COM_AXIS_NUMBER_FORMAT_EXPONENT_VISIBLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_NUMBER_FORMAT_EXPONENT_VISIBLE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_NUMBER_FORMAT_EXPONTNE_VALUE.equalsIgnoreCase(key)) {
                    Integer num = SGUtilityText.getInteger(value);
                    if (num == null) {
                        result.putResult(COM_AXIS_NUMBER_FORMAT_EXPONTNE_VALUE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setExponentValue(num.intValue()) == false) {
                        result.putResult(COM_AXIS_NUMBER_FORMAT_EXPONTNE_VALUE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_NUMBER_FORMAT_EXPONTNE_VALUE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_AXIS_NUMBER_ANGLE.equalsIgnoreCase(key)) {
                    Float num = SGUtilityText.getFloat(value);
                    if (num == null) {
                        result.putResult(COM_AXIS_NUMBER_ANGLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setNumbersAngle(num.intValue()) == false) {
                        result.putResult(COM_AXIS_NUMBER_ANGLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_AXIS_NUMBER_ANGLE,
                            SGPropertyResults.SUCCEEDED);
                }
            }
            
            return result;
        }

    }

    /**
     * 
     */
    public SGProperties getProperties() {
        List gList = this.getAxisGroupList();
        AxisProperties wp = new AxisProperties();
        wp.cp = this.getCommonProperties();
        wp.ipArray = new AxisIndividualProperties[gList.size()];
        for (int ii = 0; ii < wp.ipArray.length; ii++) {
            ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            wp.ipArray[ii] = (AxisIndividualProperties) group.getProperties();
        }
        return wp;
    }

    /**
     * 
     */
    public boolean setProperties(final SGProperties p) {

        if ((p instanceof AxisProperties) == false)
            return false;

        AxisProperties wp = (AxisProperties) p;

        if (this.setCommonProperties(wp.cp) == false) {
            return false;
        }

        AxisIndividualProperties[] array = wp.ipArray;
        for (int ii = 0; ii < array.length; ii++) {
            // ElementsGroupOfAxis group = this
            // .getAxisGroupInCube(array[ii].location);
            ElementsGroupOfAxis group = this
                    .getAxisGroup(array[ii].location);
            if (group.setProperties(array[ii]) == false) {
                return false;
            }
        }

        return true;
    }

    /**
     * 
     */
    private AxisCommonProperties getCommonProperties() {

        AxisCommonProperties p = new AxisCommonProperties();

        // p.viewDirection = this.mViewDirection;
        p.frameLinesVisibleFlag = this.mFrameLineVisibleFlag;
        p.frameLineWidth = this.getFrameLineWidth();
        p.frameLineColor = this.mFrameLineColor;
        p.lineColor = this.mLineColor;
        p.axisLineWidth = this.getAxisLineWidth();
        p.tickMarkWidth = this.getTickMarkWidth();
        p.tickMarkLength = this.mTickMarkLength;

        // strings
        p.stringColor = this.mAxisFontColor;
        p.titleFontName = this.mTitleFontName;
        p.titleFontStyle = this.mTitleFontStyle;
        p.titleFontSize = this.mTitleFontSize;
        p.scaleFontName = this.mScaleFontName;
        p.scaleFontStyle = this.mScaleFontStyle;
        p.scaleFontSize = this.mScaleFontSize;

        return p;
    }

    /**
     * 
     */
    private boolean setCommonProperties(AxisCommonProperties p) {

        // lines
        this.setFrameVisible(p.frameLinesVisibleFlag);
        this.setFrameLineWidth(p.frameLineWidth);
        this.setFrameLineColor(p.frameLineColor);
        this.setLineColor(p.lineColor);
        this.setAxisLineWidth(p.axisLineWidth);
        this.setTickMarkWidth(p.tickMarkWidth);
        this.setTickMarkLength(p.tickMarkLength);

        // strings
        this.setFontColor(p.stringColor);
        this.setTitleFontName(p.titleFontName);
        this.setTitleFontStyle(p.titleFontStyle);
        this.setTitleFontSize(p.titleFontSize);
        this.setScaleFontName(p.scaleFontName);
        this.setScaleFontStyle(p.scaleFontStyle);
        this.setScaleFontSize(p.scaleFontSize);

        return true;
    }

    /**
     * @author okumura
     */
    public static class AxisProperties extends SGProperties {
        AxisCommonProperties cp;

        /**
         * @uml.property name="ipArray"
         * @uml.associationEnd multiplicity="(0 -1)"
         */
        AxisIndividualProperties[] ipArray;
        
        AxisProperties() {
        }

        public void dispose() {
        	super.dispose();
            this.cp.dispose();
            this.cp = null;

            AxisIndividualProperties[] array = this.ipArray;
            for (int ii = 0; ii < array.length; ii++) {
                this.ipArray[ii].dispose();
            }
            this.ipArray = null;
        }

        /**
         * 
         */
        public boolean equals(final Object obj) {
            if ((obj instanceof AxisProperties) == false) {
                return false;
            }

            AxisProperties p = (AxisProperties) obj;

            if ((p.cp).equals(this.cp) == false)
                return false;
            if (p.ipArray.length != this.ipArray.length)
                return false;
            for (int ii = 0; ii < this.ipArray.length; ii++) {
                if (p.ipArray[ii].equals(this.ipArray[ii]) == false) {
                    return false;
                }
            }

            return true;
        }
    }

    /**
     * Properties common to all axes.
     */
    public static class AxisCommonProperties extends SGProperties {

        boolean frameLinesVisibleFlag;

        float frameLineWidth;

        Color frameLineColor;

        Color lineColor;

        float axisLineWidth;

        float tickMarkWidth;

        float tickMarkLength;

        // properties of strings
        Color stringColor;

        String titleFontName;

        int titleFontStyle;

        float titleFontSize;

        String scaleFontName;

        int scaleFontStyle;

        float scaleFontSize;

        public void dispose() {
        	super.dispose();
            this.frameLineColor = null;
            this.lineColor = null;
            this.stringColor = null;
            this.titleFontName = null;
            this.scaleFontName = null;
        }

        /**
         * 
         */
        public boolean equals(final Object obj) {

            if ((obj instanceof AxisCommonProperties) == false) {
                return false;
            }

            AxisCommonProperties p = (AxisCommonProperties) obj;
            if (p.frameLinesVisibleFlag != this.frameLinesVisibleFlag)
                return false;
            if (p.frameLineWidth != this.frameLineWidth)
                return false;
            if (p.frameLineColor.equals(this.frameLineColor) == false)
                return false;
            if (p.lineColor.equals(this.lineColor) == false)
                return false;
            if (p.stringColor.equals(this.stringColor) == false)
                return false;
            if (p.axisLineWidth != this.axisLineWidth)
                return false;
            if (p.tickMarkWidth != this.tickMarkWidth)
                return false;
            if (p.tickMarkLength != this.tickMarkLength)
                return false;

            // properties of strings
            if (p.titleFontSize != this.titleFontSize)
                return false;
            if (p.titleFontName.equals(this.titleFontName) == false)
                return false;
            if (p.titleFontStyle != this.titleFontStyle)
                return false;
            if (p.scaleFontSize != this.scaleFontSize)
                return false;
            if (p.scaleFontName.equals(this.scaleFontName) == false)
                return false;
            if (p.scaleFontStyle != this.scaleFontStyle)
                return false;

            return true;
        }

    }

    /**
     * Properties for each axis.
     */
    public static class AxisIndividualProperties extends SGProperties {
        int location;

        int scaleType;

        double minValue;

        double maxValue;

        boolean invertCoordinatesFlag;

        boolean autoCalcAxisNumberFlag;

        double incrementValue;

        double baseTickValue;

        String titleString;

        boolean axisVisibleFlag;

        boolean titleVisibleFlag;

        boolean scaleLinesVisibleFlag;

        boolean scaleNumbersVisibleFlag;

        boolean scaleLinesInsideFlag;

        boolean scaleNumbersIntegerFlag;

        boolean exponentFlag;

        int exponentValue;
        
        float scaleNumbersAngle;

        public void dispose() {
        	super.dispose();
            this.titleString = null;
        }

        /**
         * 
         */
        public boolean equals(final Object obj) {
            if ((obj instanceof AxisIndividualProperties) == false) {
                return false;
            }

            AxisIndividualProperties p = (AxisIndividualProperties) obj;

            if (p.location != this.location)
                return false;
            if (p.scaleType != this.scaleType)
                return false;
            if (p.minValue != this.minValue)
                return false;
            if (p.maxValue != this.maxValue)
                return false;
            if (p.invertCoordinatesFlag != this.invertCoordinatesFlag)
                return false;
            if (p.autoCalcAxisNumberFlag != this.autoCalcAxisNumberFlag)
                return false;
            if (p.incrementValue != this.incrementValue)
                return false;
            if (p.baseTickValue != this.baseTickValue)
                return false;
            if (p.titleString.equals(this.titleString) == false)
                return false;
            if (p.axisVisibleFlag != this.axisVisibleFlag)
                return false;
            if (p.titleVisibleFlag != this.titleVisibleFlag)
                return false;
            if (p.scaleLinesVisibleFlag != this.scaleLinesVisibleFlag)
                return false;
            if (p.scaleNumbersVisibleFlag != this.scaleNumbersVisibleFlag)
                return false;
            if (p.scaleLinesInsideFlag != this.scaleLinesInsideFlag)
                return false;
            if (p.scaleNumbersIntegerFlag != this.scaleNumbersIntegerFlag)
                return false;
            if (p.exponentFlag != this.exponentFlag)
                return false;
            if (p.exponentValue != this.exponentValue)
                return false;
            if (p.scaleNumbersAngle != this.scaleNumbersAngle)
                return false;

            return true;
        }

    }

    /**
     * Called when the key is pressed.
     */
    public void keyPressed(KeyEvent e) {
    }

    /**
     * Called when the key is released.
     */
    public void keyReleased(KeyEvent e) {
    }

    /**
     * Called when the key is typed.
     */
    public void keyTyped(KeyEvent e) {
	Object source = e.getSource();
	char c = e.getKeyChar();
	
	// if the text field is visible
	if (source.equals(this.mTextField)) {
	    
	    // hide the text field
	    if (c == KeyEvent.VK_ESCAPE) {
		this.hideEditField();
		this.mEditingStringElement = null;
	    }
	}
    }
    
    /**
     * Sets the dialog owner this figure element.
     * 
     * @param frame
     *            the dialog owner
     * @return true if succeeded
     */
    public boolean setDialogOwner(final Frame frame) {
        if (super.setDialogOwner(frame) == false) {
            return false;
        }
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (mPropertyDialog == null) {
                    mPropertyDialog = new SGAxisDialog(mDialogOwner, true,
                            new int[] { SGIFigureElementAxis.AXIS_HORIZONTAL_1,
                                    SGIFigureElementAxis.AXIS_HORIZONTAL_2,
                                    SGIFigureElementAxis.AXIS_PERPENDICULAR_1,
                                    SGIFigureElementAxis.AXIS_PERPENDICULAR_2});
                    mColorBarPropertyDialog = new SGColorBarDialog(mDialogOwner, true);
                }
            }
        });
        return true;
    }

    /**
     * Called when the text string in the text field is updated.
     * 
     * @param e
     *          a document event
     */
    public void changedUpdate(DocumentEvent e) {
        this.updateLabelTextField();
    }

    /**
     * Called when the text string in the text field is updated.
     * 
     * @param e
     *          a document event
     */
    public void insertUpdate(DocumentEvent e) {
        this.updateLabelTextField();
    }

    /**
     * Called when the text string in the text field is updated.
     * 
     * @param e
     *          a document event
     */
    public void removeUpdate(DocumentEvent e) {
        this.updateLabelTextField();
    }

    /**
     * Update the text field.
     *
     */
    private void updateLabelTextField() {
        
        // create the font
        final Font font = new Font(this.getTitleFontName(), this
                .getTitleFontStyle(), (int) (this.getTitleFontSize() * this
                .getMagnification()));
        
        // update the text field
        this.updateTextField(this.mTextField, font);
    }
    
    /**
     * Called when the caret in the text field is update.
     */
    public void caretUpdate(final CaretEvent e) {
//        final String str = this.mTextField.getText();
//        final Font font = new Font(this.getTitleFontName(), this
//                .getTitleFontStyle(), (int) (this.getTitleFontSize() * this
//                .getMagnification()));
//        final Rectangle2D stringRect = font.getStringBounds(str,
//                new FontRenderContext(null, false, false));
//
//        final double width = stringRect.getWidth();
//        if (width > this.mTextField.getWidth()) {
//            this.mTextField.setSize((int) (stringRect.getWidth() + this
//                    .getMagnification()
//                    * this.getTitleFontSize()), this.mTextField.getHeight());
//        }
    }

    /**
     * Called when menu items in the menu bar is selected.
     *
     */
    public void onMenuSelected() {
        if (this.mTextField.isVisible()) {
            this.closeTextField();
        }
    }

    /**
     * Returns the z-axis.
     * 
     * @return the z-axis
     */
    public SGAxis getZAxis() {
        return this.mZAxisElementsGroup.mAxis;
    }
    
    /**
     * Returns all axis groups.
     * 
     * @return the list of all axis groups
     */
    private List<ElementsGroupOfAxis> getAxisGroupList() {
        List<ElementsGroupOfAxis> list = new ArrayList<ElementsGroupOfAxis>();
        list.addAll(this.mElementsGroupList);
        return list;
    }

    /**
     * Returns all axis groups including the z-axis.
     * 
     * @return the list of all axis groups
     */
    private List<ElementsGroupOfAxis> getAllAxisGroupList() {
        List<ElementsGroupOfAxis> list = this.getAxisGroupList();
        if (this.mZAxisElementsGroup != null) {
            // add to the head of the list
            list.add(0, this.mZAxisElementsGroup);
        }
        return list;
    }

    /**
     * Returns whether the axis is available.
     * 
     * @param location
     *           location of an axis
     * @return true if the axis is available
     */
    public boolean isAxisAvailable(final String location) {
        if (SGIFigureElementAxis.AXIS_COLOR_BAR.equals(location)) {
            return true;
        } else if (SGIFigureElementAxis.AXIS_BOTTOM.equals(location)
                || SGIFigureElementAxis.AXIS_TOP.equals(location)
                || SGIFigureElementAxis.AXIS_LEFT.equals(location)
                || SGIFigureElementAxis.AXIS_RIGHT.equals(location)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Returns the color bar model.
     * 
     * @return the color bar model
     */
    public SGColorBarModel getColorBarModel() {
        return this.mColorBarModel;
    }

    
    protected ArrayList getUndoableChildList() {
        ArrayList uList = new ArrayList();
        uList.add(this.mZAxisElementsGroup);
        return uList;
    }

    /**
     * Returns the color bar if it exists.
     * 
     * @return the color bar
     */
    public SGIColorBar getColorBar() {
        return this.mZAxisElementsGroup;
    }
    
    /**
     * Returns whether the color bar is visible.
     * 
     * @return true if visible
     */
    public boolean isColorBarVisible() {
    	return this.mZAxisElementsGroup.isAxisVisible();
    }

    /**
     * Sets whether the color bar is visible.
     * 
     * @param true to set visible
     * 
     * @return true if succeeded
     */
    public boolean setColorBarVisible(final boolean b) {
        return this.mZAxisElementsGroup.setAxisVisible(b);
    }

    /**
     * Returns whether the color bar is available.
     * 
     * @return true if the color bar is available
     */
    public boolean isColorBarAvailable() {
        return this.mColorBarAvailableFlag;
    }
    

    /**
     * The class for the color bar.
     * 
     * @author kuromaru
     *
     */
    private class ElementsGroupColorBar extends ElementsGroupOfAxis 
            implements SGIColorBar, SGISelectable, SGIMovable, SGIColorBarDialogObserver,
            SGIColorBarConstants, SGIFigureConstants, SGIUndoable {

        /**
         * Relative x-coordinate from the origin of the graph rectangle
         * at the default magnification.
         */
        float mX = 0.0f;

        /**
         * Relative y-coordinate from the origin of the graph rectangle
         * at the default magnification.
         */
        float mY = 0.0f;

        /**
         * Bar width of the color bar at the default magnification.
         */
        float mBarWidth = 0.0f;

        /**
         * Length of the axis line and the color bar at the default magnification.
         */
        float mBarLength = 0.0f;
        
        /**
         * Space between an axis line and scale numbers.
         */
        float mSpaceAxisLineAndNumber;

        /**
         * Space between scale numbers and the title.
         */
        float mSpaceNumberAndTitle;

        /**
         * The direction of color bar.
         */
        String mDirection = DIRECTION_HORIZONTAL_LOWER;

        /**
         * Color of lines.
         */
        Color mLineColor = DEFAULT_LINE_COLOR;

        /**
         * Width of axis lines.
         */
        float mFrameLineWidth = DEFAULT_AXIS_LINE_WIDTH;

//        /**
//         * Width of tick mark lines.
//         */
//        float mTickMarkWidth = DEFAULT_TICK_MARK_LINE_WIDTH;

        /**
         * Length of tick mark lines.
         */
        float mTickMarkLength = DEFAULT_TICK_MARK_LINE_WIDTH;

        /**
         * Color of the title and scale strings.
         */
        Color mFontColor = DEFAULT_FONT_COLOR;

        /**
         * Font name of the title.
         */
        String mTitleFontName = DEFAULT_FONT_NAME;

        /**
         * Font size of the title.
         */
        float mTitleFontSize = DEFAULT_FONT_SIZE;

        /**
         * Font style of the title.
         */
        int mTitleFontStyle = DEFAULT_FONT_STYLE;

        /**
         * Font name of scale.
         */
        String mScaleFontName = DEFAULT_FONT_NAME;

        /**
         * Font size of the scale.
         */
        float mScaleFontSize = DEFAULT_FONT_SIZE;

        /**
         * Font size of the scale.
         */
        int mScaleFontStyle = DEFAULT_FONT_STYLE;

        /**
         * A flag whether this object is selected.
         */
        boolean mSelectedFlag = false;

        /**
         * The x-axis for the location of this object.
         */
        SGAxis mXAxis = null;
        
        /**
         * The y-axis for the location of this object.
         */
        SGAxis mYAxis = null;
        
        /**
         * The line stroke for the tick marks.
         */
        SGStroke mTickMarkStroke = new SGStroke();

        /**
         * Builds a object.
         *
         */
        ElementsGroupColorBar() {
            super();
            this.init();
        }
        
        // initialization
        private void init() {

            // set axes
            SGAxis xAxis = SGFigureElementAxis.this.getAxis(DEFAULT_COLOR_BAR_HORIZONTAL_AXIS);
            SGAxis yAxis = SGFigureElementAxis.this.getAxis(DEFAULT_COLOR_BAR_PERPENDICULAR_AXIS);
            this.mXAxis = xAxis;
            this.mYAxis = yAxis;

            // lines
            this.setFrameLineWidth(DEFAULT_AXIS_LINE_WIDTH, LINE_WIDTH_UNIT);
            this.setTickMarkWidth(DEFAULT_TICK_MARK_LINE_WIDTH, LINE_WIDTH_UNIT);
            this.setTickMarkLength(DEFAULT_TICK_MARK_LENGTH, AXIS_TICK_LENGTH_UNIT);
            this.setLineColor(DEFAULT_LINE_COLOR);

            // font
            this.setFontColor(DEFAULT_FONT_COLOR);
            this.setFontName(DEFAULT_FONT_NAME);
            this.setFontSize(DEFAULT_FONT_SIZE, FONT_SIZE_UNIT);
            this.setFontStyle(DEFAULT_FONT_STYLE);
            
            // size
            this.setBarWidth(DEFAULT_COLOR_BAR_WIDTH, COLOR_BAR_SIZE_UNIT);
            this.setBarLength(DEFAULT_COLOR_BAR_LENGTH, COLOR_BAR_SIZE_UNIT);

            // space
            this.setSpaceToScale(
                    SGIFigureConstants.DEFAULT_SPACE_AXIS_LINE_AND_NUMBER,
                    SGIFigureConstants.FIGURE_SPACE_UNIT);
            this.setSpaceToTitle(
                    SGIFigureConstants.DEFAULT_SPACE_NUMBER_AND_TITLE,
                    SGIFigureConstants.FIGURE_SPACE_UNIT);
        }

        /**
         * Create a pop-up menu.
         * @return
         *         a pop-up menu
         */
        protected JPopupMenu createPopupMenu() {
            JPopupMenu p = new JPopupMenu();
            
            p.setBounds(0, 0, 100, 100);

            p.add(new JLabel("  -- Color Bar --"));
            p.addSeparator();

            SGUtility.addItem(p, this, MENUCMD_HIDE_COLOR_BAR);

            p.addSeparator();

            SGUtility.addItem(p, this, MENUCMD_PROPERTY);

            return p;
        }

        public void actionPerformed(ActionEvent e) {
            super.actionPerformed(e);
            String command = e.getActionCommand();
            if (command.equals(MENUCMD_HIDE_COLOR_BAR)) {
                this.mVisibleFlag = false;
                repaint();
                setChanged(true);
                notifyToRoot();
                return;
            }
        }

        /**
         * Creates a drawing element of a scale number.
         */
        protected ElementStringOfScale createSingleTickMarkInstance(
                final String str) {
            ElementStringOfScale el = new ElementStringOfScale(str,
                    this.mScaleFontName, this.mScaleFontStyle,
                    this.mScaleFontSize, this.mFontColor,
                    SGFigureElementAxis.this.mMagnification,
                    this.mNumbersAngle, this.getLocationInPlane());
            return el;
        }

        /**
         * Creates a string element for the exponent.
         */
        protected SGDrawingElementString2DExtended createExponentStringElement(
                final String str) {
            SGDrawingElementString2DExtended el = new SGDrawingElementString2DExtended(
                    str, this.mScaleFontName, this.mScaleFontStyle,
                    this.mScaleFontSize, this.mFontColor,
                    SGFigureElementAxis.this.mMagnification, 0.0f);
            return el;
        }

        protected int getLocationInPlane() {
            if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)) {
                return AXIS_NORMAL_HORIZONTAL_LOWER;
            } else if (DIRECTION_HORIZONTAL_UPPER.equals(this.mDirection)) {
                return AXIS_NORMAL_HORIZONTAL_UPPER;
            } else if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)) {
                return AXIS_NORMAL_PERPENDICULAR_LEFT;
            } else if (DIRECTION_PERPENDICULAR_RIGHT.equals(this.mDirection)) {
                return AXIS_NORMAL_PERPENDICULAR_RIGHT;
            } else {
                return -1;
            }
        }
        
        protected int getLocation() {
        	return AXIS_NORMAL;
        }

        /**
         * Set properties to axis lines.
         * 
         * @return true if succeeded
         */
        protected boolean setAxisLineProperties() {
            for (int ii = 0; ii < this.mAxisLines.length; ii++) {
                this.mAxisLines[ii].setLineWidth(this.mFrameLineWidth);
                this.mAxisLines[ii].setColor(this.mLineColor);
            }
            return true;
        }

        /**
         * Set properties of the title.
         * 
         * @return true if succeeded
         */
        protected boolean setTitleProperties() {
            this.mTitle.setColor(this.mFontColor);
            this.mTitle.setFont(this.mTitleFontName,
                    this.mTitleFontStyle, this.mTitleFontSize);
            return true;
        }

        /**
         * Creates axis lines.
         * 
         * @return axis lines
         */
        protected ElementLineOfAxis[] createAxisLines() {
            ElementLineOfAxis[] array = new ElementLineOfAxis[4];
            for (int ii = 0; ii < array.length; ii++) {
                array[ii] = new ElementLineOfAxis();
            }
            return array;
        }

        /**
         * Create the scale lines.
         * 
         * @return true if succeeded
         */
        protected boolean createTickMarks() {
            // clear the list
            this.mTickMarksList.clear();

            final float lw = this.mTickMarkStroke.getLineWidth();
            
            // create
            boolean flag = true;
            switch (this.mAxis.getScaleType()) {
            case SGAxis.LINEAR_SCALE:
                flag = this.createTickMarksInLinearScale(
                        lw, this.mTickMarkLength, this.mLineColor);
                break;
            case SGAxis.LOG_SCALE:
                flag = this.createTickMarksInLogScale(
                        lw, this.mTickMarkLength, this.mLineColor);
                break;
            default:
                throw new Error();
            }

            return flag;
        }

        /**
         * Sets the location of the color bar.
         * 
         * @param x
         *           x coordinate
         * @param y
         *           y coordinate
         */
        public void setLocation(final float x, final float y) {
            this.mX = (x - mGraphRectX) / mMagnification;
            this.mY = (y - mGraphRectY) / mMagnification;
        }
        
        /**
         * Sets the size of the color bar.
         * 
         * @param len
         *           the length
         * @param barWidth
         *           the bar width
         */
        public void setSize(final float len, final float barWidth) {
            this.mBarLength = len;
            this.mBarWidth = barWidth;
        }
        
        /**
         * Returns the x-coordinate of the color bar in the fiugre.
         * @return
         *         the x-coordinate of the color bar in the fiugre
         */
        public float getX() {
            return mGraphRectX + this.mX * mMagnification;
        }

        /**
         * Returns the y-coordinate of the color bar in the fiugre.
         * @return
         *         the y-coordinate of the color bar in the fiugre
         */
        public float getY() {
            return mGraphRectY + this.mY * mMagnification;
        }
        
        /**
         * Returns the length multiplied by the magnification.
         * 
         * @return the length of the color bar multiplied by the magnification
         */
        public float getLength() {
            return this.mBarLength * mMagnification;
        }

        /**
         * Returns the bar width multiplied by the magnification.
         * 
         * @return the bar width of the color bar multiplied by the magnification
         */
        public float getBarWidth() {
            return this.mBarWidth * mMagnification;
        }
        
        /**
         * Returns the bounds of color bar.
         * 
         * @return the bounds of color bar
         */
        public Rectangle2D getColorBarRectangle() {
            final double len = this.getLength();
            final double size = this.getBarWidth();
            final double w, h;
            if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)
                    || DIRECTION_HORIZONTAL_UPPER.equals(this.mDirection)) {
                w = len;
                h = size;
            } else if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)
                    || DIRECTION_PERPENDICULAR_RIGHT.equals(this.mDirection)) {
                w = size;
                h = len;
            } else {
                throw new Error();
            }
            Rectangle2D rect = new Rectangle2D.Double(
                    this.getX(), this.getY(), w, h);
            return rect;
        }

        /**
         * Draw all objects.
         * 
         * @param g2d
         *           the graphic context
         */
        public void paintGraphics2D(final Graphics2D g2d) {
            
            if (!this.mVisibleFlag) {
                return;
            }
            
            // get the bounds of the color bar
            Rectangle2D bounds = this.getColorBarRectangle();

            final double x, y, w, h;
            final int nStart, nEnd;
            final double stepX, stepY;
            final double size = this.getBarWidth();
            if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)
                    || DIRECTION_HORIZONTAL_UPPER.equals(this.mDirection)) {
                x = bounds.getMinX();
                y = bounds.getMinY();
                w = 1.0;
                h = size;
                nStart = (int) x;
                nEnd = (int) bounds.getMaxX();
                stepX = 1.0;
                stepY = 0.0;
            } else if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)
                    || DIRECTION_PERPENDICULAR_RIGHT.equals(this.mDirection)) {
                x = bounds.getMinX();
                y = bounds.getMaxY();
                w = size;
                h = 1.0;
                nStart = (int) y;
                nEnd = (int) bounds.getMinY();
                stepX = 0.0;
                stepY = - 1.0;
            } else {
                throw new Error();
            }

            // get the value range of the color bar
            final double zMin = SGFigureElementAxis.this.mColorBarModel.getMinValue();
            final double zMax = SGFigureElementAxis.this.mColorBarModel.getMaxValue();
            final double zRange = zMax - zMin;

            // fill the rectangle
            final Rectangle2D rect = new Rectangle2D.Double();
            final int nDiff = Math.abs(nEnd - nStart);
            for (int ii = 0; ii <= nDiff; ii++) {
                rect.setRect(x + stepX * ii, y + stepY * ii, w, h);
                double ratio = (double) ii / nDiff;
                if (this.mAxis.isInvertCoordinates()) {
                    ratio = 1.0 - ratio;
                }
                final double zValue = zMin + ratio * zRange;
                final Color cl = SGFigureElementAxis.this.mColorBarModel.evaluate(
                        zValue, SGAxis.LINEAR_SCALE);
                g2d.setColor(cl);
                g2d.fill(rect);
            }
            
            // draw axis lines and numbers
            super.paintGraphics2D(g2d);
            
            // draw focused anchors
            if (SGFigureElementAxis.this.mSymbolsVisibleFlagAroundFocusedObjects
                    && this.isSelected()) {
                ArrayList pList = this.getAnchorPointList();
                SGUtilityForFigureElementJava2D.drawAnchorAsFocusedObject(
                        pList, g2d);
            }

            // draw child anchors
            if (SGFigureElementAxis.this.mSymbolsVisibleFlagAroundAllObjects) {
                ArrayList pList = this.getAnchorPointList();
                SGUtilityForFigureElementJava2D.drawAnchorAsChildObject(pList,
                        g2d);
            }
        }
        
        /**
         * Returns a list of points to draw anchors.
         * 
         * @return
         *         a list of points to draw anchors
         */
        private ArrayList getAnchorPointList() {
            ArrayList list = new ArrayList();
            Rectangle2D rect = this.getColorBarRectangle();
            final float x = (float) rect.getX();
            final float y = (float) rect.getY();
            final float w = (float) rect.getWidth();
            final float h = (float) rect.getHeight();
            Point2D nw = new Point2D.Float(x, y);
            Point2D sw = new Point2D.Float(x, y + h);
            Point2D ne = new Point2D.Float(x + w, y);
            Point2D se = new Point2D.Float(x + w, y + h);
            list.add(nw);
            list.add(sw);
            list.add(ne);
            list.add(se);
            return list;
        }
        
        /**
         * Called when the mouse is pressed.
         */
        public boolean onMousePressed(final MouseEvent e) {
            if (super.onMousePressed(e)) {
                return true;
            }
            Rectangle2D bounds = this.getColorBarRectangle();
            if (bounds.contains(e.getPoint())) {
                this.prepare();
                return true;
            }
            return false;
        }

        /**
         * Called when the mouse is released.
         */
        public boolean onMouseReleased(final MouseEvent e) {
            if (!this.mVisibleFlag) {
                return false;
            }

            if (this.mTempRange != null || this.mDraggingElement != null) {
                // if a number or a tick mark is dragged
                
                this.mTempRange = null;
                this.mDraggingElement = null;

                if (SGFigureElementAxis.this.createAllDrawingElements() == false) {
                    return false;
                }
                SGFigureElementAxis.this.initializePressedElement();

                SGProperties pTemp = this.mTemporaryProperties;
                SGProperties pPresent = this.getProperties();
                if (pTemp.equals(pPresent) == false) {
                    this.setChanged(true);
                }
                this.mTemporaryProperties = null;

                return true;
                
            } else {
                // if the color bar itself is dragged
                
                if (this.mTemporaryProperties != null) {
                    SGProperties pTemp = this.mTemporaryProperties;
                    SGProperties pPresent = this.getProperties();
                    if (pTemp.equals(pPresent) == false) {
                        this.setChanged(true);
                    }
                    this.mTemporaryProperties = null;

                    return true;
                }
            }
            
            return false;
        }
        
        /**
         * Check whether a given point is on drawing element.
         */
        public boolean onDrawingElement(final int x, final int y) {
            if (!this.mVisibleFlag) {
                return false;
            }
            if (super.onDrawingElement(x, y)) {
                return true;
            }
            Rectangle2D bounds = this.getColorBarRectangle();
            if (bounds.contains(x, y)) {
                return true;
            }
            return false;
        }

        /**
         * Called when the mouse is clicked.
         * 
         * @param e
         *          the mouse event
         * @return true if some object is clicked
         */
        public boolean onMouseClicked(final MouseEvent e) {
            if (!this.mVisibleFlag) {
                return false;
            }
            if (super.onMouseClicked(e)) {
                return true;
            }
            Rectangle2D bounds = this.getColorBarRectangle();
            if (bounds.contains(e.getPoint())) {
                // update focused objects
                SGFigureElementAxis.this.updateFocusedObjectsList(this, e);

                if (this.clicked(e)) {
                    return true;
                }
            }
            return false;
        }

        /**
         * Called when the mouse is dragged.
         */
        public boolean onMouseDragged(final MouseEvent e) {
            if (!this.mVisibleFlag) {
                return false;
            }
            if (SGFigureElementAxis.this.mPressedPoint == null) {
                return false;
            }
            final int x = SGFigureElementAxis.this.mPressedPoint.x;
            final int y = SGFigureElementAxis.this.mPressedPoint.y;

            // set the location of the color bar
            if (this.isSelected() && this.mDraggingElement == null
                    && SGFigureElementAxis.this.mPressedElement == null) {
                final int dx = e.getX() - x;
                final int dy = e.getY() - y;
                this.translate(dx, dy);
                SGFigureElementAxis.this.mPressedPoint = e.getPoint();
                this.createDrawingElements();
                return true;
            }
            
            if (super.onMouseDragged(e) == false) {
                return false;
            }

            // update the range of the color model
            SGFigureElementAxis.this.mColorBarModel.setValueRange(
                    this.mAxis.getMinValue(), this.mAxis.getMaxValue());
            return true;
        }
        
        /**
         * Sets whether this object is selected.
         * 
         * @param b
         *          true to set selected
         */
        public void setSelected(final boolean b) {
            this.mSelectedFlag = b;
        }

        /**
         * Returns whether this object is selected.
         * 
         * @return true if this object is selected
         */
        public boolean isSelected() {
            return this.mSelectedFlag;
        }
        
        /**
         * Sets the location of the axis line.
         * 
         * @return true if succeeded
         */
        protected boolean setLocationOfAxisLines() {
            float x1;
            float y1;
            float x2;
            float y2;
            if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)
                    || DIRECTION_HORIZONTAL_UPPER.equals(this.mDirection)) {
                x1 = this.getX();
                y1 = this.getY();
                x2 = x1 + this.getLength();
                y2 = y1 + this.getBarWidth();
                this.mAxisLines[0].setTermPoints(x1, y1, x2, y1);
                this.mAxisLines[1].setTermPoints(x1, y1, x1, y2);
                this.mAxisLines[2].setTermPoints(x1, y2, x2, y2);
                this.mAxisLines[3].setTermPoints(x2, y1, x2, y2);
            } else if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)
                    || DIRECTION_PERPENDICULAR_RIGHT.equals(this.mDirection)) {
                x1 = this.getX();
                y1 = this.getY();
                x2 = x1 + this.getBarWidth();
                y2 = y1 + this.getLength();
                this.mAxisLines[0].setTermPoints(x1, y2, x1, y1);
                this.mAxisLines[1].setTermPoints(x1, y2, x2, y2);
                this.mAxisLines[2].setTermPoints(x2, y2, x2, y1);
                this.mAxisLines[3].setTermPoints(x1, y1, x2, y1);
            } else {
                throw new Error();
            }
            return true;
        }

        /**
         * Creates a scale line.
         * 
         * @param value
         *           axis value of the scale line
         * @param length
         *           the length of the scale line
         * @return an instance of the scale line
         */
        protected ElementLineOfScale createSingleTickMark(final double value,
                final float width, final float length) {

            if (this.mAxis.insideRange(value) == false) {
                return null;
            }

            final float mag = SGFigureElementAxis.this.mMagnification;
            final boolean inside = this.isTickMarksInside();
            final boolean invcoord = this.isInvertCoordinates();

            double min;
            double max;
            double valueInScale;
            switch (this.mAxis.getScaleType()) {
            case SGAxis.LINEAR_SCALE: {
                min = this.mAxis.getMinValue();
                max = this.mAxis.getMaxValue();
                valueInScale = value;
                break;
            }

            case SGAxis.LOG_SCALE: {
                min = Math.log(this.mAxis.getMinValue());
                max = Math.log(this.mAxis.getMaxValue());
                valueInScale = Math.log(value);
                break;
            }

            default: {
                throw new Error();
            }
            }

            ElementLineOfScale el = null;
            final SGTuple2f start;
            final SGTuple2f end;
            final float len = this.getLength();
            if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)
                    || DIRECTION_HORIZONTAL_UPPER.equals(this.mDirection)) {
                final double x;
                if (invcoord) {
                    x = this.getX() + len * (max - valueInScale) / (max - min);
                } else {
                    x = this.getX() + len * (valueInScale - min) / (max - min);
                }
                final double yStart = this.getY() + this.getBarWidth();
                final double yEnd;
                if (inside) {
                    yEnd = yStart - length;
                } else {
                    yEnd = yStart + length;
                }
                start = new SGTuple2f((float) x, (float) yStart);
                end = new SGTuple2f((float) x, (float) yEnd);
                
            } else if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)
                    || DIRECTION_PERPENDICULAR_RIGHT.equals(this.mDirection)) {
                final double y;
                if (invcoord) {
                    y = this.getY() + len * (1.0 - (max - valueInScale) / (max - min));
                } else {
                    y = this.getY() + len * (1.0 - (valueInScale - min) / (max - min));
                }
                final double xStart = this.getX();
                final double xEnd;
                if (inside) {
                    xEnd = xStart + length;
                } else {
                    xEnd = xStart - length;
                }
                start = new SGTuple2f((float) xStart, (float) y);
                end = new SGTuple2f((float) xEnd, (float) y);

            } else {
                throw new Error();
            }
            
            el = new ElementLineOfColorBarScale(start, end);
            el.setLineWidth(width);
            el.setMagnification(mag);
            
            return el;
        }

        /**
         * Creates a scale line at the counter position.
         * 
         * @param value
         *            axis value of the scale line
         * @param length
         *            the length of the scale line
         * @return an instance of the scale line
         */
        protected ElementLineOfScale createSingleTickMarkCounter(final double value,
                final float width, final float length) {
            ElementLineOfScale el = this.createSingleTickMark(value, width, length);
            if (el == null) {
                return null;
            }
            
            // change the location
            final SGTuple2f start = el.getStart();
            final SGTuple2f end = el.getEnd();
            final boolean inside = this.isTickMarksInside();
            if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)
                    || DIRECTION_HORIZONTAL_UPPER.equals(this.mDirection)) {
                final double yStart = start.y - this.getBarWidth();
                final double yEnd;
                if (inside) {
                    yEnd = yStart + length;
                } else {
                    yEnd = yStart - length;
                }
                start.y = (float) yStart;
                end.y = (float) yEnd;
            } else if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)
                    || DIRECTION_PERPENDICULAR_RIGHT.equals(this.mDirection)) {
                final double xStart = start.x + this.getBarWidth();
                final double xEnd;
                if (inside) {
                    xEnd = xStart - length;
                } else {
                    xEnd = xStart + length;
                }
                start.x = (float) xStart;
                end.x = (float) xEnd;
            } else {
                throw new Error();
            }

            return el;
        }
        
        /**
         * Sets the location of the title.
         * 
         * @return true if succeeded
         */
        protected boolean setLocationOfTitle() {
            final float mag = SGFigureElementAxis.this.mMagnification;
            final float spaceLN = this.mSpaceAxisLineAndNumber * mag;
            final float spaceNT = this.mSpaceNumberAndTitle * mag;
            final float barWidth = this.getBarWidth();
            float scaleNumberHeight = 0.0f;
            float maxWidth = 0.0f;
            scaleNumberHeight = (float) getScaleHeight();
            maxWidth = getMaxLengthOfScaleNumbers(this);
            Rectangle2D rectTitle = this.mTitle.getElementBounds();
            if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)
                    || DIRECTION_HORIZONTAL_UPPER.equals(this.mDirection)) {
                this.mTitle.setAngle(0.0f);
                final float x = this.getX() + (this.getLength() - (float) rectTitle.getWidth()) / 2.0f;
                final float y;
                if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)) {
                    y = this.getY() + barWidth + spaceLN + scaleNumberHeight + spaceNT;
                } else {
                    y = this.getY() - (spaceLN + scaleNumberHeight + spaceNT + (float) rectTitle.getHeight());
                }
                this.mTitle.setLocation(x, y);
            } else if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)
                    || DIRECTION_PERPENDICULAR_RIGHT.equals(this.mDirection)) {
                this.mTitle.setAngle(90.0f); // rotation of 90 degree
                final float y = this.getY() + (this.getLength() + (float) rectTitle.getHeight()) / 2.0f;
                final float x;
                if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)) {
                    x = this.getX() - (spaceLN + maxWidth + spaceNT + (float) rectTitle.getWidth());
                } else {
                    x = this.getX() + barWidth + (spaceLN + maxWidth + spaceNT);
                }
                this.mTitle.setLocation(x, y);
            } else {
                throw new Error();
            }
            
            return true;
        }

        /**
         * Sets the location of scale numbers.
         * 
         * @return true if succeeded
         */
        protected boolean setLocationOfScaleNumbers() {

            final boolean invcoord = this.isInvertCoordinates();
            final int scaleType = this.mAxis.getScaleType();
            final float mag = SGFigureElementAxis.this.mMagnification;
            final float spaceLN = this.mSpaceAxisLineAndNumber * mag;

            double[] valueArrayInScale = null;
            if (scaleType == SGAxis.LINEAR_SCALE) {
                valueArrayInScale = this.mAxisValueArray;
            } else if (scaleType == SGAxis.LOG_SCALE) {
                valueArrayInScale = new double[this.mAxisValueArray.length];
                for (int ii = 0; ii < valueArrayInScale.length; ii++) {
                    valueArrayInScale[ii] = Math.log(this.mAxisValueArray[ii]);
                }
            }

            double axisMinInScale = 0.0;
            double axisMaxInScale = 0.0;
            if (scaleType == SGAxis.LINEAR_SCALE) {
                axisMinInScale = this.mAxis.getMinValue();
                axisMaxInScale = this.mAxis.getMaxValue();
            } else if (scaleType == SGAxis.LOG_SCALE) {
                axisMinInScale = Math.log(this.mAxis.getMinValue());
                axisMaxInScale = Math.log(this.mAxis.getMaxValue());
            }

            final float barWidth = this.getBarWidth();
            final float barLength = this.getLength();
            for (int ii = 0; ii < this.mNumberList.size(); ii++) {
                ElementStringOfScale el = (ElementStringOfScale) this.mNumberList
                        .get(ii);
                Rectangle2D rect = el.getElementBounds();
                final float width = (float) rect.getWidth();
                final float height = (float) rect.getHeight();
                if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)
                        || DIRECTION_HORIZONTAL_UPPER.equals(this.mDirection)) {
                    
                    final float x;
                    if (invcoord) {
                        x = (float) (this.getX() + barLength
                                * (axisMaxInScale - valueArrayInScale[ii])
                                / (axisMaxInScale - axisMinInScale));
                    } else {
                        x = (float) (this.getX() + barLength
                                * (valueArrayInScale[ii] - axisMinInScale)
                                / (axisMaxInScale - axisMinInScale));
                    }
                    
                    final float y;
                    if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)) {
                        y = this.getY() + barWidth + spaceLN;
                    } else {
                        y = this.getY() - spaceLN - (float) rect.getHeight();
                    }
                    el.setLocation((x - 0.50f * width), y);
                    
                } else if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)
                        || DIRECTION_PERPENDICULAR_RIGHT.equals(this.mDirection)) {
                    
                    final float y;
                    if (invcoord) {
                        y = (float) (this.getY() + barLength
                                * (1.0 - (axisMaxInScale - valueArrayInScale[ii])
                                        / (axisMaxInScale - axisMinInScale)));
                    } else {
                        y = (float) (this.getY() + barLength
                                * (1.0 - (valueArrayInScale[ii] - axisMinInScale)
                                        / (axisMaxInScale - axisMinInScale)));
                    }
                    
                    final float x;
                    if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)) {
                        x = (float) (this.getX() - spaceLN - rect.getWidth());
                    } else {
                        x = (float) (this.getX() + barWidth + spaceLN);
                    }
                    el.setLocation(x, (y - 0.5f * height));
                    
                } else {
                    throw new Error();
                }
            }

            // the exponent drawing element
            if (this.getExponentFlag()) {
                this.setLocationOfExponentDrawingElement();
            }

            return true;
        }

        /**
         * Sets the location of exponent object.
         * 
         * @return true if succeeded
         */
        protected boolean setLocationOfExponentDrawingElement() {
            SGDrawingElementString2D el = this.mExponentDrawingElement;

            final Rectangle2D rect = el.getElementBounds();
            final float mag = SGFigureElementAxis.this.mMagnification;
            final float spaceLN = this.mSpaceAxisLineAndNumber * mag;
            final float spaceNT = this.mSpaceNumberAndTitle * mag;
            final float barLength = this.getLength();
            final float barWidth = this.getBarWidth();

            float x = 0.0f;
            float y = 0.0f;
            
            if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)
                    || DIRECTION_HORIZONTAL_UPPER.equals(this.mDirection)) {
                x = this.getX() + barLength;
                if (DIRECTION_HORIZONTAL_LOWER.equals(this.mDirection)) {
                    y = this.getY() + barWidth + spaceLN
                            + (float) getScaleHeight() + spaceNT;
                } else {
                    y = this.getY() + - (spaceLN
                            + (float) getScaleHeight() + spaceNT + (float) rect.getHeight());
                }
            } else if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)
                    || DIRECTION_PERPENDICULAR_RIGHT.equals(this.mDirection)) {
                y = this.getY() - (float) rect.getHeight();
                if (DIRECTION_PERPENDICULAR_LEFT.equals(this.mDirection)) {
                    x = this.getX() - (float) (spaceLN + getMaxLengthOfScaleNumbers(this)
                            + spaceNT + rect.getWidth());
                } else {
                    x = this.getX() + barWidth + (float) (spaceLN 
                            + getMaxLengthOfScaleNumbers(this) + spaceNT);
                }
            } else {
                throw new Error();
            }
            
            el.setLocation(x, y);

            return true;
        }

        /**
         * Translate this object.
         * 
         * @param dx
         *          displacement to the horizontal axis direction
         * @param dy
         *          displacement to the perpendicular axis direction
         */
        public void translate(final float dx, final float dy) {
            this.setLocation(this.getX() + dx, this.getY() + dy);
        }

        /**
         * Shows the property dialog.
         * 
         */
        protected void showPropertyDialog() {
            // before showing dialog
            mCurrentPropertyDialog = mColorBarPropertyDialog;
            
            // show the dialog
            SGFigureElementAxis.this.setPropertiesOfSelectedObjects();

            // after hid the dialog
            mCurrentPropertyDialog = null;
        }
        
        public int getXAxisLocation() {
            return SGFigureElementAxis.this.getLocationInPlane(this.mXAxis);
        }

        public int getYAxisLocation() {
            return SGFigureElementAxis.this.getLocationInPlane(this.mYAxis);
        }

        public double getXValue() {
            SGAxis axis = this.mXAxis;
            double value = calcValue(this.getX(), axis, true);
            if (Double.isNaN(value)) {
                return value;
            }
            value = getNumberInRangeOrder(value, axis);
            return value;
        }

        public double getYValue() {
            SGAxis axis = this.mYAxis;
            double value = calcValue(this.getY(), axis, false);
            if (Double.isNaN(value)) {
                return value;
            }
            value = getNumberInRangeOrder(value, axis);
            return value;
        }

        public float getBarWidth(String unit) {
            return (float) SGUtilityText.convertFromPoint(this.mBarWidth, unit);
        }

        public float getBarLength(String unit) {
            return (float) SGUtilityText.convertFromPoint(this.mBarLength, unit);
        }

        public String getDirection() {
            return this.mDirection;
        }

        public boolean isReversedOrder() {
            return SGFigureElementAxis.this.mColorBarModel.isReversedOrder();
        }

        public String getColorBarStyle() {
            return mColorBarStyleManager.getColorBarName(SGFigureElementAxis.this.mColorBarModel);
        }

        public Color getFontColor() {
            return this.mFontColor;
        }

        public String getFontName() {
            return this.mTitleFontName;
        }

        public float getFontSize(String unit) {
            return (float) SGUtilityText.convertFromPoint(this.mTitleFontSize, unit);
        }

        public int getFontStyle() {
            return this.mTitleFontStyle;
        }

        public float getSpaceToScale(String unit) {
            return (float) SGUtilityText.convertFromPoint(this.mSpaceAxisLineAndNumber, unit);
        }

        public float getSpaceToTitle(String unit) {
            return (float) SGUtilityText.convertFromPoint(this.mSpaceNumberAndTitle, unit);
        }

        public float getFrameLineWidth(String unit) {
            return (float) SGUtilityText.convertFromPoint(this.mFrameLineWidth, unit);
        }

        public Color getLineColor() {
            return this.mLineColor;
        }

        public float getTickMarkWidth(String unit) {
        	final float lw = this.mTickMarkStroke.getLineWidth();
            return (float) SGUtilityText.convertFromPoint(lw, unit);
        }
        
        public float getTickMarkLength(String unit) {
            return (float) SGUtilityText.convertFromPoint(this.mTickMarkLength, unit);
        }

        public boolean setXAxisLocation(int location) {
            if (location != SGIFigureElementAxis.AXIS_HORIZONTAL_1
                    && location != SGIFigureElementAxis.AXIS_HORIZONTAL_2) {
                return false;
            }
            this.mXAxis = SGFigureElementAxis.this.getAxisInPlane(location);
            return true;
        }

        public boolean setYAxisLocation(int location) {
            if (location != SGIFigureElementAxis.AXIS_PERPENDICULAR_1
                    && location != SGIFigureElementAxis.AXIS_PERPENDICULAR_2) {
                return false;
            }
            this.mYAxis = SGFigureElementAxis.this.getAxisInPlane(location);
            return true;
        }

        public boolean setXValue(final double value) {
            SGAxis axis = this.mXAxis;

            // current values gotten from the position of legend
            double currentValue = calcValue(getComponent().getX(), axis, true);
            currentValue = getNumberInRangeOrder(currentValue, axis);

            // if values from the dialog is diffrent from the current values,
            // set the values from the dialog
            float x;
            if (value == currentValue) {
                x = this.mX;
            } else {
                final float x_ = calcLocation(value, axis, true);
                x = (x_ - mGraphRectX) / mMagnification;
            }
            this.mX = x;

            return true;
        }

        public boolean setYValue(double value) {
            SGAxis axis = this.mYAxis;

            // current values gotten from the position of legend
            double currentValue = calcValue(getComponent().getY(), axis, false);
            currentValue = getNumberInRangeOrder(currentValue, axis);

            // if values from the dialog is diffrent from the current values,
            // set the values from the dialog
            float y;
            if (value == currentValue) {
                y = mY;
            } else {
                final float y_ = calcLocation(value, axis, false);
                y = (y_ - mGraphRectY) / mMagnification;
            }
            this.mY = y;

            return true;
        }

    	/**
    	 * Sets the bar width.
    	 * 
    	 * @param w
    	 *           the bar width to set
    	 * @return true if succeeded
    	 */
    	public boolean setBarWidth(final float w) {
    		if (w < 0.0f) {
    			throw new IllegalArgumentException("w < 0.0f");
    		}
    		this.mBarWidth = w;
    		return true;
    	}

    	/**
    	 * Sets the bar width in a given unit.
    	 * 
    	 * @param w
    	 *           the bar width to set in a given unit
    	 * @param unit
    	 *           an unit of length
    	 * @return true if succeeded
    	 */
        public boolean setBarWidth(final float w, String unit) {
    		final Float sNew = SGUtility.calcPropertyValue(w, unit,
    				COLOR_BAR_SIZE_UNIT, COLOR_BAR_WIDTH_MIN, COLOR_BAR_WIDTH_MAX,
    				COLOR_BAR_SIZE_MINIMAL_ORDER);
    		if (sNew == null) {
    			return false;
    		}
    		return this.setBarWidth(sNew.floatValue());
        }
        
    	/**
    	 * Sets the bar length.
    	 * 
    	 * @param len
    	 *           the bar length to set
    	 * @return true if succeeded
    	 */
    	public boolean setBarLength(final float len) {
    		if (len < 0.0f) {
    			throw new IllegalArgumentException("len < 0.0f");
    		}
    		this.mBarLength = len;
    		return true;
    	}

    	/**
    	 * Sets the bar length in a given unit.
    	 * 
    	 * @param len
    	 *           the bar length to set in a given unit
    	 * @param unit
    	 *           an unit of length
    	 * @return true if succeeded
    	 */
        public boolean setBarLength(final float len, final String unit) {
    		final Float sNew = SGUtility.calcPropertyValue(len, unit,
    				COLOR_BAR_SIZE_UNIT, COLOR_BAR_LENGTH_MIN, COLOR_BAR_LENGTH_MAX,
    				COLOR_BAR_SIZE_MINIMAL_ORDER);
    		if (sNew == null) {
    			return false;
    		}
    		return this.setBarLength(sNew.floatValue());
        }

    	/**
    	 * Sets the space to the scale numbers.
    	 * 
    	 * @param space
    	 *           the space to the scale numbers to set
    	 * @return true if succeeded
    	 */
        public boolean setSpaceToScale(final float space) {
        	this.mSpaceAxisLineAndNumber = space;
        	return true;
        }

    	/**
    	 * Sets the space to the scale numbers in a given unit.
    	 * 
    	 * @param space
    	 *           the space to the scale numbers to set in a given unit
    	 * @param unit
    	 *           an unit of length
    	 * @return true if succeeded
    	 */
        public boolean setSpaceToScale(float space, String unit) {
    		final Float sNew = SGUtility.calcPropertyValue(space, unit,
    				FIGURE_SPACE_UNIT, FIGURE_SPACE_TO_SCALE_MIN, FIGURE_SPACE_TO_SCALE_MAX,
    				MINIMAL_SPACE_ORDER);
    		if (sNew == null) {
    			return false;
    		}
    		return this.setSpaceToScale(sNew.floatValue());
        }

    	/**
    	 * Sets the space to the title.
    	 * 
    	 * @param space
    	 *           the space to the title to set
    	 * @return true if succeeded
    	 */
        public boolean setSpaceToTitle(final float space) {
        	this.mSpaceNumberAndTitle = space;
        	return true;
        }
        
    	/**
    	 * Sets the space to the title in a given unit.
    	 * 
    	 * @param space
    	 *           the space to the title to set in a given unit
    	 * @param unit
    	 *           an unit of length
    	 * @return true if succeeded
    	 */
        public boolean setSpaceToTitle(float space, String unit) {
    		final Float sNew = SGUtility.calcPropertyValue(space, unit,
    				FIGURE_SPACE_UNIT, FIGURE_SPACE_TO_TITLE_MIN, FIGURE_SPACE_TO_TITLE_MAX,
    				MINIMAL_SPACE_ORDER);
    		if (sNew == null) {
    			return false;
    		}
    		return this.setSpaceToTitle(sNew.floatValue());
        }

        /**
         * Sets the bar direction.
         * 
         * @param direction
         *           the direction of the color bar to set
    	 * @return true if succeeded
         */
        public boolean setDirection(final String direction) {
//        	if (SGUtilityText.isEqualString(DIRECTION_HORIZONTAL_LOWER, direction)
//        			|| SGUtilityText.isEqualString(DIRECTION_HORIZONTAL_UPPER, direction)
//        			|| SGUtilityText.isEqualString(DIRECTION_PERPENDICULAR_LEFT, direction)
//        			|| SGUtilityText.isEqualString(DIRECTION_PERPENDICULAR_RIGHT, direction)) {
//                this.mDirection = direction;
//                return true;
//        	} else {
//        		return false;
//        	}
        	
        	String dir;
        	if (SGUtilityText.isEqualString(DIRECTION_HORIZONTAL_LOWER, direction)) {
        		dir = DIRECTION_HORIZONTAL_LOWER;
        	} else if (SGUtilityText.isEqualString(DIRECTION_HORIZONTAL_UPPER, direction))  {
        		dir = DIRECTION_HORIZONTAL_UPPER;
        	} else if (SGUtilityText.isEqualString(DIRECTION_PERPENDICULAR_LEFT, direction))  {
        		dir = DIRECTION_PERPENDICULAR_LEFT;
        	} else if (SGUtilityText.isEqualString(DIRECTION_PERPENDICULAR_RIGHT, direction))  {
        		dir = DIRECTION_PERPENDICULAR_RIGHT;
        	} else {
        		return false;
        	}
            this.mDirection = dir;
        	return true;
        }

        /**
         * Sets to be reversed.
         * 
         * @param b
         *          true to be reversed
    	 * @return true if succeeded
         */
        public boolean setReversedOrder(final boolean b) {
            SGFigureElementAxis.this.mColorBarStyleManager.setReversedOrder(b);
            return true;
        }

        /**
         * Sets the color bar style.
         * 
         * @param style
         *           the color bar style
    	 * @return true if succeeded
         */
        public boolean setColorBarStyle(final String style) {
            SGColorBarModel model = SGFigureElementAxis.this.mColorBarStyleManager
            		.getColorBarModel(style);
            if (model == null) {
            	return false;
            }
            // set the current value range
            final double min = SGFigureElementAxis.this.mColorBarModel.getMinValue();
            final double max = SGFigureElementAxis.this.mColorBarModel.getMaxValue();
            final int scaleType = SGFigureElementAxis.this.mColorBarModel.getScaleType();
            model.setValueRange(min, max, scaleType);
            
            SGFigureElementAxis.this.mColorBarModel = model;
            return true;
        }
        
        /**
         * Sets the frame line width.
         * 
         * @param lw
         *           the frame line width to set
         * @return true if succeeded
         */
        public boolean setFrameLineWidth(final float lw) {
        	if (lw < 0.0f) {
        		throw new IllegalArgumentException("lw < 0.0f");
        	}
        	this.mFrameLineWidth = lw;
        	return true;
        }

        /**
         * Sets the frame line width in a given unit.
         * 
         * @param lw
         *           the frame line width to set in a given unit
         * @param unit
         *           an unit of length
         * @return true if succeeded
         */
        public boolean setFrameLineWidth(final float lw, final String unit) {
        	Float lwNew = SGUtility.getLineWidth(lw, unit);
        	if (lwNew == null) {
        		return false;
        	}
            return this.setFrameLineWidth(lwNew.floatValue());
        }

        public boolean setLineColor(Color cl) {
            this.mLineColor = cl;
            return true;
        }
        
        /**
         * Sets the tick mark width.
         * 
         * @param lw
         *           the tick mark width
         * @return true if succeeded
         */
        public boolean setTickMarkWidth(final float lw) {
            if (lw < 0.0f) {
                throw new IllegalArgumentException("lw < 0.0f");
            }
//            this.mTickMarkWidth = lw;
            this.mTickMarkStroke.setLineWidth(lw);
            return true;
        }

        /**
         * Sets the tick mark width in a given unit.
         * 
         * @param lw
         *          the tick mark width in a given unit
         * @param unit
         *          the unit of length
         * @return true if succeeded
         */
        public boolean setTickMarkWidth(final float lw, final String unit) {
            final Float lwNew = SGUtility.getLineWidth(lw, unit);
            if (lwNew == null) {
                return false;
            }
            return this.setTickMarkWidth(lwNew);
        }

        /**
         * Sets the tick mark length.
         * 
         * @param len
         *           the tick mark length
         * @return true if succeeded
         */
        public boolean setTickMarkLength(final float len) {
            if (len < 0.0f) {
                throw new IllegalArgumentException("len < 0.0f");
            }
            this.mTickMarkLength = len;
            return true;
        }

        /**
         * Sets the tick mark length in a given unit.
         * 
         * @param len
         *           the tick mark length in a given unit
         * @param unit
         *           the unit of length
         * @return true if succeeded
         */
        public boolean setTickMarkLength(final float len, final String unit) {
            final Float lNew = SGUtility.calcPropertyValue(len, unit, 
                    AXIS_TICK_LENGTH_UNIT, AXIS_TICK_LENGTH_MIN, AXIS_TICK_LENGTH_MAX, 
                    TICK_MARK_LENGTH_MINIMAL_ORDER);
            if (lNew == null) {
                return false;
            }
            return this.setTickMarkLength(lNew);
        }

        /**
         * Sets the font name.
         * 
         * @param name
         *           the font name to set
         * @return true if succeeded
         */
        public boolean setFontName(final String name) {
            this.mTitleFontName = name;
            this.mScaleFontName = name;
            return true;
        }

        /**
         * Sets the font size.
         * 
         * @param size
         *            font size of the title
         * @return true if succeeded
         */
        public boolean setFontSize(final float size) {
            this.mTitleFontSize = size;
            this.mScaleFontSize = size;
            return true;
        }

        /**
         * Sets the font size in a given unit.
         * 
         * @param size
         *           the font size to set in a given unit
         * @param unit
         *           the unit of length
         * @return true if succeeded
         */
        public boolean setFontSize(final float size, final String unit) {
            final Float sNew = SGUtility.getFontSize(size, unit);
            if (sNew == null) {
                return false;
            }
            return this.setFontSize(sNew);
        }

        /**
         * Sets the font style.
         * 
         * @param style
         *            font style
         * @return true if succeeded
         */
        public boolean setFontStyle(final int style) {
            this.mTitleFontStyle = style;
            this.mScaleFontStyle = style;
            return true;
        }

        /**
         * Sets the font color.
         * 
         * @param cl
         *          the font color to set
         * @return true if succeeded
         */
        public boolean setFontColor(final Color cl) {
        	if (cl == null) {
        		throw new IllegalArgumentException("cl == null");
        	}
            this.mFontColor = cl;
            return true;
        }
        
        /**
         * Sets the range and the scale type.
         * 
         * @param minValue
         *           the minimum value
         * @param maxValue
         *           the maximum value
         * @param scaleType
         *           the scale type
         * @return true if succeeded
         */
        public boolean setScale(final Number minValue, final Number maxValue, final Integer scaleType) {
            if (super.setScale(minValue, maxValue, scaleType) == false) {
                return false;
            }
            final double min = (minValue == null) ? mColorBarModel.getMinValue() : minValue.doubleValue();
            final double max = (maxValue == null) ? mColorBarModel.getMaxValue() : maxValue.doubleValue();
            final int type = (scaleType == null) ? mColorBarModel.getScaleType() : scaleType.intValue();
            SGFigureElementAxis.this.mColorBarModel.setValueRange(min, max, type);
            return true;
        }
        
        /**
         * 
         * @param value
         * @return
         */
        public boolean setInvertCoordinates(final boolean b) {
            if (super.setInvertCoordinates(b) == false) {
                return false;
            }
            SGFigureElementAxis.this.mColorBarModel.setInvertCoordinates(b);
            return true;
        }

        /**
         * Returns whether the observer has valid x axis value.
         * 
         * @param config
         *           configuration of the x-axis
         * @param value
         *           axis value
         * @return true for valid axis value
         */
        public boolean hasValidXAxisValue(final int config, final Number value) {
            final SGAxis axis = (config == -1) ? this.mXAxis : getAxisInPlane(config);
            final double v = (value != null) ? value.doubleValue() : this
                    .getXValue();
            return axis.isValidValue(v);
        }

        /**
         * Returns whether the observer has valid y axis value.
         * 
         * @param config
         *           configuration of the y-axis
         * @param value
         *           axis value
         * @return true for valid axis value
         */
        public boolean hasValidYAxisValue(final int config, final Number value) {
            final SGAxis axis = (config == -1) ? this.mYAxis : getAxisInPlane(config);
            final double v = (value != null) ? value.doubleValue() : this
                    .getYValue();
            return axis.isValidValue(v);
        }

        public SGPropertyDialog getPropertyDialog() {
            return mColorBarPropertyDialog;
        }
        
        /**
         * Sets to be selected the color bar.
         */
        protected boolean clicked(final MouseEvent e) {
            this.setSelected(true);
            return super.clicked(e);
        }

        /**
         * Commit the change of the properties.
         * @return
         *         true if succeeded
         */
        public boolean commit() {
        
            // compare two properties
            SGProperties pTemp = this.mTemporaryProperties;
            SGProperties pPresent = this.getProperties();
            if (pTemp.equals(pPresent) == false) {
                setChanged(true);
            }
            
            // clear temporary properties
            this.mTemporaryProperties = null;

            // update drawing elements
            if (this.createDrawingElements() == false) {
                return false;
            }
            repaint();
            notifyChange();

            return true;
        }

        /**
         * Cancel the setting of properties.
         * @return
         *         true if succeeded
         */
        public boolean cancel() {
        
            // set temporay properties to drawing elements to cancel the change
            if (this.setProperties(this.mTemporaryProperties) == false) {
                return false;
            }

            // clear temporay properties
            this.mTemporaryProperties = null;
            
            // update drawing elements
            if (this.createDrawingElements() == false) {
                return false;
            }
            repaint();
            notifyChange();

            return true;
        }

        private SGProperties mTemporaryProperties = null;

        public boolean prepare() {
            this.mTemporaryProperties = this.getProperties();
            return true;
        }

        public boolean preview() {
            if (this.createDrawingElements() == false) {
                return false;
            }
            repaint();
            notifyChange();
            return true;
        }
        
        public SGProperties getProperties() {
            ColorBarProperties p = new ColorBarProperties();
            if (this.getProperties(p) == false) {
                return null;
            }
            return p;
        }
        public boolean getProperties(SGProperties p) {
            if (super.getProperties(p) == false) {
                return false;
            }
            ColorBarProperties cp = (ColorBarProperties) p;
            cp.xAxis = this.mXAxis;
            cp.yAxis = this.mYAxis;
            cp.x = this.mX;
            cp.y = this.mY;
            cp.barWidth = this.mBarWidth;
            cp.barLength = this.mBarLength;
            cp.toScale = this.mSpaceAxisLineAndNumber;
            cp.toTitle = this.mSpaceNumberAndTitle;
            cp.direction = this.mDirection;
            cp.reversedOrder = this.isReversedOrder();
            cp.colorBarStyle = this.getColorBarStyle();
            cp.frameLineWidth = this.mFrameLineWidth;
//            cp.tickMarkWidth = this.mTickMarkWidth;
            cp.tickMarkWidth = this.mTickMarkStroke.getLineWidth();
            cp.tickMarkLength = this.mTickMarkLength;
            cp.lineColor = this.mLineColor;
            cp.fontName = this.mTitleFontName;
            cp.fontStyle = this.mTitleFontStyle;
            cp.fontSize = this.mTitleFontSize;
            cp.fontColor = this.mFontColor;
            return true;
        }
        
        public boolean setProperties(SGProperties p) {
            if (super.setProperties(p) == false) {
                return false;
            }
            ColorBarProperties cp = (ColorBarProperties) p;
            this.mXAxis = cp.xAxis;
            this.mYAxis = cp.yAxis;
            this.mX = cp.x;
            this.mY = cp.y;
            this.mBarWidth = cp.barWidth;
            this.mBarLength = cp.barLength;
            this.mSpaceAxisLineAndNumber = cp.toScale;
            this.mSpaceNumberAndTitle = cp.toTitle;
            this.mDirection = cp.direction;
            SGFigureElementAxis.this.mColorBarModel = mColorBarStyleManager.getColorBarModel(cp.colorBarStyle);
            SGFigureElementAxis.this.mColorBarModel.setValueRange(
                    this.mAxis.getMinValue(), this.mAxis.getMaxValue(), this.mAxis.getScaleType());
            this.setReversedOrder(cp.reversedOrder);
            this.mFrameLineWidth = cp.frameLineWidth;
//            this.mTickMarkWidth = cp.tickMarkWidth;
            this.mTickMarkStroke.setLineWidth(cp.tickMarkWidth);
            this.mTickMarkLength = cp.tickMarkLength;
            this.mLineColor = cp.lineColor;
            this.mTitleFontName = cp.fontName;
            this.mScaleFontName = cp.fontName;
            this.mTitleFontStyle = cp.fontStyle;
            this.mScaleFontStyle = cp.fontStyle;
            this.mTitleFontSize = cp.fontSize;
            this.mScaleFontSize = cp.fontSize;
            this.mFontColor = cp.fontColor;
            return true;
        }

        /**
         * The undo manager.
         */
        private SGUndoManager mUndoManager = new SGUndoManager(this);

        public SGProperties getMemento() {
            return this.getProperties();
        }

        public boolean setMemento(SGProperties p) {
            return this.setProperties(p);
        }
        
        public boolean isUndoable() {
            return this.mUndoManager.isUndoable();
        }

        public boolean isRedoable() {
            return this.mUndoManager.isRedoable();
        }

        public boolean initPropertiesHistory() {
            return this.mUndoManager.initPropertiesHistory();
        }

        public boolean setMementoBackward() {
            if (this.mUndoManager.setMementoBackward() == false) {
                return false;
            }
            if (this.createDrawingElements() == false) {
                return false;
            }
            notifyChange();
            return true;
        }

        public boolean setMementoForward() {
            if (this.mUndoManager.setMementoForward() == false) {
                return false;
            }
            if (this.createDrawingElements() == false) {
                return false;
            }
            notifyChange();
            return true;
        }

        public boolean undo() {
            return this.setMementoBackward();
        }

        public boolean redo() {
            return this.setMementoForward();
        }

        public boolean updateHistory() {
            return this.mUndoManager.updateHistory();
        }

        public void initUndoBuffer() {
            this.mUndoManager.initUndoBuffer();
        }

        public boolean isChanged() {
            return this.mUndoManager.isChanged();
        }

        public void setChanged(boolean b) {
            this.mUndoManager.setChanged(b);
        }

        public boolean deleteForwardHistory() {
            return this.mUndoManager.deleteForwardHistory();
        }

        public boolean isChangedRoot() {
            return this.isChanged();
        }

        public void clearChanged() {
            this.setChanged(false);
        }

        public void notifyToRoot() {
            SGFigureElementAxis.this.notifyToRoot();
        }

        /**
         * Returns the list of child nodes.
         * 
         * @return the list of child nodes
         */
        public ArrayList getChildNodes() {
            return new ArrayList();
        }

        /**
         * Returns a text string that denotes this class.
         * 
         * @return a text string that denotes this class
         */
        public String getClassDescription() {
            return "Color Bar";
        }

        /**
         * Returns a text string that denotes this instance.
         * 
         * @return a text string that denotes this instance
         */
        public String getInstanceDescription() {
            return "Color Bar";
        }
        
        /**
         * Returns the tag name.
         * 
         * @return the tag name
         */
        public String getTagName() {
            return SGIColorBarConstants.TAG_NAME_COLOR_BAR;
        }

        /**
         * Write the properties to an given Element object.
         * 
         * @param el
         *          an Element object
         * @return true if succeeded
         */
        public boolean writeProperty(final Element el) {

            final int digitColorBarSize = COLOR_BAR_SIZE_MINIMAL_ORDER - 1;
            final int digitLineWidth = SGIConstants.LINE_WIDTH_MINIMAL_ORDER - 1;
            final int digitFontSize = FONT_SIZE_MINIMAL_ORDER - 1;
            final int digitSpace = MINIMAL_SPACE_ORDER - 1;
            final int digitTickMarkLength = TICK_MARK_LENGTH_MINIMAL_ORDER - 1;
            
            final float barWidth = (float) SGUtilityNumber.roundOffNumber(
                    this.getBarWidth() * SGIConstants.CM_POINT_RATIO,
                    digitColorBarSize);
            final float barLength = (float) SGUtilityNumber.roundOffNumber(
                    this.getLength() * SGIConstants.CM_POINT_RATIO,
                    digitColorBarSize);
            final float toScale = (float) SGUtilityNumber.roundOffNumber(
                    this.mSpaceAxisLineAndNumber * SGIConstants.CM_POINT_RATIO,
                    digitSpace);
            final float toTitle = (float) SGUtilityNumber.roundOffNumber(
                    this.mSpaceNumberAndTitle * SGIConstants.CM_POINT_RATIO,
                    digitSpace);
            final float tickMarkWidth = (float) SGUtilityNumber
                    .roundOffNumber(this.mTickMarkStroke.getLineWidth(), digitLineWidth);
            final float tickMarkLength =  (float) SGUtilityNumber
                    .roundOffNumber(this.mTickMarkLength * SGIConstants.CM_POINT_RATIO,
                    digitTickMarkLength);
            final float frameLineWidth = (float) SGUtilityNumber
                    .roundOffNumber(this.mFrameLineWidth, digitLineWidth);
            final float fontSize = (float) SGUtilityNumber.roundOffNumber(
                    this.mTitleFontSize, digitFontSize);
            
            final String xAxisLocation = SGFigureElementAxis.this.getLocationName(this.mXAxis);
            final String yAxisLocation = SGFigureElementAxis.this.getLocationName(this.mYAxis);
            
            el.setAttribute(KEY_VISIBLE, Boolean.toString(this.mVisibleFlag));
            el.setAttribute(KEY_X_AXIS_POSITION, xAxisLocation);
            el.setAttribute(KEY_Y_AXIS_POSITION, yAxisLocation);
            el.setAttribute(KEY_X_VALUE, Double.toString(this.getXValue()));
            el.setAttribute(KEY_Y_VALUE, Double.toString(this.getYValue()));
            el.setAttribute(KEY_COLOR_BAR_WIDTH, Float.toString(barWidth) 
                    + COLOR_BAR_SIZE_UNIT);
            el.setAttribute(KEY_COLOR_BAR_LENGTH, Float.toString(barLength) 
                    + COLOR_BAR_SIZE_UNIT);
            el.setAttribute(KEY_COLOR_BAR_SPACE_TO_SCALE, Float.toString(toScale) 
                    + FIGURE_SPACE_UNIT);
            el.setAttribute(KEY_COLOR_BAR_SPACE_TO_TITLE, Float.toString(toTitle) 
                    + FIGURE_SPACE_UNIT);
            el.setAttribute(KEY_COLOR_BAR_DIRECTION, this.mDirection);
            el.setAttribute(KEY_COLOR_BAR_STYLE, this.getColorBarStyle());
            el.setAttribute(KEY_COLOR_BAR_REVERSED_ORDER, Boolean
                    .toString(mColorBarStyleManager.isReversedOrder()));
            el.setAttribute(KEY_FRAME_LINE_WIDTH, Float.toString(frameLineWidth)
                    + LINE_WIDTH_UNIT);
            el.setAttribute(KEY_COLOR_BAR_LINE_COLOR, 
                    SGUtilityText.getColorString(this.mLineColor));
            el.setAttribute(KEY_TICK_MARK_WIDTH, Float.toString(tickMarkWidth)
                    + LINE_WIDTH_UNIT);
            el.setAttribute(KEY_TICK_MARK_LENGTH, Float.toString(tickMarkLength)
                    + AXIS_TICK_LENGTH_UNIT);
            el.setAttribute(KEY_FONT_NAME, this.mTitleFontName);
            el.setAttribute(KEY_FONT_SIZE, Float.toString(fontSize)
                    + FONT_SIZE_UNIT);
            el.setAttribute(KEY_FONT_STYLE, SGUtilityText
                    .getFontStyleName(this.mTitleFontStyle));
            el.setAttribute(KEY_STRING_COLORS, SGUtilityText
                    .getColorString(this.mFontColor));

            if (this.writeCommonProperties(el) == false) {
                return false;
            }

            return true;
        }

        /**
         * Read properties from a given Element object.
         * 
         * @param element
         *           an Element object
         * @return true if succeeded
         */
        protected boolean readProperties(final Element element) {
            
            if (super.readCommonProperties(element) == false) {
                return false;
            }
            
            String str = null;
            Number num = null;
            Color cl = null;
            Boolean b = null;
            StringBuffer sb = new StringBuffer();

            // visible
            str = element.getAttribute(KEY_VISIBLE);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean visible = b.booleanValue();

            // x axis location
            str = element.getAttribute(KEY_X_AXIS_POSITION);
            if (str.length() != 0) {
                this.mXAxis = SGFigureElementAxis.this.getAxis(str);
            }

            // y axis location
            str = element.getAttribute(KEY_Y_AXIS_POSITION);
            if (str.length() != 0) {
                this.mYAxis = SGFigureElementAxis.this.getAxis(str);
            }
            
            // x value
            str = element.getAttribute(KEY_X_VALUE);
            if (str.length() == 0) {
                return false;
            }
            num = SGUtilityText.getDouble(str);
            if (num == null) {
                return false;
            }
            final double xValue = num.doubleValue();

            // y value
            str = element.getAttribute(KEY_Y_VALUE);
            if (str.length() == 0) {
                return false;
            }
            num = SGUtilityText.getDouble(str);
            if (num == null) {
                return false;
            }
            final double yValue = num.doubleValue();

            // bar width
            str = element.getAttribute(SGIColorBarConstants.KEY_COLOR_BAR_WIDTH);
            if (str.length() == 0) {
                return false;
            }
            sb.setLength(0);
            num = SGUtilityText.getNumber(str, sb);
            if (num == null) {
                return false;
            }
            final float barWidth = num.floatValue();

            // bar length
            str = element.getAttribute(SGIColorBarConstants.KEY_COLOR_BAR_LENGTH);
            if (str.length() == 0) {
                return false;
            }
            sb.setLength(0);
            num = SGUtilityText.getNumber(str, sb);
            if (num == null) {
                return false;
            }
            final float barLength = num.floatValue();

            // to scale
            str = element.getAttribute(SGIColorBarConstants.KEY_COLOR_BAR_SPACE_TO_SCALE);
            if (str.length() == 0) {
                return false;
            }
            sb.setLength(0);
            num = SGUtilityText.getNumber(str, sb);
            if (num == null) {
                return false;
            }
            final float toScale = num.floatValue();
            
            // to title
            str = element.getAttribute(SGIColorBarConstants.KEY_COLOR_BAR_SPACE_TO_TITLE);
            if (str.length() == 0) {
                return false;
            }
            sb.setLength(0);
            num = SGUtilityText.getNumber(str, sb);
            if (num == null) {
                return false;
            }
            final float toTitle = num.floatValue();
            
            // direction
            str = element.getAttribute(SGIColorBarConstants.KEY_COLOR_BAR_DIRECTION);
            if (str.length() == 0) {
                return false;
            }
            final String direction = str;
            
            // color bar style
            str = element.getAttribute(SGIColorBarConstants.KEY_COLOR_BAR_STYLE);
            if (mColorBarStyleManager.getColorBarModel(str) == null) {
                return false;
            }
            final String style = str;
            
            // reversed order
            str = element.getAttribute(SGIColorBarConstants.KEY_COLOR_BAR_REVERSED_ORDER);
            if (str.length() == 0) {
                return false;
            }
            b = SGUtilityText.getBoolean(str);
            if (b == null) {
                return false;
            }
            final boolean reversedOrder = b.booleanValue();
            
            // line width
            str = element.getAttribute(KEY_FRAME_LINE_WIDTH);
            if (str.length() == 0) {
                return false;
            }
            sb.setLength(0);
            num = SGUtilityText.getNumber(str, sb);
            if (num == null) {
                return false;
            }
            final float frameLineWidth = num.floatValue();

            // line color
            str = element.getAttribute(KEY_LINE_COLOR);
            if (str.length() == 0) {
                return false;
            }
            cl = SGUtilityText.parseColor(str);
            if (cl == null) {
                return false;
            }
            final Color lineColor = cl;
            
            // tick mark width
            str = element.getAttribute(KEY_TICK_MARK_WIDTH);
            if (str.length() == 0) {
                return false;
            }
            sb.setLength(0);
            num = SGUtilityText.getNumber(str, sb);
            if (num == null) {
                return false;
            }
            final float tickMarkWidth = num.floatValue();
            
            // tick mark length
            str = element.getAttribute(KEY_TICK_MARK_LENGTH);
            if (str.length() == 0) {
                return false;
            }
            sb.setLength(0);
            num = SGUtilityText.getNumber(str, sb);
            if (num == null) {
                return false;
            }
            final float tickMarkLength = num.floatValue();
            
            // font name
            str = element.getAttribute(KEY_FONT_NAME);
            if (str.length() == 0) {
                return false;
            }
            final String fontName = str;
            
            // font size
            str = element.getAttribute(KEY_FONT_SIZE);
            if (str.length() == 0) {
                return false;
            }
            sb.setLength(0);
            num = SGUtilityText.getNumber(str, sb);
            if (num == null) {
                return false;
            }
            final float fontSize = num.floatValue();
            
            // font style
            str = element.getAttribute(KEY_FONT_STYLE);
            if (str.length() == 0) {
                return false;
            }
            final Integer fontStyle = SGUtilityText.getFontStyle(str);
            if (fontStyle == null) {
                return false;
            }
            
            // font color
            str = element.getAttribute(KEY_STRING_COLORS);
            if (str.length() == 0) {
                return false;
            }
            cl = SGUtilityText.parseColor(str);
            if (cl == null) {
                return false;
            }
            final Color fontColor = cl;
            
            
            //
            // set properties
            //

            if (this.setAxisVisible(visible) == false) {
                return false;
            }

            if (this.setXValue(xValue) == false) {
                return false;
            }
            
            if (this.setYValue(yValue) == false) {
                return false;
            }
            
            if (this.setBarWidth(barWidth, SGIColorBarConstants.COLOR_BAR_SIZE_UNIT) == false) {
                return false;
            }

            if (this.setBarLength(barLength, SGIColorBarConstants.COLOR_BAR_SIZE_UNIT) == false) {
                return false;
            }
            
            if (this.setSpaceToScale(toScale, SGIFigureConstants.FIGURE_SPACE_UNIT) == false) {
                return false;
            }

            if (this.setSpaceToTitle(toTitle, SGIFigureConstants.FIGURE_SPACE_UNIT) == false) {
                return false;
            }

            if (this.setDirection(direction) == false) {
                return false;
            }
            
            if (this.setColorBarStyle(style) == false) {
                return false;
            }
            
            if (this.setReversedOrder(reversedOrder) == false) {
                return false;
            }
            
            if (this.setFrameLineWidth(frameLineWidth, LINE_WIDTH_UNIT) == false) {
                return false;
            }
            
            if (this.setTickMarkWidth(tickMarkWidth, LINE_WIDTH_UNIT) == false) {
                return false;
            }
            
            if (this.setTickMarkLength(tickMarkLength, AXIS_TICK_LENGTH_UNIT) == false) {
                return false;
            }
            
            if (this.setLineColor(lineColor) == false) {
                return false;
            }
            
            if (this.setFontName(fontName) == false) {
                return false;
            }
            
            if (this.setFontSize(fontSize, FONT_SIZE_UNIT) == false) {
                return false;
            }
            
            if (this.setFontStyle(fontStyle.intValue()) == false) {
                return false;
            }
            
            if (this.setFontColor(fontColor) == false) {
                return false;
            }

            return true;
        }

        /**
		 * Sets the properties of this axis.
		 * 
		 * @param map
		 *            a map of properties
		 * @param iResult
		 *            the input result
		 * @return the updated result of setting properties
		 */
        protected SGPropertyResults setProperties(SGPropertyMap map,
        		SGPropertyResults iResult) {
            SGPropertyResults result = (SGPropertyResults) iResult.clone();
            
            result = super.setProperties(map, result);
            if (result == null) {
            	return null;
            }
            
            Iterator<String> itr = map.getKeyIterator();
            while (itr.hasNext()) {
                String key = itr.next();
                String value = map.getValueString(key);

    			if (COM_COLOR_BAR_AXIS_X.equalsIgnoreCase(key)) {
    				final int loc = SGUtility.getAxisLocation(value);
    				if (loc == -1) {
    					result.putResult(COM_COLOR_BAR_AXIS_X,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (this.setXAxisLocation(loc) == false) {
    					result.putResult(COM_COLOR_BAR_AXIS_X,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				result
    						.putResult(COM_COLOR_BAR_AXIS_X,
    								SGPropertyResults.SUCCEEDED);
    			} else if (COM_COLOR_BAR_AXIS_Y.equalsIgnoreCase(key)) {
    				final int loc = SGUtility.getAxisLocation(value);
    				if (loc == -1) {
    					result.putResult(COM_COLOR_BAR_AXIS_Y,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (this.setYAxisLocation(loc) == false) {
    					result.putResult(COM_COLOR_BAR_AXIS_Y,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				result.putResult(COM_COLOR_BAR_AXIS_Y,
							SGPropertyResults.SUCCEEDED);
    			} else if (COM_COLOR_BAR_LOCATION_X.equalsIgnoreCase(key)) {
    				Double num = SGUtilityText.getDouble(value);
    				if (num == null) {
    					result.putResult(COM_COLOR_BAR_LOCATION_X,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
    					result.putResult(COM_COLOR_BAR_LOCATION_X,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (this.setXValue(num.doubleValue()) == false) {
    					result.putResult(COM_COLOR_BAR_LOCATION_X,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				result.putResult(COM_COLOR_BAR_LOCATION_X,
    						SGPropertyResults.SUCCEEDED);
    			} else if (COM_COLOR_BAR_LOCATION_Y.equalsIgnoreCase(key)) {
    				Double num = SGUtilityText.getDouble(value);
    				if (num == null) {
    					result.putResult(COM_COLOR_BAR_LOCATION_Y,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (SGUtility.isValidPropertyValue(num.doubleValue()) == false) {
    					result.putResult(COM_COLOR_BAR_LOCATION_Y,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (this.setYValue(num.doubleValue()) == false) {
    					result.putResult(COM_COLOR_BAR_LOCATION_Y,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				result.putResult(COM_COLOR_BAR_LOCATION_Y,
    						SGPropertyResults.SUCCEEDED);
    			} else if (COM_COLOR_BAR_WIDTH.equalsIgnoreCase(key)) {
    				StringBuffer unit = new StringBuffer();
    				Number num = SGUtilityText.getNumber(value, unit);
    				if (num == null) {
    					result.putResult(COM_COLOR_BAR_WIDTH,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (this.setBarWidth(num.floatValue(), unit.toString()) == false) {
    					result.putResult(COM_COLOR_BAR_WIDTH,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				result.putResult(COM_COLOR_BAR_WIDTH,
    						SGPropertyResults.SUCCEEDED);
    			} else if (COM_COLOR_BAR_LENGTH.equalsIgnoreCase(key)) {
    				StringBuffer unit = new StringBuffer();
    				Number num = SGUtilityText.getNumber(value, unit);
    				if (num == null) {
    					result.putResult(COM_COLOR_BAR_LENGTH,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (this.setBarLength(num.floatValue(), unit.toString()) == false) {
    					result.putResult(COM_COLOR_BAR_LENGTH,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				result.putResult(COM_COLOR_BAR_LENGTH,
    						SGPropertyResults.SUCCEEDED);
    			} else if (COM_COLOR_BAR_SPACE_TO_SCALE.equalsIgnoreCase(key)) {
    				StringBuffer unit = new StringBuffer();
    				Number num = SGUtilityText.getNumber(value, unit);
    				if (num == null) {
    					result.putResult(COM_COLOR_BAR_SPACE_TO_SCALE,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (this.setSpaceToScale(num.floatValue(), unit.toString()) == false) {
    					result.putResult(COM_COLOR_BAR_SPACE_TO_SCALE,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				result.putResult(COM_COLOR_BAR_SPACE_TO_SCALE,
    						SGPropertyResults.SUCCEEDED);
    			} else if (COM_COLOR_BAR_SPACE_TO_TITLE.equalsIgnoreCase(key)) {
    				StringBuffer unit = new StringBuffer();
    				Number num = SGUtilityText.getNumber(value, unit);
    				if (num == null) {
    					result.putResult(COM_COLOR_BAR_SPACE_TO_TITLE,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				if (this.setSpaceToTitle(num.floatValue(), unit.toString()) == false) {
    					result.putResult(COM_COLOR_BAR_SPACE_TO_TITLE,
    							SGPropertyResults.INVALID_INPUT_VALUE);
    					continue;
    				}
    				result.putResult(COM_COLOR_BAR_SPACE_TO_TITLE,
    						SGPropertyResults.SUCCEEDED);
                } else if (COM_COLOR_BAR_DIRECTION.equalsIgnoreCase(key)) {
                    if (this.setDirection(value) == false) {
                        result.putResult(COM_COLOR_BAR_DIRECTION,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_COLOR_BAR_DIRECTION,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_COLOR_BAR_STYLE.equalsIgnoreCase(key)) {
                	if (this.setColorBarStyle(value) == false) {
                        result.putResult(COM_COLOR_BAR_STYLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}
                    result.putResult(COM_COLOR_BAR_STYLE,
                            SGPropertyResults.SUCCEEDED);
    			} else if (COM_COLOR_BAR_REVERSED_ORDER.equalsIgnoreCase(key)) {
                    Boolean b = SGUtilityText.getBoolean(value);
                    if (b == null) {
                        result.putResult(COM_COLOR_BAR_REVERSED_ORDER,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setReversedOrder(b.booleanValue()) == false) {
                        result.putResult(COM_COLOR_BAR_REVERSED_ORDER,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_COLOR_BAR_REVERSED_ORDER,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_COLOR_BAR_FRAME_LINE_WIDTH.equalsIgnoreCase(key)) {
                    StringBuffer unit = new StringBuffer();
                    Number num = SGUtilityText.getNumber(value, unit);
                    if (num == null) {
                        result.putResult(COM_COLOR_BAR_FRAME_LINE_WIDTH,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setFrameLineWidth(num.floatValue(), unit.toString()) == false) {
                        result.putResult(COM_COLOR_BAR_FRAME_LINE_WIDTH,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_COLOR_BAR_FRAME_LINE_WIDTH,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_COLOR_BAR_TICK_MARK_WIDTH.equalsIgnoreCase(key)) {
                    StringBuffer unit = new StringBuffer();
                    Number num = SGUtilityText.getNumber(value, unit);
                    if (num == null) {
                        result.putResult(COM_COLOR_BAR_TICK_MARK_WIDTH,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setTickMarkWidth(num.floatValue(), unit.toString()) == false) {
                        result.putResult(COM_COLOR_BAR_TICK_MARK_WIDTH,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_COLOR_BAR_TICK_MARK_WIDTH,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_COLOR_BAR_TICK_MARK_LENGTH.equalsIgnoreCase(key)) {
                    StringBuffer unit = new StringBuffer();
                    Number num = SGUtilityText.getNumber(value, unit);
                    if (num == null) {
                        result.putResult(COM_COLOR_BAR_TICK_MARK_LENGTH,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setTickMarkLength(num.floatValue(), unit.toString()) == false) {
                        result.putResult(COM_COLOR_BAR_TICK_MARK_LENGTH,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_COLOR_BAR_TICK_MARK_LENGTH,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_COLOR_BAR_LINE_COLOR.equalsIgnoreCase(key)) {
                    Color cl = SGUtilityText.getColor(value);
                    if (cl != null) {
                        if (this.setLineColor(cl) == false) {
                            result.putResult(COM_COLOR_BAR_LINE_COLOR,
                                    SGPropertyResults.INVALID_INPUT_VALUE);
                            continue;
                        }
                    } else {
                    	cl = SGUtilityText.parseColor(value);
    					if (cl == null) {
    						result.putResult(COM_COLOR_BAR_LINE_COLOR,
    								SGPropertyResults.INVALID_INPUT_VALUE);
    						continue;
    					}
    					if (this.setLineColor(cl) == false) {
    						result.putResult(COM_COLOR_BAR_LINE_COLOR,
    								SGPropertyResults.INVALID_INPUT_VALUE);
    						continue;
    					}
                    }
                    result.putResult(COM_COLOR_BAR_LINE_COLOR,
                            SGPropertyResults.SUCCEEDED);
    			} else if (COM_COLOR_BAR_FONT_NAME.equalsIgnoreCase(key)) {
                	final String name = SGUtility.findFontFamilyName(value);
                	if (name == null) {
                        result.putResult(COM_COLOR_BAR_FONT_NAME, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}
                    if (this.setFontName(name) == false) {
                        result.putResult(COM_COLOR_BAR_FONT_NAME,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_COLOR_BAR_FONT_NAME,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_COLOR_BAR_FONT_STYLE.equalsIgnoreCase(key)) {
                    Integer style = SGUtilityText.getFontStyle(value);
                    if (style == null) {
                        result.putResult(COM_COLOR_BAR_FONT_STYLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setFontStyle(style.intValue()) == false) {
                        result.putResult(COM_COLOR_BAR_FONT_STYLE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_COLOR_BAR_FONT_STYLE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_COLOR_BAR_FONT_SIZE.equalsIgnoreCase(key)) {
                    StringBuffer unit = new StringBuffer();
                    Number num = SGUtilityText.getNumber(value, unit);
                    if (num == null) {
                        result.putResult(COM_COLOR_BAR_FONT_SIZE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    if (this.setFontSize(num.floatValue(), unit.toString()) == false) {
                        result.putResult(COM_COLOR_BAR_FONT_SIZE,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                    result.putResult(COM_COLOR_BAR_FONT_SIZE,
                            SGPropertyResults.SUCCEEDED);
                } else if (COM_COLOR_BAR_FONT_COLOR.equalsIgnoreCase(key)) {
                    Color cl = SGUtilityText.getColor(value);
                    if (cl != null) {
                        if (this.setFontColor(cl) == false) {
                            result.putResult(COM_COLOR_BAR_FONT_COLOR,
                                    SGPropertyResults.INVALID_INPUT_VALUE);
                            continue;
                        }
                    } else {
                    	cl = SGUtilityText.parseColor(value);
    					if (cl == null) {
    						result.putResult(COM_COLOR_BAR_FONT_COLOR,
    								SGPropertyResults.INVALID_INPUT_VALUE);
    						continue;
    					}
    					if (this.setFontColor(cl) == false) {
    						result.putResult(COM_COLOR_BAR_FONT_COLOR,
    								SGPropertyResults.INVALID_INPUT_VALUE);
    						continue;
    					}
                    }
                    result.putResult(COM_COLOR_BAR_FONT_COLOR,
                            SGPropertyResults.SUCCEEDED);
                }
            }
            
            return result;
        }
        
        /**
         * Sets the magnification.
         * 
         * @param mag
         *           the magnification
         * @return true if succeeded
         */
        public boolean setMagnification(final float mag) {
        	if (super.setMagnification(mag) == false) {
        		return false;
        	}
        	this.mTickMarkStroke.setMagnification(mag);
        	return true;
        }

        /**
         * Overrode to create a scale line instance.
         */
        protected ElementLineOfScale createScaleLineInstance(ElementLineOfScale el) {
        	return new ElementLineOfColorBarScale(el);
        }
        
        /**
         * Line for the tick marks in the color bar.
         */
        private class ElementLineOfColorBarScale extends ElementLineOfScale {
        	
        	ElementLineOfColorBarScale(final SGTuple2f start, final SGTuple2f end) {
                super(start, end);
            }

        	ElementLineOfColorBarScale(final ElementLineOfScale el) {
                super(el);
            }

            /**
             * Overrode to return the line color of the color bar.
             * 
             * @return the line color of the color bar
             */
            public Color getColor() {
                return ElementsGroupColorBar.this.mLineColor;
            }

            /**
             * Overrode to return the stroke for the tick marks in the color bar.
             * 
             * @return the stroke for the tick marks in the color bar
             */
            public SGStroke getStroke() {
            	return ElementsGroupColorBar.this.mTickMarkStroke;
            }
        }
        
    }
    
    /**
     * Properties for the color bar.
     *
     */
    public static class ColorBarProperties extends AxisIndividualProperties {
        
        SGAxis xAxis = null;

        SGAxis yAxis = null;

        float x;
        
        float y;
        
        float barWidth;
        
        float barLength;
        
        float toScale;
        
        float toTitle;
        
        String direction;
        
        boolean reversedOrder;
        
        String colorBarStyle;

        float frameLineWidth;
        
        float tickMarkWidth;
        
        float tickMarkLength;
        
        Color lineColor;

        String fontName;
        
        int fontStyle;
        
        float fontSize;
        
        Color fontColor;

        /**
         * Dispose this object.
         */
        public void dispose() {
            this.xAxis = null;
            this.yAxis = null;
            this.lineColor = null;
            this.fontColor = null;
            this.colorBarStyle = null;
            this.fontName = null;
            super.dispose();
        }

        public boolean equals(final Object obj) {
            if ((obj instanceof ColorBarProperties) == false) {
                return false;
            }
            if (super.equals(obj) == false) {
                return false;
            }
            ColorBarProperties p = (ColorBarProperties) obj;
            if (this.xAxis != null) {
                if (this.xAxis.equals(p.xAxis) == false) {
                    return false;
                }
            } else {
                if (p.xAxis != null) {
                    return false;
                }
            }
            if (this.yAxis != null) {
                if (this.yAxis.equals(p.yAxis) == false) {
                    return false;
                }
            } else {
                if (p.yAxis != null) {
                    return false;
                }
            }
            if (this.x != p.x) {
                return false;
            }
            if (this.y != p.y) {
                return false;
            }
            if (this.barWidth != p.barWidth) {
                return false;
            }
            if (this.barLength != p.barLength) {
                return false;
            }
            if (this.toScale != p.toScale) {
                return false;
            }
            if (this.toTitle != p.toTitle) {
                return false;
            }
            if (this.direction != p.direction) {
                return false;
            }
            if (this.colorBarStyle != null) {
                if (this.colorBarStyle.equals(p.colorBarStyle) == false) {
                    return false;
                }
            } else {
                if (p.colorBarStyle != null) {
                    return false;
                }
            }
            if (this.reversedOrder != p.reversedOrder) {
                return false;
            }
            if (this.frameLineWidth != p.frameLineWidth) {
                return false;
            }
            if (this.tickMarkWidth != p.tickMarkWidth) {
                return false;
            }
            if (this.tickMarkLength != p.tickMarkLength) {
                return false;
            }
            if (this.lineColor != null) {
                if (this.lineColor.equals(p.lineColor) == false) {
                    return false;
                }
            } else {
                if (p.lineColor != null) {
                    return false;
                }
            }
            if (this.fontName != null) {
                if (this.fontName.equals(p.fontName) == false) {
                    return false;
                }
            } else {
                if (p.fontName != null) {
                    return false;
                }
            }
            if (this.fontStyle != p.fontStyle) {
                return false;
            }
            if (this.fontSize != p.fontSize) {
                return false;
            }
            if (this.fontColor != null) {
                if (this.fontColor.equals(p.fontColor) == false) {
                    return false;
                }
            } else {
                if (p.fontColor != null) {
                    return false;
                }
            }
            
            return true;
        }
        
    }
    
    /**
     * Fits axis range to the focused data.
     * 
     * @param element
     *           a figure element for data
     * @return true if succeeded
     */
    public boolean fitAxisRangeToFocusedData(final SGIFigureElementForData element) {
        List<SGData> dList = element.getFocusedDataList();
        
        return this.fitAxisRangeToData(element, dList);
    }
    
    /**
     * Fits axis range to given data objects.
     * 
     * @param element
     *           a figure element for data
     * @param dList
     *           a list of data
     * @return true if succeeded
     */
    @Override
    public boolean fitAxisRangeToData(final SGIFigureElementForData element,
            final List<SGData> dList) {

        // creates a map of data and its display group-set
        List<SGIChildObject> elList = element.getVisibleChildList();
        Map<SGData, SGISXYAxisShiftEnabled> mapDataToShift = new HashMap<SGData, SGISXYAxisShiftEnabled>();
        for (int i = 0; i < elList.size(); i++) {
            SGIChildObject child = elList.get(i);
            if (child instanceof SGElementGroupSetInGraph) {
                SGElementGroupSetInGraph gs = (SGElementGroupSetInGraph)child;
                if (gs instanceof SGISXYAxisShiftEnabled) {
                    mapDataToShift.put(gs.getData(), (SGISXYAxisShiftEnabled)gs);
                }
            }
        }

        // creates a map of data-axis information list
        Map<SGData, List<SGDataAxisInfo>> axisInfoListMap = new HashMap<SGData, List<SGDataAxisInfo>>();
        for (int ii = 0; ii < dList.size(); ii++) {
            SGData data = dList.get(ii);
            List<SGDataAxisInfo> axisInfoList = element.getAxisInfoList(data);
            axisInfoListMap.put(data, axisInfoList);
        }

//        // check data types
//        boolean allSXY = true;
//        for (int ii = 0; ii < dList.size(); ii++) {
//        	SGData data = dList.get(ii);
//            if (!(data instanceof SGISXYData)) {
//                allSXY = false;
//                break;
//            }
//        }
//        boolean allVXY = true;
//        for (int ii = 0; ii < dList.size(); ii++) {
//        	SGData data = dList.get(ii);
//            if (!(data instanceof SGIVXYTypeData)) {
//                allVXY = false;
//                break;
//            }
//        }
//    	
//        // creates a map of axes and data ranges
//        Map<SGAxis, List<SGValueRange>> axisRangeListMap = new HashMap<SGAxis, List<SGValueRange>>();
//        for (int ii = 0; ii < dList.size(); ii++) {
//            SGData data = dList.get(ii);
//            List<SGDataAxisInfo> axisInfoList = axisInfoListMap.get(data);
//            for (int jj = 0; jj < axisInfoList.size(); jj++) {
//                SGDataAxisInfo axisInfo = axisInfoList.get(jj);
//                SGValueRange range = axisInfo.getRange();
//                final double minValue = range.getMinValue();
//                final double maxValue = range.getMaxValue();
//                if (Math.abs(maxValue - minValue) < Double.MIN_VALUE) {
//					range.setRange(
//							minValue - ALTERNATIVE_AXIS_VALUE, 
//							minValue + ALTERNATIVE_AXIS_VALUE);
//                }
//                SGAxis axis = axisInfo.getAxis();
//                if (this.isHorizontal(axis)) {
//                	if (allSXY) {
//                		range = this.calcRange(minValue, maxValue, 0.0);                		
//                	} else if (allVXY) {
//                		range = this.calcRange(minValue, maxValue, 0.10);
//                	}
//                } else if (this.isPerpendicular(axis)) {
//                	if (allSXY) {
//                		range = this.calcRange(minValue, maxValue, 0.05);                		
//                	} else if (allVXY) {
//                		range = this.calcRange(minValue, maxValue, 0.10);
//                	}
//                }
//                List<SGValueRange> list = axisRangeListMap.get(axis);
//                if (list == null) {
//                    list = new ArrayList<SGValueRange>();
//                    axisRangeListMap.put(axis, list);
//                }
//                
//                // contains shift movement if object shifts in SXY graph
//                SGISXYAxisShiftEnabled shiftObject = mapDataToShift.get(data);
//                if (null != shiftObject) {
//                    Double minShift = shiftObject.getShiftMinValue(axis);
//                    Double maxShift = shiftObject.getShiftMaxValue(axis);
//                    SGTuple2d tuple = range.getRange();
//                    double min = tuple.x;
//                    double max = tuple.y;
//                    if (null != minShift) {
//                        min = min + minShift.doubleValue();
//                    }
//                    if (null != maxShift) {
//                        max = max + maxShift.doubleValue();
//                    }
//                    Double barBaseline = shiftObject.getBarBaselineForAxis(axis);
//                    if (null != barBaseline) {
//                        if (min > barBaseline.doubleValue()) {
//                            min = barBaseline.doubleValue();
//                        }
//                        if (max < barBaseline.doubleValue()) {
//                            max = barBaseline.doubleValue();
//                        }
//                    }
//                    range.setRange(min, max);
//                }
//                list.add(range);
//            }
//        }
//        
//        boolean axesChanged = false;
//        boolean colorBarChanged = false;
//        List<ElementsGroupOfAxis> gList = this.getAllAxisGroupList();
//        for (int ii = 0; ii < gList.size(); ii++) {
//        	
//            // get minimum and maximum axis values
//            ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
//            SGAxis axis = group.mAxis;
//            List<SGValueRange> rangeList = axisRangeListMap.get(axis);
//            if (rangeList == null) {
//                continue;
//            }
//            double min = Double.MAX_VALUE;
//            double max = - Double.MAX_VALUE;
//            for (int jj = 0; jj < rangeList.size(); jj++) {
//            	SGValueRange range = rangeList.get(jj);
//            	final double minValue = range.getMinValue();
//            	final double maxValue = range.getMaxValue();
//                if (minValue < min) {
//                    min = minValue;
//                }
//                if (maxValue > max) {
//                    max = maxValue;
//                }
//            }
//            if (min >= max) {
//            	min = - SGIFigureElementAxis.ALTERNATIVE_AXIS_VALUE;
//            	max = SGIFigureElementAxis.ALTERNATIVE_AXIS_VALUE;
//            }
//
//            // set new axis range
//            final SGTuple2d rangeCur = axis.getRange();
//            final SGTuple2d rangeNew = new SGTuple2d(min, max);
//            if (axis.isValidValue(min) && axis.isValidValue(max)) {
//                axis.setScale(rangeNew);
//                if (rangeNew.equals(rangeCur) == false) {
//                    axesChanged = true;
//                    if (this.isColorBarAvailable() 
//                            && axis.equals(this.mZAxisElementsGroup.mAxis)) {
//                        colorBarChanged = true;
//                    }
//                }
//            }
//        }
        
        SGData[] dataArray = new SGData[dList.size()];
        dList.toArray(dataArray);
        
        FitAxisResult result = this.fitAxes(dataArray, axisInfoListMap, mapDataToShift);
        
        final boolean axesChanged = result.axesChanged;
        final boolean colorBarChanged = result.colorBarChanged;
        if (axesChanged || colorBarChanged) {
            // update the drawing elements
            this.createAllDrawingElements();
            this.notifyChange();

            // notify the changes
            if (axesChanged) {
                this.setChanged(true);
            }
            if (colorBarChanged) {
                if (this.isColorBarAvailable()) {
                    this.mZAxisElementsGroup.setChanged(true);
                }
            }
        }

        return true;
    }
    
    static class FitAxisResult {
    	boolean axesChanged = true;
    	boolean colorBarChanged = true;
    }
    
    
    private FitAxisResult fitAxes(SGData[] dataArray, Map<SGData, List<SGDataAxisInfo>> axisInfoListMap, 
    		Map<SGData, SGISXYAxisShiftEnabled> mapDataToShift) {
    	
    	FitAxisResult result = new FitAxisResult();
    	
        // check data types
        boolean allSXY = true;
        for (int ii = 0; ii < dataArray.length; ii++) {
            if (!(dataArray[ii] instanceof SGISXYData)) {
                allSXY = false;
                break;
            }
        }
        boolean allVXY = true;
        for (int ii = 0; ii < dataArray.length; ii++) {
        	SGData data = dataArray[ii];
            if (!(data instanceof SGIVXYTypeData)) {
                allVXY = false;
                break;
            }
        }

    	List<Double> xMinList = new ArrayList<Double>();
    	List<Double> xMaxList = new ArrayList<Double>();
    	List<Double> yMinList = new ArrayList<Double>();
    	List<Double> yMaxList = new ArrayList<Double>();
    	
        // creates a map of axes and data ranges
        Map<SGAxis, List<SGValueRange>> axisRangeListMap = new HashMap<SGAxis, List<SGValueRange>>();
        for (SGData data: dataArray) {
            List<SGDataAxisInfo> axisInfoList = axisInfoListMap.get(data);
            for (SGDataAxisInfo axisInfo : axisInfoList) {
                SGValueRange range = axisInfo.getRange();
                SGAxis axis = axisInfo.getAxis();
                List<SGValueRange> list = axisRangeListMap.get(axis);
                if (list == null) {
                    list = new ArrayList<SGValueRange>();
                    axisRangeListMap.put(axis, list);
                }
                
                // contains shift movement if object shifts in SXY graph
                SGISXYAxisShiftEnabled shiftObject = mapDataToShift.get(data);
                if (null != shiftObject) {
                    Double minShift = shiftObject.getShiftMinValue(axis);
                    Double maxShift = shiftObject.getShiftMaxValue(axis);
                    SGTuple2d tuple = range.getRange();
                    double min = tuple.x;
                    double max = tuple.y;
                    if (null != minShift) {
                        min = min + minShift.doubleValue();
                    }
                    if (null != maxShift) {
                        max = max + maxShift.doubleValue();
                    }
                    Double barBaseline = shiftObject.getBarBaselineForAxis(axis);
                    if (null != barBaseline) {
                        if (min > barBaseline.doubleValue()) {
                            min = barBaseline.doubleValue();
                        }
                        if (max < barBaseline.doubleValue()) {
                            max = barBaseline.doubleValue();
                        }
                    }
                    range.setRange(min, max);
                }
                
                list.add(range);
            }
            
            // get minimum values and maximum values
            if (allVXY) {
                for (SGDataAxisInfo axisInfo : axisInfoList) {
                    SGValueRange range = axisInfo.getRange();
                    SGAxis axis = axisInfo.getAxis();
                    if (this.isHorizontal(axis)) {
                        xMinList.add(Double.valueOf(range.getMinValue()));
                        xMaxList.add(Double.valueOf(range.getMaxValue()));
                    } else if (this.isPerpendicular(axis)) {
                        yMinList.add(Double.valueOf(range.getMinValue()));
                        yMaxList.add(Double.valueOf(range.getMaxValue()));
                    }
                }
            }
        }

        // set the same minimum and maximum values for VXY data
        if (allVXY) {
        	double[] xMinArray = SGUtilityNumber.toArray(xMinList);
        	double[] xMaxArray = SGUtilityNumber.toArray(xMaxList);
        	double[] yMinArray = SGUtilityNumber.toArray(yMinList);
        	double[] yMaxArray = SGUtilityNumber.toArray(yMaxList);
        	double xMin = SGUtilityNumber.min(xMinArray);
        	double xMax = SGUtilityNumber.max(xMaxArray);
        	double yMin = SGUtilityNumber.min(yMinArray);
        	double yMax = SGUtilityNumber.max(yMaxArray);
        	double xDiff = xMax - xMin;
        	double yDiff = yMax - yMin;
        	SGValueRange xRangeNew, yRangeNew;
        	if (xDiff < yDiff) {
        		xRangeNew = this.calcRange(xMin, xMin + yDiff, 0.0);
        		yRangeNew = this.calcRange(yMin, yMax, 0.0);
        	} else {
        		xRangeNew = this.calcRange(xMin, xMax, 0.0);
        		yRangeNew = this.calcRange(yMin, yMin + xDiff, 0.0);
        	}
            for (Entry<SGAxis, List<SGValueRange>> e : axisRangeListMap.entrySet()) {
            	SGAxis axis = e.getKey();
            	if (this.isHorizontal(axis)) {
                	List<SGValueRange> rangeList = e.getValue();
                	for (SGValueRange range : rangeList) {
                		range.setRange(xRangeNew);
                	}
            	} else if (this.isPerpendicular(axis)) {
                	List<SGValueRange> rangeList = e.getValue();
                	for (SGValueRange range : rangeList) {
                		range.setRange(yRangeNew);
                	}
            	}
            }
        }

        boolean axesChanged = false;
        boolean colorBarChanged = false;
        List<ElementsGroupOfAxis> gList = this.getAllAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
        	
            // get minimum and maximum axis values
            ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            SGAxis axis = group.mAxis;
            List<SGValueRange> rangeList = axisRangeListMap.get(axis);
            if (rangeList == null) {
                continue;
            }
            boolean bMin = false;
            boolean bMax = false;
            double min = Double.MAX_VALUE;
            double max = - Double.MAX_VALUE;
            for (int jj = 0; jj < rangeList.size(); jj++) {
            	SGValueRange range = rangeList.get(jj);
            	final double minValue = range.getMinValue();
            	final double maxValue = range.getMaxValue();
            	if (!Double.isNaN(minValue)) {
                    if (minValue < min) {
                        min = minValue;
                        bMin = true;
                    }
            	}
            	if (!Double.isNaN(maxValue)) {
                    if (maxValue > max) {
                        max = maxValue;
                        bMax = true;
                    }
            	}
            }
            if (!bMin && !bMax) {
            	min = - SGIFigureElementAxis.ALTERNATIVE_AXIS_VALUE;
            	max = SGIFigureElementAxis.ALTERNATIVE_AXIS_VALUE;
            } else {
            	if (!bMin) {
            		min = max - SGIFigureElementAxis.ALTERNATIVE_AXIS_VALUE;
            	}
            	if (!bMax) {
            		max = min + SGIFigureElementAxis.ALTERNATIVE_AXIS_VALUE;
            	}
            }
            
            SGValueRange range = null;
        	if (allSXY) {
                if (this.isHorizontal(axis)) {
            		range = this.calcRange(min, max, 0.0);
                } else if (this.isPerpendicular(axis)) {
            		range = this.calcRange(min, max, 0.05);
                }
        	} else if (allVXY) {
        		range = this.calcRange(min, max, 0.10);
        	}
            if (range != null) {
            	min = range.getMinValue();
            	max = range.getMaxValue();
            }
            
            if (min >= max) {
            	min = - SGIFigureElementAxis.ALTERNATIVE_AXIS_VALUE;
            	max = SGIFigureElementAxis.ALTERNATIVE_AXIS_VALUE;
            }

            // set new axis range
            final SGTuple2d rangeCur = axis.getRange();
            final SGTuple2d rangeNew = new SGTuple2d(min, max);
            if (axis.isValidValue(min) && axis.isValidValue(max)) {
                axis.setScale(rangeNew);
                if (rangeNew.equals(rangeCur) == false) {
                    axesChanged = true;
                    if (this.isColorBarAvailable() 
                            && axis.equals(this.mZAxisElementsGroup.mAxis)) {
                        colorBarChanged = true;
                    }
                }
            }
        }
        
        result.axesChanged = axesChanged;
        result.colorBarChanged = colorBarChanged;
        
        return result;
    }
    
    /**
     * Synchronize the axes to a given data object.
     * 
     * @param element
     *           a figure element for data
     * @param data
     *           a data object
     * @return true if succeeded
     */
    private boolean synchronizeAxis(final SGIFigureElementForData element, 
            final SGData data) {

        // get axis information
        List<SGDataAxisInfo> infoList = element.getAxisInfoList(data);
        
        // update the axis title
        List<ElementsGroupOfAxis> gList = this.getAllAxisGroupList();
        for (int ii = 0; ii < gList.size(); ii++) {
            ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList.get(ii);
            for (int jj = 0; jj < infoList.size(); jj++) {
                SGDataAxisInfo info = (SGDataAxisInfo) infoList.get(jj);
                if (info.getAxis().equals(group.mAxis)) {
                    final String title = info.getTitle();
                    final int location = info.getLocation();
                    String titleNew = this.createTitleString(title, location);
                    group.setTitle(titleNew);
                }
            }
        }

        // update the visibility of axis numbers and tick marks
		if (data instanceof SGISXYData) {
			SGISXYData sxy = (SGISXYData) data;
			final Boolean tickLabelHorizontal = sxy.isTickLabelHorizontal();
			if (sxy.isTickLabelAvailable()) {
				for (int ii = 0; ii < gList.size(); ii++) {
					ElementsGroupOfAxis group = (ElementsGroupOfAxis) gList
							.get(ii);
					for (int jj = 0; jj < infoList.size(); jj++) {
						SGDataAxisInfo info = (SGDataAxisInfo) infoList.get(jj);
						if (info.getAxis().equals(group.mAxis)) {
							final boolean visible = (this
									.isHorizontal(group.mAxis) != tickLabelHorizontal
									.booleanValue());
							group.setNumbersVisible(visible);
							group.setTickMarksVisible(visible);
						}
					}
				}
			}
		}

        // create all drawing elements
        if (this.createAllDrawingElements() == false) {
        	return false;
        }

        return true;
    }

    /**
     * Synchronizes axes with data column.
     * 
     * @param element
     *           figure element as an event source
     * @param msg
     *           the message
     */
    private void synchronizeDataColumnChange(final SGIFigureElementForData element, 
            final String msg) {
        if (SGIFigureElement.NOTIFY_DATA_COLUMN_CHANGE_ON_PREVIEW.equals(msg)
                || SGIFigureElement.NOTIFY_DATA_COLUMN_CHANGE_ON_COMMIT.equals(msg)) {

            // set temporary properties
            if (this.mTemporaryProperties == null) {
                this.mTemporaryProperties = this.getProperties();
            }

            // get current data
            List<SGData> dList = element.getFocusedDataList();
            if (dList.size() == 1) {
                SGData data = dList.get(0);
                
                // fit axis range
                this.fitAxisRangeToData(element, dList);
                
                // synchronize axis titles
                this.synchronizeAxis(element, data);
                
            } else if (dList.size() == 0) {
            	this.mTemporaryProperties = null;
            	return;
            } else {
                // this should not happen
                throw new Error("Data column selection for multiple data is not supported. dList.size="+dList.size());
            }
            
            // clear temporary properties on commit
            if (SGIFigureElement.NOTIFY_DATA_COLUMN_CHANGE_ON_COMMIT.equals(msg)) {
                
                SGProperties p = this.getProperties();
                if (p.equals(this.mTemporaryProperties) == false) {
                    this.setChanged(true);
                }
                
                // clear temporary properties
                this.mTemporaryProperties = null;
            }
            
        } else if (SGIFigureElement.NOTIFY_DATA_COLUMN_CHANGE_ON_CANCEL.equals(msg)) {

        	// recover the properties
        	this.setProperties(this.mTemporaryProperties);
        	this.createAllDrawingElements();
        	this.notifyChange();
        	this.repaint();
        	
            // clear temporary properties
            this.mTemporaryProperties = null;
        }
        
    }

    /**
     * Sets the common properties.
     * 
     * @param map
     *           a map of properties
     * @return the result of setting properties
     */
    public SGPropertyResults setProperties(SGPropertyMap map) {
        SGPropertyResults result = new SGPropertyResults();
        
        // prepare
        if (this.prepare() == false) {
            return null;
        }
        
        Iterator<String> itr = map.getKeyIterator();
        while (itr.hasNext()) {
            String key = itr.next();
            String value = map.getValueString(key);
            
            if (COM_AXIS_FONT_NAME.equalsIgnoreCase(key)) {
            	final String name = SGUtility.findFontFamilyName(value);
            	if (name == null) {
                    result.putResult(COM_AXIS_FONT_NAME, SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
            	}
                if (this.setTitleFontName(name) == false) {
                    result.putResult(COM_AXIS_FONT_NAME,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_AXIS_FONT_NAME,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_AXIS_FONT_STYLE.equalsIgnoreCase(key)) {
                Integer style = SGUtilityText.getFontStyle(value);
                if (style == null) {
                    result.putResult(COM_AXIS_FONT_STYLE,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setTitleFontStyle(style.intValue()) == false) {
                    result.putResult(COM_AXIS_FONT_STYLE,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_AXIS_FONT_STYLE,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_AXIS_FONT_SIZE.equalsIgnoreCase(key)) {
                StringBuffer unit = new StringBuffer();
                Number num = SGUtilityText.getNumber(value, unit);
                if (num == null) {
                    result.putResult(COM_AXIS_FONT_SIZE,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setTitleFontSize(num.floatValue(), unit.toString()) == false) {
                    result.putResult(COM_AXIS_FONT_SIZE,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_AXIS_FONT_SIZE,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_AXIS_FONT_COLOR.equalsIgnoreCase(key)) {
                Color cl = SGUtilityText.getColor(value);
                if (cl != null) {
                    if (this.setFontColor(cl) == false) {
                        result.putResult(COM_AXIS_FONT_COLOR,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                } else {
                	cl = SGUtilityText.parseColor(value);
                	if (cl == null) {
                        result.putResult(COM_AXIS_FONT_COLOR, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}
                    if (this.setFontColor(cl) == false) {
                        result.putResult(COM_AXIS_FONT_COLOR, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                }
                result.putResult(COM_AXIS_FONT_COLOR,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_AXIS_FRAME_VISIBLE.equalsIgnoreCase(key)) {
                Boolean b = SGUtilityText.getBoolean(value);
                if (b == null) {
                    result.putResult(COM_AXIS_FRAME_VISIBLE,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setFrameVisible(b.booleanValue()) == false) {
                    result.putResult(COM_AXIS_FRAME_VISIBLE,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_AXIS_FRAME_VISIBLE,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_AXIS_FRAME_LINE_WIDTH.equalsIgnoreCase(key)) {
                StringBuffer unit = new StringBuffer();
                Number num = SGUtilityText.getNumber(value, unit);
                if (num == null) {
                    result.putResult(COM_AXIS_FRAME_LINE_WIDTH,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setFrameLineWidth(num.floatValue(), unit.toString()) == false) {
                    result.putResult(COM_AXIS_FRAME_LINE_WIDTH,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_AXIS_FRAME_LINE_WIDTH,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_AXIS_FRAME_COLOR.equalsIgnoreCase(key)) {
                Color cl = SGUtilityText.getColor(value);
                if (cl != null) {
                    if (this.setFrameLineColor(cl) == false) {
                        result.putResult(COM_AXIS_FRAME_COLOR,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                } else {
                	cl = SGUtilityText.parseColor(value);
                	if (cl == null) {
                        result.putResult(COM_AXIS_FRAME_COLOR, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                	}
                    if (this.setFrameLineColor(cl) == false) {
                        result.putResult(COM_AXIS_FRAME_COLOR, SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                }
                result.putResult(COM_AXIS_FRAME_COLOR,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_AXIS_LINE_WIDTH.equalsIgnoreCase(key)) {
                StringBuffer unit = new StringBuffer();
                Number num = SGUtilityText.getNumber(value, unit);
                if (num == null) {
                    result.putResult(COM_AXIS_LINE_WIDTH,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setAxisLineWidth(num.floatValue(), unit.toString()) == false) {
                    result.putResult(COM_AXIS_LINE_WIDTH,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_AXIS_LINE_WIDTH,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_TICK_MARK_WIDTH.equalsIgnoreCase(key)) {
                StringBuffer unit = new StringBuffer();
                Number num = SGUtilityText.getNumber(value, unit);
                if (num == null) {
                    result.putResult(COM_TICK_MARK_WIDTH,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setTickMarkWidth(num.floatValue(), unit.toString()) == false) {
                    result.putResult(COM_TICK_MARK_WIDTH,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_TICK_MARK_WIDTH,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_TICK_MARK_LENGTH.equalsIgnoreCase(key)) {
                StringBuffer unit = new StringBuffer();
                Number num = SGUtilityText.getNumber(value, unit);
                if (num == null) {
                    result.putResult(COM_TICK_MARK_LENGTH,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                if (this.setTickMarkLength(num.floatValue(), unit.toString()) == false) {
                    result.putResult(COM_TICK_MARK_LENGTH,
                            SGPropertyResults.INVALID_INPUT_VALUE);
                    continue;
                }
                result.putResult(COM_TICK_MARK_LENGTH,
                        SGPropertyResults.SUCCEEDED);
            } else if (COM_AXIS_LINE_COLOR.equalsIgnoreCase(key)) {
                Color cl = SGUtilityText.getColor(value);
                if (cl != null) {
                    if (this.setLineColor(cl) == false) {
                        result.putResult(COM_AXIS_LINE_COLOR,
                                SGPropertyResults.INVALID_INPUT_VALUE);
                        continue;
                    }
                } else {
                	cl = SGUtilityText.parseColor(value);
					if (cl == null) {
						result.putResult(COM_AXIS_LINE_COLOR,
								SGPropertyResults.INVALID_INPUT_VALUE);
						continue;
					}
					if (this.setLineColor(cl) == false) {
						result.putResult(COM_AXIS_LINE_COLOR,
								SGPropertyResults.INVALID_INPUT_VALUE);
						continue;
					}
                }
                result.putResult(COM_AXIS_LINE_COLOR,
                        SGPropertyResults.SUCCEEDED);
            }
        }
        
        // commit the changes
        if (this.commit() == false) {
            return null;
        }
        this.notifyToRoot();
        this.notifyChange();
        this.repaint();
        
        return result;
    }

    /**
     * Sets the properties of each axis.
     * 
     * @param id
     *           the ID of child object
     * @param map
     *           a map of properties
     * @return the result of setting properties
     */
    public SGPropertyResults setChildProperties(final int id,
            SGPropertyMap map) {
    	List<ElementsGroupOfAxis> gList = this.getAllAxisGroupList();
        for (ElementsGroupOfAxis group : gList) {
            if (group.getLocation() == id) {
                return group.setProperties(map);
            }
        }
        return null;
    }

}
